(*---------------------------------------------------------------------------
   :Program.    Fish.mod
   :Author.     Fridtjof Siebert
   :Address.    Nobileweg 67
   :Phone.      0711/822509
   :Shortcut.   [fbs]
   :Version.    0.9
   :Date.       10-May-88
   :Copyright.  PD
   :Language.   Modula-II
   :Translator. M2Amiga
   :Imports.    none
   :Contents.   Ein Programm, das zeigt, wie man selbst Animation pro-
   :Contents.   grammieren kann.
   :Remark.     Es ist eine Umsetzung des gleichnamigen Lattice-C-Programms
   :Remark.     von Cathy J. Wagner (Fish Disk ??).
---------------------------------------------------------------------------*)

MODULE Fish;

FROM SYSTEM IMPORT ADR,ADDRESS,SHIFT;
FROM Graphics IMPORT GelsInfoPtr,VSpritePtr,CollTablePtr,BitMap,BitMapPtr,
       AnimOb,RastPortFlags,RastPortFlagSet,CollTable,VSprite,GelsInfo,
       InitGels,LoadRGB4,SetRast,InitBitMap,AllocRaster,BltClear,Animate,
       SortGList,WaitTOF,DrawGList,WaitBlit,VSpriteFlags,VSpriteFlagSet;
FROM Intuition IMPORT IntuiMessagePtr,IDCMPFlags,IDCMPFlagSet,MakeScreen,
       WindowFlagSet,RethinkDisplay;
FROM Exec IMPORT AllocMem,MemReqs,MemReqSet,FreeMem,GetMsg,ReplyMsg;

FROM FishStart IMPORT startgfx,endgfx,myscreen,view,vp,rp,window;
FROM FishInit IMPORT GInfo,head,tail,boom,animinit,freeanimobj;

CONST
  WIDTH   = 320;
  HEIGHT  = 200;
  DEPTH   = 3;

VAR
  bpdata: ARRAY[0..15] OF CARDINAL;

  bitmap: ARRAY[0..1] OF BitMap;


PROCEDURE show();

VAR
  dispIndex: INTEGER;
  i, i2, j,k: INTEGER;
  message: IntuiMessagePtr;
  animKey: ADDRESS;
  OldIBitMap: BitMapPtr;

BEGIN
  OldIBitMap := rp^.bitMap;

  boom := AllocMem(SIZE(CollTable),MemReqSet{public,memClear});
  IF boom = NIL THEN HALT END;

  head := AllocMem(SIZE(VSprite),MemReqSet{public,memClear});
  IF head = NIL THEN HALT END;
  tail := AllocMem(SIZE(VSprite),MemReqSet{public,memClear});
  IF tail = NIL THEN HALT END;

  GInfo := AllocMem(SIZE(GelsInfo),MemReqSet{public,memClear});
  IF GInfo = NIL THEN HALT END;

  WITH GInfo^ DO
    nextLine := NIL;
    lastColor := NIL;
    leftmost := 0;
    topmost := 0;
    rightmost := 160;
    bottommost := HEIGHT - 1;
    collHandler := boom;
    sprRsrvd := -1;
  END;
  rp^.gelsInfo := GInfo;
  InitGels(head,tail,GInfo);


  LoadRGB4(vp,ADR(bpdata[0]),16);
  SetRast(rp,0);

  FOR i:=0 TO 1 DO

    InitBitMap(bitmap[i],DEPTH,WIDTH,HEIGHT);

    FOR j:=0 TO DEPTH-1 DO

      bitmap[i].planes[j] := AllocRaster(WIDTH,HEIGHT);
      IF bitmap[i].planes[j] = NIL THEN HALT END;

      BltClear(bitmap[i].planes[j],SHIFT(SHIFT(WIDTH+15,-4),1)*HEIGHT,0);
    END;
  END;

  rp^.flags := RastPortFlagSet{dBuffer};
  rp^.mask := 255;

  vp^.rasInfo^.bitMap := ADR(bitmap[1]);
  rp^.bitMap := ADR(bitmap[1]);
  SetRast(rp,0);

  vp^.rasInfo^.bitMap := ADR(bitmap[0]);
  rp^.bitMap := ADR(bitmap[0]);
  SetRast(rp,0);

  animinit(ADR(animKey),vp,rp);

  dispIndex := 0;

  LOOP
    message := GetMsg(window^.userPort);
    WHILE message=NIL DO
      Animate(ADR(animKey), rp);
      SortGList(rp);
      WaitTOF();
      DrawGList(rp,vp);
      vp^.rasInfo^.bitMap := ADR(bitmap[dispIndex]);
      WaitTOF();
      MakeScreen(myscreen);
      WaitBlit();
      RethinkDisplay();
      dispIndex := 1-dispIndex;
      rp^.bitMap := ADR(bitmap[dispIndex]);
      message := GetMsg(window^.userPort);
    END;

    IF message^.class = IDCMPFlagSet{closeWindow} THEN
      ReplyMsg(message);
      freeanimobj(rp);
      FOR i:=0 TO 1 DO
        FOR j:=0 TO DEPTH-1 DO
          FreeMem(bitmap[i].planes[j],SHIFT(SHIFT(WIDTH+15,-4),1)*HEIGHT);
        END;
      END;
      FreeMem(boom,SIZE(CollTable));
      FreeMem(head,SIZE(VSprite));
      FreeMem(tail,SIZE(VSprite));
      FreeMem(GInfo,SIZE(GelsInfo));
      vp^.rasInfo^.bitMap := OldIBitMap;
      rp^.bitMap := OldIBitMap;
      EXIT;
    END;
    ReplyMsg(message);
  END;
END show;

PROCEDURE delay();

VAR
  i,x: INTEGER;

BEGIN
  FOR i:=0 TO 9999 DO END;
END delay;

BEGIN
  bpdata[ 0] := 000H; bpdata[ 1] := 5EFH; bpdata[ 2] := 128H; bpdata[ 3] := 0D81H;
  bpdata[ 4] := 326H; bpdata[ 5] := 203H; bpdata[ 6] := 086H; bpdata[ 7] := 044H;
  bpdata[ 8] := 5EFH; bpdata[ 9] := 633H; bpdata[10] := 855H; bpdata[11] := 857H;
  bpdata[12] := 065H; bpdata[13] := 669H; bpdata[14] := 223H; bpdata[15] := 547H;

  startgfx(320,80,50,200,2,NIL,ADR("FRIDI'S FISH"),WindowFlagSet{},NIL);

  show();

  endgfx();

END Fish.
