(*---------------------------------------------------------------------------
   :Program.    ShowInputEvents
   :Author.     Fridtjof Bjrn Siebert
   :Address.    Nobileweg 67, D-7-Stgt-40
   :Phone.      (0)711/822509
   :Shortcut.   [fbs]
   :Version.    1.0
   :Date.       31-May-88
   :Copyright.  PD
   :Language.   MODULA-II
   :Translator. M2Amiga
   :Contents.   Modula-2 Version des Demo-Prgs zum Input-Device aus dem
   :Contents.   Rom Reference Manula Libraries & Devices, S.328 ff.
   :Remark.     Fehler in Programm verbessert: Forbid durch Disable ersetzt!
---------------------------------------------------------------------------*)

MODULE ShowInputEvents;

FROM SYSTEM      IMPORT ADR, ADDRESS, SETREG, REG, SHIFT, LONGSET, CAST;
FROM Arts        IMPORT Assert, detectCtrlC;

FROM Dos         IMPORT Delay;
FROM Exec        IMPORT MsgPortPtr, IOStdReq, Interrupt, MemEntry, Forbid,
                        Permit, IOStdReqPtr, OpenDevice, CloseDevice, DoIO,
                        Disable, Enable;
FROM ExecSupport IMPORT CreatePort, DeletePort, CreateStdIO, DeleteStdIO;
FROM Input       IMPORT inputName, addHandler, remHandler;
FROM InputEvent  IMPORT InputEvent, InputEventPtr, Class;

FROM InOut       IMPORT WriteString, WriteLn, WriteCard, WriteInt;

CONST
  F1KeyUp = 0D0H;

VAR
  CopyEvent, OutPutEvent: InputEvent;
  InputDevPort: MsgPortPtr;
  InputRequestBlock: IOStdReqPtr;
  HandlerStuff: Interrupt;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                          Input  Handler:                                *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

(* It's Variables are global (don't know if allocating them scratches      *)
(* A0 or A1. Who knows ? Who's got a M2Amiga-Debugger ??? Or a even        *)
(* a Disassembler, like TDI's ??                                          *)

VAR
  EV: InputEventPtr;
  Result: InputEventPtr;

PROCEDURE MyHandler();
(* That's my Input-Handler it receives 2 values, though in MODULA-2 it     *)
(* looks like a Procedure receiving and returning nothing. Anyway, there's *)
(* a Pointer returned:                                                     *)
(* A0: Pointer to EventChain the Handler should handle with.               *)
(* A1: This points to my MemEntries, i.e. contains HandlerStuff.date       *)
(* D0: Will contain the new Pointer to Eventlist or NIL if none.           *)

(* $S- This avoids checking Stack size. This Procedure gets another Stack  *)
(* than the main Programm !!                                               *)

BEGIN
  EV := ADDRESS(REG(8)); (* this gets InputEventList from A0 *)
    (* A1 is ignored here 'cause there no memory needed *)

  IF EV^.class = timer THEN
  (* this shouldn't be timer, 'cause there are too many TimerEvents (10/s) *)
    Result := NIL;
  ELSE
    Forbid();
  (* no Task switching. Else a half-changed Event could be printed *)
    CopyEvent := EV^;
    Permit();
    Result := EV;
  END;
  SETREG(0,EV);
END MyHandler;
(* that's it. I hope it to work !!! *)

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*           Main Programm (initialization, Printing and CleanUp)          *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

VAR
  error : CARDINAL;
  oldsecs,oldmicro: LONGCARD;
  oldclass: Class;

BEGIN
  detectCtrlC := FALSE;
  InputDevPort := CreatePort(NIL,0);
  Assert(InputDevPort#NIL,ADR("CreatePort failed"));
  InputRequestBlock := CreateStdIO(InputDevPort);
  Assert(InputRequestBlock#NIL,ADR("CreateStdIO failed"));
  WITH HandlerStuff DO
    data := NIL;             (* pointer to it's data (I don't have any)  *)
    code := ADR(MyHandler);  (* Thats my Handle-Procedure *)
    node.pri := 51;          (* Priority higher than Intuition's (50) *)
  END;
  OpenDevice(ADR(inputName),0,InputRequestBlock,LONGSET{});
  WriteString("Opened the Input-Device"); WriteLn;
  WITH InputRequestBlock^ DO
    command := addHandler;
    data := ADR(HandlerStuff);
  END;
  DoIO(InputRequestBlock);
  WITH CopyEvent DO
    timeStamp.secs := 0;
    timeStamp.micro:= 0;
    class := null;
    oldsecs := 0;
    oldmicro := 0;
    oldclass := null;
  END;
  LOOP
    Delay(5);
    WITH CopyEvent DO
      IF (class = rawkey) AND (code=F1KeyUp) THEN EXIT END;
      IF (timeStamp.secs#oldsecs) OR (timeStamp.micro#oldmicro) OR
        (class#oldclass) THEN
        Disable(); (* this differs from Libraries&Devices Reference Manual *)
        (* the Handler is an Interrupt and can't be stopped with Forbid !  *)
        OutPutEvent := CopyEvent;
        Enable();
        WITH OutPutEvent DO;
          oldsecs := timeStamp.secs;
          oldmicro := timeStamp.micro;
          oldclass := class;
          WriteString("New Input-Event:"); WriteLn;
          WriteString("class: "); WriteInt(ORD(class),6); WriteLn;
          WriteString("subClass: "); WriteInt(ORD(subClass),6); WriteLn;
          WriteString("Code: "); WriteCard(code,6); WriteLn;
          WriteString("Qualifier: "); WriteCard(CAST(CARDINAL,qualifier),6);
          WriteLn;
          WriteString("x: "); WriteInt(x,6); WriteLn;
          WriteString("y: "); WriteInt(y,6); WriteLn;
          WriteString("secs: "); WriteCard(timeStamp.secs,12); WriteLn;
          WriteString("micro: "); WriteCard(timeStamp.micro,12); WriteLn;
        END;
      END; (* IF CopyEvent#old *)
    END;   (* WITH *)
  END;   (* LOOP *)

  WITH InputRequestBlock^ DO
    command := remHandler;
    data := ADR(HandlerStuff);
  END;
  DoIO(InputRequestBlock);
  DeleteStdIO(InputRequestBlock);
  DeletePort(InputDevPort);
END ShowInputEvents.
