MODULE M2Code;

(*-------------------------------------------------------------------------*)
(*                            -----------------                            *)
(*                -   -   -   -  M 2 C O D E  -  -  -  -  -                *)
(*                            -----------------                            *)
(*                                                                         *)
(*             Wandelt relativen (!) Maschinencodeprogramme in             *)
(*              TDI-Modula-2 Source- and DefinitionModule um.              *)
(*                                                                         *)
(*   1988 by Fridtjof Siebert                                             *)
(*            Nobileweg 67                                                 *)
(*            7000 Stuttgart 40 (Stammheim)                                *)
(*            Germany                                                      *)
(*     Phone: 0711/822509                                                  *)
(*                                                                         *)
(*  Usage:                                                                 *)
(*  M2Code Src [-cRx:Typ] [-vRx:Typ] [-rRx:Typ] [-sRx] [-sall] [-d] [-p]   *)
(*           (Rx: 68000-Register (D0..D7/A0..A7);)                         *)
(*           (Typ: Modula Type)                                            *)
(*                                                                         *)
(*      -cRx:  konstanter Parameter wird in Rx bergeben.                  *)
(*      -vRx:  Variabler Parameter in Rx bergeben.                        *)
(*      -rRx:  Ergebnis wird aus Register Rx bergeben.                    *)
(*      -sRx:  Die Rx mu zwischengespeichert werden (bei D2..D7 & A2..A7) *)
(*      -sall: Alle Register zwischenspeichern (auer a7)                  *)
(*      -d:    Letztes Wort lschen, wenn es 0 ist.                        *)
(*      -p:    Entry- & Exit-Code weglassen (Prg mu mit RTS aufhren)     *)
(*                                                                         *)
(*  Beispiel:                                                              *)
(*   M2Code  -vA0:ADDRESS -cD0:WORD -rD1:BYTE -sD3 -sD4                    *)
(*     erzeugt folgende Procedur:                                          *)
(*  PROCEDURE (VAR a:ADDRESS; b:WORD):BYTE                                 *)
(*     D3 und D4 werden auf den Stack gerettet.                            *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

(*------  Importlist:  ------*)

FROM SYSTEM IMPORT SETREG,CODE,REGISTER,NULL,ADR,ADDRESS,BYTE,ASH;
FROM CommandLine IMPORT GetCL,CLStrings;
FROM DOSFiles IMPORT Open,Close,Read,Write,FileHandle,ModeOldFile,
      ModeNewFile;
FROM DOSLibrary IMPORT DOSName,DOSBase;
FROM Libraries IMPORT OpenLibrary,CloseLibrary;
FROM Memory IMPORT AllocMem,FreeMem,MemReqSet,MemChip,MemClear;
FROM InOut IMPORT WriteString,WriteLn;
FROM Strings IMPORT CompareResults,Delete,Copy,Concat,Compare,Length,Pos,
      SetTerminator,Assign,Insert;
FROM Conversions IMPORT ConvertToString;

(*------  Types:  ------*)

TYPE
  String2 = ARRAY[0..2] OF CHAR;

(*------  Variables:  ------*)

VAR
  argc: CARDINAL;                     (* count args *)
  argv: ARRAY[0..32] OF CLStrings;    (* the arguments *)
  ParaCount: CARDINAL;                (* Counts Parameters for Procedure *)
  ParaTypes: ARRAY[0..15] OF ARRAY[0..39] OF CHAR; (* the Types *)
  ParaRegs: ARRAY[0..15] OF CARDINAL; (* The Registers for the Parameters *)
  ParaVar: ARRAY[0..15] OF BOOLEAN;   (* VAR or CONST ? *)
  ResultReg: INTEGER;                 (* Register for result *)
  ResultType: ARRAY[0..39] OF CHAR;   (* Type of result *)
  i: CARDINAL;                        (* no special use *)
  CommString: String2;                (* the first 2 chars of a argv *)
  RegString: String2;                 (* the third and fourth char *)
  SaveRegs: BITSET;                   (* Registers to be saved *)
  LoadRegs: BITSET;                   (* Registers to be loaded *)
  DoSave: BOOLEAN;                    (* Save or not ? *)
  DelZero: BOOLEAN;                   (* Delete last word if it's a zero *)
  NoEntryCode: BOOLEAN;               (* Don't create entry & exit code *)
  Char: ARRAY[0..0] OF CHAR;          (* for changing a Char to a Array of Chars *)
  SourceFile, ModName, ImplFile, DefFile: CLStrings; (* The Files *)
  InBuffer: POINTER TO CARDINAL;      (* Buffer for Input *)
  OutBuffer: POINTER TO ARRAY[0..255] OF CHAR; (* Outputbuffer *)
  InH,OutH: FileHandle;               (* FileHandles for I/O *)
  len: LONGCARD;                      (* for saving Writes's result *)
  RegNumString: ARRAY[0..39] OF CHAR; (* the Converted Register-Number *)
  Hex: ARRAY[0..39] OF CHAR;          (* for Hex-Numbers *)
  ok,ok2: BOOLEAN;                    (* for getting boolean results *)
  code: CARDINAL;                     (* for saving read machinecode *)

(*------  Convert Registerstring (D0,a5 etc) into number (0..15) :  -------*)

PROCEDURE CalcReg(RegStr: String2): CARDINAL;

VAR
  Reg: CARDINAL;

BEGIN
  IF (RegStr[0]="d") OR (RegStr[0]="D") THEN Reg := 0;
  ELSIF (RegStr[0]="a") OR (RegStr[0]="A") THEN Reg := 8;
  ELSE WriteString("Unknown Register"); WriteLn(); HALT; END;
  Reg := Reg + ORD(RegStr[1]) - ORD("0");
  IF Reg>15 THEN WriteString("Unknown Register"); WriteLn(); HALT; END;
  RETURN Reg;
END CalcReg;

(*---------------------  String ausgeben:  --------------------------------*)

PROCEDURE WriteBuf(Lines:CARDINAL);

BEGIN
  IF Write(OutH,OutBuffer,LONGCARD(Length(OutBuffer^)))= -1D THEN
    Close(OutH);
    IF InH#0 THEN Close(InH) END;
    WriteString("Error while writing"); WriteLn; HALT;
  END;
  OutBuffer^[0] := CHR(10); len := Write(OutH,OutBuffer,1D);
  WHILE Lines>1 DO
    len := Write(OutH,OutBuffer,1D); DEC(Lines);
  END;
END WriteBuf;

(*-------------------  LineFeed ausgeben:  --------------------------------*)

PROCEDURE LF(Lines: CARDINAL);

BEGIN
  OutBuffer^[0] := CHR(10); len := Write(OutH,OutBuffer,1D);
  WHILE Lines>1 DO
    len := Write(OutH,OutBuffer,1D); DEC(Lines);
  END;
END LF;

(*-------------------------  Start  ---------------------------------------*)

BEGIN

(*------  Get Commandline:  ------*)

IF GetCL(argc,argv)=FALSE THEN WriteString("Too many parameters"); WriteLn(); HALT; END;

(*------  No Parameters? Then type Usage:  ------*)

IF argc=0 THEN
  WriteString("Usage:"); WriteLn;
  WriteString("M2Code [-cRx:Type] [-vRx:Type] [-rRx:Type] [-sRx] [-sall] [-d]"); WriteLn;
  WriteString(" (Rx: 68000-Register (d0..a7), Type: Modula TYPE-name)");WriteLn;
  WriteLn;
  WriteString("  -c    : constant Parameter"); WriteLn;
  WriteString("  -v    : variable Parameter"); WriteLn;
  WriteString("  -r    : Result");             WriteLn;
  WriteString("  -s    : save Register ");     WriteLn;
  WriteString("  -sall : save all Registers (apart from a7)"); WriteLn;
  WriteString("  -d    : delete last 2 Bytes from Source, if they are zeroes"); WriteLn;
  WriteString("  -p    : no entry and exit-code for the procedure"); WriteLn;
  WriteString(" This utility was developed using TDI's Modula-2/Amiga."); WriteLn;
  WriteString("  1988 by Fridtjof Siebert, Nobileweg 67, 7000 Stgt-40, Germany"); WriteLn;
  WriteString("    Phone: 0711/822509"); WriteLn;
  HALT;
END;

(*------  create Filename:  ------*)

SourceFile := argv[0];
IF Pos(SourceFile,".",0,i) THEN
  Copy(SourceFile,0,i,ModName);     (* Delete Suffix *)
ELSE
  ModName := SourceFile;
END;
Concat(ModName,".mod",ImplFile);    (* add .mod for Implementation *)
Concat(ModName,".def",DefFile);     (* add .def for Definition *)

(*------  Rest der Commandline auswerten:  ------*)

DoSave := FALSE; SaveRegs := {}; LoadRegs := {}; ResultReg := -1;
DelZero := FALSE; NoEntryCode := FALSE; (* default settings *)

ParaCount := 0; i := 1; (* init loop *)
WHILE i<argc DO
  Copy (argv[i],0,2,CommString);            (* `-x' nach CommString *)
  Copy (argv[i],2,2,RegString);             (* `Rx' nach RegString  *)
  IF Compare("-sall",argv[i])= Equal THEN   (* alle Regs speichern *)
    SaveRegs := {1..15}; LoadRegs := {0..14}; DoSave:= TRUE;
  ELSIF Compare("-s",CommString)=Equal THEN (* ein Reg speichern:  *)
    SaveRegs := SaveRegs + BITSET(ASH(1,15-CalcReg(RegString)));
    LoadRegs := LoadRegs + BITSET(ASH(1,CalcReg(RegString)));
    DoSave := TRUE;
  ELSIF Compare("-d",CommString)=Equal THEN (* Nullwort lschen *)
    DelZero := TRUE;
  ELSIF Compare("-p",CommString)=Equal THEN (* no Entry&Exit Code *)
    NoEntryCode := TRUE;
  ELSIF Compare("-r",CommString)=Equal THEN (* Result in Rx *)
    ResultReg := CalcReg(RegString);
    Copy(argv[i],4,Length(argv[i])-4,ResultType);
  ELSE
    IF Compare("-c",CommString)=Equal THEN  (* Konstante bergeben: *)
      ParaRegs[ParaCount] := CalcReg(RegString);
      ParaVar[ParaCount] := FALSE;
      Copy(argv[i],4,Length(argv[i])-4,ParaTypes[ParaCount]);
    ELSIF Compare("-v",CommString)=Equal THEN (* Variable bergeben: *)
      ParaRegs[ParaCount] := CalcReg(RegString);
      ParaVar[ParaCount] := TRUE;
      Copy(argv[i],4,Length(argv[i])-4,ParaTypes[ParaCount]);
    ELSE                                    (* unkorrekter Parameter: *)
      WriteString("Unknown Parameter"); WriteLn(); HALT;
    END;
    INC(ParaCount);  (* increament number of Parameters *)
    IF ParaCount=17 THEN WriteString("Too Many Parameters"); WriteLn; HALT; END;
  END;
  INC(i);
END;

DOSBase:= OpenLibrary(DOSName,0); (* Dos Library ffnen *)

InBuffer := AllocMem(272,MemReqSet{MemChip,MemClear}); (* Speicher *)
OutBuffer := ADDRESS(LONGCARD(InBuffer) + 16D);

(*------  Definition Module erzeugen:  ------*)

OutH := Open(DefFile,ModeNewFile);    (* Open defmodule for writing *)
IF OutH=0 THEN WriteString("Couldn't open "); WriteString(DefFile); WriteLn; HALT; END;

WriteLn; WriteString("Creating Definition Module: "); WriteString(DefFile);
WriteLn; WriteLn; InH := 0;

(*------  First Line:  ------*)

Assign(OutBuffer^,"DEFINITION MODULE ");
 Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,";",OutBuffer^); WriteBuf(2);

(*------  ImportList: ------*)

Assign(OutBuffer^,"FROM SYSTEM IMPORT BYTE,WORD,LONGWORD,ADDRESS,PROCESS;");
WriteBuf(2);

(*------  The Procedure:  ------*)

Assign(OutBuffer^,"(*------  "); Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,"  ------*)",OutBuffer^); WriteBuf(2);

Assign(OutBuffer^,"PROCEDURE "); Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,"(",OutBuffer^); i := 0;

(*------  Parameterliste erzeugen:  ------*)

WHILE i<ParaCount DO
  IF ParaVar[i] THEN Concat(OutBuffer^,"VAR ",OutBuffer^) END;
  Char[0]:= CHR(i+97); Concat(OutBuffer^,Char,OutBuffer^);
  Concat(OutBuffer^,ParaTypes[i],OutBuffer^); INC(i);
  IF i<ParaCount THEN Concat(OutBuffer^,";",OutBuffer^); END;
END;
Concat(OutBuffer^,")",OutBuffer^);
IF ResultReg#-1 THEN Concat(OutBuffer^,ResultType,OutBuffer^) END;
Concat(OutBuffer^,";",OutBuffer^); WriteBuf(2);

(*------  The End:  ------*)

Assign(OutBuffer^,"END "); Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,".",OutBuffer^); WriteBuf(1);

Close(OutH);  (* Definition Module fertig *)

(*------  Implementation Module erzeugen:  ------*)

InH := Open(SourceFile,ModeOldFile); (* open source for reading *)
IF InH=0 THEN WriteString(SourceFile); WriteString(" not found."); WriteLn; HALT END;

WriteString("Source File: "); WriteString(SourceFile); WriteLn; WriteLn;

OutH := Open(ImplFile,ModeNewFile); (* open ImplModule for writing *)
IF OutH=0 THEN Close(InH); WriteString("Couldn't open "); WriteString(ImplFile); WriteLn; HALT; END;

WriteString("Creating Implementation Module: "); WriteString(ImplFile);
WriteLn; WriteLn;

(*------  First Line:  ------*)

Assign(OutBuffer^,"IMPLEMENTATION MODULE ");
Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,";",OutBuffer^);
WriteBuf(2);

(*------  Entry and Exit code:  ------*)

IF NoEntryCode THEN Assign(OutBuffer^,"(*$P-*)"); WriteBuf(2) END;

(*------  Importlist:  ------*)

Assign(OutBuffer^,"FROM SYSTEM IMPORT BYTE,WORD,LONGWORD,ADDRESS,PROCESS,CODE,SETREG,REGISTER;");
WriteBuf(2);

(*------  Constants for saving registers:  ------*)

IF DoSave THEN
  Assign(OutBuffer^,"CONST"); WriteBuf(1);
  Assign(OutBuffer^,"  MOVEMS = 48E7H;"); WriteBuf(1);
  Assign(OutBuffer^,"  MOVEML = 4CDFH;"); WriteBuf(2);
END;

(*------  The Procedure:  ------*)

Assign(OutBuffer^,"(*------  "); Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,"  ------*)",OutBuffer^); WriteBuf(2);

Assign(OutBuffer^,"PROCEDURE "); Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,"(",OutBuffer^); i := 0;

(*------  Parameterliste:  ------*)

WHILE i<ParaCount DO
  IF ParaVar[i] THEN Concat(OutBuffer^,"VAR ",OutBuffer^) END;
  Char[0]:= CHR(i+97); Concat(OutBuffer^,Char,OutBuffer^);
  Concat(OutBuffer^,ParaTypes[i],OutBuffer^); INC(i);
  IF i<ParaCount THEN Concat(OutBuffer^,";",OutBuffer^); END;
END;
Concat(OutBuffer^,")",OutBuffer^);
IF ResultReg#-1 THEN Concat(OutBuffer^,ResultType,OutBuffer^) END;
Concat(OutBuffer^,";",OutBuffer^); WriteBuf(2);

(*------  begin:  ------*)

Assign(OutBuffer^,"BEGIN"); WriteBuf(2);

(*------  Save Registers:  ------*)

IF DoSave THEN
  Assign(OutBuffer^,"  CODE(MOVEMS,");
  ConvertToString(LONGCARD(SaveRegs),16,FALSE,Hex,ok);
  Insert("0000",Hex,0); Delete(Hex,0,Length(Hex)-5);
  Concat(OutBuffer^,Hex,OutBuffer^); Concat(OutBuffer^,"H);",OutBuffer^);
  WriteBuf(2);
END;

(*------  Put parameters into Registers:  ------*)

i := 0;
WHILE i<ParaCount DO
  Assign(OutBuffer^,"  SETREG(");
  ConvertToString(LONGCARD(ParaRegs[i]),10,FALSE,RegNumString,ok);
  Concat(OutBuffer^,RegNumString,OutBuffer^);
  Concat(OutBuffer^,",ADDRESS(",OutBuffer^);
  Char[0] := CHR(i+97); Concat(OutBuffer^,Char,OutBuffer^);
  Concat(OutBuffer^,"));",OutBuffer^); WriteBuf(1); INC(i);
END;
LF(1);

(*------  insert Machine-Code:  ------*)

len := Read(InH,InBuffer,2D); code := InBuffer^;
Assign(OutBuffer^,"  CODE("); i := 0;

WHILE Read(InH,InBuffer,2D)=2D DO
  ConvertToString(LONGCARD(code),16,FALSE,Hex,ok);
  Insert("0000",Hex,0); Delete(Hex,0,Length(Hex)-5);
  Concat(OutBuffer^,Hex,OutBuffer^); Concat(OutBuffer^,"H",OutBuffer^);
  INC(i);
  IF i=8 THEN
    Concat(OutBuffer^,");",OutBuffer^); WriteBuf(1);
    i := 0; Assign(OutBuffer^,"  CODE(");
  ELSE
    Concat(OutBuffer^,",",OutBuffer^);
  END;
  code := InBuffer^;
END;
IF DelZero AND (code=0) AND NOT(1 IN BITSET(i)) THEN (* delete last Zero *)
  IF i#0 THEN
    Delete(OutBuffer^,Length(OutBuffer^)-1,1);
    Concat(OutBuffer^,");",OutBuffer^);
    WriteBuf(1);
  END;
ELSE
  ConvertToString(LONGCARD(code),16,FALSE,Hex,ok);
  Insert("0000",Hex,0); Delete(Hex,0,Length(Hex)-5);
  Concat(OutBuffer^,Hex,OutBuffer^); Concat(OutBuffer^,"H);",OutBuffer^);
  WriteBuf(1);
END;
LF(1);

(*------  get Variable parameters from Registers:  ------*)

i := 0;
WHILE i<ParaCount DO
  IF ParaVar[i] THEN
    Char[0] := CHR(i+97);
    Assign(OutBuffer^,"  "); Concat(OutBuffer^,Char,OutBuffer^);
    Concat(OutBuffer^," := ",OutBuffer^);
    Copy(ParaTypes[i],1,Length(ParaTypes[i])-1,Hex); (* delete `:' *)
    Concat(OutBuffer^,Hex,OutBuffer^);
    Concat(OutBuffer^,"(REGISTER(",OutBuffer^);
    ConvertToString(LONGCARD(ParaRegs[i]),10,FALSE,RegNumString,ok);
    Concat(OutBuffer^,RegNumString,OutBuffer^);
    Concat(OutBuffer^,"));",OutBuffer^); WriteBuf(1);
  END;
  INC(i);
END;
IF ParaCount#0 THEN LF(1) END;

(*------  Return the Result:  ------*)

IF ResultReg#-1 THEN
  Assign(OutBuffer^,"  RETURN "); Concat(OutBuffer^,ResultType,OutBuffer^);
  Delete(OutBuffer^,9,1); Concat(OutBuffer^,"(REGISTER(",OutBuffer^);
  ConvertToString(LONGCARD(ResultReg),10,FALSE,RegNumString,ok);
  Concat(OutBuffer^,RegNumString,OutBuffer^); Concat(OutBuffer^,"));",OutBuffer^);
  WriteBuf(2);
END;

(*------  get saved Registers back:  ------*)

IF DoSave THEN
  Assign(OutBuffer^,"  CODE(MOVEML,");
  ConvertToString(LONGCARD(LoadRegs),16,FALSE,Hex,ok);
  Insert("0000",Hex,0); Delete(Hex,0,Length(Hex)-5);
  Concat(OutBuffer^,Hex,OutBuffer^); Concat(OutBuffer^,"H);",OutBuffer^);
  WriteBuf(2);
END;

(*------  The End:  ------*)

Assign(OutBuffer^,"END "); Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,";",OutBuffer^); WriteBuf(2);

(*------  Initialisation: (empty) ------*)

Assign(OutBuffer^,"BEGIN"); WriteBuf(1);

Assign(OutBuffer^,"END "); Concat(OutBuffer^,ModName,OutBuffer^);
Concat(OutBuffer^,".",OutBuffer^); WriteBuf(1);

(*------  Close Files:  ------*)

Close(OutH);
Close(InH);

(*------  Get Mem back:  ------*)

FreeMem(InBuffer,272);

(*------  Close DosLibrary:  ------*)

CloseLibrary(DosBase);

(*------  That's it! ------*)

END M2Code.

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*   Thanks to the flu I had the week  I wrote this. Whithout it I would   *)
(*   have had  to go to school  and write 3 (!!!)  tests. There wouldn't   *)
(*   have been any time for me to write a programme. Now this is working   *)
(*   and my flu is gone! Here comes the weekend!                           *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)
