IMPLEMENTATION MODULE RIDisplay;

FROM SYSTEM      IMPORT ADR, ADDRESS, SHIFT, BITSET, LONGSET, CAST;
FROM Arts        IMPORT Assert;

FROM Intuition   IMPORT WindowPtr, Gadget, GadgetFlags, GadgetFlagSet,
                        ActivationFlags, ActivationFlagSet, boolGadget,
                        NewWindow, IDCMPFlags, IDCMPFlagSet, WindowFlags,
                        WindowFlagSet, ScreenFlags, ScreenFlagSet,
                        OpenWindow, CloseWindow, propGadget, PropInfo,
                        PropInfoFlags, PropInfoFlagSet, Image, StringInfo,
                        strGadget, RefreshGadgets;
FROM Graphics    IMPORT RastPortPtr, Move, Text, SetAPen, SetDrMd, jam1,
                        Draw, RectFill;
FROM Dos         IMPORT Delay;  (* this should be removed later !!! *)

FROM Conversions IMPORT ValToStr;
FROM Strings     IMPORT Insert, Length, Delete, first, last;

(*
TYPE
  RIGadgetID = (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z,Size,
                Max,Speed,Key,Record,Play,StopKey,Save,Load,SaveAll,LoadAll,
                none);
  KeyQualis = (Shift,Alt,Ctrl,Amiga);
  KeyQualiSet = SET OF KeyQualis;
  RIRecordInfo = RECORD
    Size: CARDINAL;       (* The Number of Inputs it contains              *)
    Max: CARDINAL;        (* The maximum Number of Inputs it could contain *)
    Speed: CARDINAL;      (* The Speed (0..9)                              *)
    Key: CARDINAL;        (* The Key this can be activated with            *)
    KeyQual: KeyQualiSet; (* Shift,Ctrl,Alt,Amiga for that Key             *)
    Memo: ADDRESS;        (* Address of Input's Memory                     *)
    MouseX,MouseY: INTEGER; (* Intuition's Mouse-Position at begining      *)
  END;
  RIRecordInfoPtr = POINTER TO RIRecordInfo;
*)

VAR
  MyGadget: ARRAY RIGadgetID OF Gadget;
  SpeedInfo: PropInfo;
  SpeedImage: Image;
  SizeInfo,MaxInfo: StringInfo;
  SizeBuffer,MaxBuffer: ARRAY[0..7] OF CHAR;
  IDCount,IDCount2: RIGadgetID;
  X: INTEGER;
  NuWindow: NewWindow;
  Window: WindowPtr;
  RP: RastPortPtr;
  ok: BOOLEAN;

(*-------------------  GadgetStruktur initialisieren:  --------------------*)

PROCEDURE SetGadget(ID: RIGadgetID; l,t,w,h: INTEGER);

BEGIN
  WITH MyGadget[ID] DO
    leftEdge := l;
    topEdge := t;
    width := w;
    height := h;
    flags := GadgetFlagSet{};
    activation := ActivationFlagSet{gadgImmediate,toggleSelect};
    gadgetType := boolGadget;
    gadgetRender := NIL;
    selectRender := NIL;
    gadgetText := NIL;
    mutualExclude := LONGSET{};
    specialInfo := NIL;
    gadgetID := ORD(ID);
    userData := NIL;
  END;
END SetGadget;

(*------------------------  Draw a Box:  ----------------------------------*)

PROCEDURE Box(RP: RastPortPtr; x,y,w,h: INTEGER);

BEGIN
  Move(RP,x,y);
  Draw(RP,x+w,y);
  Draw(RP,x+w,y+h);
  Draw(RP,x,y+h);
  Draw(RP,x,y);
END Box;

(*--------------------  This Opens the Window:  ---------------------------*)

PROCEDURE OpenRIWindow(): WindowPtr;

BEGIN

(*----------------------  Initialize Gadgets:  ----------------------------*)

(*------  Link Gadgets:  ------*)
  FOR IDCount := a TO SaveAll DO
    IDCount2 := IDCount; INC(IDCount2);
    MyGadget[IDCount].nextGadget := ADR(MyGadget[IDCount2]);
  END;
  MyGadget[LoadAll].nextGadget := NIL;
(*------  A..Z:  ------*)
  X := 8;
  FOR IDCount := a TO z DO
    SetGadget(IDCount,X,14,17,13);
    INC(X,16);
  END;
(*------ Size:  ------*)
  SetGadget(Size   , 33,45, 65,8);
  WITH MyGadget[Size] DO
    gadgetType := strGadget;
    activation := ActivationFlagSet{longint,stringCenter,relVerify};
    specialInfo := ADR(SizeInfo);
  END;
  WITH SizeInfo DO
    buffer := ADR(SizeBuffer);
    undoBuffer := NIL;
    maxChars := 7;
    bufferPos := 0;
    dispPos := 0;
  END;
(*------  Max:  ------*)
  SetGadget(Max    ,113,45, 65,8);
  WITH MyGadget[Max] DO
    gadgetType := strGadget;
    activation := ActivationFlagSet{longint,stringCenter,relVerify};
    specialInfo := ADR(MaxInfo);
  END;
  WITH MaxInfo DO
    buffer := ADR(MaxBuffer);
    undoBuffer := NIL;
    maxChars := 7;
    bufferPos := 0;
    dispPos := 0;
  END;
(*------  Speed:  ------*)
  SetGadget(Speed  ,192,44, 81, 8);
  WITH MyGadget[Speed] DO
    specialInfo := ADR(SpeedInfo);
    activation := ActivationFlagSet{relVerify};
    gadgetType := propGadget;
    gadgetRender := ADR(SpeedImage);
  END;
  WITH SpeedInfo DO
    flags := PropInfoFlagSet{autoKnob,freeHoriz};
    horizPot := 8000H;
    horizBody := 2000H; (* 0FFFFH DIV 10 *)
  END;
(*------  Key..LoadAll:  ------*)
  SetGadget(Key    ,288,42,113,13);
  SetGadget(Record , 16,74, 89,13);
  SetGadget(Play   ,120,74, 89,13);
  SetGadget(StopKey,304,74,113,13);
  SetGadget(Save   , 16,90, 89,13);
  SetGadget(Load   ,120,90, 89,13);
  SetGadget(SaveAll,224,90, 89,13);
  SetGadget(LoadAll,328,90, 89,13);

(*------------------------  Initialize Window:  ---------------------------*)

  WITH NuWindow DO
    leftEdge := 104;
    topEdge := 58;
    width := 432;
    height := 108;
    detailPen := 0;
    blockPen := 1;
    idcmpFlags := IDCMPFlagSet{gadgetDown,gadgetUp,closeWindow};
    flags := WindowFlagSet{windowDrag,windowDepth,windowClose};
    firstGadget := ADR(MyGadget[a]);
    checkMark := NIL;
    title := ADR("RecordInput    1988 by Fridtjof Siebert");
    screen := NIL;
    bitMap := NIL;
    type := ScreenFlagSet{wbenchScreen};
  END;
  Window := OpenWindow(NuWindow);
  Assert(Window#NIL,ADR("Can't open Window !!!"));

(*------------------------  Draw into Window:  ----------------------------*)

  RP := Window^.rPort;
  SetAPen(RP,1); SetDrMd(RP,jam1);
  Move(RP, 12,23);
  Text(RP,ADR("A B C D E F G H I J K L M N O P Q R S T U V W X Y Z"),51);
  Move(RP, 48,39); Text(RP,ADR("Size:") ,5);
  Move(RP,132,39); Text(RP,ADR("Max:")  ,4);
  Move(RP,200,39); Text(RP,ADR("Speed:"),6);
  Move(RP,328,39); Text(RP,ADR("Key:")  ,4);
  Move(RP, 36,83); Text(RP,ADR("Record        Play      StopKey:"),32);
  Move(RP, 44,99);
  Text(RP,ADR("Save         Load       Save All     Load All"),45);
  FOR X:=8 TO 416 BY 16 DO
    Box(RP,X,14,16,12);
  END;
  Box(RP, 32,44, 65, 9); Box(RP,112,44, 65, 9); Box(RP,288,42,112,12);
  Box(RP, 16,74, 88,12); Box(RP,120,74, 88,12); Box(RP,304,74,112,12);
  Box(RP, 16,90, 88,12); Box(RP,120,90, 88,12);
  Box(RP,224,90, 88,12); Box(RP,328,90, 88,12);
  Move(RP,  0,58); Draw(RP,432,58);
  Move(RP,  0,70); Draw(RP,432,70);
  RETURN Window;
END OpenRIWindow;

(*-------------------------  RawKey to String:  ---------------------------*)

PROCEDURE RawToStr(Key: CARDINAL; Qual: KeyQualiSet;
                   VAR String: ARRAY OF CHAR);

VAR
  Str0t64,Str0t64s: ARRAY[0..65] OF CHAR;
  Str5At5E,Str5At5Es: ARRAY[0..6] OF CHAR;
  len: CARDINAL;

BEGIN
  Str0t64 := "`1234567890ߴ\?0qwertzuiop+?123asdfghjkl#?456<yxcvbnm,.-??789 ";
  Str0t64s:= "~!`$%&/()=?`|?0QWERTZUIOP*?123ASDFGHJKL^?456>YXCVBNM;:_??789 ";
  Str5At5E := "[]/*+";
  Str5At5Es:= "{}/*+";
  IF Key>127 THEN DEC(Key,128) END;
  len := 1;
  CASE Key OF
    0..40H:  IF NOT(Shift IN Qual) THEN
               String[0] := Str0t64[Key]; String[1] := CHAR(0);
             ELSE
               String[0] := Str0t64s[Key]; String[1] := CHAR(0);
               Qual := Qual - KeyQualiSet{Shift};
             END; |
    41H:     String := "BS";    |
    42H:     String := "TAB";   |
    43H:     String := "ENTER"; |
    44H:     String := "CR";    |
    45H:     String := "ESC";   |
    46H:     String := "DEL";   |
    4AH:     String := "-";     |
    4CH:     String := "Up";    |
    4DH:     String := "Down";  |
    4EH:     String := "Right"; |
    4FH:     String := "Left";  |
    50H..58H:String[0] := "F"; String[1] := CHAR(Key-1FH);
             String[2] := CHAR(0); |
    59H:     String := "F10";   |
    5AH..5EH:IF NOT(Shift IN Qual) THEN
               String[0] := Str5At5E[Key-5AH]; String[1] := CHAR(0);
             ELSE
               String[0] := Str5At5Es[Key-5AH]; String[1] := CHAR(0);
               Qual := Qual - KeyQualiSet{Shift};
             END; |
    5FH:     String := "HELP";

  ELSE
    String := "???";
  END;
  IF Shift IN Qual THEN
    Insert(String,last,"+SHIFT");
  END;
  IF Ctrl IN Qual THEN
    Insert(String,last,"+CTRL");
  END;
  IF Alt IN Qual THEN
    Insert(String,last,"+ALT");
  END;
  IF Amiga IN Qual THEN
    Insert(String,last,"+AMIGA");
  END;
END RawToStr;

(*-----------------------  Initialize  Gadgets:  --------------------------*)

PROCEDURE SetGadgets(Active: RIGadgetID; RI: RIRecordInfoPtr;
                     StopKey: CARDINAL; StopKeyQual: KeyQualiSet);

VAR
  KeyStr: ARRAY[0..20] OF CHAR;
  len: CARDINAL;

BEGIN
  WITH MyGadget[Active] DO
    flags := flags / GadgetFlagSet{selected};
  END;
  RefreshGadgets(Window^.firstGadget,Window,NIL);
  FOR IDCount:=a TO LoadAll DO
    WITH MyGadget[IDCount] DO
      flags := flags - GadgetFlagSet{selected};
    END;
  END;
  WITH RI^ DO
    ValToStr(Size,FALSE,SizeBuffer,10,-6,CHAR(0),ok);
    SizeInfo.numChars := Length(SizeBuffer);
    ValToStr(Max,FALSE,MaxBuffer,10,-1,CHAR(0),ok);
    MaxInfo.numChars := Length(MaxBuffer);
    SpeedInfo.horizPot := Speed;
    RawToStr(Key,KeyQual,KeyStr);
    len := Length(KeyStr);
    IF len>13 THEN
      Delete(KeyStr,13,last);
      len := 13;
    END;
    SetAPen(RP,0);
    SetDrMd(RP,jam1);
    RectFill(RP,289,43,399,53);
    SetAPen(RP,1);
    Move(RP,344-4*len,51);
    Text(RP,ADR(KeyStr),len);
  END;
  RawToStr(StopKey,StopKeyQual,KeyStr);
  len := Length(KeyStr);
  IF len>13 THEN
    Delete(KeyStr,13,last);
    len := 13;
  END;
  SetAPen(RP,0);
  SetDrMd(RP,jam1);
  RectFill(RP,305,75,415,85);
  SetAPen(RP,1);
  Move(RP,360-4*len,83);
  Text(RP,ADR(KeyStr),len);
  RefreshGadgets(Window^.firstGadget,Window,NIL);
  INCL(MyGadget[Active].flags,selected);
END SetGadgets;

END RIDisplay.
