(*---------------------------------------------------------------------------
    :Program.    PrinterSupport.mod
    :Author.     Fridtjof Siebert
    :Address.    Nobileweg 67, D-7-Stgt-40
    :Phone.      0711/822509
    :Shortcut.   [fbs]
    :Version.    1.0
    :Date.       31-Jul-88
    :Copyright.  PD
    :Language.   Modula-II
    :Translator. M2Amiga
    :Imports.    none.
    :UpDate.     none.
    :Contents.   PROCEDUREs fr Druckerhandling (Open, Write, Close etc.)
    :Remark.     hnlich wie die Prozeduren des Libraries&Devices Manual.
---------------------------------------------------------------------------*)

IMPLEMENTATION MODULE PrinterSupport;

(*------  IMPORTList:  ------*)

FROM SYSTEM      IMPORT ADR, ADDRESS, LONGSET, CAST;

FROM Exec        IMPORT MsgPortPtr, IOStdReqPtr, write, UByte, OpenDevice,
                        CloseDevice, DoIO;
FROM ExecSupport IMPORT CreatePort, DeletePort, CreateExtIO, DeleteExtIO;
FROM Graphics    IMPORT RastPortPtr, ColorMapPtr, ViewModeSet;
FROM Printer     IMPORT IOPrtCmdReqPtr, IODRPReqPtr, prtCommand, rawWrite,
                        dumpRPort, Special, SpecialSet;

VAR
  PrinterPort: MsgPortPtr;
  PrinterReqBlockStd: IOStdReqPtr;     (* This couldn't be done with one  *)
  PrinterReqBlockDrp: IODRPReqPtr;     (* RECORD, because of M&V's bloody *)
  PrinterReqBlockPcr: IOPrtCmdReqPtr;  (* compiler.                       *)

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                            Open Printer:                                *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

PROCEDURE OpenPrinter();
(* This has to be called before playing with your Printer.                 *)
(* Be careful: Error-Checking is impossible because M2Amigas-Implementation*)
(* of OpenLibrary DOESN'T return TRUE or FALSE wether it was succesful or  *)
(* not. I myself was too lazy to check out of Memory after CreatePort()    *)
(* CreateExtIO(). So don't call this if the largest chunk of Memory is less*)
(* the 200 Bytes !!!                                                       *)

BEGIN
  PrinterPort := CreatePort(ADR("PrinterPort"),0);
  PrinterReqBlockStd := CreateExtIO(PrinterPort,SIZE(PrinterReqBlockDrp^));
  PrinterReqBlockDrp := ADDRESS(PrinterReqBlockStd);
  PrinterReqBlockPcr := ADDRESS(PrinterReqBlockStd);
  OpenDevice(ADR("printer.device"),0,PrinterReqBlockStd,LONGSET{});
END OpenPrinter;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                          WritePrinter:                                  *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

PROCEDURE WritePrinter(Buffer: ADDRESS; Length: LONGINT);
(* This is similar to Dos.Write. Length Bytes are written to Printer.      *)
(* Buffer needn't be a CHIP-Mem Address (I think so).                      *)

BEGIN
  WITH PrinterReqBlockStd^ DO
    command := rawWrite;
    data := Buffer;
    length := Length;
  END;
  DoIO(PrinterReqBlockStd);
END WritePrinter;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                            PrintCommand:                                *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

PROCEDURE PrintCommand(Command: CARDINAL; p0,p1,p2,p3: UByte);
(* This Sends a Printercommand. See Printer.def for more information about *)
(* the Commands. p0..p1 contain it's parameters. Just set them to zero if  *)
(* they are unused (normally they are).                                    *)

BEGIN
  PrinterReqBlockPcr^.command := prtCommand;
  WITH PrinterReqBlockPcr^ DO
    prtCommand := Command;
    parm0 := p0;
    parm1 := p1;
    parm2 := p2;
    parm3 := p3;
  END;
  DoIO(PrinterReqBlockStd);
END PrintCommand;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                       Dump A RastPort (print Graphics):                 *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

PROCEDURE DumpRPort(RP: RastPortPtr; ColorMap: ColorMapPtr;
                    Modes: ViewModeSet; sx,sy,sw,sh: CARDINAL;
                    dc,dr: LONGINT; s: SpecialSet);
(* I didn't need this yet so it isn't tested. Only for completion.         *)
(* Use Libraries & Devices Manual (if you have) for further information.   *)

BEGIN
  WITH PrinterReqBlockDrp^ DO
    command := dumpRPort;
    rastPort := RP;
    colorMap := ColorMap;
    (* next line looks a bit weird, but I found no other way to do this: *)
    modes := CAST(LONGSET,LONGCARD(CAST(CARDINAL,Modes)));
    srcX := sx;
    srcY := sy;
    srcWidth := sw;
    srcHeight :=  sh;
    destCols := dc;
    destRows := dr;
    special := s;
  END;
  DoIO(PrinterReqBlockStd);
END DumpRPort;

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*                           Close Printer:                                *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)

PROCEDURE ClosePrinter();
(* Don't forget to call this at the end of your Programm.                  *)


BEGIN
  CloseDevice(PrinterReqBlockStd);
  DeleteExtIO(PrinterReqBlockStd);
  DeletePort(PrinterPort);
END ClosePrinter;

END PrinterSupport.
