(**********************************************************************

    :Program.  	 MathLibR2.mod
    :Contents.	 Mathematik-Bibliothek fr Vektoren und Matrizen im R 
    :Author.   	 Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Public Domain
    :Language. 	 Modula-2
    :Translator. M2Amiga AMSoft
    :ModHistory. V1.0 [bne] 30.06.88 (first PD version, Amok#4)
    
**********************************************************************)

IMPLEMENTATION MODULE MathLibR2;

FROM SYSTEM IMPORT	ADR;

FROM Arts IMPORT	Assert;

FROM MathTrans IMPORT	Sqrt;

PROCEDURE Add2(V1,V2:Vector2;VAR Ve:Vector2);
BEGIN
  WITH Ve DO
    x:=V1.x+V2.x;
    y:=V1.y+V2.y;
  END;
END Add2;
  
PROCEDURE Sub2(V1,V2:Vector2;VAR Ve:Vector2);
BEGIN
  WITH Ve DO
    x:=V1.x-V2.x;
    y:=V1.y-V2.y;
  END;
END Sub2;
  
PROCEDURE Smul2(s:Scalar;V:Vector2;VAR Ve:Vector2);
BEGIN
  WITH Ve DO
    x:=V.x*s;
    y:=V.y*s;
  END;
END Smul2;
  
PROCEDURE Skprod2(V1,V2:Vector2):Scalar;
BEGIN
  RETURN V1.x*V2.x+V1.y*V2.y;
END Skprod2;
    
PROCEDURE Cprod2(V1,V2:Vector2;VAR Ve:Vector2);
VAR	V3:Vector2;
BEGIN
  WITH V3 DO
    x:=V1.x*V2.x-V1.y*V2.y;
    y:=V1.y*V2.x+V1.x*V2.y;
  END;
  Ve:=V3;
END Cprod2;
  
PROCEDURE Abs2(V:Vector2):Scalar;
BEGIN
  WITH V DO
    RETURN Sqrt(x*x+y*y);
  END
END Abs2;
  
PROCEDURE Trans2(VAR M:Matrix2;V:Vector2;VAR Ve:Vector2);
VAR	V2:Vector2;
BEGIN
  V2:=V;
  WITH V2 DO
    Ve.x:=x*M[1,1]+y*M[1,2];
    Ve.y:=x*M[2,1]+y*M[2,2];
  END;
END Trans2;
  
PROCEDURE Invert2(VAR M:Matrix2):BOOLEAN;
VAR	D:Scalar;
	N:Matrix2;

BEGIN
  N:=M;
  D:=M[1,1]*M[2,2]-M[2,1]*M[1,2];
  IF D=0.0 THEN
    RETURN FALSE;
  END;
  M[1,1]:=N[2,2]/D;
  M[1,2]:=-N[1,2]/D;
  M[2,1]:=-N[2,1]/D;
  M[2,2]:=N[1,1]/D;
  RETURN TRUE;
END Invert2; 
  
PROCEDURE Mmul2(M1,M2:Matrix2;VAR Me:Matrix2);
VAR	i,j:CARDINAL;
	M,N:Matrix2;

BEGIN
  M:=M1;
  N:=M2;
  FOR i:=1 TO 2 DO
    FOR j:=1 TO 2 DO
      Me[i,j]:=M[i,1]*N[1,j]+M[i,2]*N[2,j];
    END;
  END;
END Mmul2;

END MathLibR2.
