(**********************************************************************

    :Program.  	 MathLibR3.mod
    :Contents.	 Mathematik-Bibliothek fr Vektoren und Matrizen im R 
    :Author.   	 Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Public Domain
    :Language. 	 Modula-2
    :Translator. M2Amiga AMSoft
    :ModHistory. V1.0 [bne] 30.06.88 (first PD version, Amok#4)
    
**********************************************************************)

IMPLEMENTATION MODULE MathLibR3;

FROM SYSTEM IMPORT	ADR;

FROM Arts IMPORT	Assert;

FROM MathTrans IMPORT	Sqrt;

(* privat *)
PROCEDURE det(a1,b1,a2,b2:Scalar):Scalar;
BEGIN
  RETURN a1*b2-a2*b1;
END det;

PROCEDURE Add3(V1,V2:Vector3;VAR Ve:Vector3);
BEGIN
  WITH Ve DO
    x:=V1.x+V2.x;
    y:=V1.y+V2.y;
    z:=V1.z+V2.z;
  END;
END Add3;
  
PROCEDURE Sub3(V1,V2:Vector3;VAR Ve:Vector3);
BEGIN
  WITH Ve DO
    x:=V1.x-V2.x;
    y:=V1.y-V2.y;
    z:=V1.z-V2.z;
  END;
END Sub3;
  
PROCEDURE Smul3(s:Scalar;V:Vector3;VAR Ve:Vector3);
BEGIN
  WITH Ve DO
    x:=V.x*s;
    y:=V.y*s;
    z:=V.z*s;
  END;
END Smul3;
  
PROCEDURE Skprod3(V1,V2:Vector3):Scalar;
BEGIN
  RETURN V1.x*V2.x+V1.y*V2.y+V1.z*V2.z;
END Skprod3;
    
PROCEDURE Xprod3(V1,V2:Vector3;VAR Ve:Vector3);
VAR	V3:Vector3;
BEGIN
  WITH V3 DO
    x:=det(V1.y,V2.y,V1.z,V2.z);
    y:=-det(V1.x,V2.x,V1.z,V2.z);
    z:=det(V1.x,V2.x,V1.y,V2.y);
  END;
  Ve:=V3;
END Xprod3;
  
PROCEDURE Abs3(V:Vector3):Scalar;
BEGIN
  WITH V DO
    RETURN Sqrt(x*x+y*y+z*z);
  END
END Abs3;
  
PROCEDURE Trans3(VAR M:Matrix3;V:Vector3;VAR Ve:Vector3);
VAR	V2:Vector3;
BEGIN
  V2:=V;
  WITH V2 DO
    Ve.x:=x*M[1,1]+y*M[1,2]+z*M[1,3];
    Ve.y:=x*M[2,1]+y*M[2,2]+z*M[2,3];
    Ve.z:=x*M[3,1]+y*M[3,2]+z*M[3,3];
  END;
END Trans3;
  
PROCEDURE Invert3(VAR M:Matrix3):BOOLEAN;
VAR	D:Scalar;
	N:Matrix3;

BEGIN
  N:=M;
  D:=N[1,1]*det(N[2,2],N[2,3],N[3,2],N[3,3])
    -N[1,2]*det(N[2,1],N[2,3],N[3,1],N[3,3])
    +N[1,3]*det(N[2,1],N[2,2],N[3,1],N[3,2]);
  IF D=0.0 THEN
    RETURN FALSE;
  END;
  M[1,1]:=det(N[2,2],N[2,3],N[3,2],N[3,3])/D;
  M[1,2]:=det(N[3,2],N[3,3],N[1,2],N[1,3])/D;
  M[1,3]:=det(N[1,2],N[1,3],N[2,2],N[2,3])/D;
  M[2,1]:=det(N[3,1],N[3,3],N[2,1],N[2,3])/D;
  M[2,2]:=det(N[1,1],N[1,3],N[3,1],N[3,3])/D;
  M[2,3]:=det(N[2,1],N[2,3],N[1,1],N[1,3])/D;
  M[3,1]:=det(N[2,1],N[2,2],N[3,1],N[3,2])/D;
  M[3,2]:=det(N[3,1],N[3,2],N[1,1],N[1,2])/D;
  M[3,3]:=det(N[1,1],N[1,2],N[2,1],N[2,2])/D;
  RETURN TRUE;
END Invert3; 
  
PROCEDURE Mmul3(M1,M2:Matrix3;VAR Me:Matrix3);
VAR	i,j:CARDINAL;
	M,N:Matrix3;

BEGIN
  M:=M1;
  N:=M2;
  FOR i:=1 TO 3 DO
    FOR j:=1 TO 3 DO
      Me[i,j]:=M[i,1]*N[1,j]+M[i,2]*N[2,j]+M[i,3]*N[3,j];
    END;
  END;
END Mmul3;

END MathLibR3.
