(*********************************************************************
 *                                                                   *
 *  :Program.    TestOfAVL.mod                                       *
 *  :Author.     Michael Frie                                       *
 *  :Address.    Kernerstr. 22a                                      *
 *  :shortcut.   [MiF]                                               *
 *  :Version.    1.0                                                 *
 *  :Date.       22.09.88                                            *
 *  :Copyright.  PD                                                  *
 *  :Language.   Modula-II                                           *
 *  :Translator. M2Amiga                                             *
 *  :Imports.    AVL       (at least V1.0 --> Amok#7)                *
 *  :Contents.   Test of module AVL                                  *
 *                                                                   *
 *********************************************************************)

MODULE TestOfAVL;

FROM SYSTEM   IMPORT BYTE, ADR;
FROM InOut    IMPORT WriteInt, WriteString, WriteLn;
FROM FileSystem IMPORT Response;
IMPORT AVL;

TYPE data = ARRAY [1..30] OF CHAR;

VAR t    : AVL.tree;
    res  : Response;
    Data : data;
    Key  : INTEGER;
    b    : BOOLEAN;

PROCEDURE Compare (VAR a1, a2: ARRAY OF BYTE) : INTEGER;
 VAR i1, i2: POINTER TO INTEGER;
 BEGIN
  i1 := ADR(a1);
  i2 := ADR(a2);
  RETURN (i1^ - i2^)
 END Compare;

PROCEDURE Action (k: ARRAY OF BYTE; VAR d: ARRAY OF BYTE);
 VAR Key : POINTER TO INTEGER;
     Data : POINTER TO data;
 BEGIN
  Key := ADR(k);
  Data := ADR(d);
  WriteInt (Key^,3);
  WriteString (" ");
  WriteString (Data^);
  WriteLn
 END Action;

BEGIN
  WriteString ("Test of module AVL:"); WriteLn;
  WriteLn;

  WriteString ("* Initialize tree t"); WriteLn;
  AVL.Init (t, "TestBaum", SIZE(Key), SIZE(Data), Compare);

  WriteString ("* insert first element"); WriteLn;
  Key := 2;
  Data := "First entry";
  AVL.Write (t, Key, Data, b);
  IF NOT b (* not overwritten -> inserted *) THEN
    WriteString ("* inserting successful")
  ELSE
    WriteString ("* error while inserting")
  END;
  WriteLn;

  WriteString ("* insert second element"); WriteLn;
  Key := 5;
  Data := "Second entry";
  AVL.Write (t, Key, Data, b);
  IF NOT b THEN
    WriteString ("* inserting successful")
  ELSE
    WriteString ("* error while inserting")
  END;
  WriteLn;

  WriteString ("* insert third element"); WriteLn;
  Key := 9;
  Data := "Third entry";
  AVL.Write (t, Key, Data, b);
  IF NOT b THEN
    WriteString ("* inserting successful")
  ELSE
    WriteString ("* error while inserting")
  END;
  WriteLn;

  WriteString ("* read first element"); WriteLn;
  Key := 2;
  AVL.Read (t, Key, Data, b);
  IF b THEN
    WriteString ("* reading successful")
  ELSE
    WriteString ("* error while reading")
  END;
  WriteLn;
  WriteString ("2 ");
  WriteString (Data); WriteLn;

  WriteString ("* insert fourth element"); WriteLn;
  Key := 6;
  Data := "Fourth entry";
  AVL.Write (t, Key, Data, b);
  IF NOT b THEN
    WriteString ("* inserting successful")
  ELSE
    WriteString ("* error while inserting")
  END;
  WriteLn;

  WriteString ("* delete third element"); WriteLn;
  Key := 9;
  AVL.Delete (t, Key, b);
  IF b THEN
    WriteString ("* deleting successful")
  ELSE
    WriteString ("* error while deleting")
  END;
  WriteLn;

  WriteString ("* overwrite first element"); WriteLn;
  Key := 2;
  Data := "new first entry";
  AVL.Write (t, Key, Data, b);
  IF NOT b THEN
    WriteString ("* error while overwriting")
  ELSE
    WriteString ("* successful overwriting")
  END;
  WriteLn;

  Key := 1;
  Data := "number one";
  AVL.Write (t, Key, Data, b);
  Key := 3;
  Data := "number three";
  AVL.Write (t, Key, Data, b);
  Key := 4;
  Data := "number four";
  AVL.Write (t, Key, Data, b);
  Key := 7;
  Data := "number seven";
  AVL.Write (t, Key, Data, b);
  Key := 8;
  Data := "number eight";
  AVL.Write (t, Key, Data, b);

  WriteString ("* writing further 618 items!");
  WriteLn;
  Data := "some data";
  FOR Key := 10 TO 638 DO
   AVL.Write (t, Key, Data, b)
  END;

  WriteString ("number of elements: ");
  WriteInt (AVL.Number(t),4);
  WriteLn;

  WriteString ("* read first fifty items");
  WriteLn;
  FOR Key := 1 TO 50 DO
    AVL.Read (t, Key, Data, b);
    WriteInt(Key,4);
    IF b THEN
      WriteString (" ");
      WriteString (Data);
      WriteLn
    ELSE
      WriteString (" Error reading Data");
      WriteLn
    END
  END;
  WriteLn;

  WriteString ("* read all in infix order");
  WriteLn;
  AVL.Do (t, Action, AVL.in);
  WriteLn;


  WriteString ("* save tree in file AVLTest.tree");
  WriteLn;
  AVL.Save (t, "AVLTest.tree", res);
  IF res # done THEN
   WriteString ("* error while saving");
   WriteLn
  END;

  WriteString ("* discard tree"); WriteLn;
  AVL.Discard (t);

  WriteString ("* Init tree again for loading");
  WriteLn;
  AVL.Init (t, "TestBaum", SIZE(Data), SIZE(Key), Compare);

  WriteString ("* load tree from file AVLTest.tree");
  WriteLn;
  AVL.Load (t, "AVLTest.tree", res);
  IF res # done THEN
   WriteString ("* error while loading");
   WriteLn
  END;

  WriteString ("* read all in infix order");
  WriteLn;
  AVL.Do (t, Action, AVL.in);
  WriteLn;

  WriteString ("* End of Test"); WriteLn;

END TestOfAVL.
