(*********************************************************************
 *                                                                   *
 *  :Program.    List.mod                                            *
 *  :Author.     Michael Frie                                       *
 *  :Address.    Kernerstr. 22a                                      *
 *  :shortcut.   [MiF]                                               *
 *  :Version.    1.0                                                 *
 *  :Date.       13.09.88                                            *
 *  :Copyright.  PD                                                  *
 *  :Language.   Modula-II                                           *
 *  :Translator. M2Amiga                                             *
 *  :Imports.    MemSystem (at least V1.0 --> Amok#5)                *
 *  :Contents.   Generic data type: List                             *
 *  :Remark.     Thanks to Amok                                      *
 *                                                                   *
 *********************************************************************)

IMPLEMENTATION MODULE List;
(* MODULE List: (C) Copyright 1988 by Michael Frie *)

FROM SYSTEM    IMPORT BYTE, ADDRESS, ADR;
FROM MemSystem IMPORT Allocate, Deallocate;
FROM Exec      IMPORT CopyMem;

CONST Copyright = "MODULE List: (C) Copyright 1988 by Michael Frie";

TYPE list       = POINTER TO listbody;
     elementptr = POINTER TO element;
     element    = RECORD
                   Data : ADDRESS;
                   Next : elementptr;
                  END;
       listbody = RECORD
                   Top, Act : elementptr;
                  END;

PROCEDURE Init (VAR l:list);
 BEGIN
  Allocate (l, SIZE(listbody));
  IF l = NIL THEN HALT END;
  Allocate (l^.Top, SIZE(element));
  IF l^.Top = NIL THEN HALT END;
  l^.Act := l^.Top;
  l^.Act^.Next := NIL
 END Init;


PROCEDURE Reset (l:list);
 BEGIN
  l^.Act := l^.Top
 END Reset;


PROCEDURE Insert (l:list; Data : ARRAY OF BYTE);

 VAR p : elementptr;

 BEGIN
  Allocate (p, SIZE(element));
  IF p = NIL THEN HALT END;        (* stop program *)
  IF l^.Act^.Next = NIL THEN
    p^.Next := NIL;
    l^.Act^.Next := p;
    p := l^.Act
  ELSE
    p^.Next := l^.Act^.Next;
    l^.Act^.Next := p;
    l^.Act := p
  END;
  Allocate (p^.Data, HIGH(Data)+1);
  IF p^.Data = NIL THEN HALT END;  (* stop program *)
  CopyMem (ADR(Data), p^.Data, HIGH(Data)+1)
 END Insert;


PROCEDURE Write  (l:list; Data : ARRAY OF BYTE);
 BEGIN
  IF l^.Act^.Next = NIL THEN
   Allocate (l^.Act^.Next, SIZE(element));
   IF l^.Act^.Next = NIL THEN HALT END; (* stop program *)
   l^.Act^.Next^.Next := NIL;
   Allocate (l^.Act^.Data, HIGH(Data)+1);
   IF l^.Act^.Data = NIL THEN HALT END  (* stop program *)
  END;
  CopyMem (ADR(Data), l^.Act^.Data, HIGH(Data)+1);
  l^.Act := l^.Act^.Next
 END Write;


PROCEDURE Read (l:list; VAR Data : ARRAY OF BYTE);
 BEGIN
  IF l^.Act^.Next # NIL THEN
   CopyMem (l^.Act^.Data, ADR(Data), HIGH(Data)+1);
   l^.Act := l^.Act^.Next;
  END
 END Read;

PROCEDURE Delete (l:list);
 VAR p : elementptr;
 BEGIN
  IF (l^.Act^.Next # NIL) THEN
   Deallocate (l^.Act^.Data);
   l^.Act^.Data := l^.Act^.Next^.Data;
   p := l^.Act^.Next^.Next;
   Deallocate (l^.Act^.Next);
   l^.Act^.Next := p
  END
 END Delete;

PROCEDURE End (l:list) : BOOLEAN;
 BEGIN
  RETURN l^.Act^.Next = NIL
 END End;

PROCEDURE Do (l:list; Action : action);
 VAR More : BOOLEAN;
 BEGIN
  More := TRUE;
  WHILE (l^.Act^.Next # NIL) AND More DO
   Action (l^.Act^.Data^, More);
   l^.Act := l^.Act^.Next
  END
 END Do;

PROCEDURE Discard (VAR l:list);

 BEGIN
  WHILE l^.Top^.Next # NIL DO
   l^.Act := l^.Top^.Next;
   Deallocate (l^.Top^.Data);
   Deallocate (l^.Top);
   l^.Top := l^.Act;
  END;
  Deallocate (l^.Top);
  Deallocate (l);
 END Discard;

END List.
