(*---------------------------------------------------------------------------
    :Program.    PP.mod
    :Author.     Hans-Joachim Siebert
    :Address.    Nobileweg 67, D-7000 Stuttgart-40
    :Phone.      0711/822509
    :Shortcut.   [acs]
    :Version.    1.0
    :Date.       14-Aug-88
    :Copyright.  PD
    :Language.   Modula-II
    :Translator. M2Amiga
    :Imports.    none.
    :UpDate.     none.
    :Contents.   Program for text printouts on Epson LQ (or other).
    :Remark.     Usage: PP <Filename> [bcdeijlmpqrsux]
---------------------------------------------------------------------------*)

MODULE PP;

FROM SYSTEM    IMPORT ADR, ADDRESS;

FROM Dos       IMPORT Open, Close, Write, Read, FileHandlePtr, newFile, oldFile;

FROM Exec      IMPORT AllocMem, FreeMem, MemReqSet, MemReqs;

FROM Arguments IMPORT NumArgs, GetArg;

FROM Arts      IMPORT TermProcedure;

FROM InOut     IMPORT WriteString, WriteLn;

FROM Strings   IMPORT Length;

VAR
  MyFile,YourFile: FileHandlePtr;
  Buffer         : POINTER TO ARRAY[0..511] OF CHAR;
  len            : INTEGER;
  Laenge         : LONGINT;
  Options        : ARRAY[0..25] OF CHAR;
  Name           : ARRAY[0..127] OF CHAR;
  k              : INTEGER;
    
PROCEDURE CleanUp();

BEGIN

  IF Buffer # NIL THEN FreeMem(Buffer,SIZE(Buffer^)) END;
  IF MyFile # NIL THEN Close(MyFile) END;
  IF YourFile # NIL THEN Close(YourFile) END;

END CleanUp;

PROCEDURE Send(a,b,c : INTEGER);

VAR n : LONGINT;

BEGIN
  
  IF b = -1 THEN
    n := 1;
    Buffer^[0] := CHAR(a);
  ELSIF c = -1 THEN
    n := 2;
    Buffer^[0] := CHAR(a);
    Buffer^[1] := CHAR(b);
  ELSE
    n := 3;
    Buffer^[0] := CHAR(a);
    Buffer^[1] := CHAR(b);
    Buffer^[2] := CHAR(c);
  END;
  
  n := Write(MyFile,Buffer,n);

END Send;

PROCEDURE Reset();

BEGIN

  Send(27,64,-1);
  
END Reset;
    
PROCEDURE Auswertung();

VAR i : CARDINAL;

BEGIN
  FOR i := 0 TO Length(Options)-1 DO
    
    CASE Options[i] OF
      "b" : Send(27,69,-1)   (* bold                               *)
     |"c" : Send(15,-1,-1)   (* condensed, pitch 15                *)
     |"d" : Send(27,71,-1)   (* doublestrike                       *)
     |"e" : Send(27,77,-1)   (* elite, pitch 12                    *)
     |"i" : Send(27,52,-1)   (* italics                            *)
     |"j" : Send(27,108,10); (* left margin 10                     *)
            Send(27,81,70);  (* right margin 70                    *)
            Send(27,97, 3)   (* full justification                 *)
     |"l" : Send(27,120,1)   (* letter quality                     *)
     |"m" : Send(27,103,-1)  (* micro print                        *)
     |"p" : Send(27,112,1)   (* proportional                       *)
     |"q" : Send(27,107,1)   (* Sans Serif                         *)
     |"r" : Send(27,108,5);  (* left and right margins             *)
            Send(27,81,75)   (*                                    *)
     |"s" : Send(27,78, 6)   (* skip perforation                   *)
     |"u" : Send(27,85, 1)   (* unidirectional print               *)
     |"x" : Send(27,87, 1)   (* expanded                           *) 
     | ELSE END;
     
  END (* FOR *);
  
END Auswertung;

PROCEDURE Umlaute();

BEGIN
      
      FOR k := 0 TO Laenge-1 DO

        CASE Buffer^[k] OF
          "" : Buffer^[k] := CHAR(08EH)
         |"" : Buffer^[k] := CHAR(099H)
         |"" : Buffer^[k] := CHAR(09AH)
         |"" : Buffer^[k] := CHAR(084H)
         |"" : Buffer^[k] := CHAR(094H)
         |"" : Buffer^[k] := CHAR(081H)
         |"" : Buffer^[k] := CHAR(0E1H)
         | ELSE END;
     
      END (* FOR *);	  
      
END Umlaute;

BEGIN  (*----------------------------MAIN-------------------------------------*)

  MyFile := NIL; Buffer := NIL; YourFile := NIL;
  TermProcedure(CleanUp);
  Buffer := AllocMem(SIZE(Buffer^),MemReqSet{chip,memClear});
  MyFile := Open(ADR("par:"),newFile);
  
  IF NumArgs() = 0 THEN

    WriteLn;
    WriteString("   PP - a perfect printing program for Epson LQ      ");
    WriteLn;
    WriteString("                     by                              ");
    WriteLn;
    WriteString("             Hans-Joachim Siebert                    ");
    WriteLn;
    WriteLn;
    WriteString("   USAGE: PP <filename> [bcdeijlmpqrsux]             ");
    WriteLn;
    WriteLn;
    WriteString("b = bold,           c = condensed, d = doublestrike  ");
    WriteLn;
    WriteString("e = Elite,          i = italics,   j = justified     ");
    WriteLn;
    WriteString("l = letter quality, m = micro ,    p = proportional  ");
    WriteLn;
    WriteString("q = Sans Serif,     r = margins ,  s = skip perforation");
    WriteLn;
    WriteString("u = unidirectional, x = expanded");
    WriteLn;
    WriteLn;
                                                                       
  ELSE
  
  GetArg(1,Name,len);
  YourFile := Open(ADR(Name),oldFile);
  
    IF YourFile = NIL THEN WriteString("Couldn't find ");
                           WriteString(Name);
                           WriteLn;
    ELSE
  
    Reset();

      IF NumArgs() >= 2 THEN
  
        GetArg(2,Options,len);
        Auswertung();
    
      END;
      
    Send(27,116,1);  (* Graphic character set for German Umlaute *)
    
    REPEAT
    
      Laenge := Read(YourFile,Buffer,SIZE(Buffer^));
      
      Umlaute();

      IF Laenge <> 0 THEN
        Laenge := Write(MyFile,Buffer,Laenge);
      END;
    
    UNTIL Laenge = 0;
    
    Send(12,-1,-1);   (* Form Feed                 *) 
    
    END; (* IF YourFile = NIL THEN... ELSE...      *)
  
  END; (* IF NumArgs() = 0 THEN ... ELSIF ...      *)
                  
END PP.  
