(*---------------------------------------------------------------------------
    :Program.    SoundTask
    :Author.     Fridtjof Siebert
    :Address.    Nobileweg 67, D-7-Stgt-40
    :Shortcut.   [fbs]
    :Version.    .1
    :Date.       17-Jul-88
    :Copyright.  PD
    :Language.   Modula-II
    :Translator. M2Amiga
    :Imports.    MemSystem [bne], Amok #5
    :UpDate.     none.
    :Contents.   Programm to play a sound from a second task.
    :Remark.     This is a MODULA-II version of the Libs&Devs' Audio-example
---------------------------------------------------------------------------*)

MODULE SoundTask;

FROM SYSTEM      IMPORT ADR, ADDRESS, SHIFT, FFP, LONGSET, INLINE, SETREG,
                        CAST;

FROM Arts        IMPORT TermProcedure, Assert;

FROM Audio       IMPORT IOAudioPtr, audioName, allocFailed, perVol;

FROM Dos         IMPORT Delay;

FROM Exec        IMPORT FindTask, MsgPortPtr, DevicePtr, UByte, Byte,
                        MsgPortAction, NodeType, write, IOFlagSet, ReplyMsg,
                        sigAbort, GetMsg, OpenDevice, Wait, CloseDevice,
                        TaskPtr, AddTask, RemTask;

FROM ExecSupport IMPORT CreatePort, DeletePort, NewList, BeginIO;

FROM InOut       IMPORT WriteString;

FROM MemSystem   IMPORT ErrHeader, Allocate, AllocMem, Deallocate,
                        YesNoRequest, RETRY, CANCEL, ExitQuiet;


CONST
  BufferSize      = 250;
  SineTablePower2 = 9;
  SineTableSize   = 512;
  SineTableStep   = 2.0 * 3.141593 / 512.0;
  OpenFail        = -1;    (* Where can this be imported from ??? *)

VAR
  AllocPort:  MsgPortPtr;
  AllocIOB:   IOAudioPtr;
  Device:     DevicePtr;
  SoundPort:  MsgPortPtr;
  Buffer:     ARRAY[0..1] OF ADDRESS;
  SoundIOB:   ARRAY[0..1] OF IOAudioPtr;
  AllocationMap: ARRAY[0..3] OF Byte;

  SineTable:  ARRAY[0..SineTableSize-1] OF Byte;
  Angle:      LONGCARD;
  Frequency:  LONGCARD;
  LastYMouse: Byte;
  YMouse[0DFF00BH]: Byte;
  XMouse[0DFF00AH]: Byte;

  i: INTEGER;
  sine,cosine: FFP;
  PlayTask: TaskPtr;
  PlayStack: ADDRESS;

  Ciapra [0BFE001H]: SET OF (s0,s1,s2,s3,s4,s5,lmb);

(*------  CleanUp:  ------*)

PROCEDURE CleanUp();

BEGIN
  IF PlayTask#NIL  THEN
    RemTask(PlayTask);
    Deallocate(PlayTask);
  END;
  IF PlayStack#NIL THEN Deallocate(PlayStack) END;
  IF Device#NIL THEN CloseDevice(AllocIOB) END;
  FOR i:=0 TO 1 DO
    IF SoundIOB[i]#NIL THEN Deallocate(SoundIOB[i]) END;
    IF Buffer[i]  #NIL THEN Deallocate(Buffer[i]  ) END;
  END;
  IF SoundPort#NIL THEN Deallocate(SoundPort) END;
  IF AllocIOB #NIL THEN Deallocate(AllocIOB ) END;
  IF AllocPort#NIL THEN DeletePort(AllocPort) END;
END CleanUp;

(*------  Task to Create SoundData:  ------*)

PROCEDURE NewBuffer(); (* $S- $V- $R- *)

VAR
  i: INTEGER;
  IOA: IOAudioPtr;
  Buffer: POINTER TO Byte;
  MouseChange,CurYMouse: Byte;
  NewFreq: LONGCARD;
  Proc: PROCEDURE;
  Adr: ADDRESS;

BEGIN

(*------  Get Message:  ------*)

  LOOP

    IOA := GetMsg(SoundPort);
    IF (IOA#NIL) AND (IOA^.request.error=0) THEN

(*------  Get MouseChange:  ------*)

      CurYMouse := YMouse;
      MouseChange :=  CurYMouse - LastYMouse;
      LastYMouse := CurYMouse;

(*------  Calculate new Frequency:  ------*)

      NewFreq := Frequency + LONGCARD(LONGINT(MouseChange) *
                 LONGINT(SHIFT(Frequency,-6)));
      IF (NewFreq>200000H) AND (NewFreq<40000000H) THEN
        Frequency := NewFreq;
      END;

(*------  New WaveForm:  ------*)

      Buffer := IOA^.data;
      FOR i:=0 TO BufferSize-1 DO
        INC(Angle,Frequency);
        Buffer^ := SineTable[SHIFT(Angle,SineTablePower2-32)];
        INC(Buffer);
      END;

(*------  Send Write-Event:  ------*)

      BeginIO(IOA);

    END;   (* IF Msg ok THEN *)

  END;   (* LOOP *)

END NewBuffer; (* $S+ $V+ $R+ *)


(*------  Fun Starts:  ------*)


BEGIN

(*------  Set Default values:  ------*)

  AllocPort := NIL; AllocIOB  := NIL; Device      := NIL; SoundPort   := NIL;
  Buffer[0] := NIL; Buffer[1] := NIL; SoundIOB[0] := NIL; SoundIOB[1] := NIL;
  Angle     := 0;   sine      := 0.0; cosine      := 1.0; PlayStack   := NIL;
  PlayTask  := NIL; Frequency := 20000000H;
  ErrHeader := "SoundDemo has some trouble!";
  TermProcedure(CleanUp);

(*------  Set up sine-table:  ------*)

  FOR i:=0 TO SineTableSize-1 DO
    SineTable[i] := Byte(127.0*sine + 0.5);
    cosine := cosine - SineTableStep *   sine;
    sine   := sine   + SineTableStep * cosine;
  END;

(*------  Initialization:  ------*)

  LastYMouse := YMouse;

  AllocPort := CreatePort(ADR("Sound Example"),0);
  Assert(AllocPort#NIL,ADR("CreatePort failed"));
  Allocate(AllocIOB,SIZE(AllocIOB^));
  IF AllocIOB=NIL THEN ExitQuiet() END;

(*------  Allocation Precedence and Channel:  ------*)

  AllocIOB^.request.message.node.pri := -40;
  AllocIOB^.request.message.replyPort := AllocPort;

  AllocationMap[0] := 1;
  AllocationMap[1] := 8;
  AllocationMap[2] := 2;
  AllocationMap[3] := 4;
  AllocIOB^.data   := ADR(AllocationMap);
  AllocIOB^.length := SIZE(AllocationMap);

(*------  Open Audio-Device:  ------*)

  OpenDevice(ADR(audioName),0,AllocIOB,LONGSET{});
(* Why doesn't OpenDevice() return it's error ??? *)

  CASE AllocIOB^.request.error OF
  OpenFail:    Assert(FALSE,ADR("OpenDevice failed")); |
  allocFailed: Assert(FALSE,ADR("No audio channel available")); |
  ELSE (* no error ! *)
  END;

  Device := AllocIOB^.request.device;

(*------  Initialize ReplyPort to signal PlayTask:  ------*)

  Allocate(SoundPort,SIZE(SoundPort^));
  IF SoundPort=NIL THEN ExitQuiet END;
  WITH SoundPort^ DO
    flags     := signal;
    sigTask   := PlayTask;
    node.type := msgPort;
  END;
  NewList(ADR(SoundPort^.msgList));

(*------  Start Task:  ------*)


  Allocate(PlayTask,SIZE(PlayTask^));
  IF PlayTask=NIL THEN ExitQuiet END;
  Allocate(PlayStack,1000);
  IF PlayStack=NIL THEN ExitQuiet END;
  WITH PlayTask^ DO
    spLower   := PlayStack;
    spUpper   := ADDRESS(LONGCARD(PlayStack) + 1000);
    spReg     := spUpper;
    node.type := task;
    node.name := ADR("PlayTask");
  END;
  AddTask(PlayTask,ADR(NewBuffer),NIL);


(*------  Initialize I/O-Blocks:  ------*)

  FOR i:=0 TO 1 DO
    AllocMem(Buffer[i],BufferSize,TRUE);
    IF Buffer[i]=NIL THEN ExitQuiet END;
    Allocate(SoundIOB[i],SIZE(SoundIOB[i]^));
    IF SoundIOB[i]=NIL THEN ExitQuiet END;
    WITH SoundIOB[i]^ DO
      request.message.replyPort := SoundPort;
      request.device  := Device;
      request.unit    := AllocIOB^.request.unit;
      request.command := write;
      request.flags   := IOFlagSet{4}; (* perVol *)
      allocKey        := AllocIOB^.allocKey;
      data            := Buffer[i];
      length          := BufferSize;
      period          := 400;
      volume          := 64;
      cycles          := 1;
    END;
    ReplyMsg(SoundIOB[i]);
  END;

  WriteString("Press Mouse to stop !");
  REPEAT
    Delay(16);
  UNTIL NOT(lmb IN Ciapra);

END SoundTask.
