(* WarpText Quickie Test for new WarpText routines.
 *
 * Thanx to Anson Mah for his C-Demo. I made it Modula.
 *  -- Fridtjof.
 *)

MODULE NewWarpDemo;

FROM SYSTEM    IMPORT ADR, ADDRESS, SHIFT;

FROM Arts      IMPORT Assert, TermProcedure;

FROM Exec      IMPORT AllocMem, MemReqSet, MemReqs, FreeMem, WaitPort,
                      GetMsg, ReplyMsg;

FROM Graphics  IMPORT TextAttr, FontStyles, FontStyleSet, FontFlags,
                      FontFlagSet, OpenFont, CloseFont, TextFontPtr;

FROM Intuition IMPORT NewWindow, WindowPtr, WindowFlags, WindowFlagSet,
                      IDCMPFlags, IDCMPFlagSet, ScreenFlags, ScreenFlagSet,
                      IntuiMessagePtr, OpenWindow, CloseWindow;

FROM Strings   IMPORT Length;

FROM WarpText  IMPORT SetUpFont, NewWarp, NewWarpInfo;

VAR
  w: WindowPtr;
  nwinfo: NewWarpInfo;
  taTopaz80: TextAttr;
  wdef: NewWindow;
  text: ARRAY[0..16] OF POINTER TO ARRAY[0..79] OF CHAR;
  msg: IntuiMessagePtr;
  class: IDCMPFlagSet;
  i: CARDINAL;
  Font: TextFontPtr;

PROCEDURE CloseStuff();

BEGIN
  IF w#NIL     THEN CloseWindow(w) END;
  WITH nwinfo DO
    IF fontData#NIL THEN FreeMem(fontData,2048) END;
  END;
END CloseStuff;

BEGIN
  w := NIL;
  nwinfo.fontData := NIL;
  WITH taTopaz80 DO
    name  := ADR("topaz.font");
    ySize := 8;
    style := FontStyleSet{};
    flags := FontFlagSet{romFont};
  END;
  WITH wdef DO
    leftEdge    := 0;
    topEdge     := 0;
    width       := 640;
    height      := 200;
    detailPen   := 0;
    blockPen    := 1;
    idcmpFlags  := IDCMPFlagSet{closeWindow};
    flags       := WindowFlagSet{windowClose,windowDepth,activate};
    firstGadget := NIL;
    checkMark   := NIL;
    title       := ADR("WarpText Test");
    screen      := NIL;
    bitMap      := NIL;
    type        := ScreenFlagSet{wbenchScreen};
  END;
  TermProcedure(CloseStuff);

  w := OpenWindow(wdef);
  Assert(w#NIL,ADR("kudndopnwndo"));

  text[ 0] := ADR("Hello World!");
  text[ 1] := ADR("   This is a Demonstration of Bill Kelly's WarpText routines");
  text[ 2] := ADR("I transferred to MODULA-II. As you see they are as quick");
  text[ 3] := ADR("as the original C implementation is. So all Modula fans and");
  text[ 4] := ADR("C - Haters like me are able to use them. Soon the whole");
  text[ 5] := ADR("world will speak MODULA-II! This Demonstration was written");
  text[ 6] := ADR("first in C by Anson Mah. Then I made it Modula-II. I could");
  text[ 7] := ADR("have written a new demo in Modula-II, but was too lazy.");
  text[ 8] := ADR("    Thanx to Bill for his machinecode routines and to");
  text[ 9] := ADR("Anson for his demo!");
  text[10] := ADR("    I've just assemblered Bill's source pc-relative and");
  text[11] := ADR("splittet it into the Procedures. The code is included in the");
  text[12] := ADR("Modula source via SYSTEM.INLINE(). So you can easily compile");
  text[13] := ADR("it or make some changes in it. The Procedures get their");
  text[14] := ADR("stuff directly via the 68000-Registers. That's why I");
  text[15] := ADR("had to make some small changes in Bill's code.");
  text[16] := ADR("                                          Fridjof");

  (* set up newwarpinfo *)
  WITH nwinfo DO
    fontData := AllocMem(2048,MemReqSet{});
    Assert(fontData#NIL,ADR("out of Memory!"));
  END;
  Font := OpenFont(ADR(taTopaz80));
  SetUpFont(ADR(nwinfo),Font);

  (* warp out text *)

  WITH nwinfo DO
    bitPlane := w^.wScreen^.bitMap.planes[0];
    FOR i:=0 TO 16 DO
      xLoc := 10; yLoc := 2+i;
      NewWarp(ADR(nwinfo),text[i],Length(text[i]^));
    END;
    xLoc := 25; yLoc := 19;
    NewWarp(ADR(nwinfo),ADR("Hey, I'm here!"),14);
    xLoc := 10; yLoc := 20;
    NewWarp(ADR(nwinfo),ADR("And here!!"),10);
  END;

  CloseFont(Font);

  LOOP
    WaitPort(w^.userPort);
    msg := GetMsg(w^.userPort);
    class := msg^.class;
    ReplyMsg(msg);
    IF closeWindow IN class THEN EXIT END;
  END;

END NewWarpDemo.
