(*---------------------------------------------------------------------------
    :Program.    CreateAmokDisk.mod
    :Author.     Fridtjof Siebert
    :Address.    Nobileweg 67, D-7-Stgt-40
    :Phone.      0711/822509
    :Shortcut.   [fbs]
    :Version.    1.1
    :Date.       06.08.88
    :Copyright.  PD
    :Language.   Modula-II
    :Translator. M2Amiga
    :Imports.    IconSupport [fbs].
    :UpDate.     29-Oct-88, Support for Muchmore, [fbs]
    :Contents.   This creates Icons for a Amok-PD-Disc.
    :Contents.   Syntax: CreateAmokDisk <Disk or Directory>
    :Remark.     Thanks to Pit for his woderful Icons !!!
---------------------------------------------------------------------------*)

MODULE CreateAmokDisk;

(*------  ImportList:  ------*)

FROM SYSTEM      IMPORT ADR, ADDRESS, CAST, SHIFT, BITSET, LONGSET, INLINE;
FROM Arts        IMPORT Terminate, detectCtrlC;

FROM Dos         IMPORT FileLockPtr, UnLock, Lock, Examine, ExNext,
                        FileInfoBlockPtr, sharedLock, IoErr, CurrentDir,
                        noMoreEntries;
FROM Exec        IMPORT AllocMem, FreeMem, MemReqs, MemReqSet;
FROM Intuition   IMPORT Image;
FROM Workbench   IMPORT WBObjectType;

FROM Arguments   IMPORT NumArgs, GetArg;
FROM InOut       IMPORT WriteString, WriteInt, WriteLn, ReadString;
FROM Strings     IMPORT Copy, Occurs, Insert, Length, first, last;

FROM IconSupport IMPORT CreateIcon;

IMPORT AmokRun, AmokDir, AmokDisk, AmokDoc, AmokSource; (* Images *)

(*------  Variables:  ------*)

TYPE
  ImageData = ARRAY[0..31] OF CARDINAL;
  Icons = (Run, Dir, Disk, Doc, Source);

VAR
  Images: ARRAY Icons OF Image;
  HImages: ARRAY Icons OF Image;
  IconCount: Icons;
  IconOK: BOOLEAN;
  len: INTEGER;
  Name, DiskName: ARRAY[0..79] OF CHAR;
  LockPtr: FileLockPtr;

(*------------------------  Main Procedure:  ------------------------------*)

PROCEDURE DoIt(lock: FileLockPtr; Depth: CARDINAL): BOOLEAN;
(* a Result of FALSE indicates out of Memory-Error. Result is used to be   *)
(* able to quit this easily and quick if system runs out of Memory.        *)
(* This Procedure is Recursiv, i.e. it calls itself if t finds Directories *)

TYPE
  Name = ARRAY[0..49] OF CHAR;
  NameTypePtr = POINTER TO NameType;
  NameType = RECORD
    next: NameTypePtr;
    name: Name;
  END;

VAR
  FileInfo: FileInfoBlockPtr;
  Err: LONGINT;
  Lock2, old, exists: FileLockPtr;
  FirstName, ActName, Search: NameTypePtr;
  IconName: Name;

  (*------  Tab jump:  ------*)

  PROCEDURE Tab();

  VAR i: CARDINAL;

  BEGIN
    i:=0;
    WHILE i<=Depth DO
      WriteString("  ");
      INC(i);
    END;
  END Tab;

  (*------  Free Names:  ------*)

  PROCEDURE FreeNames(Name: NameTypePtr);

  BEGIN
    IF Name^.next#NIL THEN
      FreeNames(Name^.next);
    END;
    FreeMem(Name,SIZE(NameType));
  END FreeNames;

BEGIN

(*------  Init:  ------*)

  FirstName := NIL; ActName := NIL;
  FileInfo := AllocMem(SIZE(FileInfo^),MemReqSet{chip,memClear});
  IF FileInfo=NIL THEN RETURN(FALSE) END;

(*------  Loop to get Names:  ------*)

  old := CurrentDir(lock);   (* make it Current Directory *)

  IF Examine(lock,FileInfo)#0 THEN

    WHILE ExNext(lock,FileInfo)#0 DO

      IF FileInfo^.dirEntryType>0 THEN

(*------  Directory: Call DoIt():  ------*)

        WITH FileInfo^ DO
          Copy(IconName,fileName,first,49);
          Insert(IconName,last,".info");
          exists := Lock(ADR(IconName),sharedLock);
          IF exists#0 THEN
            UnLock(exists);
          ELSE
            IF NOT(CreateIcon(ADR(fileName),ADR(Images[Dir]),
                     ADR(HImages[Dir]),TRUE,drawer,NIL)) THEN
              Tab; WriteString("Error while creating Icon !!!"); WriteLn;
            END;
          END;
          Tab; WriteString(FileInfo^.fileName);
          WriteString("  (Directory)"); WriteLn;
          Lock2 := Lock(ADR(FileInfo^.fileName),sharedLock);
          IF Lock2#0 THEN
            IF NOT(DoIt(Lock2,Depth+1)) THEN
              FreeMem(FileInfo,SIZE(FileInfo^));
              IF FirstName#NIL THEN FreeNames(FirstName) END;
              UnLock(Lock2);
              RETURN(FALSE);
            END;
          END;
          UnLock(Lock2);
        END;

      ELSE

(*------  Save new Name:  ------*)

        IF FirstName=NIL THEN
          FirstName := AllocMem(SIZE(NameType),MemReqSet{public,memClear});
          ActName := FirstName;
        ELSE
          ActName^.next := AllocMem(SIZE(NameType),MemReqSet{public,memClear});
          ActName := ActName^.next;
        END;
        IF ActName=NIL THEN
          IF FirstName#NIL THEN FreeNames(FirstName) END;
          RETURN(FALSE);
        END;
        Copy(ActName^.name,FileInfo^.fileName,first,49);

      END;   (* IF Directory THEN ELSE *)

    END;   (* WHILE ExNext()#0 DO *)

  END;   (* IF Exmanin()#0 THEN *)

(*------  Read Error if any occured:  ------*)

  Err := IoErr();
  IF (Err#noMoreEntries) THEN
    Tab; WriteString("IO-Error: "); WriteInt(Err,6); WriteLn;
  END;

(*------  Create Icons:  ------*)

  ActName := FirstName;
  WHILE ActName#NIL DO
    WITH ActName^ DO
      IF Occurs(name,first,".info",FALSE)=last THEN
        IconName := name;
        Insert(IconName,last,".info");
        exists := Lock(ADR(IconName),sharedLock);
        IF exists#0 THEN
          UnLock(exists);
        ELSE
          Tab; WriteString(name); WriteLn;
          IF (Occurs(name,first,".mod" ,FALSE)#last) OR
             (Occurs(name,first,".def" ,FALSE)#last) OR
             (Occurs(name,first,".asm" ,FALSE)#last) OR
             (Occurs(name,first,".code",FALSE)#last) OR
             (Occurs(name,first,".c"   ,FALSE)#last) OR
             (Occurs(name,first,".h"   ,FALSE)#last) OR
             (Occurs(name,first,".bas" ,FALSE)#last) OR
             (Occurs(name,first,".qt"  ,FALSE)#last)
           THEN
            IconOK := CreateIcon(ADR(name),ADR(Images[Source]),
                        ADR(HImages[Source]),TRUE,project,ADR(":c/muchmore"));
          ELSIF (Occurs(name,first,"doc"    ,FALSE)#last)
             OR (Occurs(name,first,".dok"    ,FALSE)#last)
             OR (Occurs(name,first,"reaedme",FALSE)#last)
             OR (Occurs(name,first,"liesmich",FALSE)#last)
          THEN
            IconOK := CreateIcon(ADR(name),ADR(Images[Doc]),ADR(HImages[Doc]),
                                 TRUE,project,ADR(":c/muchmore"));
          ELSE
            IconOK := CreateIcon(ADR(name),ADR(Images[Run]),ADR(HImages[Run]),
                                 TRUE,tool,NIL);
          END;
          IF NOT(IconOK) THEN
            Tab; WriteString("Error while creating Icon !!!"); WriteLn;
          END;
        END;
      END;
      ActName := next;
    END;
  END;

  old := CurrentDir(old);   (* Back to old Directory *)

(*------  Give Memory back:  ------*)

  FreeMem(FileInfo,SIZE(FileInfo^));
  IF FirstName#NIL THEN FreeNames(FirstName) END;

  RETURN(TRUE);

END DoIt;

(*----------------------------  MAIN:  ------------------------------------*)

BEGIN

  WriteString("Amok -- Icon creation tool. 1988 by Fridtjof Siebert");
  WriteLn;

(*------  Get Argument:  ------*)

  IF NumArgs()#1 THEN
    WriteString("Usage: CreateAmok <Name>"); WriteLn;
    Terminate(0);
  END;
  GetArg(1,Name,len);

(*------  Get Image:  ------*)

  FOR IconCount := Run TO Source DO
    WITH Images[IconCount] DO
      leftEdge := 0;
      topEdge := 0;
      depth := 2;
      planePick := 3;
      planeOnOff := 0;
      nextImage := NIL;
    END;
    WITH HImages[IconCount] DO
      leftEdge := 0;
      topEdge := 0;
      depth := 2;
      planePick := 3;
      planeOnOff := 0;
      nextImage := NIL;
    END;
  END;

(* Run *)
  WITH Images[Run] DO
    width := AmokRun.Imgwidth;
    height := AmokRun.Imgheight;
    imageData := AmokRun.ImgPtr[0];
  END;
  WITH HImages[Run] DO
    width := AmokRun.Imgwidth;
    height := AmokRun.Imgheight;
    imageData := AmokRun.ImgPtr[1];
  END;

(* Dir *)
  WITH Images[Dir] DO
    width := AmokDir.Imgwidth;
    height := AmokDir.Imgheight;
    imageData := AmokDir.ImgPtr[0];
  END;
  WITH HImages[Dir] DO
    width := AmokDir.Imgwidth;
    height := AmokDir.Imgheight;
    imageData := AmokDir.ImgPtr[1];
  END;

(* Disk *)
  WITH Images[Disk] DO
    width := AmokDisk.Imgwidth;
    height := AmokDisk.Imgheight;
    imageData := AmokDisk.ImgPtr[0];
  END;
  WITH HImages[Disk] DO
    width := AmokDisk.Imgwidth;
    height := AmokDisk.Imgheight;
    imageData := AmokDisk.ImgPtr[1];
  END;

(* Doc *)
  WITH Images[Doc] DO
    width := AmokDoc.Imgwidth;
    height := AmokDoc.Imgheight;
    imageData := AmokDoc.ImgPtr[0];
  END;
  WITH HImages[Doc] DO
    width := AmokDoc.Imgwidth;
    height := AmokDoc.Imgheight;
    imageData := AmokDoc.ImgPtr[1];
  END;

(* Source *)
  WITH Images[Source] DO
    width := AmokSource.Imgwidth;
    height := AmokSource.Imgheight;
    imageData := AmokSource.ImgPtr[0];
  END;
  WITH HImages[Source] DO
    width := AmokSource.Imgwidth;
    height := AmokSource.Imgheight;
    imageData := AmokSource.ImgPtr[1];
  END;

(*------  Do It:  ------*)

  detectCtrlC := FALSE; (* aviod to be desturbed *)

  LockPtr := Lock(ADR(Name),sharedLock);
  IF Name[Length(Name)-1] = ":" THEN
    Copy(DiskName,Name,0,49);
    Insert(DiskName,last,"Disk");
    IF NOT(CreateIcon(ADR(DiskName),ADR(Images[Disk]),ADR(HImages[Disk]),
                      FALSE,disk,ADR("SYS:SYSTEM/DiskCopy"))) THEN
      WriteString("Error while creating Icon !!!"); WriteLn;
    END;
  END;
  IF LockPtr=NIL THEN
    WriteString("IO-Error: "); WriteInt(IoErr(),6); WriteLn;
    Terminate(0);
  END;

  IF NOT(DoIt(LockPtr,0)) THEN
    WriteString("Ran out of Memory !!!"); WriteLn;
  END;
  UnLock(LockPtr);
END CreateAmokDisk.
