(*---------------------------------------------------------------------------
    :Program.    PlayRep.mod
    :Author.     Fridtjof Siebert
    :Address.    Nobileweg 67, D-7-Stgt-40
    :Shortcut.   [fbs]
    :Version.    1.0
    :Date.       19-Sep-88
    :Copyright.  PD
    :Language.   Modula-II
    :Translator. M2Amiga
    :Imports.    IFF8SVXLoad [fbs], MemSystem [bne]
    :UpDate.     none.
    :Contents.   Program to play a sampled Sound repeatedly.
    :Remark.     Usage: PlayRep <iff-soundfile>
---------------------------------------------------------------------------*)

MODULE PlayRep;

FROM SYSTEM      IMPORT ADR, ADDRESS, SHIFT, BITSET, LONGSET, CAST;

FROM Arts        IMPORT Terminate, Assert;

FROM Arguments   IMPORT NumArgs, GetArg;

FROM Exec        IMPORT WaitPort;

FROM Intuition   IMPORT DisplayBeep, NewWindow, WindowPtr, WindowFlags,
                        WindowFlagSet, ScreenFlags, ScreenFlagSet,
                        IDCMPFlags, IDCMPFlagSet, OpenWindow, CloseWindow;

FROM IFF8SVXLoad IMPORT Read8SVX, Dealloc8SVX, IFF8SVXInfoPtr, PlaySample,
                        OpenAudio, CloseAudio, WaitPlay, AllChannels;


CONST
  ProgName = "PlayRep,  1988 by Fridtjof Siebert [Amok]";


VAR
  Name: ARRAY[0..79] OF CHAR; (* the Sound's Name                          *)
  length: INTEGER;            (* dummy for receiving Name's Length         *)
  Info: IFF8SVXInfoPtr;       (* Sound's Data                              *)
  Window: WindowPtr;
  NuWindow: NewWindow;
  Channel: CARDINAL;          (* Active Channel                            *)


BEGIN

(*------  Get Name:  ------*)

  IF NumArgs()#0 THEN
    GetArg(1,Name,length);
  ELSE
    Terminate(0);
  END;

(*------  Open Window:  ------*)

  WITH NuWindow DO
    leftEdge := 400;
    topEdge  := 40;
    width    := 240;
    height   := 24;
    detailPen:= 0;
    blockPen := 1;
    idcmpFlags := IDCMPFlagSet{closeWindow};
    flags := WindowFlagSet{windowSizing, windowDrag, windowDepth,
                           windowClose, activate};
    firstGadget := NIL;
    checkMark := NIL;
    title := ADR("Sound loader  AMOK");
    screen := NIL;
    bitMap := NIL;
    minWidth := 30;
    minHeight := 16;
    maxWidth := -1;
    maxHeight := -1;
    type := ScreenFlagSet{wbenchScreen};
  END;
  Window := OpenWindow(NuWindow);
  Assert(Window#NIL,ADR("Shit!"));

(*------  Read and Play Sound:  ------*)

  Info := Read8SVX(Name,16000,FALSE);       (* load sound  *)

  IF Info#NIL THEN

    IF OpenAudio(1,0) THEN                  (* open audio  *)

      IF PlaySample(Info,Channel,65535,Channel) THEN END;

      WaitPort(Window^.userPort);

      CloseAudio();                         (* close audio *)

    ELSE
      DisplayBeep(NIL);
    END;

    Dealloc8SVX(Info);                      (* free sound  *)

  ELSE
    DisplayBeep(NIL);                       (* error       *)
  END;

  CloseWindow(Window);

END PlayRep.
