(**********************************************************************

    :Program.  	 FlightDemo.mod
    :Contents.	 perspective srolling 
    :Author.   	 Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.   1988 Nicolas Benezan, all rights reserved !
    :Language. 	 Modula-2
    :Translator. M2Amiga AMSoft
    :History.	 V1.0a [bne] 26.04.1988
    
**********************************************************************)

MODULE FlightDemo;

FROM Intuition	IMPORT NewScreen,ScreenPtr,NewWindow,WindowPtr,
		customScreen,IDCMPFlagSet,IDCMPFlags,WindowFlagSet,
                WindowFlags,OpenScreen,OpenWindow,CloseScreen,
                CloseWindow,stdScreenHeight,RethinkDisplay;
FROM Graphics	IMPORT ViewModeSet,ViewModes,RastPortPtr,ViewPortPtr,
		Move,Draw,SetAPen,SetBPen,SetRast,UCopListPtr,
                CBump,CMove,CWait,SetRGB4;
FROM SYSTEM 	IMPORT ADR,ADDRESS,SHIFT;
FROM Hardware	IMPORT custom;
FROM Exec 	IMPORT MemReqs, MemReqSet, AllocMem;
FROM Arts	IMPORT Assert,TermProcedure,BreakPoint;
FROM MathLib0 	IMPORT sin,cos,arctan,pi;
FROM InOut	IMPORT WriteString,WriteCard,WriteInt,WriteLn;

CONST	Width=320.0;		(* Bildschirmbreite *)
	EHeight=160;		(* Augenhhe *)			
	IWidth=TRUNC(Width)-1;
	Offset=Width/2.0-0.5;	(* Bildschirmmitte + Rundung *)
        Horizont=105;		(* bei CopperList *)
        TopSpace=25;
        sqrt2=1.4142136;
        VcountRange=(MAX(CARDINAL)+1)*256;
        HcountRange=MAX(CARDINAL)+1;
        PosRange=(IWidth+1)*64;
        
TYPE	USHORT=[0..255];
	SHORT=[-128..127];
	PosEnum=[0..IWidth];
	PosEntry=RECORD
	  d:CARDINAL;
          e:CARDINAL;
          f:INTEGER;
          g:INTEGER;
          h:INTEGER;
          k:INTEGER;
          nl:USHORT;
          nr:USHORT;
          flag:BOOLEAN;
        END;
        PosTab=ARRAY PosEnum OF PosEntry;

VAR	Screen:ScreenPtr;
	Window:WindowPtr;
        ViewPort:ViewPortPtr;
        RastPort:RastPortPtr;
        
	Position:PosTab;
        Pos:PosEnum;
        LongPos:INTEGER;
        DeltaPos:INTEGER;
        Hstep:INTEGER;
        Vcount,Hcount:CARDINAL;
        Vstep,LongVcount:LONGINT;
        ProjFaktor:CARDINAL;
	n:USHORT;
        i:INTEGER;
        
PROCEDURE InitPosTab():CARDINAL;
CONST	a=160.0;	(* Abstand Auge Bildebene *)
	b=REAL(EHeight);(* Augenhhe *)
        c=280.0;	(* Seitenlnge der Flchen *)
        Far=2000.0;	(* Grte Sichtweite *)
        MaxInt=REAL(MAX(INTEGER));
        phi0=a*2.0/MaxInt;

VAR	phi:REAL;
        RealD,RealE:REAL;
        i:CARDINAL;

PROCEDURE Round(x:REAL):INTEGER;
BEGIN
  RETURN TRUNC(x+0.5);
END Round;

BEGIN
  FOR i:=0 TO IWidth DO
    phi:=arctan((Offset-REAL(i))/Width*2.0);
    WITH Position[i] DO
    
      RealD:=ABS(c/sin(pi/4.0-phi));
      IF RealD>MaxInt THEN
        d:=MAX(INTEGER);
      ELSE
        d:=Round(RealD);
      END;
      
      RealE:=ABS(c/sin(pi/4.0+phi));
      IF RealE>MaxInt THEN
        e:=MAX(INTEGER);
      ELSE
        e:=Round(RealE);
      END;
      
      flag:=ABS(phi)>phi0;
      IF flag THEN
        f:=Round(Offset+cos(phi)/sin(phi)*a);
        h:=Round(-c/sin(phi));
      ELSE
        f:=IWidth DIV 2;
        h:=0;
      END;
       
      g:=Round(c/cos(phi));
      k:=Round(sin(phi)/cos(phi)*MaxInt+1.0);
        
      nl:=Round(Far/RealD);
      nr:=Round(Far/RealE);
    END; (* WITH *)
  END; (* FOR *)
  RETURN TRUNC(a*b*sqrt2+0.5);
END InitPosTab;

PROCEDURE InitScreen():ScreenPtr;
VAR	MyScreen:NewScreen;

BEGIN
  WITH MyScreen DO
    leftEdge:=0;
    topEdge:=0;
    width:=320;
    height:=stdScreenHeight;
    depth:=1;
    detailPen:=0;
    blockPen:=1;
    viewModes:=ViewModeSet{};
    type:=customScreen;
    font:=NIL;
    defaultTitle:=ADR("Scrolldemo by Bene");
    gadgets:=NIL;
    customBitMap:=NIL;
  END;
  RETURN OpenScreen(MyScreen);
END InitScreen;

PROCEDURE InitWindow(Screen:ScreenPtr):WindowPtr;
VAR	MyWindow:NewWindow;

BEGIN
  WITH MyWindow DO
    leftEdge:=0;
    topEdge:=71;(*stdScreenHeight-EHeight-TopSpace*)
    width:=320;
    height:=185;(*stdScreenHeight-topEdge-10;*)
    detailPen:=1;
    blockPen:=0;
    idcmpFlags:=IDCMPFlagSet{};
    flags:=WindowFlagSet{borderless};
    firstGadget:=NIL;
    checkMark:=NIL;
    title:=NIL;
    bitMap:=NIL;
    minWidth:=0;
    minHeight:=0;
    maxWidth:=0;
    maxHeight:=0;
    type:=customScreen;
  END;
  MyWindow.screen:=Screen;
  RETURN OpenWindow(MyWindow);
END InitWindow;

PROCEDURE InitCopper(ViewPort:ViewPortPtr;Horizont:INTEGER);
CONST	Anz=31;
TYPE	list=ARRAY [0..Anz] OF INTEGER;
VAR	CopList:UCopListPtr;
 	i: CARDINAL;
 	z,f0,f1: list;
BEGIN
  (* Copper List *)
  z[0]:=-60; f0[0]:=0001H; f1[0]:=0001H;
  z[1]:=-50; f0[1]:=0002H; f1[1]:=0002H;
  z[2]:=-42; f0[2]:=0003H; f1[2]:=0003H;
  z[3]:=-35; f0[3]:=0004H; f1[3]:=0004H;
  z[4]:=-29; f0[4]:=0005H; f1[4]:=0005H;
  z[5]:=-24; f0[5]:=0006H; f1[5]:=0006H;
  z[6]:=-20; f0[6]:=0007H; f1[6]:=0007H;
  z[7]:=-16; f0[7]:=0008H; f1[7]:=0008H;
  z[8]:=-13; f0[8]:=0009H; f1[8]:=0009H;
  z[9]:=-10; f0[9]:=000AH; f1[9]:=000AH;
  z[10]:=-8;f0[10]:=000BH;f1[10]:=000BH;
  z[11]:=-6;f0[11]:=000CH;f1[11]:=000CH;
  z[12]:=-4;f0[12]:=000DH;f1[12]:=000DH;
  z[13]:=-3;f0[13]:=000EH;f1[13]:=000EH;
  z[14]:=-2;f0[14]:=000FH;f1[14]:=000FH;
  z[15]:=-1;f0[15]:=001FH;f1[15]:=001FH;
  z[16]:=0;  f0[16]:=0888H;f1[16]:=0888H;
  z[17]:=2;  f0[17]:=0888H;f1[17]:=0787H;
  z[18]:=4;  f0[18]:=0887H;f1[18]:=0777H;
  z[19]:=7;  f0[19]:=0877H;f1[19]:=0676H;
  z[20]:=11; f0[20]:=0877H;f1[20]:=0666H;
  z[21]:=16; f0[21]:=0876H;f1[21]:=0565H;
  z[22]:=22; f0[22]:=0866H;f1[22]:=0555H;
  z[23]:=29; f0[23]:=0865H;f1[23]:=0454H;
  z[24]:=37; f0[24]:=0765H;f1[24]:=0444H;
  z[25]:=46; f0[25]:=0755H;f1[25]:=0343H;
  z[26]:=56; f0[26]:=0754H;f1[26]:=0333H;
  z[27]:=67; f0[27]:=0754H;f1[27]:=0232H;
  z[28]:=79; f0[28]:=0744H;f1[28]:=0222H;
  z[29]:=82; f0[29]:=0743H;f1[29]:=0121H;
  z[30]:=96; f0[30]:=0743H;f1[30]:=0111H;
  z[31]:=102;f0[31]:=0743H;f1[31]:=0010H;
  
  CopList:=AllocMem(SIZE(CopList^), MemReqSet{chip, memClear});
  FOR i:=0 TO Anz DO
   CWait(CopList, Horizont+z[i], 0); CBump(CopList);
   CMove(CopList, ADR(custom.color[0]), f0[i]); CBump(CopList);
   CMove(CopList, ADR(custom.color[1]), f1[i]); CBump(CopList);
  END;

  CWait(CopList, 10000, 255); CBump(CopList);
  ViewPort^.uCopIns:=CopList;
  RethinkDisplay
END InitCopper;

PROCEDURE CleanUp;
BEGIN
  IF Window#NIL THEN
    CloseWindow(Window);
  END;
  IF Screen#NIL THEN
    CloseScreen(Screen);
  END;
END CleanUp;  

PROCEDURE Scroll;

VAR	Lpos,Rpos:CARDINAL;
	Lclip,Rclip,Qclip:INTEGER;
        Rflag,Lflag:BOOLEAN;

CONST	Horizont=TopSpace;
	Bottom=TopSpace+EHeight;
        Far=MAX(INTEGER);
        Clip=CARDINAL(TRUNC(sqrt2*REAL(EHeight)));

  PROCEDURE FixMul(x,y:CARDINAL):CARDINAL;
  BEGIN
    RETURN LONGCARD(x)*LONGCARD(y)DIV HcountRange;
  END FixMul;
 
BEGIN
  Pos:=SHIFT(LongPos,-6);
  WITH Position[Pos] DO
    Lpos:=FixMul(Hcount,e);	(* Querlinien *)
    Rpos:=FixMul(Hcount,d);
    Qclip:=f+INTEGER((LONGINT(Hcount)*LONGINT(h))DIV HcountRange);

    SetRast(RastPort,0);	(* ClearScreen *)
    
    FOR n:=1 TO 8 DO
      Lflag:=Lpos>=Clip;
      Rflag:=Rpos>=Clip;
      IF (Lflag AND Rflag) THEN
        Move(RastPort,0,Horizont+(ProjFaktor DIV Lpos));
        Draw(RastPort,IWidth,Horizont+(ProjFaktor DIV Rpos));
      ELSIF (Lflag AND flag) THEN
        Move(RastPort,0,Horizont+(ProjFaktor DIV Lpos));
        Draw(RastPort,Qclip,Bottom);
      ELSIF (flag AND Rflag) THEN
        Move(RastPort,Qclip,Bottom);
        Draw(RastPort,IWidth,Horizont+(ProjFaktor DIV Rpos));
      ELSE
        INC(Qclip,h);
      END;
      IF Lpos<Far THEN
        INC(Lpos,e);
      END;
      IF Rpos<Far THEN
        INC(Rpos,d);
      END;
    END;
    
    Lpos:=FixMul(Vcount,d);
    Rpos:=e-FixMul(Vcount,e);
    Lclip:=INTEGER(Pos)-INTEGER(FixMul(Vcount,g));
    Rclip:=Lclip+g;
    
    FOR n:=1 TO nl DO		(* Lngslinien links *)
      Move(RastPort,Pos,Horizont);	(* Fluchtpunkt *)
      IF Lpos<Clip THEN
        Draw(RastPort,Lclip,Bottom);
        DEC(Lclip,g);
      ELSE
        Draw(RastPort,0,Horizont+(ProjFaktor DIV Lpos));
      END;
      INC(Lpos,d);
    END;
    
    FOR n:=1 TO nr DO		(* Lngslinien rechts *)
      Move(RastPort,Pos,Horizont);	(* Fluchtpunkt *)
      IF Rpos<Clip THEN
        Draw(RastPort,Rclip,Bottom);
        INC(Rclip,g);
      ELSE
        Draw(RastPort,IWidth,Horizont+(ProjFaktor DIV Rpos));
      END;
      INC(Rpos,e);
    END;    

  END;
  LongPos:=(LongPos+DeltaPos);
  IF LongPos<0 THEN
    LongPos:=LongPos MOD PosRange;
    Vcount:=Hcount;
    Hcount:=HcountRange-1-LongVcount DIV 256;
    LongVcount:=LONGINT(Vcount)*256;    
  ELSIF LongPos>=PosRange THEN
    LongPos:=LongPos MOD PosRange;
    Vcount:=HcountRange-1-Hcount;
    Hcount:=LongVcount DIV 256;
    LongVcount:=LONGINT(Vcount)*256;
  END;
  Vstep:=LONGINT(Hstep)*LONGINT(Position[Pos].k)DIV 128;
  LongVcount:=(LongVcount+Vstep)MOD VcountRange;
  Vcount:=LongVcount DIV 256;
  Hcount:=(LONGINT(Hcount)-LONGINT(Hstep))MOD HcountRange;
END Scroll;
        
BEGIN
  ProjFaktor:=InitPosTab();
  Screen:=NIL;
  Window:=NIL;
  TermProcedure(CleanUp);
  Screen:=InitScreen();
  Assert(Screen#NIL,ADR("Screen klemmt"));
  ViewPort:=ADR(Screen^.viewPort);
  SetRGB4(ViewPort,0,0,0,0);
  Window:=InitWindow(Screen);
  Assert(Window#NIL,ADR("Window klemmt"));
  RastPort:=Window^.rPort;
  InitCopper(ViewPort,Horizont);
  SetAPen(RastPort,1);
  SetBPen(RastPort,0);
  Hcount:=0;
  Vcount:=0;
  LongVcount:=0;
  LongPos:=160*64;
  REPEAT
    DeltaPos:=-(Screen^.mouseX-160)*2;
    Hstep:=ABS(Screen^.mouseY*16);
    Scroll;
  UNTIL Screen^.mouseY<0;
  
END FlightDemo.
