(**********************************************************************
   :Program.    GraphicSupport
   :Author.     Frank Staudte. [fst]
   :Address.    Hasenbergstr. 91 D-7000 Stuttgart 1
   :Support.    [bne] & [pit] Thanks!
   :History     AUG-88 1.0 first PD Amok #9 all basic funktions
   :History     APR-89 1.1 AddBitPlane, FreeBitPlane added
   :History     MAY-89 1.2 Patched for Compiler Version 3.2d
   :Copyright.  (C) Frank Staudte 1989. This is in the public domain.
   :Language.   Modula-II.
   :Translator. M2Amiga. V3.2d (V1.0 - V1.1 compiles only under 3.11d)
   :Imports.    MemSystem V1.0 [bne] Amok#5. But you could use Heap!!
   :Contents.   Supports the Graphic-Primitives. (See documentation!)
   :Remark.     Please read the Copyright-file about copyright!!!
**********************************************************************)


IMPLEMENTATION MODULE GraphicSupport;

FROM SYSTEM     IMPORT ADR, CAST;
FROM Intuition  IMPORT ViewAddress, ScreenPtr, WindowPtr;
FROM Graphics   IMPORT ViewPortPtr, ViewModeSet, ViewPtr, InitBitMap,
  BltClear, InitVPort, GetColorMap, ViewModes, RasInfoPtr, InitRastPort,
  RastPortPtr, InitView, FreeCprList, LoadView, MrgCop, LoadRGB4,
  MakeVPort, FreeColorMap, FreeVPortCopLists, BitMap, SetRGB4,
  RastPort, ViewPort, RasInfo, View, BitMapPtr;
FROM MemSystem  IMPORT Allocate, AllocMem, Deallocate;
FROM IFFSupport IMPORT NuScreen, ReadILBM, ReadILBMFlags, ReadILBMFlagSet,
  IFFInfo;
FROM Exec       IMPORT FreeMem;

PROCEDURE RasSize(width,height:CARDINAL):CARDINAL;
BEGIN
  RETURN 2*height*((width-1)DIV 16+1);
END RasSize;

PROCEDURE CreateBitMap(width,height,depth: INTEGER):BitMapPtr;
VAR
  i,j: CARDINAL;
  bitMap:BitMapPtr;

BEGIN
  IF NOT (depth > maxDepth) THEN
    Allocate(bitMap,SIZE(BitMap));
    IF bitMap#NIL THEN
      InitBitMap(bitMap^,depth,width,height);
      WITH bitMap^ DO
        FOR i:=0 TO depth-1 DO
          AllocMem(planes[i],RasSize(width,height),TRUE);
          IF planes[i] = NIL THEN
            FOR j:=0 TO i-1 DO
              Deallocate(planes[j]);
            END;
            Deallocate(bitMap);
            RETURN NIL;
          ELSE
            BltClear(planes[i],RasSize(width,height),0)
          END;
        END;
      END;
    END;
    RETURN bitMap;
  ELSE
    RETURN NIL;
  END;
END CreateBitMap;

PROCEDURE FreeILBMBitMap(bitMap: BitMapPtr);
VAR
  i: INTEGER;
BEGIN
  IF bitMap # NIL THEN
    WITH bitMap^ DO
      FOR i:=0 TO depth-1 DO
        FreeMem(planes[i],RasSize(bytesPerRow*8,rows));
      END;
    END;
  FreeMem(bitMap,SIZE(bitMap^));
  END;
END FreeILBMBitMap;

PROCEDURE FreeBitMap(bitMap: BitMapPtr);
VAR
  i: INTEGER;
BEGIN
  IF bitMap # NIL THEN
    WITH bitMap^ DO
      FOR i:=0 TO depth-1 DO
        Deallocate(planes[i]);
      END;
    END;
  Deallocate(bitMap);
  END;
END FreeBitMap;

PROCEDURE CreateViewPort(VAR rInfo: RasInfoPtr;
                             nextViewPort: ViewPortPtr;
                             leftEdge,topEdge: INTEGER;
                             width,height: INTEGER;
                             Bitmap: BitMapPtr;
                             Modes: ViewModeSet;
                             NumberOfColors: INTEGER): ViewPortPtr;

VAR     viewPort:ViewPortPtr;
BEGIN
  Allocate(viewPort,SIZE(ViewPort));
  IF viewPort#NIL THEN
    Allocate(rInfo,SIZE(RasInfo));
    IF rInfo=NIL THEN
      Deallocate(viewPort);
    ELSE
      InitVPort(viewPort^);
      INCL(Modes,sprites);
      WITH viewPort^ DO
        next:=nextViewPort;
        colorMap:=GetColorMap(NumberOfColors);
        dWidth:=width;
        dHeight:=height;
        modes:=Modes;
        dxOffset:=leftEdge;
        dyOffset:=topEdge;
        WITH rInfo^ DO
          next:=NIL;
          bitMap:=Bitmap;
          rxOffset:=0;
          ryOffset:=0;
        END; (* With rasInfo *)
        rasInfo:=rInfo;
      END; (* With ViewPort *)
    END; (* IF rInfo = NIL *)
  END; (* IF ViewPort # NIL *)
  RETURN viewPort;
END CreateViewPort;

PROCEDURE FreeViewPort(viewPort: ViewPortPtr; rInfo: RasInfoPtr);
BEGIN
  IF rInfo # NIL THEN
    Deallocate(rInfo);
  END;
  IF viewPort # NIL THEN
    IF viewPort^.colorMap # NIL THEN
      FreeColorMap(viewPort^.colorMap);
    END;
    FreeVPortCopLists(viewPort);
    Deallocate(viewPort);
  END;
END FreeViewPort;

PROCEDURE MakeVPDualPF(VAR viewPort: ViewPortPtr;
                           Bitmap: BitMapPtr): RasInfoPtr;
VAR     rInfo: RasInfoPtr;
BEGIN
  Allocate(rInfo,SIZE(rInfo^));
  IF rInfo#NIL THEN
    INCL(viewPort^.modes,dualpf);
    INCL(viewPort^.modes,pfba);
    WITH rInfo^ DO
      next:=NIL;
      rxOffset:=0;
      ryOffset:=0;
      bitMap:=Bitmap;
    END;
    viewPort^.rasInfo^.next:=rInfo;
  END;
  RETURN rInfo;
END MakeVPDualPF;

PROCEDURE CreateRastPort(bitMap: BitMapPtr): RastPortPtr;
VAR     rastPort:RastPortPtr;
BEGIN
  Allocate(rastPort,SIZE(RastPort));
  IF rastPort#NIL THEN
    InitRastPort(rastPort^);
    rastPort^.bitMap:=bitMap;
  END;
  RETURN rastPort;
END CreateRastPort;

PROCEDURE FreeRastPort(rastPort: RastPortPtr);
BEGIN
  IF rastPort # NIL THEN
    Deallocate(rastPort);
  END;
END FreeRastPort;

PROCEDURE CreateView(FirstViewPort: ViewPortPtr):ViewPtr;
VAR     view:ViewPtr;
BEGIN
  Allocate(view,SIZE(View));
  IF view#NIL THEN
    InitView(view^);
    WITH view^ DO
      viewPort:=FirstViewPort;
    END;
  END;
  RETURN view;
END CreateView;

PROCEDURE FreeView(VAR NewView: ViewPtr; ActiveView,OldView: ViewPtr);
BEGIN
  IF ActiveView # NIL THEN
    FreeCprList(ActiveView^.lofCprList);
    FreeCprList(ActiveView^.shfCprList);
    MrgCop(OldView);
    LoadView(OldView);
    NewView:=OldView;
    Deallocate(ActiveView);
  END;
END FreeView;

PROCEDURE Swap(VAR rastPort: RastPortPtr; VAR rInfo: RasInfoPtr);
VAR
  HelpMap: BitMapPtr;
BEGIN
  HelpMap:=rInfo^.bitMap;
  rInfo^.bitMap:=rastPort^.bitMap;
  rastPort^.bitMap:=HelpMap;
END Swap;

PROCEDURE Remake(view: ViewPtr; viewPort: ViewPortPtr);
BEGIN
  MakeVPort(view,viewPort);
  MrgCop(view);
  LoadView(view);
END Remake;

PROCEDURE ScrollVPort(viewPort: ViewPortPtr; rInfo: RasInfoPtr;
                      pixX, pixY: INTEGER): BOOLEAN;
VAR     i               : INTEGER;
        xdelta,ydelta   : CARDINAL;
        BitMapX,BitMapY : INTEGER;
        Okay            : BOOLEAN;

BEGIN
  Okay:=TRUE;
  WITH rInfo^ DO
    BitMapX:=bitMap^.bytesPerRow*8;
    BitMapY:=bitMap^.rows;
    IF (rxOffset+pixX < 0) OR
       (rxOffset+pixX >= BitMapX-viewPort^.dWidth)
    THEN
    Okay:=FALSE;
    pixX:=0;
    END;
    IF (ryOffset+pixY < 0) OR
       (ryOffset+pixY >= BitMapY-viewPort^.dHeight)
    THEN
      Okay:=FALSE;
      pixY:=0;
    END;

    INC(rxOffset, pixX);
    INC(ryOffset, pixY);
  END;

  RETURN Okay;
END ScrollVPort;

PROCEDURE ReadILBMBitMap(name: ARRAY OF CHAR;
                         VAR Width,Height,Depth: INTEGER;
                         VAR Colors: ILBMColors;
                         VAR err: BOOLEAN): BitMapPtr;

VAR     MyScreen: ScreenPtr;
        MyWindow: WindowPtr;
        colPtr: POINTER TO ILBMColors;

BEGIN
  err:=FALSE;
  IF NOT ReadILBM(name,ReadILBMFlagSet{dontopen},MyScreen,MyWindow) THEN
    err:=TRUE; RETURN NIL;
  ELSE
    WITH IFFInfo DO
      WITH BMHD DO
        Width:=width;
        Height:=height;
        Depth:=INTEGER(depth);
      END;
      colPtr:=ADR(CMAP);
      Colors:=colPtr^;
    END;
    RETURN NuScreen.customBitMap;
  END;
END ReadILBMBitMap;

PROCEDURE SetILBMColors(viewPort: ViewPortPtr; Colors: ILBMColors);
VAR     i: CARDINAL;
BEGIN
  FOR i:=0 TO Colors.ColCount DO
    SetRGB4(viewPort,i,CARDINAL(Colors.red[i]),CARDINAL(Colors.green[i]),
      CARDINAL(Colors.blue[i]));
  END;
END SetILBMColors;

PROCEDURE AddBitPlane(VAR bitMap: BitMapPtr; number: INTEGER): BOOLEAN;
 VAR err: BOOLEAN;
     loop, reverse: INTEGER;
 BEGIN
   err := FALSE;
   IF bitMap # NIL THEN
     WITH bitMap^ DO
       IF (INTEGER(depth) + number <= maxDepth) THEN
         FOR loop:=INTEGER(depth) TO INTEGER(depth)+number-1 DO
           AllocMem(planes[loop],RasSize(bytesPerRow*8,rows),TRUE);
           IF planes[loop] # NIL THEN
             BltClear(planes[loop],RasSize(bytesPerRow*8,rows),0);
             INC(depth,number);
           ELSE
             FOR reverse:=loop TO depth DO
               Deallocate(planes[loop]);
               RETURN FALSE; (* Out of Mem *)
             END;
           END;
         END; (* FOR *)
         RETURN TRUE; (* Succsess *)
       ELSE
         RETURN FALSE (* number to big *)
       END;
     END; (* WITH *)
       RETURN TRUE;
   ELSE
     RETURN FALSE; (* No BitMap given! *)
   END;
 END AddBitPlane;

PROCEDURE FreeBitPlane(VAR bitMap: BitMapPtr; number: INTEGER): BOOLEAN;
 VAR loop: INTEGER;
 BEGIN
   IF bitMap # NIL THEN
     WITH bitMap^ DO
       IF (INTEGER(depth) - number >= 0) THEN
         FOR loop:=INTEGER(depth)-1 TO INTEGER(depth)-number DO
           Deallocate(planes[loop]);
           DEC(depth,number);
           RETURN TRUE; (* Successful *)
         END;
       ELSE
         RETURN FALSE; (* number too big *);
       END;
     END; (* WITH *)
   ELSE
     RETURN FALSE; (* No BitMap given *)
   END;
 END FreeBitPlane;

END GraphicSupport.MOD
