(**********************************************************************

    :Program.    Trees.mod
    :Contents.   generic data type: binary trees
    :Author.     Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Public Domain
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    TaskMemory [bne]
    :History.    V1.0a [bne] 25.Jun.1989

**********************************************************************)

IMPLEMENTATION MODULE Trees;

FROM Exec       IMPORT CopyMem;
FROM SYSTEM     IMPORT ADDRESS, ADR, BYTE;

CONST
  NodeSize=SIZE(TreeNode);

TYPE
  Tree=POINTER TO Root;
  Root=RECORD
    rootNode: TreeNodePtr;
    numNodes: LONGINT;
    cmp: CompareProc;
  END;

PROCEDURE InitTree(VAR T: Tree;
                       Cmp: CompareProc): BOOLEAN;
  BEGIN
    TreesAllocProc(T, SIZE(Root));
    IF T#NIL THEN
      WITH T^ DO
        rootNode:=NIL;
        numNodes:=0;
        cmp:=Cmp;
      END;
      RETURN TRUE;
    END;
    RETURN FALSE;
  END InitTree;

PROCEDURE DiscardTree(VAR T: Tree);
  VAR
    Next:TreeNodePtr;

  PROCEDURE Delete(Node: TreeNodePtr);
    BEGIN
      WHILE Node#NIL DO
        Delete(Node^.l);
        Next:=Node^.r;
        TreesDeallocProc(Node);
        Node:=Next;
      END;
    END Delete;

  BEGIN
    Delete(T^.rootNode);
    TreesDeallocProc(T);
  END DiscardTree;

PROCEDURE Add(T: Tree;
              Node: ADDRESS);
  VAR
    NodePtr, Parent: TreeNodePtr;
  BEGIN
    NodePtr:=Node;
    WITH NodePtr^ DO
      l:=NIL;
      r:=NIL;
    END;
    WITH T^ DO
      INC(numNodes);
      Parent:=rootNode;
      IF Parent=NIL THEN
        rootNode:=NodePtr;
        RETURN
      END;
      LOOP
        IF cmp(LONGINT(NodePtr)+NodeSize, LONGINT(Parent)+NodeSize)<0 THEN
          IF Parent^.l=NIL THEN
            Parent^.l:=NodePtr;
            EXIT
          ELSE
            Parent:=Parent^.l;
          END;
        ELSIF Parent^.r=NIL THEN
          Parent^.r:=NodePtr;
          EXIT
        ELSE
          Parent:=Parent^.r;
        END; (* IF *)
      END; (* LOOP *)
    END; (* WITH *)
  END Add;

PROCEDURE Put(T: Tree;
              Data: ARRAY OF BYTE): ADDRESS;
  VAR
    Node: TreeNodePtr;
    DataSize: CARDINAL;
  BEGIN
    DataSize:=HIGH(Data)+1;
    TreesAllocProc(Node, NodeSize+DataSize);
    IF Node#NIL THEN
      Add(T, Node);
      INC(Node, NodeSize);
      CopyMem(ADR(Data), Node, DataSize);
    END;
    RETURN Node;
  END Put;

PROCEDURE Get(T: Tree;
              Data: ADDRESS): ADDRESS;
  VAR
    Node: TreeNodePtr;
    Diff: INTEGER;
  BEGIN
    WITH T^ DO
      Node:=rootNode;
      WHILE Node#NIL DO
        Diff:=cmp(Data, LONGINT(Node)+NodeSize);
        IF Diff<0 THEN
          Node:=Node^.l;
        ELSIF Diff=0 THEN
          INC(Node, NodeSize);
          RETURN Node;
        ELSE
          Node:=Node^.r;
        END;
      END;
      RETURN NIL;
    END;
  END Get;

PROCEDURE Do(T: Tree;
             Action: TreeProc;
             Ref: ADDRESS);

  PROCEDURE Visit(Node: TreeNodePtr);
    BEGIN
      WHILE Node#NIL DO
        WITH Node^ DO
          Visit(l);
          Action(LONGINT(Node)+NodeSize, Ref);
          Node:=r;
        END;
      END;
    END Visit;

  BEGIN
    Visit(T^.rootNode);
  END Do;

PROCEDURE NodesInTree(T: Tree): LONGINT;
  BEGIN
    RETURN T^.numNodes
  END NodesInTree;

END Trees.

