(*
  :Program.       HotKey
  :Author.        Volker Rudolph
  :Address.       Medicusstr. 31 / 6750 Kaiserslautern
  :Phone.         0631/17160
  :ShortCut.      [vor]
  :Version.       1.0
  :Date.          1.7.1989
  :Copyright.     PD
  :Language.      Modula-II
  :Translator.    M2Amiga 3.2d
  :Imports.       Nichts
  :Contents.      HotKey hilft Programme zu schreiben, die wie z.B.
  :Contents.      'Blitz' im Hintergrund verschwinden, und auf Tasten-
  :Contents.      druck wieder erscheinen.
*)
IMPLEMENTATION MODULE HotKey;

FROM Arts IMPORT TermProcedure, Assert;
FROM Exec IMPORT OpenDevice, CloseDevice, DoIO, Forbid, Permit, AllocSignal,
                 FreeSignal, Signal, FindTask, Wait,
                 Interrupt, IOStdReqPtr, MsgPortPtr, TaskPtr;
FROM ExecSupport IMPORT CreatePort, CreateStdIO, DeletePort, DeleteStdIO;
FROM Input  IMPORT inputName, addHandler, remHandler;
FROM InputEvent IMPORT InputEventPtr, Qualifiers, QualifierSet, Class;
FROM SYSTEM IMPORT ADR, ADDRESS, LONGSET, INLINE, REG, SETREG;

VAR
  handlerStuff:Interrupt;
  inputPort:MsgPortPtr;
  inputReq:IOStdReqPtr;
  sigNum:LONGINT;
  signal:LONGSET;
  myTask:TaskPtr;
  hotKey:CARDINAL;
  hotQual:QualifierSet;

  (* $S- *)
  PROCEDURE InputHandler():InputEventPtr;
  VAR
    event:InputEventPtr;
  BEGIN
    event := ADDRESS(REG(8));  (* a0:InputEventPtr *)
    Forbid;
    WITH event^ DO
     IF (class = rawkey) AND
        (code = hotKey)  AND
        ((hotQual * qualifier) # QualifierSet{}) THEN
       Signal(myTask,signal);
     END; (* IF *)
    END; (* WITH *)
    Permit;
    RETURN event;
  END InputHandler;

  (* $E- *)
  PROCEDURE InputHandlerInterface;
  BEGIN
    INLINE(048E7H, 03F3EH);       (* movem.l  d2-d7/a2-a6,-(sp) *)
    SETREG(0,InputHandler());     (* d0 : InputHandler()        *)
    INLINE(04CDFH, 07CFCH);       (* movem.l  (sp)+,d2-d7/a2-a6 *)
    INLINE(04E75H);               (* rts                        *)
  END InputHandlerInterface;

  PROCEDURE AddHandler;
  BEGIN
    inputPort := CreatePort(ADR("HotKey.port"),0);
    inputReq := CreateStdIO(inputPort);
    OpenDevice(ADR(inputName),0,inputReq,LONGSET{});
    Assert(inputReq^.error = 0,ADR("AddHandler: Can't open input.device"));
    WITH handlerStuff DO
      code := InputHandlerInterface;
      node.pri := 60;
      node.name := ADR("HotKey.handler");;
    END; (* WITH *)
    inputReq^.command := addHandler;
    inputReq^.data := ADR(handlerStuff);
    DoIO(inputReq);
    Assert(inputReq^.error = 0,ADR("AddHandler: Can't add handler"));
    sigNum := AllocSignal(-1);
    Assert(sigNum # -1,ADR("AddHandler: Can't get signal"));
    signal := LONGSET{sigNum};
  END AddHandler;

  PROCEDURE RemHandler;
  BEGIN
    IF inputReq # NIL THEN
      WITH inputReq^ DO
        IF device # NIL THEN
          command := remHandler;
          data := ADR(handlerStuff);
          DoIO(inputReq);
          CloseDevice(inputReq);
        END; (* IF *)
      END; (* WITH *)
      DeleteStdIO(inputReq);
      inputReq := NIL;
    END; (* IF *)
    IF inputPort # NIL THEN
      DeletePort(inputPort);
      inputPort := NIL;
    END; (* IF *)
    IF signal # LONGSET{} THEN
      FreeSignal(sigNum);
      signal := LONGSET{};
    END; (* IF *)
  END RemHandler;

  PROCEDURE WaitHotKey(rawCode:CARDINAL;qualifier:QualifierSet);
  VAR
    sig:LONGSET;
  BEGIN
    hotKey := rawCode;
    hotQual := qualifier;
    AddHandler;
    sig := Wait(signal);
    RemHandler;
  END WaitHotKey;

BEGIN
  myTask := FindTask(NIL);
  TermProcedure(RemHandler);
END HotKey.
