(**********************************************************************

    :Program.    ClusterTest.mod
    :Contents.   Test module for Clusters [bne]
    :Author.     Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Public Domain
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    Clusters, MemSystem [bne]
    :History.    V1.0 [bne] 07.Jul.1989
    :History.    V1.1 [bne] 09.Jul.1989 (+ WriteHeap)

**********************************************************************)

MODULE ClusterTest;

FROM Arts         IMPORT Assert;
FROM BigSets      IMPORT BitInSet;
FROM Clusters     IMPORT Allocate, ClusterPtr, Deallocate, HeapArray,
                         InitMemManager;
FROM InOut        IMPORT Write, WriteHex, WriteInt, WriteLn, WriteString;
FROM RandomNumber IMPORT RND;
FROM SYSTEM       IMPORT ADR;
IMPORT MemSystem;

CONST
  Max=512;
  LongIntSize=SIZE(LONGINT);

VAR
  Test: ARRAY [0..Max] OF POINTER TO LONGINT;
  Sizes: ARRAY [0..0] OF LONGINT;
  i, j: INTEGER;

PROCEDURE WriteHeap;
  VAR
    Cl: ClusterPtr;
    Bit: CARDINAL;
    BitStr: ARRAY [0..63] OF CHAR;
  BEGIN
    WITH HeapArray^ DO
      WriteString(" firstFreeCluster:");
      WriteHex(LONGINT(firstFreeCluster),6);
      WriteString(" clusterSize:");
      WriteInt(clusterSize,1);
      WriteString(" blockSize:");
      WriteInt(blockSize,1);
      WriteString(" blocksPerCluster:");
      WriteInt(blocksPerCluster,1);
      WriteLn;
      WriteString(" Cluster, freeBlocks, firstFreeBlock, blockAllocMap:");
      WriteLn;
      Cl:=clusterList;
      WHILE Cl#NIL DO
        WITH Cl^ DO
          Write(" ");
          WriteHex(LONGINT(Cl),6);
          Write(",");
          WriteInt(freeBlocks,2);
          Write(",");
          WriteInt(firstFreeBlock,2);
          Write(",");
          FOR Bit:=0 TO blocksPerCluster-1 DO
            IF BitInSet(blockAllocMap, Bit) THEN
              BitStr[Bit]:="*";
            ELSE
              BitStr[Bit]:="";
            END;
          END;
          BitStr[blocksPerCluster]:=CHR(0);
          WriteString(BitStr);
          Cl:=next;
        END;
        WriteLn;
      END;
    END;
    WriteLn;
  END WriteHeap;

BEGIN
  Sizes[0]:=LongIntSize;
  IF InitMemManager(MemSystem.Allocate, MemSystem.NoCareDeallocate,
                    512, Sizes) THEN
    FOR i:=0 TO Max DO
      Test[i]:=NIL;
    END;
    FOR j:=0 TO 600 DO
      i:=RND(Max);
      IF Test[i]=NIL THEN
        Allocate(Test[i], LongIntSize);
        IF Test[i]#NIL THEN
          WriteString("Allocate: ");
          WriteHex(LONGINT(Test[i]), 6);
          WriteLn;
          WriteHeap;
          Test[i]^:=i;
        END;
      ELSE
        Assert(Test[i]^=i,ADR("value bug"));
        WriteString("Deallocate: ");
        WriteHex(LONGINT(Test[i]), 6);
        WriteLn;
        Deallocate(Test[i]);
        WriteHeap;
      END;
    END;
    FOR i:=0 TO Max DO
      IF Test[i]#NIL THEN
        Assert(Test[i]^=i,ADR("value bug"));
        WriteString("Deallocate: ");
        WriteHex(LONGINT(Test[i]), 6);
        WriteLn;
        Deallocate(Test[i]);
        WriteHeap;
      END;
    END;
  END;
END ClusterTest.

