(**********************************************************************
  :Program.    TFD.mod
  :Contents.   Some File-Operations using TurboFiles
  :Author.     Stefan Salewski
  :Copyright.  Public Domain
  :Language.   Modula-2
  :Translator. M2Amiga AMSoft V3.2d
  :History.    V1.0 1.Jul.1989
  :Address.    Stolper Weg 3, D-2160 Stade
  :Imports.    TurboFiles and Assembler2
**********************************************************************)

MODULE TFD;
  FROM TurboFiles IMPORT Lookup,CloseFile,FilePtr,TurboResult,SetPosMode,
       TurboRead,TurboWrite,Search,TurboGetPos,TurboFileLength,TurboSetPos,
       ReadWrite,ReadOnly,NewFile,Code;
  FROM InOut IMPORT WriteString,ReadString,WriteLn,Write,WriteInt,WriteHex;
  FROM SYSTEM IMPORT ADR,ADDRESS,CAST,LONGSET;
  FROM Arguments IMPORT NumArgs,GetArg;
  FROM Str IMPORT Length;
  FROM RandomNumber IMPORT PutSeed,RND;
  FROM Exec IMPORT Byte,UByte,CopyMem;
  FROM ASCII IMPORT eol,cr,lf,esc;
  FROM Conversions IMPORT ValToStr;
  FROM Intuition IMPORT Preferences,GetPrefs;
  FROM Assembler2 IMPORT MinLongInt;
  
  CONST
    underlineOn='[4m';
    colorText='[33m';
    normalCharSet='[0m';
    
    BufSize=512;
    StringLength=80;
    
  TYPE
    TwoChars=ARRAY[0..2] OF CHAR;
    FileName=ARRAY[0..StringLength] OF CHAR;
    BigString=ARRAY[0..255] OF CHAR;
  VAR
    fPtr:FilePtr;
    i,act:LONGINT;
    numArgs:INTEGER;
    c:CHAR;
    arg1:FileName;
    arg2:TwoChars;
    arg3:BigString;
    short:TwoChars;
    len1,len2,len3:INTEGER;
    
  (*
  PROCEDURE Printable(c:CHAR):BOOLEAN;
  BEGIN
    RETURN (c>CHAR(31)) AND ((c<CHAR(128)) OR (c>CHAR(159)))
  END Printable;
  *)
  
  PROCEDURE WriteStr(s:ARRAY OF CHAR);
  BEGIN
    WriteString(s);
    WriteLn;
  END WriteStr;
  
  PROCEDURE GetArg3(text:ARRAY OF CHAR);
  BEGIN
    IF arg3[0]=0C THEN
      WriteString(text);
      ReadString(arg3);
      len3:=Length(arg3);
      WriteLn;
    END;
  END GetArg3;
  
  PROCEDURE Point(VAR c:CHAR;ok:CHAR);
  BEGIN
    IF (c#ok) AND ((c<=CHAR(31)) OR ((c>=CHAR(128)) AND (c<=CHAR(159)))) THEN
      c:='.'
    END
  END Point;
  
  PROCEDURE Usage;
  BEGIN
    GetArg(0,arg3,len3);
    WriteStr(arg3);
    WriteLn;
    WriteStr('(c) 1989 by Stefan Salewski');
    WriteStr('            Stolper Weg 3');
    WriteStr('            D-2160 Stade');
    WriteLn;
    WriteStr('Usage:');
    WriteString(arg3);
    WriteStr(" FileName (C|D|E|H|I|L|N|P|S|T) [String]");
    WriteStr("The letter determines the Action:");
    WriteStr("C: Codes the File 'FileName' with 'String'");
    WriteStr("D: Decodes the File 'FileName' with 'String'");
    WriteStr("E: Compares the file 'FileName' with file 'String'");
    WriteStr("H: Types the file 'FileName' as HexDump");
    WriteStr("I: Transforms the File 'FileName' to M2-INLINE-Code");
    WriteStr("L: Prints the file 'FileName' with Linenumbers");
    WriteStr("N: Types the file 'FileName' with LineNumbers");
    WriteStr("P: Prints the file 'FileName' with header and pagenumbers");
    WriteStr("S: Search in the File 'FileName' for 'String'");
    WriteStr("T: Types the File 'FileName'");
  END Usage;
    
  PROCEDURE Print(numbers:BOOLEAN);
    CONST
      Top=4;
      Header=2;
      Bottom=4;
    VAR
      err:BOOLEAN;
      printer:FilePtr;
      str1:ARRAY[0..10] OF CHAR;
      i,line,page,pageSize,paperLength:CARDINAL;
      fileSize:LONGINT;
      
    PROCEDURE GetPaperLength(VAR len:CARDINAL);
    VAR
      p:Preferences;
    BEGIN
      GetPrefs(ADR(p),SIZE(Preferences));
      len:=p.paperLength
    END GetPaperLength;
    
    PROCEDURE NewLine(lines:INTEGER);
      VAR
        i:INTEGER;
        s:ARRAY[0..1] OF CHAR;
    BEGIN
      s[0]:=cr;
      s[1]:=lf;
      FOR i:=1 TO lines DO
        TurboWrite(printer,ADR(s),2);
      END
    END NewLine;
    
  BEGIN
    GetPaperLength(paperLength);
    line:=0;
    page:=0;
    c:=eol;
    pageSize:=paperLength-Top-Header-Bottom;
    WriteString('Printing the File: ');
    WriteStr(arg1);
    WriteLn;
    IF Lookup(fPtr,arg1,BufSize,ReadOnly)=done THEN
      str1:='PRT:';
      IF Lookup(printer,str1,BufSize,NewFile)=done THEN
        fileSize:=TurboFileLength(fPtr);
        str1[0]:=esc;
        str1[1]:="#";
        str1[2]:="1";
        TurboWrite(printer,ADR(str1),3);
        WHILE (fPtr^.res=done) AND (printer^.res=done) DO
          IF c=eol THEN
            IF ((line MOD pageSize)=0) AND (TurboGetPos(fPtr)<fileSize) THEN
              IF line>0 THEN
                NewLine(Bottom+1);
              END;
              INC(page);
              NewLine(Top);
              str1:='Page ';
              TurboWrite(printer,ADR(str1),5);
              ValToStr(page,FALSE,str1,10,-6,' ',err);
              TurboWrite(printer,ADR(str1),6);
              TurboWrite(printer,ADR(arg1),len1);
              NewLine(Header);
            ELSE
              NewLine(1)
            END;
            INC(line);
            IF numbers AND (TurboGetPos(fPtr)<fileSize) THEN
              ValToStr(line,FALSE,str1,10,4,' ',err);
              str1[4]:=' ';
              str1[5]:=' ';
              TurboWrite(printer,ADR(str1),6);
            END
          ELSE
            TurboWrite(printer,ADR(c),1);
          END;
          TurboRead(fPtr,ADR(c),1,act);
        END;
        i:=line MOD pageSize;
        IF i>1 THEN
          NewLine(paperLength-Top-Header-Bottom-i+1)
        END;
        NewLine(Bottom);
        WriteStr('Ready');
        CloseFile(printer);
      ELSE
        WriteStr('Cannot open Printer!');
      END;
      CloseFile(fPtr);
    ELSE
      WriteStr('File not found');
    END;
  END Print;
  
  PROCEDURE Type(numbers:BOOLEAN);
  VAR
    line,fileLength:LONGINT;
  BEGIN
    WriteString('Typing the File: ');
    WriteStr(arg1);
    WriteLn;
    IF Lookup(fPtr,arg1,BufSize,ReadOnly)=done THEN
      fileLength:=TurboFileLength(fPtr);
      line:=1;
      i:=0;
      IF numbers THEN
        WriteInt(line,5);
        Write(' ');
      END;
      WHILE fPtr^.res=done DO
        TurboRead(fPtr,ADR(arg3[i]),1,act);
        IF (arg3[i]=eol) OR (i>=80) OR (act=0) THEN
          arg3[i+act]:=0C;
          WriteString(arg3);
          IF numbers AND (arg3[i]=eol) AND (TurboGetPos(fPtr)<fileLength) THEN
            INC(line);
            WriteInt(line,5);
            arg3:='  ';
            i:=2
          ELSE 
            i:=0
          END
        ELSE
          INC(i)
        END;
      END;
      CloseFile(fPtr);
    ELSE
      WriteStr('File not found');
    END;
  END Type;
  
  PROCEDURE TypeHex;
  VAR
    line:CARDINAL;
  BEGIN
    WriteString('Hexdump of: ');
    WriteStr(arg1);
    WriteLn;
    IF Lookup(fPtr,arg1,BufSize,ReadOnly)=done THEN
      line:=0;
      WHILE fPtr^.res=done DO
        TurboRead(fPtr,ADR(arg3),16,act);
        IF act>0 THEN  
          WriteHex(line,4);
          WriteString(': ');
          FOR i:=0 TO 15 DO
            IF (CAST(LONGSET,i)*LONGSET{0..1})=LONGSET{} THEN
              Write(' ');
            END;
            IF i<act THEN
              WriteHex(CAST(UByte,arg3[i]),2)
            ELSE
              WriteString('  ')
            END;
          END;
          WriteString('  ');
          FOR i:=0 TO 15 DO
            Point(arg3[i],'.');
          END;
          arg3[act]:=0C;
          WriteStr(arg3);
          INC(line,16);
        END;
      END;
      CloseFile(fPtr);
    ELSE
      WriteStr('File not found');
    END;
  END TypeHex;
  
  PROCEDURE CodeIt(decode:BOOLEAN);
  BEGIN
    GetArg3('CodeWord: ');
    IF Code(arg1,arg3,decode) THEN
      WriteString('File ');
      WriteString(arg1);
      Write(' ');
      IF decode THEN
        WriteString('de')
      END;
      WriteString('coded with CodeWord: ');
      WriteStr(arg3)
    ELSE
      WriteStr('File not found');
    END;
  END CodeIt;
  
  PROCEDURE Equal;
  VAR
    file:FilePtr;
    s:ARRAY[0..6] OF CHAR;
    act1,act2:LONGINT;
  BEGIN
    GetArg3('File to Compare with: ');
    IF Lookup(fPtr,arg1,BufSize,ReadOnly)=done THEN
      IF Lookup(file,arg3,BufSize,ReadOnly)=done THEN
        s:='  |  ';
        REPEAT
          TurboRead(fPtr,ADR(s[0]),1,act1);
          TurboRead(file,ADR(s[4]),1,act2);
        UNTIL (fPtr^.res#done) OR (file^.res#done) OR (s[0]#s[4]);
        IF (act1=0) AND (act2=0) THEN
          WriteStr('The Files are identical!')
        ELSIF act1=0 THEN
          WriteString('End of File ');
          WriteString(arg1);
          WriteStr(' reached!')
        ELSIF act2=0 THEN
          WriteString('End of File ');
          WriteString(arg3);
          WriteStr(' reached!')
        ELSE
          WriteString('Differences at position ');
          WriteInt(TurboGetPos(fPtr),6);
          WriteStr(' detected:');
          i:=0;
          WHILE (i<10) AND (fPtr^.res=done) AND (file^.res=done) DO
            INC(i);
            Point(s[0],'.');
            Point(s[4],'.');
            WriteStr(s);
            TurboRead(fPtr,ADR(s[0]),1,act1);
            TurboRead(file,ADR(s[4]),1,act2);
          END;
        END;
        CloseFile(file)
      ELSE
        WriteString('File ');
        WriteString(arg3);
        WriteStr(' not found');
      END;
      CloseFile(fPtr);
    ELSE
      WriteString('File ');
      WriteString(arg1);
      WriteStr(' not found');
    END;
  END Equal;
  
  PROCEDURE Find;
    VAR
      pos:LONGINT;
  BEGIN
    GetArg3('String to search: ');
    IF (len3>0) AND (Lookup(fPtr,arg1,BufSize,ReadOnly)=done) THEN
      REPEAT
        pos:=Search(fPtr,arg3,len3);
        IF pos=-1 THEN
          WriteStr('String not found');
          short[0]:='N';
        ELSE
          WriteString('String found at position: ');
          WriteInt(pos,5);
          i:=MinLongInt(pos,180);
          IF TurboSetPos(fPtr,-i,current) THEN END;
          c:=0C; (* #eol *)
          WHILE (i>0) AND (c#eol) DO
            TurboRead(fPtr,ADR(c),1,act);
            DEC(i);
          END;
          IF i=0 THEN
            IF TurboSetPos(fPtr,-MinLongInt(pos,180),current) THEN END;
          END;
          WriteLn;
          WriteLn;
          WHILE TurboGetPos(fPtr)<pos DO
            TurboRead(fPtr,ADR(c),1,act);
            Point(c,eol);
            Write(c);
          END;
          Write(esc);
          WriteString(underlineOn);
          Write(esc);
          WriteString(colorText);
          i:=0;
          pos:=0;
          REPEAT
            TurboRead(fPtr,ADR(c),1,act);
            IF c=eol THEN
              INC(pos)
            END;
            Point(c,eol);
            Write(c);
            INC(i);
            IF i=len3 THEN
              Write(esc);
              WriteString(normalCharSet);
            END;
          UNTIL (pos=4) OR (fPtr^.res#done) OR (i=180);
          IF TurboSetPos(fPtr,len3-i,current) THEN END;
          WriteLn;
          WriteString('Search for next occurence? Y-J/N : ');
          ReadString(short);
          WriteLn;
        END;
      UNTIL (CAP(short[0])#'Y') AND (CAP(short[0])#'J');
      CloseFile(fPtr);
    ELSE
      WriteStr('File not found');
    END;
  END Find;
  
  PROCEDURE Inline;
  VAR
    err:BOOLEAN;
    destFile:FilePtr;
    twoByte:ARRAY[0..1] OF UByte;
    buf:ARRAY[0..12] OF CHAR;
    act,i:LONGINT;
    l:INTEGER;
  BEGIN
    GetArg3('Name of Destination File: ');
    IF len3=0 THEN
      WriteStr('No Destination File');
      RETURN
    END;
    IF Lookup(fPtr,arg1,BufSize,ReadOnly)=done THEN
      IF Lookup(destFile,arg3,BufSize,NewFile)=done THEN
        i:=0;
        buf:='    INLINE(';
        TurboWrite(destFile,ADR(buf),11);
        LOOP
          IF (i MOD 16)=0 THEN
            buf:='       ';
            buf[0]:=eol;
            TurboWrite(destFile,ADR(buf),7)
          END;
          TurboRead(fPtr,ADR(twoByte),2,act);
          IF act=2 THEN
            l:=CAST(INTEGER,twoByte)
          ELSIF act=1 THEN
            l:=twoByte[0]
          ELSE
            EXIT
          END;  
          INC(i,2);
          ValToStr(l,FALSE,buf,16,5,'0',err);
          buf[5]:='H';
          buf[6]:=',';
          TurboWrite(destFile,ADR(buf),7);
        END;
        buf[0]:=')';
        buf[1]:=';';
        buf[2]:=eol;
        IF TurboSetPos(destFile,-1,current) THEN END;
        TurboWrite(destFile,ADR(buf),3);
        WriteInt(i,6);
        WriteStr(' Bytes Written.');
      ELSE
        WriteStr('Cannot open Destination File')
      END;
    ELSE
      WriteStr('File not found')
    END;
  END Inline;
  
BEGIN
  numArgs:=NumArgs();
  GetArg(1,arg1,len1);
  GetArg(2,arg2,len2);
  GetArg(3,arg3,len3);
  IF numArgs=0 THEN
    WriteStr("Usage: FileName 'x' [String]");
    WriteString('FileName: '); 
    ReadString(arg1);
    len1:=Length(arg1);
  END;
  IF numArgs<2 THEN
    WriteString("Command: ");
    ReadString(arg2);
    len2:=Length(arg2);
  END;
  WriteLn;
  IF (len2=1) AND (len1>0) THEN
    CASE CAP(arg2[0]) OF
      |'T':Type(FALSE)
      |'S':Find
      |'C':CodeIt(FALSE)
      |'D':CodeIt(TRUE)
      |'H':TypeHex
      |'N':Type(TRUE)
      |'P':Print(FALSE);
      |'L':Print(TRUE);
      |'I':Inline
      |'E':Equal
      |ELSE
        Usage;
    END;
  ELSE
    Usage
  END;
  WriteLn;
END TFD.

