(**********************************************************************

    :Program.    Environment.mod
    :Contents.   Console and argument handler for Make [bne]
    :Author.     Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Amok internal, NOT in the public domain !
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    ErrorReq1.2, ExDos, Arguments1.1 [bne]
    :History.    V1.0b [bne] 9.Feb.1989
    :History.    V1.1a [bne] 7.Feb.1989 (+ SystemDir, MainDir)
    :History.    V1.2a [bne] 7.Feb.1989 (+ CurrentDir)
    :History.    V1.3 [bne] 22.Feb.1989 (+ OriginDir, SetDir() )
    :History.    V1.4c [bne] 5.Apr.1989 (+ Options, cosmetics)
    :History.    V1.5 [bne] 18.May.1989 (+ ExDos)
    :History.    V1.6 [bne] 22.Aug.1989 (bugs in Arguments fixed)

**********************************************************************)

IMPLEMENTATION MODULE Environment;

FROM Arguments  IMPORT GetArg, NumArgs, GetLock;
FROM Arts       IMPORT TermProcedure, wbStarted;
FROM Dos        IMPORT Close, FileHandlePtr, newFile, Open, Output,
                       Write;
FROM ErrorReq   IMPORT Assert, ExitQuiet, ErrHeader, AssertMessage;
FROM ExDos      IMPORT DupLock, FileLockPtr, GetCurrentDir, Lock,
                       sharedLock;
FROM Exec       IMPORT FindTask;
FROM Icon       IMPORT FindToolType, FreeDiskObject, GetDiskObject,
                       MatchToolValue;
FROM Str        IMPORT Copy, last, LastPos, Length, noOccur;
FROM Strings    IMPORT Compare, Delete;
FROM Workbench  IMPORT DiskObjectPtr;
FROM SYSTEM     IMPORT ADDRESS, ADR, SETREG;
IMPORT ASCII, Strings;

CONST
  DefaultOutput   = "CON:100/50/440/100/M2Make";
  DefaultPath     = "M2:";
  OUTPUT          = "OUTPUT";
  SYSTEMPATH      = "SYSTEMPATH";
  OPTIONS         = "OPTIONS";

  (* Error messages *)
  IllOption       = "Illegal Option";
  SysPathNotFound = "SYSTEMPATH not found";
  ModuleNotFound  = "Module not found";
  InvalidOutput   = "Couldn't open Output";
  TooManyArgs     = "Too many arguments";
  Usage           = "USAGE: M2M [-adiqx] MainModule";
  MakeError       = "M2Make Error:";
  MakeAborted     = "M2Make Aborted: ";

TYPE
  String=ARRAY [0..80] OF CHAR;

VAR
  OptionSet:OptionFlagSet;
  OutputFileName,SystemPath:String;
  OutputFile:FileHandlePtr;
  MainModule,TestPath:String;
  TestDir:FileLockPtr;

(**)
(* Read CLI or Workbench arguments and the default parameters           *)
(**)
PROCEDURE ExamineEnvironment;
VAR
  MyOwnName:ARRAY [0..32] OF CHAR;
  InfoFile:DiskObjectPtr;
  Num,Len,Pos,MainArg:INTEGER;
  Option:ARRAY [0..5] OF CHAR;
  OptionPos,PathLen:INTEGER;
  MainPath:String;
  Dir:FileLockPtr;

(**)
(* Strings.Occurs() for constants                                       *)
(**)
  PROCEDURE Occurs(Array:ARRAY OF CHAR;Char:CHAR;VAR Pos:INTEGER):BOOLEAN;
    VAR
      Count:INTEGER;
    BEGIN
      FOR Count:=0 TO HIGH(Array) DO
        IF CAP(Char)=Array[Count] THEN
          Pos:=Count;
          RETURN TRUE;
        END;
      END;
      RETURN FALSE;
    END Occurs;

(**)
(* Examine the ToolType field of our icon                               *)
(**)
  PROCEDURE ReadToolTypeInfo(Info:ADDRESS);
    VAR
      SysPathPtr,OutFileNamePtr,OptionPtr:POINTER TO String;
      OptionChar:POINTER TO CHAR;
      OptionNum:OptionFlags;
    BEGIN
      OutFileNamePtr:=FindToolType(Info,ADR(OUTPUT));
      IF OutFileNamePtr#NIL THEN
        Copy(OutputFileName,OutFileNamePtr^);
      END;
      SysPathPtr:=FindToolType(Info,ADR(SYSTEMPATH));
      IF SysPathPtr#NIL THEN
        Copy(SystemPath,SysPathPtr^);
      END;
      OptionPtr:=FindToolType(Info,ADR(OPTIONS));
      IF OptionPtr#NIL THEN
        Option[1]:=ASCII.nul;
        FOR OptionNum:=MIN(OptionFlags) TO MAX(OptionFlags) DO
          OptionChar:=ADR(OptionChars);
          INC(OptionChar,ORD(OptionNum));
          Option[0]:=OptionChar^;
          IF MatchToolValue(OptionPtr,ADR(Option))#0 THEN
            INCL(OptionSet,OptionNum);
          END;
        END;
      END;
    END ReadToolTypeInfo;

  BEGIN
    GetArg(0,MyOwnName);
    OptionSet:=OptionFlagSet{};
    InfoFile:=GetDiskObject(ADR(MyOwnName));
    IF InfoFile#NIL THEN
      ReadToolTypeInfo(InfoFile^.toolTypes);
      FreeDiskObject(InfoFile);
    END;
    MainArg:=0;
    FOR Num:=1 TO NumArgs() DO
      GetArg(Num,Option);
      IF Option[0]="-" THEN
        FOR OptionPos:=1 TO Length(Option)-1 DO
          Assert(Occurs(OptionChars,Option[OptionPos],Pos),ADR(IllOption));
          INCL(OptionSet,VAL(OptionFlags,Pos));
        END;
      ELSE
        Assert(MainArg=0,ADR(TooManyArgs));
        MainArg:=Num;
      END;
    END;
    Assert(MainArg#0,ADR(Usage));
    GetArg(MainArg,MainModule);
    Assert(MainModule[0]#"?",ADR(Usage));
    Len:=Length(MainModule);
    IF Len>3 THEN
      IF Compare(MainModule,Len-4,4,".mod",FALSE)=0 THEN
        Delete(MainModule,Len-4,4);       (* cut ".mod" *)
      END;
    END;
    PathLen:=LastPos(MainModule,last,"/");
    IF PathLen=noOccur THEN
      PathLen:=LastPos(MainModule,last,":");
    END;
    IF PathLen#noOccur THEN
      IF PathLen=0 THEN
        Strings.Copy(MainPath,MainModule,0,1);
      ELSE
        Strings.Copy(MainPath,MainModule,0,PathLen);
      END;
      MainDir:=Lock(MainPath,sharedLock);
      Delete(MainModule,0,PathLen+1);
    ELSE
      MainDir:=GetLock(MainArg);
      IF MainDir=NIL THEN
        MainDir:=GetCurrentDir();
      END;
      MainDir:=DupLock(MainDir);
    END;
    Assert(MainDir#NIL,ADR(ModuleNotFound));
    SystemDir:=Lock(SystemPath,sharedLock);
    Assert(SystemDir#NIL,ADR(SysPathNotFound));
  END ExamineEnvironment;

PROCEDURE GetArguments(VAR MainName:ARRAY OF CHAR;VAR Options:OptionFlagSet);
  BEGIN
    Copy(MainName,MainModule);
    Options:=OptionSet;
  END GetArguments;

PROCEDURE InitInOut;
  BEGIN
    IF wbStarted THEN
      OutputFile:=Open(ADR(OutputFileName),newFile);
      Assert(OutputFile#NIL,ADR(InvalidOutput));
    ELSE
      OutputFile:=Output();
    END;
  END InitInOut;

PROCEDURE WriteString(String:ARRAY OF CHAR);
  VAR
    Dummy:LONGINT;
  BEGIN
    Dummy:=Write(OutputFile,ADR(String),Length(String));
  END WriteString;

PROCEDURE WriteLn;
  VAR
    LineFeed:CHAR;
    Dummy:LONGINT;
  BEGIN
    LineFeed:=ASCII.lf;
    Dummy:=Write(OutputFile,ADR(LineFeed),1);
  END WriteLn;

PROCEDURE CloseInOut;
  BEGIN
    IF wbStarted THEN
      Close(OutputFile);
      OutputFile:=NIL;
    END;
  END CloseInOut;

(**)
(* Termination procedure                                                *)
(**)
PROCEDURE CleanUp;
  BEGIN
    IF OutputFile#NIL THEN
      CloseInOut;
    END;
  END CleanUp;

BEGIN
  OutputFile:=NIL;
  SystemDir:=NIL;
  MainDir:=NIL;
  SystemPath:=DefaultPath;
  OutputFileName:=DefaultOutput;
  ErrHeader:=MakeError;
  AssertMessage:=MakeAborted;
  TermProcedure(CleanUp);
  ExamineEnvironment;
END Environment.

