IMPLEMENTATION MODULE lparser;

FROM SYSTEM 		  IMPORT 	ADR;
FROM Strings 		  IMPORT 	Occurs,first,last,Length;
FROM NewMathLib	   	  IMPORT 	SQRT,EXP,LN,LOG,SIN,COS,TAN,ARCTAN,
					SINh,COSh,TANh,ARCSINh,ARCCOSh,ARCTANh,
					FACT,SIGN,DegToRad,RadToDeg,ROUND,
					LTRUNC,ARCCOS,ARCSIN,ABSOLUTE,POW,
					COTAN,ARCCOTAN;
FROM MathLibLong          IMPORT	e,pi;
FROM LongRealConversions  IMPORT 	StrToReal;
FROM Arts 		  IMPORT 	Error,BreakPoint;


PROCEDURE lfastnull(x:LONGREAL):BOOLEAN;
BEGIN
  RETURN ( ABS(x) < 0.00000000001 );
END lfastnull;


PROCEDURE lparse(argument: ARRAY OF CHAR;X:LONGREAL):LONGREAL;

  CONST
    CLS     = CHR(12);
    NULL    = "";
    TOKSIZE = 20;

  VAR
    now         	: CARDINAL;
    arglen,lauf   	: INTEGER;
    exprs       	: ARRAY [0..90]  OF CHAR;
    ans,Xres        	: LONGREAL;  


PROCEDURE binary(a,b:LONGREAL; tok: ARRAY OF CHAR):LONGREAL;
  
  VAR
    c       : LONGREAL;
    index   : INTEGER;

BEGIN

  CASE tok[0] OF
    | "+": RETURN (a+b)
    | "-": RETURN (a-b)
    | "*": RETURN (a*b)
    | "/": IF NOT lfastnull(b) THEN
             RETURN (a/b);
           ELSIF ((a>0.0) AND (b>0.0)) OR ((a<0.0) AND (b<0.0)) THEN
             RETURN MAXLONGREAL;
           ELSE
             RETURN MINLONGREAL;
           END;
    | "^": c:= a;
           FOR index:= 1 TO INTEGER(ABS(b))-1 DO
             a:= a*c;
           END;
           IF b<-0.0000000001 THEN
             IF NOT lfastnull(a) THEN
               a:= 1.0/a;
             ELSE                   (* IF 2     *)
               a:= MAXLONGREAL;
             END;                   (* IF 2     *)
           END;                     (* IF 1     *)
           IF lfastnull(b) THEN
             a:= 1.0;
           END;
           RETURN a;        
    | "&": RETURN (POW(a,b)) |
  ELSE
    BreakPoint(ADR("!   Operator nicht gefunden   !"));
    RETURN 0.0;
  END;                          (* CASE 1   *)
END binary;



PROCEDURE stripleer;

BEGIN

  WHILE (exprs[now]=" ") DO
    INC(now);
  END;

END stripleer;



PROCEDURE getfn(VAR tok: ARRAY OF CHAR);

  VAR
    toklauf : CARDINAL;

BEGIN
  FOR toklauf:= 0 TO TOKSIZE DO
    tok[toklauf]:=" ";
  END;
  toklauf:=0;
  stripleer;
  WHILE ( (exprs[now]>="A") AND (exprs[now]<="Z") ) DO
     tok[toklauf]:=exprs[now];
     INC(toklauf);
     INC(now);
  END;

END getfn;



PROCEDURE getnumb(VAR tok: ARRAY OF CHAR);

  VAR
    toklauf: CARDINAL;
BEGIN
  
  FOR toklauf:= 0 TO TOKSIZE DO
    tok[toklauf]:=" ";
  END;
  stripleer;
  toklauf:=0;
  WHILE 
  (  ( (exprs[now]>="0") AND (exprs[now]<="9") ) OR (exprs[now]=".") ) DO
    tok[toklauf]:= exprs[now];
    INC(toklauf);
    INC(now);
  END;

END getnumb;



PROCEDURE gettoken(VAR tok: ARRAY OF CHAR);

  VAR
    c: CHAR;

BEGIN
  
  stripleer;
  c:=exprs[now];
  IF (
      (c="+") OR (c="-") OR (c="*") OR (c="/") OR (c="^") OR (c=")") OR 
      (c="&")
     ) THEN
    tok[0]:= exprs[now];
    INC(now);
  ELSE
    tok[0]:= NULL;
  END;
  
END gettoken;

  

PROCEDURE parse0():LONGREAL; FORWARD;



PROCEDURE parse5():LONGREAL;

  VAR
    tok,zahlstr     	: ARRAY[0..TOKSIZE] OF CHAR;
    a           	: LONGREAL;
    err         	: BOOLEAN;
    toklen,index    	: CARDINAL;
  
  
  PROCEDURE fneval():LONGREAL;

  BEGIN
    
    IF tok[0]="X" THEN
      RETURN Xres;
    ELSIF (Occurs(tok,first,"PI",FALSE)#last) THEN
      RETURN pi;
    ELSIF (Occurs(tok,first,"SQRT",FALSE)#last) THEN
      RETURN (SQRT(parse5()));
    ELSIF (Occurs(tok,first,"EXP",FALSE)#last) THEN
      RETURN (EXP(parse5()));
    ELSIF (Occurs(tok,first,"LN",FALSE)#last) THEN
      RETURN (LN(parse5()));
    ELSIF (Occurs(tok,first,"ATAN",FALSE)#last) THEN
      RETURN (ARCTAN(parse5()));
    ELSIF (Occurs(tok,first,"ACOT",FALSE)#last) THEN
      RETURN (ARCTAN(1.0/parse5()));
    ELSIF (Occurs(tok,first,"SIN",FALSE)#last) THEN
      RETURN (SIN(parse5()));
    ELSIF (Occurs(tok,first,"COS",FALSE)#last) THEN
      RETURN (COS(parse5()));
    ELSIF (Occurs(tok,first,"LOG",FALSE)#last) THEN
      RETURN (LOG(parse5()));
    ELSIF (Occurs(tok,first,"TAN",FALSE)#last) THEN
      RETURN (TAN(parse5()));
    ELSIF (Occurs(tok,first,"COTAN",FALSE)#last) THEN
      RETURN (COTAN(parse5()));
      
    ELSIF (Occurs(tok,first,"SINH",FALSE)#last) THEN
      RETURN (SINh(parse5()));
    ELSIF (Occurs(tok,first,"COSH",FALSE)#last) THEN
      RETURN (COSh(parse5()));
    ELSIF (Occurs(tok,first,"TANH",FALSE)#last) THEN
      RETURN (TANh(parse5()));
    ELSIF (Occurs(tok,first,"ASINH",FALSE)#last) THEN
      RETURN (ARCSINh(parse5()));
    ELSIF (Occurs(tok,first,"ACOSH",FALSE)#last) THEN
      RETURN (ARCCOSh(parse5()));
    ELSIF (Occurs(tok,first,"ATANH",FALSE)#last) THEN
      RETURN (ARCTANh(parse5()));
    ELSIF (Occurs(tok,first,"RTOD",FALSE)#last) THEN
      RETURN (RadToDeg(parse5()));
    ELSIF (Occurs(tok,first,"DTOR",FALSE)#last) THEN
      RETURN (DegToRad(parse5()));
    ELSIF (Occurs(tok,first,"SIGN",FALSE)#last) THEN
      RETURN (SIGN(parse5()));
    ELSIF (Occurs(tok,first,"ASIN",FALSE)#last) THEN
      RETURN (ARCSIN(parse5()));
    ELSIF (Occurs(tok,first,"ACOS",FALSE)#last) THEN
      RETURN (ARCCOS(parse5()));
      
    ELSIF (Occurs(tok,first,"FACT",FALSE)#last) THEN
      RETURN (FACT(parse5()));
    ELSIF (Occurs(tok,first,"ABS",FALSE)#last) THEN
      RETURN (ABSOLUTE(parse5()));
    ELSIF (Occurs(tok,first,"TRUNC",FALSE)#last) THEN
      RETURN (LTRUNC(parse5()));
    ELSIF (Occurs(tok,first,"ROUND",FALSE)#last) THEN
      RETURN (ROUND(parse5()));
    ELSIF (Occurs(tok,first,"E",FALSE)#last) THEN
      RETURN e;
    ELSE
      BreakPoint(ADR("! Funktion nicht implementiert !"));
      RETURN 0.0;
    END;

  END fneval;



BEGIN                           (* parse5 Main  *)
  getfn(tok);
  IF tok[0]#" " THEN
    RETURN (fneval());
  END;
  getnumb(tok);
  IF tok[0]#" " THEN
    StrToReal(tok,a,err);
    IF err THEN
      BreakPoint(ADR("!  Fehler bei StrToReal  !"));
      RETURN 0.0;
    ELSE                        (* IF 2     *)
      RETURN(a)
    END;                        (* IF 2     *)
  END;                          (* IF 1     *)
  IF (exprs[now]="(") THEN
    INC(now);
    a:= parse0();
    IF (exprs[now]=")") THEN
      INC(now);
    ELSE                        (* IF 4     *)
      Error(ADR("!   Falscher Klammerausdruck   !"),ADR("Nicht behebbar"));
    END;                        (* IF 4     *)
    RETURN a;
  END;                          (* IF 3     *)
  BreakPoint(ADR("!   Ausdruck nicht erkannt   !"));
  RETURN 0.0;

END parse5;



PROCEDURE parse4():LONGREAL;

  VAR
    tok     : ARRAY [0..TOKSIZE] OF CHAR;

BEGIN
  
  gettoken(tok);
  IF (tok[0]="-") THEN
    RETURN (-(parse4()));
  ELSIF (tok[0]="+") THEN
    RETURN(parse4());
  ELSIF (tok[0]=NULL) THEN
    RETURN (parse5());
  END;
  DEC(now);
  RETURN(parse5());

END parse4;



PROCEDURE parse3():LONGREAL;

  VAR
    tok     : ARRAY [0..TOKSIZE] OF CHAR;
    a       : LONGREAL;

BEGIN
  
  a:= parse4();
  WHILE (exprs[now]#NULL) DO
    gettoken(tok);
    IF (  (tok[0]="^") OR (tok[0]="&")  ) THEN
      a:= binary(a,parse4(),tok);
    ELSE
      DEC(now);
    END;
    RETURN a;
  END;
  RETURN a;

END parse3;



PROCEDURE parse2():LONGREAL;

  VAR
    tok     : ARRAY [0..TOKSIZE] OF CHAR;
    a       : LONGREAL;

BEGIN
  
  a:= parse3();
  WHILE (exprs[now]#NULL) DO
    gettoken(tok);
    IF ((tok[0]="*") OR (tok[0]="/")) THEN
      a:= binary(a,parse3(),tok);
    ELSE                        (* IF 1     *)
      DEC(now);
      RETURN a;
    END;                        (* IF 1     *)
  END;                          (* WHILE 1  *)
  RETURN a;

END parse2;



PROCEDURE parse0():LONGREAL;

  VAR
    tok     : ARRAY [0..TOKSIZE] OF CHAR;
    a       : LONGREAL;

BEGIN 
  
  a:= parse2();
  WHILE (exprs[now]#NULL) DO
    gettoken(tok);
    IF ((tok[0]="+") OR (tok[0]="-")) THEN
      a:= binary(a,parse2(),tok);
    ELSE                        (* IF 1     *)
      DEC(now);
      RETURN a;
    END;                        (* IF 1     *)
  END;                          (* WHILE 1  *)
  RETURN a;

END parse0;




BEGIN
  Xres:=X;
  arglen:= Length(argument);
  FOR lauf:= 1 TO arglen DO
    exprs[lauf]:= CAP(argument[lauf-1]);
  END;
  exprs[0]:= NULL;
  exprs[arglen+1]:=NULL;
  now:=1;
  ans:= parse0();
  IF exprs[1]=NULL THEN
    Error(ADR("! String nicht zu parsen !"),ADR("Nicht behebbar"));
  END; 
  RETURN ans;

END lparse;


END lparser.
