(*---------------------------------------------------------------------------
    :Program.    NewMathlib
    :Author.     Philippe Gressly
    :Address.    Nfenhaus, CH-8926 Kappel a/Albis
    :Phone.      
    :Shortcut.   [Philu]
    :Version.    1.0
    :Date.       (7.) 1989
    :Copyright.  PD.
    :Language.   Modula-II
    :Translator. M2Amiga
    :Imports.    MathLibLong
    :UpDate.     JWE 28.09.89 , noch mehr Formeln aus MathIEEEDoubTrans
    :Contents.   More Formulas then MathLibLong
    :Remark.     none
---------------------------------------------------------------------------*)
IMPLEMENTATION MODULE NewMathLib;


FROM MathIEEEDoubTrans IMPORT Exp, Log, Log10, Sqrt, Sin, Cos, Tan, Acos, Asin,
			      Atan, Pow, Cosh, Sinh, Tanh;


CONST PI   = 3.141592653589793;
      E    = 2.718281828459045;
      ln10 = 2.302585093      ; (* bitte ergnzen wenn bekannt *)


PROCEDURE SQRT(x: LONGREAL): LONGREAL;
BEGIN
   IF x < 0.0 THEN ArEr := ArNDef; RETURN 0.0 END;
   ArEr := ArNoEr;
   RETURN Sqrt(x)
END SQRT;



PROCEDURE EXP(x: LONGREAL): LONGREAL;
BEGIN
   IF x > 709.0 THEN ArEr := ArOverFl; RETURN x END;
   ArEr := ArNoEr;
   RETURN Exp(x)
END EXP;


PROCEDURE LN(x: LONGREAL): LONGREAL;
BEGIN
   IF x <=  0.0 THEN ArEr := ArNDef; RETURN 0.0 END;
   ArEr := ArNoEr;
   RETURN Log(x)
END LN;


PROCEDURE LOG(x:LONGREAL): LONGREAL;
BEGIN
   IF x <= 0.0 THEN ArEr := ArNDef; RETURN 0.0 END;
   ArEr := ArNoEr;
   RETURN Log10(x)
END LOG;


PROCEDURE SIN(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN Sin(x)
END SIN;


PROCEDURE COS(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN Cos(x)
END COS;


PROCEDURE TAN(x: LONGREAL): LONGREAL;
BEGIN
   IF Cos(x) = 0.0 THEN ArEr := ArNDef; RETURN 0.0 END;
   ArEr := ArNoEr;
   RETURN Tan(x)
END TAN;



PROCEDURE ARCTAN(x:LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN Atan(x)
END ARCTAN;


PROCEDURE SINh(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN Sinh(x)
   (*  RETURN (EXP(x) - EXP(-x)) / 2.0 *)
END SINh;


PROCEDURE COSh(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN Cosh(x)
   (*  RETURN (EXP(x) + EXP(-x)) / 2.0 *)
END COSh;

PROCEDURE TANh(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN Tanh(x)
   (*  RETURN SINh(x) / COSh(x) *)
END TANh;


PROCEDURE ARCSINh(x:LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN LN(x + SQRT(x * x + 1.0))
END ARCSINh;


PROCEDURE ARCCOSh(x:LONGREAL): LONGREAL;
BEGIN
   IF x < 1.0 THEN ArEr := ArNDef; RETURN 0.0 END;
   ArEr := ArNoEr;
   RETURN LN(x + SQRT(x * x - 1.0))
END ARCCOSh;


PROCEDURE ARCTANh(x:LONGREAL): LONGREAL;
BEGIN
   IF (x <= -1.0) OR (x >= 1.0) THEN ArEr := ArNDef; RETURN 0.0 END;
   ArEr := ArNoEr;
   RETURN LN((1.0 + x)/(1.0-x))/2.0
END ARCTANh;



PROCEDURE FACT(x: LONGREAL): LONGREAL;
VAR r: LONGREAL;
BEGIN
   IF (x > 170.0) THEN ArEr := ArOverFl; RETURN 1.0 END;
   IF (x < 0.0) OR (x # LONGREAL(LONGINT(x))) THEN
      ArEr := ArNDef; RETURN 0.0
   END;
   ArEr := ArNoEr;
   IF x <= 1.0 THEN RETURN 1.0 END;
   r := 1.0;
   WHILE x > 1.0 DO
      r := r * x;
      x := x - 1.0
   END;
   RETURN r
END FACT;


PROCEDURE SIGN(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   IF x < 0.0 THEN RETURN -1.0
   ELSIF x > 0.0 THEN RETURN 1.0
   ELSE RETURN 0.0
   END
END SIGN;


PROCEDURE DegToRad(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN x / 180.0 * PI
END DegToRad;


PROCEDURE RadToDeg(x: LONGREAL): LONGREAL;
BEGIN
   ArEr := ArNoEr;
   RETURN x / PI * 180.0
END RadToDeg;


PROCEDURE ROUND(x:LONGREAL):LONGREAL;
BEGIN
   ArEr:= ArNoEr;
   IF x<0.0 THEN
     RETURN(LONGREAL(TRUNC(x-0.5)));
   ELSE
     RETURN(LONGREAL(TRUNC(x+0.5)));
   END;
END ROUND;


PROCEDURE LTRUNC(x:LONGREAL):LONGREAL;
BEGIN
   ArEr:= ArNoEr;
   RETURN (LONGREAL(TRUNC(x)))
END LTRUNC;

PROCEDURE ARCCOS(x:LONGREAL):LONGREAL;
BEGIN
  IF (x < -1.0) OR (x > 1.0) THEN ArEr:= ArNDef; RETURN 0.0 END;
  ArEr:= ArNoEr;
  RETURN Acos(x)
END ARCCOS;

PROCEDURE ARCSIN(x:LONGREAL):LONGREAL;
BEGIN
  IF (x < -1.0) OR (x > 1.0) THEN ArEr:= ArNDef; RETURN 0.0 END;
  ArEr:= ArNoEr;
  RETURN Asin(x)
END ARCSIN;

PROCEDURE ABSOLUTE(x:LONGREAL):LONGREAL;
BEGIN
  ArEr:= ArNoEr;
  RETURN ABS(x)
END ABSOLUTE;

PROCEDURE COTAN(x:LONGREAL):LONGREAL;
BEGIN
  IF Sin(x) = 0.0 THEN ArEr:= ArNDef; RETURN 0.0 END;
  ArEr:= ArNoEr;
  RETURN 1.0 / Tan(x)
END COTAN;

PROCEDURE ARCCOTAN(x:LONGREAL):LONGREAL;
BEGIN
  IF x = 0.0 THEN ArEr:= ArNDef; RETURN 0.0 END;
  ArEr:= ArNoEr;
  RETURN Atan(1.0/x)
END ARCCOTAN;

PROCEDURE POW(x,y:LONGREAL):LONGREAL;
BEGIN
  IF x<0.0 THEN ArEr:= ArNDef; RETURN 0.0 END;
  ArEr:= ArNoEr;
  RETURN Exp(y*Log(x));
END POW;

  
BEGIN
  ArEr := ArNoEr
END NewMathLib.

