(*--------------------------------------------------------------------------
    :Program.      UsePrefs.mod
    :Author.       Norbert Sdorf
    :Address.      Krnerstrae 6, D 6840 Lampertheim 1
    :Phone.        06206/2212
    :Copyright.    Norbert Sdorf 1989.(program freely distributable)
    :Language.     Modula-II
    :Translator.   M2Amiga 3.2d
    :History.      V1.0, 26-August-1989
    :History.      V1.1, 2-September-1989 (bug fixes)
    :Contents.     alternative 'preferences' per Mausklick oder ber CLI.
    :Remark.       Anwendung: useprefs [<(path/)File>]
    :Remark.       UsePrefs sollte in den Prefs-Ordner gelegt werden.
---------------------------------------------------------------------------*)
MODULE UsePrefs;

(* $R- $V- $S- $F- *)

(* Funktion:
   Das Programm wird von CLI oder Workbench gestartet und erwartet als
   Argument den Namen der Datei, die eine alternative System-Configuration
   enthlt.
   Das Argument wird als Dateiaufruf verwendet (Pfadinformationen werden
   bernommen). Falls kein Argument angegeben wird, versucht 'UsePrefs'
   die ursprnglichen 'preferences' aus 'DEVS:system-configuration'
   zu installieren. Wird als Argument ein Fragezeichen angegeben, gibt
   'UsePrefs' eine kurze Information zur Anwendung aus.

   zum Listing:
   Der Einfachheit wegen wird die Konfigurationsdatei in eine Zeichenkette
   eingelesen (Feld-Variable 'Buffer').
   Aus Sicherheitsgrnden versucht 'Useprefs' mehr Zeichen, als eine
   normale Konfigurationsdatei enthlt, einzulesen. Konnten mehr oder
   weniger als 232 Zeichen (Gre der Preference-Struktur) gelesen werden,
   wird das Programm mit einer Fehlermeldung abgebrochen (Keine Konfigura-
   tionsdatei). Weiterhin wird der Inhalt des ersten Bytes berprft. Dort
   mu ein Character mit der Ordnungszahl 8 oder 9 (Preferences.fontHeights)
   stehen.
   Die eingelesenen Daten werden nicht interpretiert, sondern sofort mit
   'SetPrefs' im System installiert.

   brigens: In dem Buch "AMIGA Modula-2" von Holger Gzella, erschienen
             im Markt&Technik Verlag 1989, ist die Funktion des dritten
             Parameter der Intuition-Anweisung "SetPrefs" falsch inter-
             pretiert (Seite 122). Der selbe Fehler findet sich auch in
             dem Buch "Amiga Programmierpraxis Intuition" von Peter Woll-
             schlaeger, erschienen im selben Verlag 1988 (Seite 243).
             Dort steht, da bei Angabe von TRUE die Daten in den Speicher
             und auf Diskette, bei Angabe von FALSE nur in den Speicher
             geschrieben werden.
             Laut "Intuition Reference Manual" (Appendix C, Seite 2)
             und meinen eigenen Erfahrungen bedeutet ein Wert von TRUE
             nicht, da die neuen Einstellungen auf Diskette gespeichert
             werden, sondern lediglich, da die nderung von Intuition
             als Event global weitervermittelt wird, damit z.B. die
             Workbench davon erfhrt und entsprechend reagiert (nder-
             ung der Schriftgre unter den Icons). Da die Fenster
             nicht von Workbench sondern von Intuition selbst verwaltet
             werden, und Intuition diese nur refresh-t, wenn es das fr
             ntig hlt (Fenster werden vom User verschoben, berlappt
             oder in der Gre verndert, etc.), mu mein Programm sich
             kurzzeitig um den Refresh der Fensterrahmen kmmern.
 *)

IMPORT Intuition;
FROM SYSTEM    IMPORT ADR,ADDRESS;
FROM Arts      IMPORT wbStarted,Requester;
FROM Arguments IMPORT GetArg;
FROM ASCII     IMPORT eol;
FROM Dos       IMPORT Open,Close,Read,Write,Output,FileHandlePtr,oldFile;

TYPE String = ARRAY [0..79] OF CHAR;

VAR
      ConFigFile            : FileHandlePtr;
      IBase                 : Intuition.IntuitionBasePtr;
      WBWindow              : Intuition.WindowPtr;
      Buffer,ConFigName     : ARRAY [0..255] OF CHAR;
      x                     : CARDINAL;
      len                   : INTEGER;
      written,rLength       : LONGINT;
      OutMessage            : ARRAY [0..2] OF String;
      MessLen               : ARRAY [0..2] OF CARDINAL;
      done                  : BOOLEAN;

PROCEDURE MakeMessage(mode : CARDINAL);

BEGIN
IF NOT(wbStarted) THEN
  CASE mode OF
 1: OutMessage[1]:="    USAGE: UsePrefs [<(path/)name of configuration file>]";
    OutMessage[2]:="ANWENDUNG: UsePrefs [<(Pfad/)Name der Konfigurationsdatei>]";
    MessLen[1]:=58; MessLen[2]:=60;|
 2: OutMessage[1]:="  Default 'preferences' installed.";
    OutMessage[2]:="  Ursprngliche 'preferences' installiert.";
    MessLen[1]:=34; MessLen[2]:=42;|
 3: OutMessage[1]:="  Alternate 'preferences' installed.";
    OutMessage[2]:="  Alternative 'preferences' installiert.";
    MessLen[1]:=36; MessLen[2]:=40;|
 4: OutMessage[1]:="  ERROR  : not a configuration file!";
    OutMessage[2]:="  FEHLER : keine Konfigurationsdatei!";
    MessLen[1]:=36; MessLen[2]:=37;|
 5: OutMessage[1]:="  ERROR  : couldn't open file!";
    OutMessage[2]:="  FEHLER : konnte Datei nicht ffnen!";
    MessLen[1]:=30; MessLen[2]:=37;|
   ELSE
   END (*CASE*);
    FOR x:=0 TO 2 DO
        OutMessage[x,MessLen[x]]:=eol; OutMessage[x,MessLen[x]+1]:=eol;
        written:=Write(Output(),ADR(OutMessage[x]),MessLen[x]+2);
    END;
ELSE
   CASE mode OF
   4: done:=Requester(ADR("UsePrefs: invalid file !"),
                      ADR("UsePrefs: ungltige Datei !"),NIL,ADR("QUIT"));|
   5: done:=Requester(ADR("UsePrefs: couldn't open file !"),
                      ADR("UsePrefs: Fehler beim ffnen !"),NIL,ADR("QUIT"));|
   ELSE
   END (*CASE*);
END;

END MakeMessage;

BEGIN

OutMessage[0]:="'UsePrefs' 1.1  by N.Sdorf '89(freely distributable)";
MessLen[0]:=56;

IBase:=ADR(Intuition);

   GetArg(1,ConFigName,len);
IF (len>=0) AND (ConFigName[0]<>"?") THEN
  IF len>0 THEN
       ConFigFile:=Open(ADR(ConFigName),oldFile);
  ELSIF len=0 THEN
       ConFigFile:=Open(ADR("DEVS:system-configuration"),oldFile);
  END;
    IF ConFigFile # NIL THEN
       rLength:=Read(ConFigFile,ADR(Buffer),235);
       Close(ConFigFile);
       IF (rLength=232) AND ((Buffer[0]=CHAR(8)) OR (Buffer[0]=CHAR(9))) THEN

          Intuition.SetPrefs(ADR(Buffer),232,TRUE); (* TRUE ! *)

          WBWindow:=IBase^.activeScreen^.firstWindow;

  (* Refresh der Fensterrahmen aller Fenster auf der Workbench, damit
     diese die eventuell genderte Schriftgre im Titel bernehmen.   *)

          LOOP
              IF WBWindow # NIL THEN
                 Intuition.RefreshWindowFrame(WBWindow);
                 WBWindow:=WBWindow^.nextWindow;
              ELSE
                 EXIT;
              END;
          END (*LOOP*);

          IF len<>0 THEN MakeMessage(3) ELSE MakeMessage(2) END;
       ELSE
          MakeMessage(4);
       END;
    ELSE
       MakeMessage(5);
    END;
ELSIF (len=1) AND (ConFigName[0]="?") THEN
      MakeMessage(1);
END;

END UsePrefs.
