(**********************************************************************
  :Program.    ILBMHandler.mod
  :Contents.   Procedures for loading and saving ILBM-Pictures
  :Author.     Stefan Salewski
  :Copyright.  Public Domain
  :Language.   Modula-2
  :Translator. M2Amiga AMSoft V3.2d
  :History.    V1.1 3.Jul.1989
  :Address.    Stolper Weg 3, D-2160 Stade
  :Imports.    TurboFiles, Assembler2
**********************************************************************)

IMPLEMENTATION MODULE ILBMHandler;
(*
  Created:   1/89
  Changed:   14.2.88/23.2.89/25.2.89/10.3.89/14.3.89/17.3.89/29.3.89/
             8.6.89/6.7.89/14.7.89 by
             Stefan Salewski
             Stolper Weg 3
             2160 Stade   West-Germany
             
*)
  FROM Assembler2 IMPORT MinLongInt,MaxLongInt,MaxInt,MinCard,Fill,Equal,
                         MinInt;
  FROM SYSTEM     IMPORT ADR,ADDRESS,SHIFT,BITSET,LONGSET,CAST,INLINE,REG,BYTE,
                         SETREG;
  FROM Dos        IMPORT Delay;
  FROM Arts       IMPORT TermProcedure,Assert,CurrentLevel;
  FROM Exec       IMPORT AllocMem,FreeMem,MemReqs,MemReqSet,CopyMem,UByte,Byte,
                         Interrupt,AddIntServer,RemIntServer,NodeType;
  FROM Intuition  IMPORT NewScreen,ScreenPtr,ScreenFlags,ScreenFlagSet,
                         customScreen,IDCMPFlags,IDCMPFlagSet,DisplayBeep,
                         MakeScreen,RethinkDisplay;
  IMPORT Graphics;
  FROM Graphics   IMPORT SetRGB4,RastPort,RastPortPtr,BitMap,BitMapPtr,
                         ViewModes,ViewModeSet,InitBitMap,AllocRaster,
                         FreeRaster,ViewPortPtr,RectanglePtr,GfxBasePtr,
                         Rectangle,GetRGB4,BltBitMapRastPort;
  FROM Hardware   IMPORT vertb;
  FROM TurboFiles IMPORT TurboRead,TurboWrite,Lookup,CloseFile,TurboSetPos,
                         TurboGetPos,FilePtr,TurboResult,
                         DeleteFile,SetPosMode,ReadOnly,NewFile;
  
  CONST
    FormName="FORM";
    IlbmName="ILBM";
    BmhdName="BMHD";
    CmapName="CMAP";
    CamgName="CAMG";
    GrabName="GRAB";
    DestName="DEST";
    CrngName="CRNG";
    BodyName="BODY";
    
    LoResStandWidth	=320;
    LoResOverscanWidth	=704 DIV 2;
    NTSCNonIntStandH	=200;
    PalNonIntStandH	=256;
    NTSCNonIntOverH	=262; (* or 464 DIV 2 or 452 DIV 2 ??? *)
    PalNonIntOverH	=290;(*572 DIV 2*)
    
    MaxImageWidth=2048; (* currently 1024 *)
    FullSpeed=4000H;
    MOVEMS=48E7H; (* that's the 68000-Instruction MOVEM to save Registers*)
    MOVEML=4CDFH; (* that's MOVEM to load Registers                      *)
    Buffer0Size=(MaxImageWidth DIV 8)*2;
    Buffer1Size=Buffer0Size;
    BufferSize=Buffer0Size+Buffer1Size;
    
  TYPE
    BytePtr=POINTER TO Byte;
    UBytePtr=POINTER TO UByte;
    BitSetPtr=POINTER TO BITSET;
    ILBMInfoPtr=POINTER TO ILBMInfo;
    FourChar=ARRAY[0..3] OF CHAR;
        
  VAR
    myLevel:INTEGER;
    inFile,outFile:FilePtr;
    cyc1,cyc2,cyc3:UByte;
    buffer:ADDRESS;              (* Buffer for Read,Write,Shift              *)
    buffer1:ADDRESS;             (* ADR(buffer)+BufferSize/2                 *)
    datenAdr:ADDRESS;
    len:LONGINT;                 (* Receives Length from Read/Write()        *)
    infoPtr:ILBMInfoPtr;         (* Interrupt's ILBMInfo                     *)
    defaultRect:Rectangle;
    null:UByte;
    integer:INTEGER;
    cardinal:CARDINAL;
    longint:LONGINT;
    
(**********************************************************************)
  
  PROCEDURE ScreenSize(VAR width,height:INTEGER;overscan:BOOLEAN);
  (* Returns the size of a lores non-interlace standard- or
     overscan-screen. Uses gfxBase.displayFlags to decide if the
     program runs on a PAL- or NTSC-Amiga. I hope this works correct.
  *)
    VAR
      gfxBasePtr:GfxBasePtr;
  BEGIN
    gfxBasePtr:=ADR(Graphics);
    IF overscan THEN
      width:=LoResOverscanWidth;
      IF Graphics.ntsc IN gfxBasePtr^.displayFlags THEN
        height:=NTSCNonIntOverH
      ELSE
        height:=PalNonIntOverH
      END
    ELSE
      width:=LoResStandWidth;
      IF Graphics.ntsc IN gfxBasePtr^.displayFlags THEN
        height:=NTSCNonIntStandH
      ELSE
        height:=PalNonIntStandH
      END
    END
  END ScreenSize;
  
  PROCEDURE CenterOverScanScreen(sPtr:ScreenPtr);
    CONST
      NormalX=129; (* Top left corner of a normal screen *)
      NormalY=44;
      MaxLoresDx=-16; (* Should be -16, but this uses not the full screen *)
      MaxNoLaceDy=-16;
    VAR
      gfxPtr:GfxBasePtr;
      standW,standH:INTEGER;
      viewDx,viewDy:INTEGER;
      dx,dy:INTEGER;
  BEGIN
    (* If the screen is greater than a normal screen, then we move it
       to the upper left corner to use the whole display.
       The maximal displacement should be 32 (hires or interlace) or 16
       (ROM-Manual). This uses not really the whole display of the monitor.
       For example, if I use a x-displacement of 32 instead of 16 for the lores
       picture HUBY (should be on a AMOK-Disk) I can see a greater part of it.
       But when I use a x-displacement of 20 or 24, I have a black vertical
       bar on the right side of my monitor. I don't know why, so I now use
       the save values from the ROM-Manual.
       (If I would make this values much greater, the display would crash.)
     *) 
    gfxPtr:=ADR(Graphics);
    viewDx:=gfxPtr^.actiView^.dxOffset;
    viewDy:=gfxPtr^.actiView^.dyOffset;
    ScreenSize(standW,standH,FALSE);
    IF hires IN sPtr^.viewPort.modes THEN
      dx:=MaxInt((standW*2-sPtr^.width) DIV 2,2*MaxLoresDx);
    ELSE
      dx:=MaxInt((standW-sPtr^.width) DIV 2,MaxLoresDx);
    END;
    sPtr^.viewPort.dxOffset:=MinInt(NormalX+dx-viewDx,0);
    IF lace IN sPtr^.viewPort.modes THEN
      dy:=MaxInt((standH*2-sPtr^.height) DIV 2,2*MaxNoLaceDy);
    ELSE
      dy:=MaxInt((standH-sPtr^.height) DIV 2,MaxNoLaceDy)
    END;
    sPtr^.viewPort.dyOffset:=MinInt(NormalY+dy-viewDy,0);
    MakeScreen(sPtr);
    RethinkDisplay();
  END CenterOverScanScreen;
   
(**********************************************************************)   
  
  PROCEDURE UpperWord(i:INTEGER):INTEGER;
  BEGIN
    RETURN (i+15)-((i+15) MOD 16)
  END UpperWord;

(**********************************************************************)
  
  PROCEDURE LowerWord(i:INTEGER):INTEGER;
  BEGIN
    RETURN i-(i MOD 16)
  END LowerWord;
  
(**********************************************************************)
  
  PROCEDURE InitScreen(VAR nS:NewScreen;VAR info:ILBMInfo;overscan:BOOLEAN);
    VAR w,h:INTEGER;
  BEGIN
    ScreenSize(w,h,overscan);
    nS.width:=MaxInt(info.bmhd.width,info.bmhd.scrnWidth);
    nS.height:=MaxInt(info.bmhd.height,info.bmhd.scrnHeight);
    nS.leftEdge:=LowerWord(info.bmhd.left);(* currently leftEdge=0 *)
    nS.topEdge:=info.bmhd.top;
    nS.depth:=info.bmhd.depth;
    nS.viewModes:=ViewModeSet{};
    IF (nS.width>w) AND (nS.depth<5) THEN INCL(nS.viewModes,hires) END;
    IF nS.height>h THEN INCL(nS.viewModes,lace) END;
    IF camg IN info.ILBMTitle THEN
      WITH info.camg DO
        IF (Lace  IN viewType) THEN INCL(nS.viewModes,lace  ) END;
        IF (HoMod IN viewType) THEN INCL(nS.viewModes,ham   ) END;
        IF (Hires IN viewType) THEN INCL(nS.viewModes,hires ) END;
        IF (DblPF IN viewType) THEN INCL(nS.viewModes,dualpf) END;
        IF (Extra IN viewType) THEN INCL(nS.viewModes,extraHalfbrite) END;
      END;
    END;
    nS.detailPen:=0;
    nS.blockPen:=1;
    nS.type:=customScreen+ScreenFlagSet{screenQuiet};
    nS.font:=NIL;
    nS.defaultTitle:=NIL;
    nS.gadgets:=NIL;
    nS.customBitMap:=NIL;
  END InitScreen;
  
  (**********************************************************************)

  PROCEDURE DimOn(screenPtr:ScreenPtr;VAR info:ILBMInfo;delay:UByte);
    VAR
      m,n:CARDINAL;
  BEGIN
    IF (delay=0) OR (ham IN screenPtr^.viewPort.modes) THEN
      m:=15
    ELSE
      m:=0;
      DEC(delay);
    END;
    WITH info.cmap DO
      REPEAT
        INC(m);
        FOR n:=0 TO colorCnt-1 DO
          cardinal:=3*n;
          SetRGB4(ADR(screenPtr^.viewPort),n,
                  (colors[cardinal+Red]*m) DIV 16,
                  (colors[cardinal+Green]*m) DIV 16,
                  (colors[cardinal+Blue]*m) DIV 16);
        END;
        IF delay>0 THEN
          Delay(delay)
        END;
      UNTIL m>=16;
    END;
  END DimOn;
  
(**********************************************************************)

  PROCEDURE DimOff(screenPtr:ScreenPtr;VAR info:ILBMInfo;delay:UByte);
    VAR m,n:CARDINAL;
  BEGIN
    IF NOT(ham IN screenPtr^.viewPort.modes) THEN
      WITH info.cmap DO
        IF delay=0 THEN
          m:=1
        ELSE
          m:=16;
          DEC(delay);
        END;
        REPEAT
          DEC(m);
          FOR n:=0 TO colorCnt-1 DO
            cardinal:=3*n;
            SetRGB4(ADR(screenPtr^.viewPort),n,
                    (colors[cardinal+Red]*m) DIV 16,
                    (colors[cardinal+Green]*m) DIV 16,
                    (colors[cardinal+Blue]*m) DIV 16);
          END;
          IF delay>0 THEN
            Delay(delay)
          END;
        UNTIL m<=0
      END;
    ELSE
      DimOn(screenPtr,info,0)
    END;
  END DimOff;
  
(**********************************************************************)
  
  PROCEDURE MakeBitMap(width,height:Pixel;depth:Depth):BitMapPtr;
  VAR
    i:Depth;
    error:BOOLEAN;
    bitMapPtr:BitMapPtr;
  BEGIN
    bitMapPtr:=AllocMem(SIZE(BitMap),MemReqSet{public});
    InitBitMap(bitMapPtr^,depth,width,height);
    i:=0;
    REPEAT
      bitMapPtr^.planes[i]:=AllocRaster(width,height);
      error:=bitMapPtr^.planes[i]=NIL;
      INC(i);
    UNTIL (i=depth) OR error;
    IF error THEN
      WHILE i>=1 DO
        DEC(i);
        FreeRaster(bitMapPtr^.planes[i],width,height);
      END;
      FreeMem(bitMapPtr,SIZE(BitMap));
      RETURN NIL;
    ELSE
      RETURN bitMapPtr
    END
  END MakeBitMap;
  
(**********************************************************************)
  
  PROCEDURE FreeBitMap(VAR bmPtr:BitMapPtr);
  VAR
    i:CARDINAL;
  BEGIN
    IF bmPtr#NIL THEN
      FOR i:=0 TO bmPtr^.depth-1 DO
        FreeRaster(bmPtr^.planes[i],bmPtr^.bytesPerRow*8,bmPtr^.rows);
      END;
      FreeMem(bmPtr,SIZE(BitMap));
    END;
    bmPtr:=NIL;
  END FreeBitMap;
  
(**********************************************************************)
  
  PROCEDURE ReadInfo(VAR name:ARRAY OF CHAR;VAR info:ILBMInfo;
                     VAR bodyPos:LONGINT):BOOLEAN;
    VAR
      bitSetPtr:POINTER TO BITSET ;
      bitSet:BITSET;
      chName:FourChar;
      bodyFound,ok:BOOLEAN;
      uByte:UByte;
    
  BEGIN
    bodyFound:=FALSE;
    bodyPos:=-1;
    info.ILBMTitle:=ILBMTitleSet{};
    info.crng.count:=0;
    IF Lookup(inFile,name,256+128,ReadOnly)#done THEN
      ilbmError:=openFileFailedILBM;
      RETURN FALSE
    END;
    TurboRead(inFile,ADR(chName),4,len);
    ok:=Equal(ADR(chName),ADR(FormName),4);
    IF TurboSetPos(inFile,4,current) THEN END;
    TurboRead(inFile,ADR(chName),4,len);
    IF ok AND Equal(ADR(chName),ADR(IlbmName),4) THEN
      ilbmError:=noErrorILBM
    ELSE
      ilbmError:=noILBM
    END;
    WHILE (ilbmError=noErrorILBM) AND NOT(bodyFound) DO
      TurboRead(inFile,ADR(chName),4,len);
      IF Equal(ADR(chName),ADR(BmhdName),4) THEN
        INCL(info.ILBMTitle,bmhd);
        TurboRead(inFile,ADR(info.bmhd),SIZE(BMHD),len);
      ELSIF Equal(ADR(chName),ADR(CmapName),4) THEN
        INCL(info.ILBMTitle,cmap);
        TurboRead(inFile,ADR(info.cmap.size),4,len);
        TurboRead(inFile,ADR(info.cmap)+4,info.cmap.size,len);
        WITH info.cmap DO
          colorCnt:=size DIV 3;
          FOR integer:=0 TO size-1 DO
            colors[integer]:=colors[integer] DIV 16;
          END
        END;
      ELSIF Equal(ADR(chName),ADR(CamgName),4) THEN
        INCL(info.ILBMTitle,camg);
        TurboRead(inFile,ADR(info.camg),SIZE(CAMG),len);
      ELSIF Equal(ADR(chName),ADR(GrabName),4) THEN
        INCL(info.ILBMTitle,grab);
        TurboRead(inFile,ADR(info.grab),SIZE(GRAB),len);
      ELSIF Equal(ADR(chName),ADR(DestName),4) THEN
        INCL(info.ILBMTitle,dest);
        TurboRead(inFile,ADR(info.dest),SIZE(DEST),len);
      ELSIF Equal(ADR(chName),ADR(CrngName),4) THEN
        TurboRead(inFile,ADR(info.crng.data[info.crng.count]),SIZE(CRNG),len);
        WITH info.crng.data[info.crng.count] DO
          bitSetPtr:=ADR(on);
          bitSet:=bitSetPtr^;
          on:=(0 IN bitSet);
          forward:=NOT(1 IN bitSet);
          on:=on AND (low<info.cmap.colorCnt)  AND (rate<=FullSpeed)
                 AND (high<info.cmap.colorCnt) AND (rate>0) AND (low#high);
          IF on THEN
            IF low>high THEN
              uByte:=low;
              low:=high;
              high:=uByte;
              forward:=FALSE;
            END;
            INC(info.crng.count);
            INCL(info.ILBMTitle,crng);
          END;
        END;
      ELSIF Equal(ADR(chName),ADR(BodyName),4) THEN
        bodyFound:=TRUE;
        bodyPos:=TurboGetPos(inFile)-4;
      ELSE (* Ignore unknown data *)
        TurboRead(inFile,ADR(longint),4,len);
        IF TurboSetPos(inFile,longint,current) THEN END;
      END;
      IF inFile^.res#done THEN
        ilbmError:=readErrorILBM;
      END;
    END;
    CloseFile(inFile);
    RETURN bodyFound AND (info.ILBMTitle>=ILBMTitleSet{bmhd,cmap});
  END ReadInfo;
  
(**********************************************************************)

  PROCEDURE MaskPlane(VAR info:ILBMInfo):Depth;
  BEGIN
    IF info.bmhd.masking=1 THEN
      RETURN 1
    ELSE
      RETURN 0
    END
  END MaskPlane;
  
(**********************************************************************)
  
  PROCEDURE ReadBody(VAR name:ARRAY OF CHAR;bitmap:BitMap;bodyPos:LONGINT;
                     imageWidth,imageX,imageY,destX,destY,width,height:Pixel;
                     planesToIgnore:Depth;imageIsCompressed:BOOLEAN):BOOLEAN;
    VAR
      i,j,k:CARDINAL;
      depth:UByte;
      chunkName:FourChar;
      myPlanes:ARRAY[1..MaxPlanes] OF ADDRESS;
      byte:UByte;
      bytes:ARRAY[0..1] OF UByte;
      (* Um nicht immer wieder Einzelne Bytes von FileSystem anfordern zu
         muessen, lese ich immmer mit den Bilddaten den naesten Zaehler mit
         ein; bytes[1] ist dieser Zaehler.
      *)
      (* bytes[0] is the byte to copy in the bitmap, bytes[1] is a counter that
         determines how much times I must copy the next byte in the bitmap
      *)
      d:POINTER TO UByte;
      count:CARDINAL;
      actBuffer:ADDRESS;
      bodyLength:LONGINT;

  BEGIN
    imageWidth:=UpperWord(imageWidth) DIV 8;
    width :=((imageX MOD 8) +width+7) DIV 8;
    imageX:=imageX DIV 8;
    destX :=destX  DIV 8;
    IF (imageX+width>imageWidth) OR
       (*(imageY+height>imageHeight) OR ==> ReadError*)
       (destX+width>bitmap.bytesPerRow) OR
       (destY+height>bitmap.rows) THEN
       ilbmError:=wrongSizeILBM;
       RETURN FALSE
    END;
    ilbmError:=noErrorILBM;
    depth:=bitmap.depth;
    IF Lookup(inFile,name,1024,ReadOnly)#done THEN
      ilbmError:=openFileFailedILBM;
      RETURN FALSE;
    END;
    IF TurboSetPos(inFile,bodyPos,beginning) THEN END;
    TurboRead(inFile,ADR(chunkName),4,len);
    IF NOT(Equal(ADR(chunkName),ADR(BodyName),4)) THEN
      ilbmError:=noILBM
    ELSE
      actBuffer:=buffer+LONGINT(imageX);
      TurboRead(inFile,ADR(bodyLength),4,len);
      longint:=LONGINT(destY*bitmap.bytesPerRow+destX);
      FOR i:=1 TO depth DO
        myPlanes[i]:=bitmap.planes[i-1]+longint;
      END;
      IF imageIsCompressed THEN
        TurboRead(inFile,ADR(bytes[1]),1,len);
        FOR i:=1 TO imageY DO (* Ignore the first imageY lines *)
          FOR j:=1 TO depth+planesToIgnore DO
            d:=buffer;
            count:=0;
            WHILE (count<imageWidth) (* AND (len#0)*) DO
              IF bytes[1]<128 THEN
                INC(bytes[1]);
                TurboRead(inFile,d,bytes[1]+1,len);
                INC(d,bytes[1]);
                INC(count,bytes[1]);
                bytes[1]:=d^;
              ELSE
                byte:=257-bytes[1];
                TurboRead(inFile,ADR(bytes[0]),2,len);
                INC(count,byte);
              END
            END;
          END;
        END;
        FOR i:=1 TO height DO (* Read the desired part of the picture *)
          FOR j:=1 TO depth+planesToIgnore DO
            d:=buffer;
            count:=0;
            WHILE (count<imageWidth)(* AND (len#0)*) DO
              IF bytes[1]<128 THEN
                INC(bytes[1]);
                TurboRead(inFile,d,bytes[1]+1,len);
                INC(d,bytes[1]);
                INC(count,bytes[1]);
                bytes[1]:=d^;
              ELSE
                byte:=257-bytes[1];
                TurboRead(inFile,ADR(bytes[0]),2,len);
                Fill(d,bytes[0],byte);
                INC(count,byte);               
                INC(d,byte);                   
              END
            END;
            IF j<=bitmap.depth THEN
              CopyMem(actBuffer,myPlanes[j],width);
              INC(myPlanes[j],bitmap.bytesPerRow);
            END;
          END;
        END;
      ELSE
      (*FOR i:=1 TO imageY DO
          FOR j:=1 TO depth+planesToIgnore DO
            TurboRead(inFile,buffer,imageWidth,len);
          END;
        END;*)
        IF TurboSetPos(inFile,imageWidth*imageY*(depth+planesToIgnore),current)
           THEN
        END;
        FOR i:=1 TO height DO
          FOR j:=1 TO depth DO
            TurboRead(inFile,buffer,imageWidth,len);
            CopyMem(actBuffer,myPlanes[j],width);
            INC(myPlanes[j],bitmap.bytesPerRow);
          END;
        (*FOR j:=1 TO planesToIgnore DO
            TurboRead(inFile,buffer,imageWidth,len);
          END;*)
          IF TurboSetPos(inFile,imageWidth*planesToIgnore,current) THEN END;
        END;
      END;
    END;
    IF len=0 THEN
      ilbmError:=readErrorILBM;
    END;
    CloseFile(inFile);
    RETURN (ilbmError=noErrorILBM);
  END ReadBody;
  
(**********************************************************************)
  
  PROCEDURE LoadInRastPort(VAR name:ARRAY OF CHAR;rPtr:RastPortPtr;
                           bodyPos:LONGINT;imageWidth,imageX,imageY,destX,
                           destY,width,height:INTEGER;planesToIgnore:Depth;
                           imageIsCompressed:BOOLEAN):BOOLEAN;
    VAR
      tmpBMPtr:BitMapPtr;
  BEGIN
    tmpBMPtr:=MakeBitMap(width,height,rPtr^.bitMap^.depth);
    IF tmpBMPtr=NIL THEN
      ilbmError:=outOfMemILBM;
      RETURN FALSE
    END;
    IF NOT ReadBody(name,tmpBMPtr^,bodyPos,imageWidth,imageX,imageY,
                    0,0,width,height,planesToIgnore,imageIsCompressed) THEN
      RETURN FALSE
    ELSE
      BltBitMapRastPort(tmpBMPtr,imageX MOD 8,0,
                      rPtr,destX,destY,width,height,0C0H);
      FreeBitMap(tmpBMPtr);
      RETURN TRUE
    END;
  END LoadInRastPort;
  
(**********************************************************************)
  
  (* $S-
    We have to turn off stack-checking because this interruptserver-
    procedure runs in supervisor-modus and so uses the supervisor-stack.
  *)    
  PROCEDURE CycleInterrupt; (* InterruptServerProcedure *)
  BEGIN
    INLINE(MOVEMS,3F3EH); (* Movem.l D2-D7/A2-A6,+(SP) *)
    infoPtr:=ADDRESS(REG(9));
    FOR cyc1:=0 TO infoPtr^.crng.count-1 DO
      IF infoPtr^.crng.data[cyc1].on THEN
        INC(infoPtr^.cycleInfo.speedCnt[cyc1],infoPtr^.crng.data[cyc1].rate);
        IF infoPtr^.cycleInfo.speedCnt[cyc1]>=FullSpeed THEN
          DEC(infoPtr^.cycleInfo.speedCnt[cyc1],FullSpeed);
          IF infoPtr^.crng.data[cyc1].forward THEN
            IF infoPtr^.cycleInfo.base[cyc1]<=
              infoPtr^.crng.data[cyc1].low THEN
              infoPtr^.cycleInfo.base[cyc1]:=infoPtr^.crng.data[cyc1].high;
            ELSE
              DEC(infoPtr^.cycleInfo.base[cyc1]);
            END;
          ELSE
            IF infoPtr^.cycleInfo.base[cyc1]>=
            infoPtr^.crng.data[cyc1].high THEN
              infoPtr^.cycleInfo.base[cyc1]:=
              infoPtr^.crng.data[cyc1].low;
            ELSE
              INC(infoPtr^.cycleInfo.base[cyc1]);
            END;
          END;
          cyc3:=infoPtr^.cycleInfo.base[cyc1];
          FOR cyc2:=infoPtr^.crng.data[cyc1].low TO 
                    infoPtr^.crng.data[cyc1].high DO
                    cardinal:=3*cyc3;
            SetRGB4(infoPtr^.cycleInfo.vpPtr,
                    cyc2,infoPtr^.cmap.colors[cardinal+Red],
                    infoPtr^.cmap.colors[cardinal+Green],
                    infoPtr^.cmap.colors[cardinal+Blue]);
            INC(cyc3);
            IF cyc3>infoPtr^.crng.data[cyc1].high THEN
              cyc3:=infoPtr^.crng.data[cyc1].low
            END;
          END;
        END;
      END;
    END;
    INLINE(MOVEML,7CFCH); (* Movem.l (SP)-,D2-D7/A2-A6 *)
  END CycleInterrupt;
(* $S= *)

(***************************************************************************)

  PROCEDURE StartCycling(VAR info:ILBMInfo;screenPtr:ScreenPtr):BOOLEAN;
    VAR j:INTEGER;
  BEGIN
    IF (crng IN info.ILBMTitle) AND (screenPtr#NIL) THEN
      info.cycleInfo.vpPtr:=ADR(screenPtr^.viewPort);
      FOR cardinal:=0 TO info.crng.count-1 DO
        info.cycleInfo.base[cardinal]:=info.crng.data[cardinal].low;
        info.cycleInfo.speedCnt[cardinal]:=0;
      END;
      WITH info.cycleInfo.int DO
        node.type:=interrupt;
        node.pri:=-60;
        node.name:=NIL;
        data:=ADR(info);
        code:=CycleInterrupt;
      END;
      SETREG(0,0); (* Only for V3.2 , Compiler-ERROR !!! *)
      AddIntServer(vertb,ADR(info.cycleInfo.int));
      RETURN TRUE;
    ELSE
      RETURN FALSE
    END
  END StartCycling;

(***************************************************************************)

  PROCEDURE EndCycling(VAR info:ILBMInfo);
  BEGIN
    Assert(info.cycleInfo.vpPtr#NIL,ADR('EndCycling without StartCycling!'));
    SETREG(0,0); (* Only for V3.2 , Compiler-ERROR !!! *)
    RemIntServer(vertb,ADR(info.cycleInfo.int));
  END EndCycling;
  
(***************************************************************************)

  PROCEDURE InitILBMInfo(VAR info:ILBMInfo;
                        rpPtr:RastPortPtr;
                        vpPtr:ViewPortPtr);
    VAR
      colorConv:LONGCARD;
      i:LONGINT;
  BEGIN
    WITH info.bmhd DO
      (* 
      size:=SIZE(BMHD)-4;\
      width               | will be initialiezed by SaveBitMap
      height             /
      *)
      left:=0;
      top:=0;
      depth:=rpPtr^.bitMap^.depth;
      masking:=0;
      (* compression will be initialiezed by SaveBitMap *)
      pad:=0;
      transCol:=0;
      scrnWidth:=rpPtr^.bitMap^.bytesPerRow*8;
      scrnHeight:=rpPtr^.bitMap^.rows;
      IF hires IN vpPtr^.modes THEN
        xAspect:=5;
      ELSE
        xAspect:=10;
      END;
      IF lace IN vpPtr^.modes THEN
        xAspect:=xAspect*2;
      END;
      yAspect:=11;
    END;
    WITH info.cmap DO
      colorCnt:=vpPtr^.colorMap^.count;
      (* size:=colorCnt*3; will be initialiezed by SaveBitMap*)
      FOR i:=0 TO (colorCnt-1)*3 BY 3 DO
        colorConv:=GetRGB4(vpPtr^.colorMap,i DIV 3);
        colors[i+Red]  :=(colorConv DIV 256) MOD 16;
        colors[i+Green]:=(colorConv DIV 16 ) MOD 16;
        colors[i+Blue] := colorConv MOD 16;
      END;
    END;
    WITH info.camg DO
      (* size:=SIZE(CAMG)-4; will be initialiezed by SaveBitMap*)
      viewType:=ViewTypeSet{};
      IF lace           IN vpPtr^.modes THEN INCL(viewType,Lace)  END;
      IF hires          IN vpPtr^.modes THEN INCL(viewType,Hires) END;
      IF dualpf         IN vpPtr^.modes THEN INCL(viewType,DblPF) END;
      IF ham            IN vpPtr^.modes THEN INCL(viewType,HoMod) END;
      IF extraHalfbrite IN vpPtr^.modes THEN INCL(viewType,Extra) END;
    END;
    info.ILBMTitle:=ILBMTitleSet{bmhd,cmap,camg};
  END InitILBMInfo;
  
(***********************************************************************)

  PROCEDURE SaveBitMap(VAR name:ARRAY OF CHAR;
                       VAR info:ILBMInfo;
                       bmPtr:BitMapPtr;
                       rectPtr:RectanglePtr;
                       compressIt:BOOLEAN):BOOLEAN;

    VAR
      myPlanes:ARRAY[0..MaxPlanes-1] OF ADDRESS;
      bytesPerImageLine,wordsPerImageLine:INTEGER;
      fileSize,bodySizePos,bodySize:LONGINT;
      needToShift:BOOLEAN;
      bitsToShift:[0..15];
      lowerLeft,upperWidth:INTEGER;(* Word-aligned Offset & Width *)
      bitSetPtr:BitSetPtr;
      bitSet:BITSET;
      rMask,lMask:LONGSET;
      newLength:INTEGER;
      i,j:INTEGER;
    
    PROCEDURE Compress(source,dest:ADDRESS;
                       sourceLength:INTEGER;VAR destLength:INTEGER);
(*  Die sourceLength Bytes, die an der Position source stehen,
    werden komprimiert und an der Speicherstelle dest abgespeichert.
    destLength ist die Laenge dieser kompriemierten Daten in Bytes.
    Die komprimierten Daten sind folgendermassen zu interpretieren:
    Ist ein Byte
    0..127 :  bedeutet das: Die naechsten n+1 (1..128) Bytes
              unveraendert uebernehmen.
    129..255: Das folgende Byte wird 257-n mal dubliziert.
    128:      keine Bedeutung.
   
    Der Speicher dest muss mindestens 1.5*sourceLength gross sein,
    denn die Komprimierung kann theoretisch bei sehr unguenstigen Daten
    auch negativ ausfallen!
   
*)    CONST
        PackLimit=2; (* Wenn mehr als PackLimit Bytes gleich sind,
                        so wird gepackt. PackLimit=2 ist Standart
                        und wohl auch optimal, die Prozedur funktioniert
                        aber fuer Werte von 1 bis 128 *)
      VAR
        pos,byte1,byte2:UBytePtr;
        count:BytePtr;
        gleiche,i:UByte;
        ungleicheGleichNull:BOOLEAN;
        len:INTEGER;
    BEGIN
      len:=0;
      ungleicheGleichNull:=TRUE;
      pos:=dest;
      byte1:=source;
      DEC(byte1);
      byte2:=source;
      LOOP
        gleiche:=0;
        REPEAT
          INC(gleiche);
          INC(byte1);INC(byte2);INC(len);
        UNTIL (byte2^#byte1^) OR (len>=sourceLength) OR (gleiche>=128);
        IF gleiche>PackLimit THEN (* wir koennen packen *)
          pos^:=257-gleiche;
          INC(pos);
          pos^:=byte1^;
          INC(pos);
          ungleicheGleichNull:=TRUE;
        ELSIF len<=sourceLength THEN (* zuwenig gleiche Bytes gefunden *)
          IF ungleicheGleichNull OR (count^>=128-PackLimit) THEN
            ungleicheGleichNull:=FALSE;
            count:=ADDRESS(pos);
            count^:=-1;
            INC(pos);
          END;
          FOR i:=1 TO gleiche DO
            INC(count^);
            pos^:=byte1^;
            INC(pos);
          END;
        ELSE
          EXIT
        END;
      END;
      destLength:=LONGINT(pos)-dest;
    END Compress;

    PROCEDURE Shift(from,to:ADDRESS;words,count:INTEGER);
    (* Kopiert 2*words bytes von der Speicherstelle from nach to,
       wobei diese Bytes gleichzeitig um count Bits nach links verschoben
       werden. Es muss gelten: 0<= count <=15 .
    *)
      VAR
        lwPtr,toPtr:POINTER TO LONGSET;
        h:LONGSET;
        i:INTEGER;
    BEGIN
      toPtr:=to;
      IF count=0 THEN
        CopyMem(from,to,words*2);
        INC(toPtr,(words-2)*2)
      ELSE
        lwPtr:=from;
        toPtr^:=CAST(LONGSET,SHIFT(CAST(LONGCARD,lwPtr^),count));
        FOR i:=0 TO words-2 DO
          INC(lwPtr,2);
          INC(toPtr,2);
          h:=CAST(LONGSET,SHIFT(CAST(LONGCARD,lwPtr^),count));
          h:=h-lMask;
          toPtr^:=toPtr^*lMask;
          toPtr^:=h+toPtr^;
        END;
        DEC(toPtr,2);
      END;
      toPtr^:=toPtr^-rMask;
    END Shift;
    
  BEGIN (* SaveBitMap *)
    IF NOT(info.ILBMTitle>=ILBMTitleSet{bmhd,cmap}) THEN
      ilbmError:=noILBM;
      RETURN FALSE
    END;
    IF rectPtr=NIL THEN
      rectPtr:=ADR(defaultRect);
      WITH rectPtr^ DO
        minX:=0;
        minY:=0;
        maxX:=bmPtr^.bytesPerRow*8-1;
        maxY:=bmPtr^.rows-1;
      END;
    END;
    info.bmhd.width:=rectPtr^.maxX-rectPtr^.minX+1;
    info.bmhd.height:=rectPtr^.maxY-rectPtr^.minY+1;
    lowerLeft:=LowerWord(rectPtr^.minX);
    upperWidth:=UpperWord(info.bmhd.width);
    longint:=rectPtr^.minY*INTEGER(bmPtr^.bytesPerRow)+(lowerLeft DIV 8);
    FOR i:=0 TO bmPtr^.depth-1 DO
      myPlanes[i]:=bmPtr^.planes[i]+longint;
    END;
    IF compressIt THEN
      info.bmhd.compression:=1
    ELSE
      info.bmhd.compression:=0
    END;
    IF Lookup(outFile,name,1024,NewFile)#done THEN
      ilbmError:=openFileFailedILBM;
      RETURN FALSE;
    END;
    TurboWrite(outFile,ADR(FormName),8);
    (* formName ist nur 4 Bytes gross,der Rest ist beliebig *)
    TurboWrite(outFile,ADR(IlbmName),4);
    info.bmhd.size:=SIZE(BMHD)-4;
    TurboWrite(outFile,ADR(BmhdName),4);
    TurboWrite(outFile,ADR(info.bmhd),SIZE(BMHD));
    info.cmap.size:=info.cmap.colorCnt*3;
    FOR i:=0 TO MinLongInt(info.cmap.size-1,3*MaxColors-1) DO
      info.cmap.colors[i]:=info.cmap.colors[i]*16
    END;
    TurboWrite(outFile,ADR(CmapName),4);
    TurboWrite(outFile,ADR(info.cmap),info.cmap.size+4);
    IF grab IN info.ILBMTitle THEN
      info.grab.size:=SIZE(GRAB)-4;
      TurboWrite(outFile,ADR(GrabName),4);
      TurboWrite(outFile,ADR(info.grab),SIZE(GRAB));
    END;
    IF dest IN info.ILBMTitle THEN
      info.dest.size:=SIZE(DEST)-4;
      TurboWrite(outFile,ADR(DestName),4);
      TurboWrite(outFile,ADR(info.dest),SIZE(DEST));
    END;
    IF camg IN info.ILBMTitle THEN
      info.camg.size:=SIZE(CAMG)-4;
      TurboWrite(outFile,ADR(CamgName),4);
      TurboWrite(outFile,ADR(info.camg),SIZE(CAMG));
    END;
    IF crng IN info.ILBMTitle THEN
      WITH info.crng DO
        FOR i:=0 TO MinCard(count-1,15) DO
          data[i].size:=SIZE(CRNG)-4;
          data[i].pad:=0;
          bitSetPtr:=ADR(data[i].on);
          IF data[i].on AND data[i].forward THEN
            bitSetPtr^:=BITSET{0,1}
          ELSIF data[i].on THEN
            bitSetPtr^:=BITSET{0}
          ELSE
            bitSetPtr^:=BITSET{}
          END;
          TurboWrite(outFile,ADR(CrngName),4);
          TurboWrite(outFile,ADR(data[i]),SIZE(CRNG));
        END;
      END;
    END;
    (* WriteBody, body-size ist noch unbekannt *)
    TurboWrite(outFile,ADR(BodyName),8);
    bodySizePos:=TurboGetPos(outFile)-4;
    bytesPerImageLine:=upperWidth DIV 8; (* Laenge einer Zeile in Bytes *)
    wordsPerImageLine:=bytesPerImageLine DIV 2;
    needToShift:=(lowerLeft#rectPtr^.minX) OR (upperWidth#info.bmhd.width);
  (* Wenn wir einen Ausschnitt aus der Grafik (BitMap) speichern wollen,
     der n*16+x Pixel von linken Rand der BitMap entfernt ist, so
     so lesen wir die Daten ab der Position n*16 aus der BitMap und
     verschieben dann alle Bits einer Zeile um x nach links.
     Dabei sind n und x positive ganze Zahlen. ( n*16+x=rectPtr.minX )
     
     Wenn nur (upperWidth#info.bmhd.width), dann wird zwar nicht geschoben,
     aber wir fuellen jede Zeile bis auf Wordgroesse auf. Die Prozedur
     Shift macht dann eine UND-Verknuepfung mit rMask und dem letzten
     Langwort der Zeile. Dadurch werden die Auffuellbits auf Null gesetzt
  *)
    IF needToShift THEN
      bitsToShift:=rectPtr^.minX-lowerLeft; (* 0<=bitsToShift<16 *)
      lMask:=LONGSET{}; (* Maske die die Prozedur Shift braucht *)
      FOR i:=31 TO 16+bitsToShift BY -1 DO
        INCL(lMask,i)
      END;
      rMask:=LONGSET{}; (* Maske die die Prozedur Shift braucht *)
      FOR i:=0 TO upperWidth-info.bmhd.width-1 DO
        INCL(rMask,i)
      END;
    END;
    FOR i:=0 TO info.bmhd.height-1 DO
      FOR j:=0 TO info.bmhd.depth-1 DO
        (* datenAdr gibt immer den Ort an, wo die zu speichernden
           Daten augenblicklich stehen *)
        IF needToShift THEN
          Shift(myPlanes[j],buffer1,wordsPerImageLine,bitsToShift);
          datenAdr:=buffer1;
        ELSE
          datenAdr:=myPlanes[j];
        END;
        IF compressIt THEN
          Compress(datenAdr,buffer,bytesPerImageLine,newLength);
          datenAdr:=buffer;
        ELSE
          newLength:=bytesPerImageLine
        END;
        TurboWrite(outFile,datenAdr,newLength);
        INC(myPlanes[j],bmPtr^.bytesPerRow);
      END;
    END;
    fileSize:=TurboGetPos(outFile);
    bodySize:=fileSize-bodySizePos-4;
    IF ODD(fileSize) THEN
      TurboWrite(outFile,ADR(null),1);
      INC(fileSize);
    END;
    IF TurboSetPos(outFile,bodySizePos,beginning) THEN END;
    TurboWrite(outFile,ADR(bodySize),4);
    IF TurboSetPos(outFile,4,beginning) THEN END;
    DEC(fileSize,8);
    TurboWrite(outFile,ADR(fileSize),4);
    IF outFile^.res#done THEN
      CloseFile(outFile);
      IF DeleteFile(name) THEN END;
      ilbmError:=writeErrorILBM;
      RETURN FALSE;
    ELSE
      CloseFile(outFile);
      RETURN TRUE;
    END;
  END SaveBitMap;
  
(**********************************************************************)

  PROCEDURE SaveRastPort(VAR name:ARRAY OF CHAR;
                         rpPtr:RastPortPtr;
                         vpPtr:ViewPortPtr;
                         rectPtr:RectanglePtr;
                         compressIt:BOOLEAN):BOOLEAN;

  VAR
    ilbmInfo:ILBMInfo;
  BEGIN
    InitILBMInfo(ilbmInfo,rpPtr,vpPtr);
    RETURN SaveBitMap(name,ilbmInfo,rpPtr^.bitMap,
                      rectPtr,compressIt);
  END SaveRastPort;
  
(**********************************************************************)

  PROCEDURE SaveScreen(VAR name:ARRAY OF CHAR;
                       screenPtr:ScreenPtr;
                       rectPtr:RectanglePtr;
                       compressIt:BOOLEAN):BOOLEAN;
                            
  BEGIN
    RETURN SaveRastPort(name,ADR(screenPtr^.rastPort),
                        ADR(screenPtr^.viewPort),rectPtr,compressIt);
  END SaveScreen;
  
(**********************************************************************)

  PROCEDURE CleanUp;
  BEGIN
    IF myLevel>=CurrentLevel() THEN
      FreeMem(buffer,BufferSize)
    END
  END CleanUp;
  
BEGIN
  myLevel:=CurrentLevel();
  buffer:=AllocMem(BufferSize,MemReqSet{public,memClear});
  Assert(buffer#NIL,ADR("Sorry, but I need more RAM"));
  buffer1:=buffer+Buffer0Size;
  null:=0;
  TermProcedure(CleanUp);
END ILBMHandler.
