(**********************************************************************

    :Program.    FastLister
    :Contents.   Pop-Up Textanzeiger
    :Usage.      FastLister <Directory>
    :Author.     Thomas Stolze
    :Copyright.  ShareWare
    :Language.   Modula-2
    :Translator. M2Amiga AMSoft V3.2
    :Imports.    StringSupport
    :History.    V1.0 Thomas Stolze

**********************************************************************)

(* $R- $V- $S- SN- SF- *)
MODULE FastLister;

FROM Arts          IMPORT Assert,Requester,Terminate,TermProcedure;
FROM Arguments     IMPORT GetArg,NumArgs;
FROM SYSTEM        IMPORT ADDRESS,ADR,CAST,LONGSET;
FROM FileSystem    IMPORT Close,File,GetPos,Length,Lookup,ReadBytes,ReadChar,
                          Response,SetPos;
FROM StringSupport IMPORT LeftStr,StringStr,ValStr;
FROM Str           IMPORT Concat,Copy,FirstPos;
FROM Graphics      IMPORT ViewModes,ViewModeSet;
FROM Intuition     IMPORT CloseScreen,CloseWindow,customScreen,IDCMPFlags,
                          IDCMPFlagSet,IntuiMessagePtr,ModifyIDCMP,NewScreen,
                          NewWindow,OpenScreen,OpenWindow,ScreenPtr,selectDown,
                          Window,WindowFlags,WindowFlagSet,WindowPtr,
                          DisplayBeep;
FROM Exec          IMPORT CloseDevice,DoIO,FindPort,GetMsg,Interrupt,IOStdReq,
                          IOStdReqPtr,MsgPortPtr,OpenDevice,ReplyMsg,WaitPort,
                          Forbid,Permit,Enable,Disable;
FROM ExecSupport   IMPORT CreateExtIO,CreatePort,DeleteExtIO,DeletePort;
FROM DirSorter     IMPORT CleanDir,DirEntries,DirFeld,StoreVerzeichnis;
FROM InputEvent    IMPORT InputEvent,InputEventPtr;
FROM Input         IMPORT addHandler,inputName,remHandler;
FROM Dos           IMPORT Delay;
FROM InputDevice   IMPORT ESC,Handler,HELP,inputMessagePtr,readIOPtr,sleepPort;

CONST programmName = "FastLister V1.01";

VAR Zeile        : ARRAY [0..99] OF CHAR;
    argument     : ARRAY [0..56] OF CHAR;
    Speicher     : ADDRESS;
    Output,
    Input        : File;
    DateiLaenge  : LONGINT;
    arglen       : INTEGER;
    window       : WindowPtr;
    screenptr    : ScreenPtr;
    wind         : NewWindow;
    scr          : NewScreen;
    writeIOPtr   : IOStdReqPtr;
    writePortPtr,
    readPortPtr  : MsgPortPtr;
    msg          : IntuiMessagePtr;
    class        : IDCMPFlagSet;
    inputmsg     : inputMessagePtr;
    interrupted  : Interrupt;
    inputeventptr: InputEventPtr;
    consoleclosed,
    req          : BOOLEAN;
    portptr      : MsgPortPtr;

PROCEDURE ConsoleWrite(command : CARDINAL; len : LONGCARD; adr : ADDRESS);
BEGIN
  writeIOPtr^.command:=command;
  writeIOPtr^.length:=len;
  writeIOPtr^.data:=adr;
  DoIO(writeIOPtr);
  Assert(writeIOPtr^.error = 0,ADR("I/O Fehler"));
  writeIOPtr^.command:=5; DoIO(writeIOPtr);
  Assert(writeIOPtr^.error = 0,ADR("I/O Fehler"));
END ConsoleWrite;

PROCEDURE InputRead(command : CARDINAL; len : LONGCARD; adr : ADDRESS);
BEGIN
  readIOPtr^.command:=command;
  readIOPtr^.length:=len;
  readIOPtr^.data:=adr;
  DoIO(readIOPtr);
  Assert(readIOPtr^.error = 0,ADR("I/O Fehler"));
END InputRead;

PROCEDURE MakeCommand(ohneCSI : ARRAY OF CHAR);
VAR string : ARRAY [0..10] OF CHAR;
BEGIN
  string[0]:=233C; string[1]:=00C; Concat(string,ohneCSI);
  ConsoleWrite(3,HIGH(ohneCSI)+2,ADR(string));
END MakeCommand;

PROCEDURE FensterDefinition;
BEGIN
  MakeCommand ("c"); MakeCommand("1;1;H"); MakeCommand("J");
  MakeCommand("2y");  MakeCommand("80u");
  MakeCommand("29t");
END FensterDefinition;

PROCEDURE Fenster(sc : ScreenPtr);
BEGIN
  WITH wind DO
    leftEdge:=0; topEdge:=0; width:=640; height:=256;
    detailPen:=0; blockPen:=1;
    idcmpFlags:=IDCMPFlagSet{closeWindow,rawKey,mouseButtons};
    flags:=WindowFlagSet{windowDepth,windowClose,gimmeZeroZero,
                         noCareRefresh,borderless,activate,windowActive};
    title:=ADR(programmName);
    screen:=sc; minWidth:=80; minHeight:=20; maxWidth:=640; maxHeight:=256;
    type:=customScreen;
  END;
  window:=OpenWindow(wind);
    Assert(window # NIL,ADR("Fenster nicht geffnet"));
END Fenster;

PROCEDURE OeffneFenster;
BEGIN
  WITH scr DO
    leftEdge:=0; topEdge:=0; width:=640; height:=256;
    detailPen:=0; blockPen:=0; depth:=2;
    viewModes:=ViewModeSet{hires}; type:=customScreen;
    font:=NIL; defaultTitle:=ADR(programmName); gadgets:=NIL;
    customBitMap:=NIL;
  END;

  sleepPort:=CreatePort(ADR("FastLister_Port"),0);
      Assert(sleepPort # NIL,ADR("MsgPort nicht allokiert"));

END OeffneFenster;

PROCEDURE OeffneDatei(Name : ARRAY OF CHAR): BOOLEAN;
BEGIN
  Lookup(Input,Name,10240,FALSE);
  IF (Input.res = done) THEN
     Length(Input,DateiLaenge);
     RETURN TRUE;
  END;
  RETURN FALSE;
END OeffneDatei;

PROCEDURE ChooseFile(): INTEGER;
VAR get      : ARRAY [0..35] OF CHAR;
    input    : ARRAY [0..3] OF CHAR;
    i,j,code : CARDINAL;
    len,count: INTEGER;

    PROCEDURE Schreibe(name : ARRAY OF CHAR);
    BEGIN
      ConsoleWrite(3,HIGH(name)-2,ADR(name));
      MakeCommand("1E");
    END Schreibe;

BEGIN
  ModifyIDCMP(window,IDCMPFlagSet{closeWindow,vanillaKey}); MakeCommand(14C);
  MakeCommand("10y"); MakeCommand("1x"); MakeCommand("26u");
  MakeCommand("26t");

  j:=1;  ConsoleWrite(5,0,NIL); len:=0;
  REPEAT
    i:=1; get:=("");
    REPEAT
      StringStr(get,j,2); Concat(get,". "); Concat(get,DirFeld[j]^.fileName);
      LeftStr(get,get,26);
      Schreibe(get);
      INC(i); INC(j);
    UNTIL ((i > 25) OR (j > DirEntries));
    INC(len);

    CASE len OF
      1 : MakeCommand("208x"); MakeCommand("26t");
    | 2 : MakeCommand("424x"); MakeCommand("26t");
    ELSE
    END;

    MakeCommand("1;1;H");
  UNTIL (j > DirEntries);

  MakeCommand("1x"); MakeCommand("1;1;H"); MakeCommand("29t");
  MakeCommand("80u"); MakeCommand("27E"); get:=("--------------------");
  FOR i:=0 TO 3 DO
    ConsoleWrite(3,20,ADR(get)); MakeCommand("D");
  END;
  MakeCommand("1E"); input:=(""); count:=0; get:=(""); len:=1;

    LOOP
      WaitPort(window^.userPort);
        msg:=GetMsg(window^.userPort);
        class:=msg^.class;
        code:=msg^.code;
     ReplyMsg(msg);
     IF (closeWindow IN class) THEN
         RETURN 0;
     ELSIF (vanillaKey IN class) THEN
         get[0]:=CHR(code); get[1]:=00C; input[count]:=CHR(code); INC(count);
         ConsoleWrite(3,1,ADR(get));
         IF code  = 27 THEN RETURN 0 END;
         IF code = 13 THEN DEC(count) END;

         IF (count = 2) OR (code = 13) THEN
            input[count]:=00C;
            len:=ValStr(input);
            IF ((len <= CAST(INTEGER,DirEntries)) AND (len > 0))
                                                      OR (code = 13) THEN
               EXIT;
            ELSE
              input:=(""); count:=0; get:=(""); MakeCommand(15C);
            END;
         END;
      END;
    END;
  MakeCommand(14C);

  ModifyIDCMP(window,IDCMPFlagSet{closeWindow,rawKey,mouseButtons});
  ConsoleWrite(5,0,NIL);
  RETURN len;
END ChooseFile;

PROCEDURE NeueAuswahl(): BOOLEAN;
VAR index : CARDINAL;
BEGIN
  Close(Input);  Copy(Zeile,argument); Concat(Zeile,"/");
  index:=ChooseFile();
  IF (index # 0) THEN
     Concat(Zeile,DirFeld[index]^.fileName);
     RETURN TRUE;
  END;
  RETURN FALSE;
END NeueAuswahl;

PROCEDURE Lister(): BOOLEAN;
VAR Eingabe     : ARRAY [0..2] OF CHAR;
    gelesen,
    geschrieben,
    pos,getpos,
    page        : LONGINT;
    zeilen,code : CARDINAL;
    Zeichen,chr : CHAR;
    listen      : BOOLEAN;

    PROCEDURE Lesen;
    BEGIN
      zeilen:=0; pos:=0; page:=0;
      REPEAT
        Zeile:=("");
        ReadBytes(Input,ADR(Zeile),98,gelesen);
        pos:=FirstPos(Zeile,0,Zeichen);
        GetPos(Input,getpos);
           ConsoleWrite(3,pos+1,ADR(Zeile)); (* 3 = Write *)
        SetPos(Input,getpos-(gelesen-pos)+1);
        INC(zeilen); page:= page + pos+1;
      UNTIL (Input.eof OR (zeilen = 28));
    END Lesen;
BEGIN
  gelesen:=0; geschrieben:=0; Zeichen:=12C;
   FensterDefinition; Lesen;
  LOOP
    WaitPort(window^.userPort);
       msg:=GetMsg(window^.userPort);
       class:=msg^.class;
       code:=msg^.code;
    ReplyMsg(msg);
    IF (closeWindow IN class) THEN
       EXIT;
    ELSIF (rawKey IN class) THEN
       CASE code OF
         44H,43H :  EXIT;
       | 40H,4DH :  IF NOT Input.eof THEN
                       FensterDefinition; Lesen;
                    END;
       | 4CH     :  GetPos(Input,getpos);
                    IF getpos - 2*page < 0 THEN
                       SetPos(Input,0);
                    ELSE
                       SetPos(Input,getpos - 2*page);
                       REPEAT
                         ReadChar(Input,chr);
                       UNTIL (chr = Zeichen);
                    END;
                    IF Input.res = done THEN
                       FensterDefinition; Lesen;
                    END;
       | 4FH     :  SetPos(Input,0);
                    IF Input.res = done THEN
                       FensterDefinition; Lesen;
                    ELSE
                       EXIT;
                    END;
       | 4EH     :  IF (DateiLaenge-600 >= 0) THEN
                       SetPos(Input,DateiLaenge-600);
                       REPEAT
                         ReadChar(Input,chr);
                       UNTIL (chr = Zeichen);
                       IF Input.res = done THEN
                          FensterDefinition; Lesen;
                       END;
                    END;
       | 45H,5FH :  RETURN FALSE;
       ELSE
       END;
    ELSIF (mouseButtons IN class) THEN
       CASE code OF
         selectDown :
                    IF NOT Input.eof THEN
                       FensterDefinition; Lesen;
                    END;

       ELSE
       END;
    END;
  END;
  RETURN TRUE;
END Lister;

PROCEDURE Erwache;
VAR ende : BOOLEAN;
BEGIN
  REPEAT
    IF NeueAuswahl() THEN
       IF OeffneDatei(Zeile) THEN
          ende:=Lister();
       END;
    ELSE
       ende:=TRUE;
    END;
  UNTIL ende;
END Erwache;

PROCEDURE InstallHandler(adrdata : ADDRESS);
BEGIN
  interrupted.data:=adrdata;
  interrupted.code:=ADR(Handler);
  interrupted.node.pri:=51;

  readIOPtr^.data:=ADR(interrupted);
  readIOPtr^.command:=addHandler;

  DoIO(readIOPtr);
  Assert(readIOPtr^.error = 0,ADR("I/O Fehler"));

END InstallHandler;

PROCEDURE Wache;
VAR event : InputEvent;
    code  : CARDINAL;
BEGIN
  readPortPtr:=CreatePort(NIL,0);
    Assert(readPortPtr # NIL,ADR("MsgPort nicht allokiert"));
  readIOPtr:=CreateExtIO(readPortPtr,SIZE(IOStdReq));
    Assert(readIOPtr # NIL,ADR("IOStdReq nicht allokiert"));
  readIOPtr^.length:=SIZE(InputEvent);

  OpenDevice(ADR(inputName),0,readIOPtr,LONGSET{});
  OeffneFenster;

  writePortPtr:=CreatePort(NIL,0);
    Assert(writePortPtr # NIL,ADR("MsgPort nicht allokiert"));
  writeIOPtr:=CreateExtIO(writePortPtr,SIZE(IOStdReq));
    Assert(writeIOPtr # NIL,ADR("IOStdReq nicht allokiert"));

  consoleclosed:=TRUE;
  LOOP
    InstallHandler(NIL);

    WaitPort(sleepPort);
      inputmsg:=GetMsg(sleepPort);
      code:=inputmsg^.event.code;
    ReplyMsg(inputmsg);

    InputRead(remHandler,0,ADR(interrupted));
    interrupted.code:=NIL;
    Delay(5);

    CASE code OF
       HELP  :
                screenptr:=OpenScreen(scr);
                  Assert(screenptr # NIL,ADR("Screen nicht geffnet"));
                Fenster(screenptr);

                writeIOPtr^.data:=window;
                writeIOPtr^.length:=SIZE(Window);
                OpenDevice(ADR("console.device"),0,writeIOPtr,LONGSET{});
                Delay(5);
                consoleclosed:=FALSE;
                  Erwache;
                CloseDevice(writeIOPtr);  consoleclosed:=TRUE;

                CloseWindow(window); window:=NIL;
                CloseScreen(screenptr); screenptr:=NIL;

    | ESC    :  EXIT;
    ELSE
    END;

  END;
END Wache;

PROCEDURE Cleanup;
BEGIN
 Forbid(); Disable();
  Close(Input);
  IF interrupted.code # NIL THEN
     InputRead(remHandler,0,ADR(interrupted));
  END;
  IF (sleepPort # NIL) THEN
     DeletePort(sleepPort);
  END;
  IF (window # NIL) THEN
     CloseWindow(window)
  END;
  IF (screenptr # NIL) THEN
     CloseScreen(screenptr);
  END;
  IF NOT consoleclosed THEN
     CloseDevice(writeIOPtr);
  END;
  CloseDevice(readIOPtr);
  DeleteExtIO(writeIOPtr);  DeleteExtIO(readIOPtr);
  DeletePort(writePortPtr); DeletePort(readPortPtr);
  CleanDir;
 Permit(); Enable();
END Cleanup;

BEGIN
  portptr:=FindPort(ADR("FastLister_Port"));
  IF portptr = NIL THEN
     IF NumArgs() # 0 THEN
        TermProcedure(Cleanup);
        GetArg(1,argument,arglen);
        IF (argument[arglen-1] = "/") THEN
           argument[arglen-1]:=00C;
        END;
        StoreVerzeichnis(ADR(argument));

        Wache;

        DisplayBeep(NIL);
        Terminate(0);
     END;
  END;
  DisplayBeep(NIL);
END FastLister.
