(********************************************************************
  :Program.       mcdTree.mod
  :Author.        Ludwig Geromiller
  :Address.       Filderstr. 63, 7000 Stuttgart 1
  :Phone.         0711/6409664
  :History.       V1.0, Sep-89, Ludwig Geromiller
  :Copyright.     PD
  :Language.      Modula-II
  :Translator.    M2Amiga 3.2d
  :Imports.       Nichts
  :Contents.      Analyse der Directorystruktur des aktuellen Laufwerks
  :Contents.      Ausgabe im ASCII-Format von Directory-Level und
  :Contents.      Directory-Namen
  :Usage.         Aufruf aus CLI mcdTree >:Tree.mcd
  :Usage.         Ausgabeumleitung auf Datei Tree.mcd im Root-Directory
  :Usage.         :Tree.mcd enthlt die Directorystruktur und
  :Usage.         wird von mcd ausgewertet (s.a. mcd.mod)
********************************************************************)

MODULE mcdTree;

FROM Arts       IMPORT Assert, TermProcedure, Terminate;
FROM Str        IMPORT Concat;
FROM SYSTEM     IMPORT ADDRESS, ADR, CAST, BPTR;
FROM Terminal   IMPORT Write, WriteString, WriteLn;
FROM InOut      IMPORT WriteInt;
FROM Dos        IMPORT BSTR, FileInfoBlockPtr, UnLock, FileLockPtr,
                       sharedLock, Lock, Examine, ExNext, FileInfoBlock;
FROM Exec       IMPORT AllocMem, FreeMem, MemReqs, MemReqSet;

TYPE String     = ARRAY [0..107] OF CHAR;
     FLockPtr   = POINTER TO FLock;
     FLock      = RECORD (* Listenelement *)
                    pathname    : String;
                    infoptr     : FileInfoBlockPtr;
                    filelockptr : FileLockPtr;
                    previous    : FLockPtr;
                    next        : FLockPtr
                  END;

VAR newcdptr: FileLockPtr;
    lockptr : FLockPtr;
    erfolg,
    root    : BOOLEAN;
    Count   : INTEGER;


PROCEDURE LookUpDir(dirname:ARRAY OF CHAR);
  BEGIN
    lockptr^.next := AllocMem(SIZE(FLock),MemReqSet{memClear});
    Assert(lockptr^.next#NIL,ADR("No Memory for FLock"));
    WITH lockptr^.next^ DO
      infoptr := AllocMem(SIZE(FileInfoBlock),MemReqSet{memClear});
      Assert (infoptr#NIL,ADR ("No Memory for FileInfoBlock!"))
    END;
    lockptr^.next^.previous := lockptr;
    lockptr := lockptr^.next;
    IF root THEN
      Concat(lockptr^.pathname,dirname);
      Concat(lockptr^.pathname,":");
      root := FALSE
    ELSE
      lockptr^.pathname:=lockptr^.previous^.pathname;
      Concat(lockptr^.pathname,dirname);
      Concat(lockptr^.pathname,"/")
    END;
    lockptr^.filelockptr := Lock(ADR(lockptr^.pathname),sharedLock);
    Assert(lockptr^.filelockptr#NIL,ADR("No lock for directory!"));
    erfolg := Examine(lockptr^.filelockptr,lockptr^.infoptr);
    Assert(erfolg,ADR("Examine failed!"));
    Assert(lockptr^.infoptr^.dirEntryType>0,
             ADR("Argument isn`t a Directory!"));
    WHILE ExNext(lockptr^.filelockptr,lockptr^.infoptr) DO
      IF lockptr^.infoptr^.dirEntryType > 0 THEN (* Neues Dir *)
          INC(Count);
          WriteInt(Count,2);WriteString("  ");
          WriteString(lockptr^.infoptr^.fileName);WriteLn;
          LookUpDir(lockptr^.infoptr^.fileName); (* rekursiver Aufruf *)
      END;
    END;
    WITH lockptr^ DO
      FreeMem(infoptr,SIZE(FileInfoBlock));
      infoptr:=NIL;
      UnLock (filelockptr);
      filelockptr := NIL;
    END;
    lockptr := lockptr^.previous;
    FreeMem(lockptr^.next,SIZE(FLock));
    lockptr^.next := NIL;
    DEC(Count);
  END LookUpDir;

  PROCEDURE BSTRtoString(bstr:BSTR; VAR string: ARRAY OF CHAR);
    VAR aptr   : POINTER TO String;
        counter: CARDINAL;
    BEGIN
      aptr := ADDRESS(CAST(BPTR,bstr));
      IF CARDINAL(aptr^[0]) = 0 THEN string:="";RETURN END;
      FOR counter:=0 TO CARDINAL(aptr^[0])-1 DO
        string[counter]:=aptr^[counter+1]
      END;
      string[ORD(aptr^[0])] := 0C
    END BSTRtoString;

PROCEDURE DiskSearch; (* Untersuche aktuelles Laufwerk *)
  VAR volumeName        : String;

  BEGIN
    newcdptr := Lock (ADR(""),sharedLock);
  (* get current volumeName *)
    BSTRtoString(newcdptr^.volume^.name,volumeName);
    root := TRUE;
    Count:=0;
    WriteInt(Count,2);WriteString("  ");WriteString(volumeName);WriteLn;
    UnLock(newcdptr);
    newcdptr:=NIL;
    LookUpDir(volumeName);
  END DiskSearch;

  PROCEDURE Ausstieg;   (* Freigeben der Resourcen *)
    BEGIN
      IF lockptr # NIL THEN
        WHILE lockptr^.next # NIL DO
          lockptr := lockptr^.next
        END;
        WHILE lockptr^.previous # NIL DO
          IF lockptr^.filelockptr # NIL THEN
            UnLock(lockptr^.filelockptr)
          END;
          IF lockptr^.infoptr # NIL THEN
            FreeMem(lockptr^.infoptr,SIZE(FileInfoBlock))
          END;
          lockptr := lockptr^.previous;
          FreeMem(lockptr^.next,SIZE(FLock))
        END;
        FreeMem(lockptr,SIZE(FLock))
      END;
    END Ausstieg;

BEGIN (* mcd *)
  TermProcedure(Ausstieg);

  lockptr := AllocMem(SIZE(FLock),(* Dummy-Eintrag*)
                        MemReqSet{memClear});
  Assert(lockptr#NIL,ADR("Keinen Speicher fr FLock!"));
  DiskSearch;
END mcdTree.
