(*---------------------------------------------------------------------------
    :Program.    Rechner
    :Author.     Philippe Gressly
    :Address.    Nfenhaus, CH-8926 Kappel a/Albis
    :History.    V1.0, October 89, Philippe Gressly
    :Copyright.  PD.
    :Language.   Modula-II
    :Translator. M2Amiga v3.2
    :Imports.    UPN
    :Contents.   Ein Rechner
---------------------------------------------------------------------------*)

MODULE Rechner;

IMPORT UPN; (* Damit knnen Formeln ausgewertet werden *)
FROM Arts IMPORT Terminate, CurrentLevel, Assert, TermProcedure, wbStarted;
FROM SYSTEM IMPORT ADR;
FROM InOut IMPORT ReadString, WriteString, WriteLn, Write;
FROM LongRealInOut IMPORT WriteReal;
FROM Str IMPORT Length, Copy;

CONST MAX = 256;

VAR VarName, Formula: ARRAY[0..MAX] OF CHAR;
    f: UPN.UPN;
    Wert: LONGREAL;
    i,j: CARDINAL;
    done: BOOLEAN;


PROCEDURE Separate();
(* In Formula steht die Eingabezeile. Nun wird diese in Variable und
 * Formel zerlegt und in "VarName" und in "Formula" wieder abgespeichert.
 * Beispiel: KugelVolumen=4/3*PI*Radius^3
 *           VarName      Formula
 *)
BEGIN
   VarName[0] := 0C; (* Falls eine Formel ohne Zuweisung gegeben wurde *)
   i := 0;
   WHILE (i < Length(Formula)) AND (Formula[i] # "=") DO INC(i) END;
   IF i >= Length(Formula) THEN RETURN END;
   Copy(VarName, Formula);
   VarName[i] := 0C;
   IF i >= 1 THEN
      FOR j := i + 1 TO Length(Formula) DO
         Formula[j - i - 1] := Formula[j]
      END
   END
END Separate;




PROCEDURE Close;
BEGIN
   UPN.KillUPN(f); (* Gibt den Speicher wieder frei. *)
   IF wbStarted THEN
      WriteString("Mit CloseGadget beenden.")
   END
END Close;



BEGIN (* Rechner *)
   Assert(UPN.AllocUPN(f), ADR("Speicher fehlt"));
   TermProcedure(Close);
   WriteString("Beenden mit einem Punkt am Anfang der Eingabezeile.");
   WriteLn;
   WriteString("(Es drfen keine Leerzeichen in der Formel stehen, da");
   WriteLn;
   WriteString("ReadString nur bis zum Leerschlag liest!)");
   WriteLn;
   WriteString("Mit (?) knnen alle Variablen angezeigt werden.");
   WriteLn;
   LOOP
      WriteLn;
      WriteString("");
      ReadString(Formula);
      IF Formula[0] = "." THEN Terminate(CurrentLevel()) END;
      IF Formula[0] = "?" THEN
         i := UPN.GetNumVars(f);
         IF i > 0 THEN
            FOR j := 1 TO i DO
               UPN.GetVarName(f,j,VarName);
               WriteString(VarName); WriteString("=");
               done := UPN.Compile(f, VarName);
               WriteReal(UPN.Execute(f), 10, 10);
               WriteLn
            END
         END
      ELSE
         Separate; (* Separates Formula in Formula und Varname *)

         IF (Formula[0] = 0C) OR (NOT UPN.Compile(f, Formula)) THEN
            WriteString("  "); WriteString(Formula); WriteLn;
            FOR i := 1 TO UPN.UPNError.Wo DO
               WriteString(" ");
            END;
            WriteString(" /^\"); UPN.WriteError;
         ELSE
            Wert := UPN.Execute(f);
            IF UPN.UPNError.ET = UPN.None THEN (* No Error *)
               IF Length(VarName) > 0 THEN
                  done := TRUE;
                     IF UPN.GetVarNr(f, VarName) = 0 THEN
                     done := UPN.AddVarsToUPN(f, VarName);
                     IF NOT done THEN
                        WriteString("Kann Variable nicht allokieren!");
                        WriteLn
                     END
                  END;
                  IF done THEN
                     UPN.SetVar(f, UPN.GetVarNr(f, VarName), Wert)
                  END
               END;
               WriteReal(Wert, 10, 10)
            ELSE (* Execute Error *)
               UPN.WriteError; WriteLn
            END  (* IF Execute Error *)
         END  (* IF Compile Error *)
      END  (* IF "?" *)
   END (* LOOP *)
END Rechner.


