(*---------------------------------------------------------------------------
   :Program.    3DStern.mod
   :Version.    1.0
   :Contents.   Red/Green-Glasses should be available
   :Remark.     Rewritten in Modula-2 from C by M.Peuckert
   :History.    V1.0 First C-Version by ... (wei nich mehr), Fishdisks (#33 ?)
   :History.    V1.0 M2, Markus Peuckert, November 89
   :Author.     Markus Peuckert
   :Address.    Schtzenstr. 50, D-3550 Marburg, West-Germany,
   :Copyright.  PD
   :Language.   Modula-2
   :Translator. M2Amiga V3.1d
---------------------------------------------------------------------------*)

MODULE Stern;

FROM SYSTEM    IMPORT ADR, ADDRESS, SHIFT;
FROM Arts      IMPORT Assert,TermProcedure;
FROM Graphics  IMPORT RastPortPtr,ViewPortPtr,ViewModes,ViewModeSet,
                      WritePixel, SetAPen, SetRGB4, SetRast;
FROM Intuition IMPORT customScreen,IDCMPFlags,IDCMPFlagSet,NewScreen,NewWindow,
                      ScreenFlags,ScreenFlagSet,ScreenPtr,WindowFlags,
                      WindowFlagSet,WindowPtr,CloseScreen,CloseWindow,
                      OpenScreen,OpenWindow;
FROM Exec      IMPORT MessagePtr, GetMsg, ReplyMsg;
FROM RandomNumber IMPORT RND;


CONST  WIDTH  = 352;
       HEIGHT = 282;
       DEPTH  = 4;

       nStars = 32;
       inc    = 3;
       Magic  = 128;

TYPE   ArrayTyp = ARRAY [0..nStars-1] OF INTEGER;

VAR
 screen : ScreenPtr;
 window : WindowPtr;
 rp     : RastPortPtr;
 vp     : ViewPortPtr;

 x, y, z,
 xlo, xro, yo  : ArrayTyp;
 xls, xrs, ys, fz : INTEGER;


PROCEDURE Cleanup;
BEGIN
 IF window#NIL THEN
  CloseWindow(window)
 END;
 IF screen#NIL THEN
  CloseScreen(screen)
 END
END Cleanup;

PROCEDURE CreateScreen(w,h,d: INTEGER;t: ADDRESS): ScreenPtr;
VAR
 ns: NewScreen;
BEGIN
 WITH ns DO
  leftEdge:=0;topEdge:=0;width:=w;height:=h;depth:=d;
  detailPen:=0;blockPen:=1;
  viewModes:=ViewModeSet{};
  type:=customScreen;
  font:=NIL;
  defaultTitle:=t;
  gadgets:=NIL;
  customBitMap:=NIL
 END;
 RETURN OpenScreen(ns)
END CreateScreen;

PROCEDURE CreateWindow(x,y,w,h: INTEGER; if: IDCMPFlagSet; wf: WindowFlagSet;
                       g,s,t: ADDRESS): WindowPtr;
VAR
 nw: NewWindow;
BEGIN
 WITH nw DO
  leftEdge:=x;topEdge:=y;width:=w;height:=h; detailPen:=0;blockPen:=1;
  idcmpFlags:=if; flags:=wf; firstGadget:=g; checkMark:=NIL;
  title:=t;screen:=s;bitMap:=NIL;minWidth:=50;minHeight:=50;
  maxWidth:=w;maxHeight:=h; type:=customScreen
 END;
 RETURN OpenWindow(nw)
END CreateWindow;

PROCEDURE MakePoint (i : INTEGER);
BEGIN
 x[i] := RND (256) - 128;
 y[i] := RND (150) - 75;
 z[i] := 255;
END MakePoint;


VAR dum : LONGINT;
    Msg : MessagePtr;
    i   : INTEGER;
BEGIN
 window:=NIL; screen:=NIL;
 TermProcedure(Cleanup);
 screen:=CreateScreen(WIDTH, HEIGHT, DEPTH, NIL);
 Assert(screen#NIL,ADR("Kann Schirm nicht ffnen"));
 window:=CreateWindow(0,0,WIDTH, HEIGHT, IDCMPFlagSet{mouseButtons},
                      WindowFlagSet{borderless}, NIL,screen,NIL);
 Assert(window#NIL,ADR("Kann Fenster nicht ffnen"));
 rp:=window^.rPort;
 vp:=ADR(screen^.viewPort);
(*--------------- Initialisierung abgeschlossen ----------------------------*)
 SetRGB4 (vp, 0, 0,0,0);  SetRast (rp, 0);
 xls:=1;
 FOR i:=1 TO 7 DO
  SetRGB4 (vp, i, 0, xls, 0);
  SetRGB4 (vp, i+8, xls, 0, 0);
  INC (xls, 1);
 END;

 FOR i:=0 TO nStars-1 DO
  MakePoint (i);
 END;
 REPEAT
  FOR i:=0 TO nStars-1 DO
   DEC (z[i], inc);
   IF z[i]<=0 THEN  MakePoint (i); END;
   fz := z[i];
   xls := (x[i] - (SHIFT (fz, -4))) * Magic / fz + 176;
   xrs := (x[i] + (SHIFT (fz, -4))) * Magic / fz + 176;
   ys  := y[i] * Magic / fz + 141;
   SetAPen (rp, 0);
   dum:=WritePixel (rp, xlo[i], yo[i]);
   dum:=WritePixel (rp, xro[i], yo[i]);
   IF (xls<0) OR (xls>352) OR (xrs<0) OR (xrs>352) OR (ys<0) OR (ys>282) THEN
    MakePoint (i);
   ELSE
    SetAPen (rp, 256 - SHIFT(fz, -5));
    dum:=WritePixel (rp, xls, ys);
    SetAPen (rp, 8+ (256 - (SHIFT(fz, -5))));
    dum:=WritePixel (rp, xrs, ys);
    xlo[i] := xls; xro[i] := xrs; yo[i] := ys;
   END;
  END;
  Msg := GetMsg (window^.userPort);
  IF Msg#NIL THEN
   ReplyMsg (Msg);
  END
 UNTIL Msg # NIL

END Stern.
