(* ------------------------------------------------------------------------
  :Program.       Tron
  :Contents.      Simple Game! (inspired by Tron)
  :Usage.         Use ToolTypes to change default-Keys
  :Usage.         Just drive around but do not touch a wall!
  :Author.        Kai Bolay
  :Address.       Hoffmannstrae 168, 7250 Leonberg
  :Phone.         07152 / 22135
  :History.       v1.0 Kai Bolay 22-Dec-89 Initial Version (Took 1h to code)
  :History.       v1.1 Kai Bolay 03-Jan-90 Thanks to Bully (5 mins??)
  :History.       v2.0 Kai Bolay 18-Jan-90 Tooltypes-feature added!
  :Copyright.     PD (Quelltext nicht beachten! Abschreckendes Beispiel)
  :Language.      Modula-2
  :Translator.    M2Amiga 3.2d
  :Imports.       IntuStruct1.3 [bne] (Meri, Bene!)
------------------------------------------------------------------------ *)
MODULE Tron;

(* FOLD: IMPORT *)
FROM SYSTEM      IMPORT ADR, CAST;
FROM Arts        IMPORT TermProcedure, Assert;
FROM Exec        IMPORT GetMsg, ReplyMsg, WaitPort;
FROM Dos         IMPORT Delay;
FROM Icon        IMPORT GetDiskObject, FreeDiskObject, FindToolType,
                        MatchToolValue;
FROM Intuition   IMPORT ScreenPtr, NewScreen, OpenScreen, CloseScreen,
                        WindowPtr, NewWindow, OpenWindow, CloseWindow,
                        WindowFlags, WindowFlagSet, customScreen,
                        IDCMPFlags, IDCMPFlagSet, IntuiMessagePtr,
                        DisplayBeep, SetWindowTitles, RefreshWindowFrame;
FROM Graphics    IMPORT ViewModes, ViewModeSet, ReadPixel, WritePixel,
                        SetRast, SetRGB4, SetAPen;
FROM Workbench   IMPORT DiskObjectPtr;
FROM Conversions IMPORT StrToVal;
FROM Arguments   IMPORT GetArg;
FROM IntuiStruct IMPORT StructScreen, StructWindow;
(* ENDFD *)
(* FOLD: TYPE *)
TYPE Direction = (up, left, right, down);
     Pos       = RECORD
                    x, y : INTEGER;
                 END; (* RECORD *)
     Nr        = [1..2];
(* ENDFD *)
(* FOLD: VAR *)
VAR TronScreen   : ScreenPtr;
    TronWindow   : WindowPtr;
    key          : CARDINAL;
    KeyExit      : CARDINAL;
    KeyDir       : ARRAY Direction OF ARRAY Nr OF CARDINAL;
    KeyFaster    : CARDINAL;
    KeySlower    : CARDINAL;
    KeyPause     : CARDINAL;
    KeyReset     : CARDINAL;
    PlayerDir    : ARRAY Nr OF Direction;
    PlayerPos    : ARRAY Nr OF Pos;
    PlayerLost   : ARRAY Nr OF [0..999];
    Crash, Start : BOOLEAN;
    delay        : CARDINAL;
    Pause        : BOOLEAN;
    TitleStr     : ARRAY [0..80] OF CHAR;
(* ENDFD *)

(* FOLD: CloseDown *)
PROCEDURE CloseDown;

BEGIN
   IF TronWindow # NIL THEN
      CloseWindow (TronWindow);
      TronWindow := NIL;
   END; (* IF *)
   IF TronScreen # NIL THEN
      CloseScreen (TronScreen);
      TronScreen := NIL;
   END; (* IF *)
END CloseDown;
(* ENDFD *)
(* FOLD: OpenUp *)
PROCEDURE OpenUp;

CONST Title = "Tron v2.0   1990 by Kai Bolay";

VAR NewTronScreen : NewScreen;
    NewTronWindow : NewWindow;
    Modes         : ViewModeSet;

   (* FOLD: InitDefaults *)
   PROCEDURE InitDefaults;

   TYPE StringPtr = POINTER TO ARRAY [0..999] OF CHAR;

   VAR TronName : ARRAY [0..200] OF CHAR;
       StrPtr   : StringPtr;
       TronIcon : DiskObjectPtr;
       dummy    : INTEGER;
       err      : BOOLEAN;

      (* FOLD: InitCard *)
      PROCEDURE InitCard (VAR card : CARDINAL; default : CARDINAL;
                          name : StringPtr);
      VAR int : LONGINT;
          sig : BOOLEAN;

      BEGIN
         card := default;
         IF TronIcon # NIL THEN
            StrPtr := FindToolType (TronIcon^.toolTypes, name);
            IF StrPtr # NIL THEN
               StrToVal (StrPtr^, int, sig, 10, err);
               IF NOT (err) AND NOT (sig) THEN card := int; END;
            END; (* IF *)
         END; (* IF *)
         Assert (card < 256, ADR ("Illegal number in Tooltypes!"));
      END InitCard;
      (* ENDFD *)

   BEGIN
      GetArg (0, TronName, dummy);
      TronIcon := GetDiskObject (ADR (TronName));
      Modes := ViewModeSet {};
      InitCard (KeyDir[up, 1], 119, ADR ("UP1"));
      InitCard (KeyDir[up, 2], 056, ADR ("UP2"));
      InitCard (KeyDir[left, 1], 097, ADR ("LEFT1"));
      InitCard (KeyDir[left, 2], 052, ADR ("LEFT2"));
      InitCard (KeyDir[right, 1], 115, ADR ("RIGHT1"));
      InitCard (KeyDir[right, 2], 054, ADR ("RIGHT2"));
      InitCard (KeyDir[down, 1], 121, ADR ("DOWN1"));
      InitCard (KeyDir[down, 2], 050, ADR ("DOWN2"));
      PlayerLost[1] := 0; PlayerLost[2] := 0;
      InitCard (KeyFaster, 43, ADR ("FASTER"));
      InitCard (KeySlower, 45, ADR ("SLOWER"));
      InitCard (KeyPause, 112, ADR ("PAUSE"));
      InitCard (KeyReset, 114, ADR ("RESET"));
      InitCard (KeyExit, 27, ADR ("EXIT"));
      InitCard (delay, 3, ADR ("SPEED"));
      IF delay > 4 THEN delay := 4; END; IF delay < 0 THEN delay := 0; END;
      delay := 4 - delay;
      IF TronIcon # NIL THEN
         StrPtr := FindToolType (TronIcon^.toolTypes, ADR ("VIEWMODES"));
         IF StrPtr # NIL THEN
            IF MatchToolValue (StrPtr, ADR ("hires")) THEN
               INCL (Modes, hires);
            END; (* IF *)
            IF MatchToolValue (StrPtr, ADR ("lace")) THEN
               INCL (Modes, lace);
            END; (* IF *)
         END; (* IF *)
         FreeDiskObject (TronIcon);
      END; (* IF *)
   END InitDefaults;
   (* ENDFD *)

BEGIN
   InitDefaults;
   TronWindow := NIL; TronScreen := NIL;
   TermProcedure (CloseDown);
   StructScreen (NewTronScreen, 2, 0, 1, Modes, customScreen, ADR (Title));
   TronScreen := OpenScreen (NewTronScreen);
   Assert (TronScreen # NIL, ADR ("Can't open Screen!"));
   WITH TronScreen^ DO
      SetRGB4 (ADR (viewPort), 0,  0,  0,  0);
      SetRGB4 (ADR (viewPort), 1, 15, 15, 15);
      SetRGB4 (ADR (viewPort), 2, 15,  0,  0);
      SetRGB4 (ADR (viewPort), 3, 15, 15,  0);
   END; (* WITH *)
   StructWindow (NewTronWindow, 0, 11, TronScreen^.width,
                 TronScreen^.height - 11, 0, 3, IDCMPFlagSet {vanillaKey},
                 WindowFlagSet {backDrop, activate, rmbTrap},
                 NIL, TronScreen, customScreen);
   TronWindow := OpenWindow (NewTronWindow);
   Assert (TronWindow # NIL, ADR ("Can't open Window!"));
END OpenUp;
(* ENDFD *)
(* FOLD: InitPos *)
PROCEDURE InitPos;

BEGIN
   WITH TronWindow^ DO
      WaitPort (userPort);
      SetRast (rPort, 0);
      RefreshWindowFrame (TronWindow);
      PlayerPos[1].x := width DIV 2 - 10;
      PlayerPos[2].x := width DIV 2 + 10;
      PlayerPos[1].y := height DIV 2 - 1;
      PlayerPos[2].y := height DIV 2;
      PlayerDir[1] := left; PlayerDir[2] := right;
   END; (* WITH *)
END InitPos;
(* ENDFD *)
(* FOLD: HandleMessage *)
PROCEDURE HandleMessage;

VAR TronMessage : IntuiMessagePtr;
    Dir         : [up..down];
    Player      : Nr;

BEGIN
   TronMessage := GetMsg (TronWindow^.userPort);
   IF TronMessage # NIL THEN
      key := TronMessage^.code;
      ReplyMsg (TronMessage);
      FOR Player := 1 TO 2 DO
         FOR Dir := up TO down DO
            IF key = KeyDir[Dir, Player] THEN PlayerDir[Player] := Dir; END;
         END; (* FOR *)
      END; (* FOR *)
      IF  key = KeyFaster THEN
         IF delay > 0 THEN DEC (delay); END;
      ELSIF key = KeySlower THEN
         IF delay < 4 THEN INC (delay); END;
      ELSIF key = KeyPause THEN
         WaitPort (TronWindow^.userPort);
      ELSIF key = KeyReset THEN
         SetWindowTitles (TronWindow, -1, TronScreen^.defaultTitle);
         PlayerLost[1] := 0; PlayerLost[2] := 0;
         InitPos;
      END; (* IF *)
   END; (* IF *)
END HandleMessage;
(* ENDFD *)
(* FOLD: Movement *)
PROCEDURE Movement;

   (* FOLD: Looser *)
   PROCEDURE Looser (Player : Nr);

      (* FOLD: ValToStrPos *)
      PROCEDURE ValToStrPos (Val, Pos : CARDINAL);

      VAR d, Pot, Digit : CARDINAL;

      BEGIN
         Pot := 100;
         FOR Digit := Pos-1 TO Pos+1 DO
            d := Val DIV Pot;
            TitleStr[Digit] := CHAR (d + 48); (* CAST will nicht! *)
            DEC (Val, Pot * d);
            Pot := Pot DIV 10;
         END; (* FOR *)
      END ValToStrPos;
      (* ENDFD *)

   VAR EatIt : IntuiMessagePtr;

   BEGIN
      DisplayBeep (TronScreen);
      IF PlayerLost[Player] # 999 THEN
         INC (PlayerLost[Player]);
      ELSE
         PlayerLost[Player] := 0;
      END; (* IF *)

      IF (hires IN TronScreen^.viewPort.modes) THEN
         TitleStr := "  !! Bad luck !!   Player #x is the looser!   !! Bad luck !!  ";
         TitleStr[32] := CAST (CHAR, (Player + 48));
      ELSE
         TitleStr := "   Player #x is the looser!  ";
         TitleStr[14] := CAST (CHAR, (Player + 48));
      END; (* IF *)
      SetWindowTitles (TronWindow, -1, ADR (TitleStr));

      Delay (1 * 50);
      REPEAT EatIt := GetMsg (TronWindow^.userPort) UNTIL EatIt = NIL;
 
      IF (hires IN TronScreen^.viewPort.modes) THEN
         TitleStr := "Player #1 : xxx      Tron v2.0   1990 by Kai Bolay      Player #2 : xxx";
         ValToStrPos (PlayerLost[1], 70);
      ELSE
         TitleStr := "Player #1 : xxx     Player #2 : xxx";
         ValToStrPos (PlayerLost[1], 33);
      END; (* IF *)
      ValToStrPos (PlayerLost[2], 13);
      SetWindowTitles (TronWindow, -1, ADR (TitleStr));
      InitPos;
   END Looser;
   (* ENDFD *)

VAR Player : Nr;

BEGIN
   FOR Player := 1 TO 2 DO
      WITH PlayerPos[Player] DO
         CASE PlayerDir[Player] OF
         | up    : DEC (y);
         | left  : DEC (x);
         | right : INC (x);
         | down  : INC (y);
         END; (* CASE *)
         WITH TronWindow^ DO
            SetAPen (rPort, Player);
            IF ReadPixel (rPort, x, y) = 0 THEN
               IF WritePixel (rPort, x, y) THEN
                  Looser (Player);
               END; (* IF *)
            ELSE
               Looser (Player);
            END; (* IF *)
         END; (* WITH *)
      END; (* WITH *)
   END; (* FOR *)
END Movement;
(* ENDFD *)

BEGIN
   OpenUp;
   InitPos;
   REPEAT
      HandleMessage;
      Movement;
      Delay (delay);
   UNTIL key = KeyExit;
END Tron.
