(*
  :Program.       DCF
  :Author.        Martin Horneffer
  :Address.       Semester: Ssterfeldstr.30, 5100 Aachen
  :Address.       sonst:    Stenzelbergstr.23, 5330 Knigswinter 41
  :Address.       Zerberus-Net: KOX@MIDI.ZER
  :Address.       Maus-Net: Martin Horneffer @ BN
  :Address.              &  Martin Horneffer @ BN2
  :History.       V1.0 26.12.89 Martin Horneffer
  :Copyright.     PD
  :Language.      MODULA-II
  :Translator.    M2Amiga 3.11d
  :Imports.       TimerSupport V1.0, Hartmut Hhn

  :Contents.      Dekodierung des deutschen Zeitzeichens DCF-77
  :Contents.      und optional Setzen der Systemzeit

  :Remark.        Das Prg braucht natrlich eine entsprechende Hardware
  :Remark.        (77,5 kHz Empfnger), die dem Computer das Zeitsignal
  :Remark.        zugnglich macht.
  :Remark.        So, wie es ist, geht das Prg davon aus, da der TTL-
  :Remark.        Ausgang der Empfngers am 'fire'-Pin des (rechten)
  :Remark.        Joystick-Ports anliegt.
  :Remark.        Das Prg lt sich aber leicht an eine andere Beschaltung
  :Remark.        anpassen. Bei Problemen an mich wenden.
  :Remark.        Je nach Empfnger mssen die Konstanten gendert werden,
  :Remark.        vor allem 'maxShort'
  :Remark.        Vielen Dank an [fbs] frs Zeigen, wie man
  :Remark.        InterruptServer installiert.

    Achtung !!!
      Wer das Mischen von englischen und deutschen Bezeichnern und
    Texten nicht haben kann, sollte besser nicht weiterlesen.

*)

MODULE DCF ;

FROM TimerSupport       IMPORT OpenTimer, CloseTimer, SetTime, TimerWait ;
FROM InOut              IMPORT WriteString, WriteLn, WriteInt, Write ;
FROM Arts               IMPORT TermProcedure, Terminate ;
FROM Arguments          IMPORT NumArgs, GetArg ;

FROM SYSTEM IMPORT ADR,INLINE;
FROM Exec IMPORT Interrupt,NodeType,AddIntServer,RemIntServer;
FROM Hardware IMPORT vertb;

TYPE    StatusSet       = SET OF (valid,running,startBit,tst1,tst2,tst3) ;

CONST   WaitTime        = 20000 ; (* Verzgerung in us *)
        maxSec          = 1000000 DIV WaitTime ; (* max. Wert f. eine Sek. *)
        maxShort        =   90000 DIV WaitTime ; (* max. f. kurzen Impuls *)
        maxLong         =  200000 DIV WaitTime ; (* max. f. langen Impuls *)
        esc             = 33C ;
        csi             = 233C ;

    MOVEMS = 48E7H; (* that's the 68000-Instruction MOVEM to save Registers*)
    MOVEML = 4CDFH; (* that's MOVEM to load Registers                      *)

VAR     Ciaapra [0BFE001H]      : SET OF (s0,s1,s2,s3,s4,s5,lmb,DCF) ;
        (* Empfnger liegt am Feuerknopf vom (rechten) Joystick-Port *)
        (* Das kann bei anderer Beschaltung natrlich anders sein. *)
        (* Diese Variable wird aber nur an EINER Stelle abgefragt *)

        Jahr,Monat,Tag,Stunde,
        Minute, WoTag, sec,sec2 : LONGINT ;
        Z1, Z2, R, A1, A2       : BOOLEAN ;
        WochenTag               : ARRAY [1..7] OF ARRAY [0..10] OF CHAR ;
        Status                  : StatusSet;
        HighTime, LowTime, check: LONGINT ;
        TimeOutHigh, TimeOutLow,
        longMins, shortMins     : LONGINT ;
        last, currBit           : BOOLEAN ;
        VertBIntr               : Interrupt; (* Meine Interrupt-Struktur *)
        CliOption               : ARRAY [0..7] OF CHAR ;
        Debug,SetClock,ShowTime : BOOLEAN ;
        MaxErrors, i, l         : INTEGER ;

PROCEDURE reset ;
  BEGIN
    Status := Status - StatusSet{valid,startBit,tst1,tst2,tst3} ;
    Jahr:=1900; Monat:=0; Tag:=0; Stunde:=0; Minute:=0; WoTag:=0;
  END reset ;

PROCEDURE Auswerten ( s : LONGINT; bit : BOOLEAN ) ;
(* Ordnet jedes Bit nach seiner Bedeutung ein *)
  BEGIN
    IF s = 0 THEN  check:=0  END ;
    IF bit THEN  INC(check)  END ;
    CASE s OF
      14 : reset |
      15 :  R := bit |
      16 : A1 := bit |
      17 : Z1 := bit |
      18 : Z2 := bit |
      19 : A2 := bit |
      20 : IF bit THEN  INCL(Status,startBit)  END |  (* Startbit *)
      28 : IF (check MOD 2) = 0 THEN  INCL(Status,tst1)  END ;
           check := 0 |                            (* Prfbit 1 *)
      35 : IF (check MOD 2) = 0 THEN  INCL(Status,tst2)  END ;
           check := 0 |                            (* Prfbit 2 *)
      58 : INCL(Status,valid) ;                    (* Daten gltig *)
           IF (check MOD 2) = 0 THEN  INCL(Status,tst3)  END ;
           check := 0 |                            (* Prfbit 3 *)
      ELSE
        IF s >= 59 THEN
          Status := Status - StatusSet{valid,running} ;
          INC(longMins) ;
        ELSIF (s>=21) AND bit THEN
          CASE s OF
            21 : INC(Minute,1) |
            22 : INC(Minute,2) |
            23 : INC(Minute,4) |
            24 : INC(Minute,8) |
            25 : INC(Minute,10) |
            26 : INC(Minute,20) |
            27 : INC(Minute,40) |
            29 : INC(Stunde,1) |
            30 : INC(Stunde,2) |
            31 : INC(Stunde,4) |
            32 : INC(Stunde,8) |
            33 : INC(Stunde,10) |
            34 : INC(Stunde,20) |
            36 : INC(Tag,1) |
            37 : INC(Tag,2) |
            38 : INC(Tag,4) |
            39 : INC(Tag,8) |
            40 : INC(Tag,10) |
            41 : INC(Tag,20) |
            42 : INC(WoTag,1) |
            43 : INC(WoTag,2) |
            44 : INC(WoTag,4) |
            45 : INC(Monat,1) |
            46 : INC(Monat,2) |
            47 : INC(Monat,4) |
            48 : INC(Monat,8) |
            49 : INC(Monat,10) |
            50 : INC(Jahr,1) |
            51 : INC(Jahr,2) |
            52 : INC(Jahr,4) |
            53 : INC(Jahr,8) |
            54 : INC(Jahr,10) |
            55 : INC(Jahr,20) |
            56 : INC(Jahr,40) |
            57 : INC(Jahr,80) |
          END ; (* CASE *)
        END ; (* ELSIF *)
    END ; (* CASE,ELSE *)
  END Auswerten ;

PROCEDURE DCFIntProc() ;
(* InterruptServer zum Decodieren der TTL-Singnale *)
  BEGIN
    INLINE(MOVEMS,3F3EH); (* this is MOVEM d2-d7/a2-a6,-(sp) *)

    (* Die folgende Zeile mu bei anderer Hardware ggf. gendert werden. *)
    (* Sie ist dann aber die einzige *)
    IF DCF IN Ciaapra THEN
      IF last THEN      (* high *)
        INC(HighTime);
        IF HighTime > 2*maxSec THEN
          HighTime := 0 ;
          EXCL(Status,running) ; (* mu neu initialisiert werden *)
          INC(TimeOutHigh) ;
        END ;
      ELSE              (* steigende Flanke *)
        last := TRUE ;
        HighTime := 0 ;
        IF running IN Status THEN
          currBit := (LowTime > maxShort) ;
          Auswerten(sec,currBit);
        END ;
      END ; (* IF last *)
    ELSE
      IF last THEN      (* fallende Flanke *)
        last := FALSE;
        INC(sec) ;
        LowTime := 0 ;
        IF HighTime > maxSec THEN
          IF sec <59 THEN  INC(shortMins)  END ;
          sec := 0 ;
          INCL(Status,running) ;
        END ;
      ELSE              (* immer noch unten *)
        INC(LowTime) ;
        IF LowTime > maxLong THEN
          LowTime := 0 ;
          EXCL(Status,running) ; (* mu neu initialisiert werden *)
          INC(TimeOutLow) ;
        END ;
      END ; (* IF last *)
    END ; (* IF DCF IN Ciaapra *)

    INLINE(MOVEML,7CFCH); (* this is MOVEM (sp)+,d2-d7/a2-a6 *)
  END DCFIntProc ;

PROCEDURE StatusAusgabe ;
  BEGIN
    Write(esc); WriteString("[33m");
    IF currBit THEN  WriteString("+++ ");  ELSE  WriteString("--- ");  END ;
    WriteInt(sec,3);
    WriteString(" Err.: ");
    WriteInt(TimeOutHigh,1); Write('/');
    WriteInt(TimeOutLow,1); Write('/');
    WriteInt(longMins,1); Write('/');
    WriteInt(shortMins,1);  Write(csi); Write('K');
    IF valid IN Status THEN  WriteString(" valid");  END;
    IF running IN Status THEN  WriteString(" running"); END;
    IF startBit IN Status THEN  WriteString(" sb")  END ;
    IF tst1 IN Status THEN  WriteString(" pb1")  END ;
    IF tst2 IN Status THEN  WriteString(" pb2")  END ;
    IF tst3 IN Status THEN  WriteString(" pb3")  END ;

    WriteLn; Write(esc); WriteString("[0m"); Write(esc); Write('M');
  END StatusAusgabe ;

PROCEDURE ZeitAusgabe ;
  BEGIN
    WochenTag[1] := "Montag";     WochenTag[2] := "Dienstag";
    WochenTag[3] := "Mittwoch";   WochenTag[4] := "Donnertsag";
    WochenTag[5] := "Freitag";    WochenTag[6] := "Samstag";
    WochenTag[7] := "Sonntag";
    WriteLn;
    WriteString("Wir haben heute "); WriteString(WochenTag[WoTag]);
    WriteString(", den "); WriteInt(Tag,2); Write('.'); WriteInt(Monat,2);
    Write('.'); WriteInt(Jahr,4); WriteLn;
    WriteString("Es ist jetzt genau: ");
    WriteInt(Stunde,1); WriteString(" Uhr, ");
    WriteInt(Minute,1); WriteString(" Minuten und ");
    WriteInt(sec,1); WriteString(" Sekunde(n) ");
    IF Z1 THEN  WriteString("MESZ")  END ;
    IF Z2 THEN  WriteString("MEZ")  END ;
    IF A1 THEN  WriteString("-A1")  END ;
    IF A2 THEN  WriteString("-A2")  END ;
    WriteLn ;
    IF R THEN  WriteString("Der Reserve-Sender ist in Betrieb !"); WriteLn END;
    Write(esc); WriteString("[0m"); WriteLn;
  END ZeitAusgabe ;

PROCEDURE Ende ;
  BEGIN
    RemIntServer(vertb,ADR(VertBIntr));
  END Ende ;

PROCEDURE Hilfe ; (* Hilfstext zeigen *)
  BEGIN
    GetArg( 0, CliOption, l) ;
    WriteString("Syntax:"); WriteLn;
    WriteString(CliOption); WriteString(" [-s|+s] [-d|+d] [-t|+t]"); WriteLn;
    WriteString("Default: "); WriteString(CliOption);
    WriteString("+s -d +t"); WriteLn;
    WriteString("+s : Systemzeit setzen;  -s : nicht setzen"); WriteLn;
    WriteString("+d : Debug-Mode;         -d : kein Debugging"); WriteLn;
    WriteString("+t : Zeit/Datum zeigen;  -t : nicht zeigen"); WriteLn;
    WriteLn;
    Terminate(0);
  END Hilfe ;

BEGIN
  (* Titel anzeigen *)
  WriteString("DCF-77-Decoder   26.12.1989 by Martin Horneffer"); WriteLn;

  (* Commandline auswerten *)
  SetClock := TRUE ; Debug := FALSE ; ShowTime := TRUE ; MaxErrors := 5 ;
  i := 1 ;
  WHILE i <= NumArgs() DO
    GetArg( i, CliOption, l) ;
    IF CliOption[0] = '?' THEN  Hilfe ;
    ELSIF CliOption[1] = 's' THEN
      IF    CliOption[0] = '-' THEN  SetClock := FALSE ;
      ELSIF CliOption[0] = '+' THEN  SetClock := TRUE ;  END ;
    ELSIF CliOption[1] = 'd' THEN
      IF    CliOption[0] = '+' THEN  Debug := TRUE ;
      ELSIF CliOption[0] = '-' THEN  Debug := FALSE ;  END ;
    ELSIF CliOption[1] = 't' THEN
      IF    CliOption[0] = '+' THEN  ShowTime := TRUE ;
      ELSIF CliOption[0] = '-' THEN  ShowTime := FALSE ;  END ;
    ELSE
      WriteString("Unbekannte Option: ") ;
      WriteString(CliOption) ; WriteLn ;
    END ; (* IF *)
    INC(i) ;
  END ; (* WHILE *)

  (* initialisieren *)
  reset ; sec := 0 ; Status := StatusSet{} ;
  TimeOutHigh:=0; TimeOutLow:=0; longMins:=0; shortMins:=-1;
  OpenTimer(TRUE);
  TermProcedure(CloseTimer);

  (* Interrupt installieren *)
  WITH VertBIntr DO
    node.type   := interrupt;  (* Typ ist Interrupt *)
    node.pri    := -60;
    node.name   := ADR("DCF77-Interrupt");  (* Name (nur fr Debugging) *)
    data        := NIL;        (* keine Daten. data ist beim Interrupt in A1 *)
    code        := ADR(DCFIntProc);  (* Zeiger auf InterruptProzedur *)
  END;
  AddIntServer(vertb,ADR(VertBIntr));
  TermProcedure(Ende);

  (* Hauptschleife: von Zeit zu Zeit nachschauen, ob das Datun da ist *)
  (*                ggf. laufend den Status anzeigen *)
  REPEAT
    TimerWait( 0, 750000) ;               (* 3/4 Sekunden warten *)
    IF Debug THEN  StatusAusgabe  END ;
  UNTIL
    (Status=StatusSet{valid,running,startBit,tst1,tst2,tst3}) AND (sec<10)
    OR (TimeOutHigh+TimeOutLow+longMins+shortMins > MaxErrors) ;

  (* Ende. Meldungen ausgeben. *)
  IF TimeOutHigh+TimeOutLow+longMins+shortMins > MaxErrors THEN
    Write(esc); WriteString("[3;33mDCF-77 failed. ");
    IF TimeOutLow+longMins+shortMins < 0
      THEN  WriteString("Receiver not connected. ") ;  END ;
    Write(esc); WriteString("[0m"); WriteLn;
  ELSE
    IF ShowTime THEN  ZeitAusgabe ;  END ;
    IF SetClock THEN
      sec2 := sec ;
      (* Auf Sekundenbeginn warten *)
      REPEAT  TimerWait( 0, 10000)  UNTIL sec2 # sec ;
      SetTime(Jahr,Monat,Tag,Stunde,Minute,sec) ;
      Write(esc); WriteString("[3;33mDCF-77 successfull. clock set.");
      Write(esc); WriteString("[0m"); WriteLn;
    ELSE
      Write(esc); WriteString("[3;33mDCF-77 successfull."); Write(esc);
      WriteString("[0m"); WriteLn;
    END ;
  END ;
  (* Timer, InterruptServer usw. werden durch die TermProcedues aufgerumt *)
END DCF.
