(*-------------------------------------------------------------------------

  Kleine Demonstration zu den Prozeduren des Moduls AVL. Es benutzt einen
  inhomogenen Baum zum Speichern von 3 Verschiedenen Lebewesensarten.
  Auerdem ist dies ein schnes Beispiel fr das intensive Arbeiten mit
  Recorderweiterungen und Type-Tests.

-------------------------------------------------------------------------*)

MODULE Bio;

IMPORT io, AVL;

TYPE
  TierPtr = POINTER TO Tier;
  Tier = RECORD (AVL.SNode)
           beine: LONGINT;
           augen: LONGINT;
         END;

  PflanzePtr = POINTER TO Pflanze;
  Pflanze = RECORD (AVL.SNode)
              bluetenfarbe: AVL.String;
              groesse: LONGINT;
            END;

  EinzellerPtr = POINTER TO Einzeller;
  Einzeller = RECORD (AVL.SNode)
                groesse: LONGINT;
              END;

VAR
  head: AVL.SRoot;
  Eingabe: LONGINT;

PROCEDURE ReadInt(s: ARRAY OF CHAR; VAR x: LONGINT);
BEGIN
  LOOP
    io.WriteString(s);
    IF io.ReadInt(x) THEN RETURN END;
  END;
END ReadInt;

PROCEDURE ReadName(VAR name: ARRAY OF CHAR);
BEGIN
  io.WriteString("\n  Name: "); io.ReadString(name);
END ReadName;


PROCEDURE AddLebe;

VAR
  Eingabe: LONGINT;
  new: AVL.SNodePtr;

  PROCEDURE NewTier(): TierPtr;
  VAR Tier: TierPtr;
  BEGIN
    NEW(Tier); ReadName(Tier.name);
    ReadInt("\n  Anzahl Beine: ",Tier.beine);
    ReadInt("\n  Anzahl Augen: ",Tier.augen);
    RETURN Tier;
  END NewTier;

  PROCEDURE NewPflanze(): PflanzePtr;
  VAR Pflanze: PflanzePtr;
  BEGIN
    NEW(Pflanze); ReadName(Pflanze.name);
    io.WriteString("\n  Bltenfarbe: "); io.ReadString(Pflanze.bluetenfarbe);
    ReadInt("\n  Gre (in cm): ",Pflanze.groesse);
    RETURN Pflanze;
  END NewPflanze;

  PROCEDURE NewEinzeller(): EinzellerPtr;
  VAR Einzeller: EinzellerPtr;
  BEGIN
    NEW(Einzeller); ReadName(Einzeller.name);
    ReadInt("\n  Gre (in m): ",Einzeller.groesse);
    RETURN Einzeller;
  END NewEinzeller;

BEGIN
  LOOP
    io.WriteString("\n   Lebewesen hinzufgen:\n");
    io.WriteString("\n (1) Tier");
    io.WriteString("\n (2) Pflanze");
    io.WriteString("\n (3) Einzeller");
    io.WriteString("\n (4) zurck\n");
    ReadInt("\nin> ",Eingabe);
    IF (Eingabe<1) OR (Eingabe>3) THEN RETURN END;
    CASE Eingabe OF
    1: new := NewTier()     |
    2: new := NewPflanze()  |
    3: new := NewEinzeller()|
    4: EXIT |
    END;
    io.WriteLn;
    IF AVL.SAdd(head,new) THEN io.WriteString("ok\n")
                          ELSE io.WriteString(new.name);
                               io.WriteString(" existiert schon!\n") END;
  END;
END AddLebe;


PROCEDURE FindLebe;
VAR
  name: AVL.String;
  node: AVL.SNodePtr;
BEGIN
  ReadName(name);
  node := AVL.SFind(head,name);
  IF node=NIL THEN
    io.WriteString("\n  sorry, nicht gefunden!\n");
  ELSE
    io.WriteString("\n  "); io.WriteString(name); io.WriteString(" ist ein");
    IF node IS Tier THEN
      WITH node: Tier DO
        io.WriteString(" Tier mit "); io.WriteInt(node.beine,4);
        io.WriteString(" Beinen und "); io.WriteInt(node.augen,4);
        io.WriteString(" Augen.\n");
      END;
    ELSIF node IS Pflanze THEN
      WITH node: Pflanze DO
        io.WriteString("e Pflanze mit "); io.WriteString(node.bluetenfarbe);
        io.WriteString("en Blten, die"); io.WriteInt(node.groesse,6);
        io.WriteString(" cm gro wird.\n");
      END;
    ELSE
      io.WriteString(" Einzeller, der"); io.WriteInt(node(Einzeller).groesse,6);
      io.WriteString(" m gro ist.\n");
    END;
  END;
END FindLebe;


PROCEDURE RemoveLebe;
VAR name: AVL.String;
    node: AVL.SNodePtr;
BEGIN
  ReadName(name);
  node := AVL.SFind(head,name);
  IF (node#NIL) AND AVL.Remove(head,node) THEN
    io.WriteString("\n  gelscht.\n");
  ELSE
    io.WriteString("\n  sorry, nicht gefunden!\n");
  END;
END RemoveLebe;


PROCEDURE TypeName(n: AVL.NodePtr);
BEGIN
  io.WriteString(n(AVL.SNode).name); io.WriteLn;
END TypeName;


BEGIN
  AVL.SInit(head);
  io.WriteString("\n      Bio-Lexikon\n");
  io.WriteString("    ===============\n");
  LOOP
    io.WriteString("\n  (1)  Lebewesen hinzufgen\n");
    io.WriteString("  (2)  Lebewesen suchen\n");
    io.WriteString("  (3)  Lexikon anzeigen\n");
    io.WriteString("  (4)  Lebewesen lschen\n");
    io.WriteString("  (5)  Ende\n\n");
    ReadInt("in> ",Eingabe);
    CASE Eingabe OF
    1: AddLebe    |
    2: FindLebe   |
    3: io.WriteLn; AVL.DoForward(head,TypeName) |
    4: RemoveLebe |
    5: EXIT       |
    ELSE END;
  END;
  AVL.Dispose(head);
END Bio.

