(* FileIO        Simple IO with more than one file     Moe 18.01.84
   ======        =================================
This module provides procedures which are similar to those of InOut,
except that they can be used with more than one file (even with the
console).
------------------------------------------------------------------------*)
IMPLEMENTATION MODULE FileIO;

FROM FileSystem IMPORT
  Close, File, ReadChar, WriteChar;
FROM SYSTEM IMPORT 
  WORD, ADR;
IMPORT Terminal;

CONST CR = 15C; LF = 12C; BS = 10C;

(* Read             Read a character from file f
------------------------------------------------------------------------*)
PROCEDURE Read(VAR f:File; VAR ch:CHAR);
BEGIN
  IF ADR(f) = ADR(con)
    THEN Terminal.Read(ch);
    ELSE
      ReadChar(f,ch);
      IF ch = LF THEN ch := EOL END;
      IF f.eof THEN ch:=EF; END;
      END;
  END Read;

PROCEDURE ReadLn(VAR f:File);
VAR ch: CHAR;
BEGIN
  LOOP
    Read(f, ch);
    IF ch = EOL THEN EXIT END;
  END;
END ReadLn;
    

(* ReadCard    Read a CARDINAL-constant from file f
-------------------------------------------------------------------*)
PROCEDURE ReadCard(VAR f:File; VAR val:CARDINAL);
VAR
  ch: CHAR;
  i:  CARDINAL;
BEGIN
  IF ADR(f) = ADR(con)
    THEN
      i:=0; val:=0;
      REPEAT Terminal.Read(ch); UNTIL ch<>" ";
      WHILE ch>" " DO
        IF ch=DEL THEN
          IF i>0 THEN
            Terminal.Write(ch); DEC(i); val:=val DIV 10;
            END;
        ELSIF (ch>="0") AND (ch<="9") AND
          ((val<6553) OR ((val=6553) AND (ch<="5"))) THEN
          Terminal.Write(ch); INC(i); val:=10*val+CARDINAL(ORD(ch)-ORD("0"));
          END;
        Terminal.Read(ch);
        END;
      Done:=i>0;
    ELSE
      val:=0; Done:=TRUE;
      REPEAT Read(f,ch) UNTIL ch<>" ";
      WHILE ch>" " DO
        IF (ch>="0") AND (ch<="9") AND Done AND
          ((val<6553) OR ((val=6553) AND (ch<="5")))
          THEN val:=10*val+CARDINAL(ORD(ch)-ORD("0"));
          ELSE Done:=FALSE; val:=0;
          END;
        Read(f,ch);
        END;
    END;
  termCH:=ch;
  END ReadCard;  

(* ReadString    Read a string of characters from file f
-------------------------------------------------------------------*)
PROCEDURE ReadString(VAR f:File; VAR s:ARRAY OF CHAR);
VAR
  i:  INTEGER;
  ch: CHAR;
BEGIN
  IF ADR(f) = ADR(con)
    THEN
      REPEAT Terminal.Read(ch); UNTIL ch # ' ';
      i:=-1;
      WHILE ch>" " DO
        IF ch=DEL THEN
          IF i>=0 THEN
            Terminal.Write(ch); DEC(i);
            END;
        ELSIF i<INTEGER(HIGH(s)) THEN
          Terminal.Write(ch); INC(i); s[i]:=ch;
          END;
        Terminal.Read(ch);
        END;
    ELSE
      REPEAT Read(f,ch); UNTIL  (ch # ' ') AND (ch # EOL);
      i:=-1;
      IF FALSE(*ch = '"'*) THEN
        s[0] := '"'; i := 0;
        LOOP
          Read(f,ch);
          IF i=INTEGER(HIGH(s)) THEN
            s[i] := '"';
            EXIT
          END;
          (* Strings in "...." werden mit Quotes eingelesen *)
          INC(i);   s[i]:=ch;
          IF ch = '"' THEN EXIT END;
        END;
      ELSE
        WHILE ch>" " DO
          IF i<INTEGER(HIGH(s)) THEN INC(i); s[i]:=ch; END;
          Read(f,ch);
        END;
      END;
    END;
  termCH:=ch;
  INC(i);
  IF i<=INTEGER(HIGH(s)) THEN s[i]:=0C; END;
  END ReadString;                        


(* ReadInt   Read an INTEGER-constant from file f
-------------------------------------------------------------------*)
PROCEDURE ReadInt(VAR f:File; VAR val:INTEGER);   
VAR
  ch:   CHAR;
  sign: INTEGER;
  x:    CARDINAL;
  s:    ARRAY[1..80] OF CHAR;
  i:    CARDINAL;
BEGIN
  ReadString(f,s);
  x:=0; val:=0; i:=1;
  IF s[i]="-" THEN sign:=-1; INC(i); ELSE sign:=1; END;
  ch:=s[i];
  LOOP
    IF ch=0C THEN Done:=TRUE; EXIT; END;
    IF (ch<"0") OR (ch>"9") THEN Done:=FALSE; EXIT; END;
    IF (x>3276) OR ((x=3276) AND (ch>"8")) THEN Done:=FALSE; EXIT; END;
    x:=10*x+CARDINAL(ORD(ch)-ORD("0"));
    INC(i); ch:=s[i];
    END;
  IF Done THEN
    IF x<=32767 THEN val:=sign*INTEGER(x);
    ELSIF sign=-1 THEN val:=-32767-1;
    ELSE Done:=FALSE; END;
    END;
  END ReadInt;  


(* ReadWord   Read a word from File f without conversion
-----------------------------------------------------------------------*)
PROCEDURE ReadWord(VAR f:File; VAR w:CARDINAL);  
VAR i,j: CHAR;
BEGIN
  Read(f,i); Read(f,j);
  w:=256*ORD(i) + ORD(j);
  END ReadWord;


(* Write            Write a character to list file
------------------------------------------------------------------------*)
PROCEDURE Write(VAR f:File; ch:CHAR);
BEGIN
  IF ADR(f) = ADR(con)
    THEN IF ch = EOL THEN Terminal.WriteLn
         ELSIF (ch = DEL) OR (ch = BS)
         THEN Terminal.Write(BS);
              Terminal.Write(' '); Terminal.Write(BS);
         ELSE Terminal.Write(ch)
         END
    ELSIF ch = EOL THEN (*WriteChar(f,CR);*) WriteChar(f,LF);
    ELSE WriteChar(f,ch); 
    END;
  END Write;


(* WriteCard        Write a cardinal to list file
------------------------------------------------------------------------*)
PROCEDURE WriteCard(VAR f:File; nr:CARDINAL; w:CARDINAL);
VAR
 l,d: CARDINAL;
 t:   ARRAY[1..5] OF CHAR;
BEGIN
  l:=0;
  REPEAT
    d:=nr MOD 10; nr:=nr DIV 10;
    INC(l); t[l]:=CHR(ORD("0")+d);
    UNTIL nr=0;
  WHILE w>l DO Write(f," "); DEC(w); END;
  WHILE l>0 DO Write(f,t[l]); DEC(l); END;
  END WriteCard;


(* WriteHex         Write length bytes from a
-----------------------------------------------------------------------*)
(*PROCEDURE WriteHex(VAR f:File; s:ARRAY OF WORD;          25.11.,D,Dob
                     length:CARDINAL);
VAR
  i,j,w: CARDINAL;

  PROCEDURE WriteHexDigit(b:CARDINAL);
  BEGIN
    IF b<10
      THEN Write(f,CHR(b+ORD("0")));
      ELSE Write(f,CHR(b-10+ORD("A"))); END;
    END WriteHexDigit;

BEGIN
  j:=0;
  FOR i:=1 TO length DO
    IF ODD(i)
      THEN w:=CARDINAL(s[j]) DIV 256;
      ELSE w:=CARDINAL(s[j]) MOD 256; INC(j); END;
    Write(f," ");
    WriteHexDigit(w DIV 16);
    WriteHexDigit(w MOD 16);
    END;
  END WriteHex;                                                        *)


(* WriteInt   Write an INTEGER-value to file f
-------------------------------------------------------------------*)
PROCEDURE WriteInt(VAR f:File; i:INTEGER; w:CARDINAL);
VAR
  l,d,x:  CARDINAL;
  t:      ARRAY[1..5] OF CHAR;
  sign:   CHAR;
BEGIN
  IF i<0
    THEN sign:="-"; x:=ABS(i+1); INC(x);
    ELSE sign:=" "; x:=ABS(i);
    END;
  l:=0;
  REPEAT
    d:=x MOD 10; x:=x DIV 10;
    INC(l); t[l]:=CHR(ORD("0")+d);
    UNTIL x=0;
  WHILE w>l+1 DO Write(f," "); DEC(w); END;
  IF (sign="-") OR (w>l) THEN Write(f,sign); END;
  WHILE l>0 DO Write(f,t[l]); DEC(l); END;
  END WriteInt; 


(* WriteLn          Skip to new line on list file
------------------------------------------------------------------------*)
PROCEDURE WriteLn(VAR f:File);
BEGIN
  Write(f,EOL);
  (*Write(f, CR); Write(f, LF);*)
  END WriteLn;


(* WriteString        Write a string to list file
------------------------------------------------------------------------*)
PROCEDURE WriteString(VAR f:File; s:ARRAY OF CHAR);
VAR i: CARDINAL;
BEGIN
  i:=0;
  LOOP
    IF i>CARDINAL(HIGH(s)) THEN EXIT;
    ELSIF s[i]="$" THEN WriteLn(f);
    ELSIF s[i]=0C THEN EXIT;
    ELSE Write(f,s[i]); END;
    INC(i);
    END;
  END WriteString;


(* WriteText          Write text to list file
------------------------------------------------------------------------*)
PROCEDURE WriteText(VAR f:File; t:ARRAY OF CHAR; l:CARDINAL);
VAR i: CARDINAL;
BEGIN
  IF l>0 THEN
    FOR i:=0 TO l-1 DO Write(f,t[i]); END;
    END;
  END WriteText;


(* WriteWord   Write a word to File f without conversion
-----------------------------------------------------------------------*)
PROCEDURE WriteWord(VAR f:File; w:CARDINAL);
BEGIN
  Write(f,CHR(w DIV 256));                               (*25.11.,C,Dob*)
  Write(f,CHR(w MOD 256));                               (*25.11.,C,Dob*)
  END WriteWord;


BEGIN  (*FileIO*)
END FileIO.
