(* cocolex:      lexical analyzer for coco                 moe 83.03.27
   =======       =========================                     83.12.23
This is the Coco-scanner. It
  a) reads and prints the input grammar
  b) returns symbol numbers and terminal attributes to the parser
  c) hashes names and strings into a name list (permanently or
     temporarily)
  d) converts number-strings to values
All symbols, which are no terminals of Cocol get the symbol type
'nococosy' and are hashed into the name list.
---------------------------------------------------------------------*)
IMPLEMENTATION MODULE OberonLex;
IMPORT InOut;
FROM Arts       IMPORT TermProcedure, Assert;
FROM SYSTEM     IMPORT ADR;
FROM FileIO     IMPORT con, WriteCard, WriteString, WriteText;
IMPORT FileIO;
FROM Errors     IMPORT CompErr, SemErr, Restriction;
FROM FileSystem IMPORT File, Lookup, Close, Response;
FROM ASCII      IMPORT eol;
CONST
  EF         =    4C;
  EOL        =   36C;

TYPE
  Charclass =
    (none,letter,digit,chquote,strquote,eql, period,variant,lpar,rpar,lbrack,
     rbrack,lconbr,rconbr,latpar,ratpar,semicolon,colon,comma,endfile,
     endline,dollar,minus(*fr Simpl*),times,divide,plus,noteql,
     (*fuer Oberon*) arrow,and,not,atsign,attention);

VAR
  c:       CHAR;
  class:   ARRAY CHAR OF Charclass;      (*class OF input character*)
  queue:   ARRAY[0..10] OF CARDINAL;     (*attribute queue*)
  qf,ql:   CARDINAL;                     (*first pointer, last pointer*)
  unread:  BOOLEAN;
  obnName: ARRAY[0..19] OF CHAR;

(* EnQueue      Put attribute value into queue
---------------------------------------------------------------------*)
PROCEDURE EnQueue(w:CARDINAL);
BEGIN
  queue[ql]:=w; IF ql=10 THEN ql:=0; ELSE INC(ql); END;
  IF qf=ql THEN Restriction(4); END;
  END EnQueue;


(* DeQueue      Get attribute value from queue
--------------------------------------------------------------------*)
PROCEDURE DeQueue(VAR w:CARDINAL);
BEGIN
  IF qf=ql THEN CompErr(2); END;
  w:=queue[qf]; IF qf=10 THEN qf:=0; ELSE INC(qf); END;
  END DeQueue;


(* EmptyQueue      Is queue empty?
--------------------------------------------------------------------*)
PROCEDURE EmptyQueue():BOOLEAN;
BEGIN RETURN qf=ql; END EmptyQueue;

(* IO           Manages input file and output file
=====================================================================*)
MODULE IO;
IMPORT
  class, Close, CompErr, con, digit, EOL, EF, File, letter, src, Lookup,
  lst, Response, Restriction, SemErr, WriteCard,
  WriteString, WriteText, zeile, WriteLn, unread;
IMPORT FileIO;
EXPORT
  ch, col, IOInitListFile, line, NewPage, NextCh, print, WriteLineNumber,
  IOWriteLn;
CONST
  FF            = 14C;
  maxpagelength = 62;                                      (* 24.11.86 La *)
VAR
  ch:         CHAR;               (*current input character*)
  col:        CARDINAL;           (*start of current column*)
  firstpage:  BOOLEAN;            (*switch: TRUE if first page*)
  line:       CARDINAL;           (*current output line number*)
  fnlength:   CARDINAL;           (*length of list file name*)
  oldsrc:     File;               (*old source file*)
  page:       CARDINAL;           (*current page number*)
  pagelength: CARDINAL;           (*length of current page*)
  print:      BOOLEAN;
  lstn:       ARRAY[1..30] OF CHAR; (*list file name*)


(* NewPage      Skip to new page in list file
---------------------------------------------------------------------*)
PROCEDURE NewPage(VAR f:File);
VAR i: CARDINAL;
BEGIN
  IF firstpage THEN firstpage:=FALSE; ELSE FileIO.Write(f,FF); END;
  INC(page);
  WriteString(f,"COCO Compiler-Compiler Vs 3.1 (S)");      (* 24.11.86 La *)
  WriteText(f,"                                              ",
            36-fnlength);
  WriteText(f,lstn,fnlength); FileIO.Write(f,"/"); WriteCard(f,page,4);
  FileIO.Write(f,EOL); FileIO.Write(f,EOL); FileIO.Write(f,EOL);
  pagelength:=3;
  END NewPage;


(* NextCh          Get next input character (ch,col global)
---------------------------------------------------------------------*)
PROCEDURE NextCh;
VAR c1: CARDINAL;
BEGIN
  (*IF unread THEN
    unread := FALSE;
  ELSE*)
  IF TRUE THEN
    (*WriteString(con, '< ');*)
    FileIO.Read(src,ch);
    (*
    IF ((ch >= 'a') AND (ch <= 'z')) OR ((ch >= 'A') AND (ch <= 'Z')) THEN
      FileIO.Write(con, ch);
    ELSE
      WriteCard(con, CARDINAL(ORD(ch)), 3);
    END;
    WriteString(con, '> ');
    *)
    zeile[col] := ch;
    col:=col+1;
    zeile[col] := 0C;
    IF print AND (ch>=" ") THEN FileIO.Write(lst,ch); END;
  END;
END NextCh;


(* IOWriteLn      Skip to next line and write form feed if necessary
---------------------------------------------------------------------*)
PROCEDURE IOWriteLn(VAR f:File);
BEGIN
  FileIO.Write(f,EOL); INC(pagelength);
  IF pagelength>maxpagelength THEN NewPage(f); END;
  END IOWriteLn;


(* WriteLineNumber   Write next line number to file f
---------------------------------------------------------------------*)
PROCEDURE WriteLineNumber(VAR f:File);
CONST DEL = 177C;
VAR i: CARDINAL;
BEGIN
  INC(line); WriteCard(f,line,4); FileIO.Write(f," ");
  IF (line MOD 16)=0 THEN  (*update counter on screen*)
    (*IF line>16 THEN
      FOR i:=1 TO 5 DO FileIO.Write(con,DEL); END;
      END;*)
    WriteCard(con,line,5); WriteLn(con);
    END;
  END WriteLineNumber;

(* InitListFile     Initialize list file
---------------------------------------------------------------------*)
PROCEDURE IOInitListFile(VAR fn:ARRAY OF CHAR; p:BOOLEAN);
BEGIN
  fnlength:=0;
  WHILE fn[fnlength]<>0C DO
    lstn[fnlength+1]:=fn[fnlength]; INC(fnlength);
    END;
  print:= TRUE(*p*);
  NewPage(lst); WriteLineNumber(lst);
  END IOInitListFile;


BEGIN  (*IO*)
  col:=0; line:=0; page:=0; ch:=" ";
  pagelength:=0; firstpage:=TRUE;
  END IO;

PROCEDURE InitListFile(VAR fn:ARRAY OF CHAR; p:BOOLEAN);
BEGIN
  IOInitListFile(fn, p);
END InitListFile;

PROCEDURE WriteLn(VAR f:File);
BEGIN
  IOWriteLn(f);
END WriteLn;

(* NAMES        Manages lists for identifiers and strings
=====================================================================*)
MODULE NAMES;
IMPORT ch, Charclass, class, col, EOL, EF, SemErr, line, NextCh,
       Restriction, tokenClass;
EXPORT
  NAGetName, keys, ReadName, ReadString, NARestartHash, NAStopHash;
CONST
  idmax      = 4980;        (*max.length of identifier-list*)
  htmax      =  359;        (*max.length of hash table*)
VAR
  i:       CARDINAL;
  id:      ARRAY[1..idmax+20] OF CHAR;  (*identifiers*)
  idact:   CARDINAL;                    (*last element IN id*)
  keys:    CARDINAL;                    (*position OF last keyword IN id*)
  ht:      ARRAY[0..htmax] OF INTEGER;  (*hash table*)
  storeid: BOOLEAN;                     (*store id. permanently?*)


(* Hash         Hash an identifier and return its spix
---------------------------------------------------------------------*)
PROCEDURE Hash(idp:CARDINAL; VAR spix: CARDINAL);
VAR
  h,l:   CARDINAL;
  d:     INTEGER;

  PROCEDURE EqualId(x,y,l:CARDINAL):BOOLEAN;
  VAR i: CARDINAL;
  BEGIN
    i:=0;
    WHILE (i<l) AND (id[x+i]=id[y+i]) DO INC(i); END;
    RETURN i=l;
    END EqualId;

BEGIN
  l:=idp-idact; spix:=idact+1;
  h:=(CARDINAL(id[spix])*7 + CARDINAL(id[spix+1]) + l) * 17 MOD htmax;
  d:= -htmax;
  LOOP
    IF ht[h]=0 THEN                        (*new identifier*)
      IF storeid THEN ht[h]:=spix; idact:=idp; END;
      EXIT;
    ELSIF EqualId(ht[h],spix,l) THEN       (*old identifier*)
      spix:=ht[h]; EXIT;
    ELSE                                   (*collision*)
      d:=d+2;
      IF d=htmax THEN Restriction(1); END;      (*hash table full*)
      h:=(h+CARDINAL(ABS(d))) MOD htmax;
      END;
    END;    (*LOOP*)
  IF idp>idmax THEN Restriction(2); END;        (*identifier list full*)
  END Hash;


(* EnterKey       Enter a keyword to the identifier list
---------------------------------------------------------------------*)
PROCEDURE EnterKey(sy:CARDINAL; key:ARRAY OF CHAR);
VAR idp,i: CARDINAL;
BEGIN
  INC(idact); id[idact]:=CHR(sy); idp:=idact; (*store symbol number*)
  FOR i:=0 TO HIGH(key) DO                    (*store keyword*)
    INC(idp); id[idp]:=key[i];
    END;
  INC(idp); id[idp]:=0C;
  Hash(idp,keys);   (*keys contains the last keyword spix at any time*)
  END EnterKey;


(* GetName      Get the name of an identifier from the name list
---------------------------------------------------------------------*)
PROCEDURE NAGetName(spix:CARDINAL;VAR name:ARRAY OF CHAR;VAR l:CARDINAL);
VAR i: CARDINAL;
BEGIN
  i:=spix; l:=0;
  (* Damit das 0 Byte auch noch mitkommt *)
  LOOP
    name[l]:=id[i];
    IF id[i] = 0C THEN EXIT END;
    INC(l);
    IF l>CARDINAL(HIGH(name)) THEN EXIT END;
    INC(i);
  END;
  (*WHILE (id[i]<>0C) AND (l<=CARDINAL(HIGH(name))) DO
    name[l]:=id[i]; INC(i); INC(l);
    END;*)
  END NAGetName;


(* ReadName      Read identifier or keyword
---------------------------------------------------------------------*)
PROCEDURE ReadName(VAR klasse: tokenClass;
                   VAR val:CARDINAL);
VAR typ, spix,idp: CARDINAL;
BEGIN
  idp:=idact;
  WHILE (class[ch]=letter) OR (class[ch]=digit) DO
    INC(idp); id[idp]:=ch; NextCh;
    END;
  INC(idp); id[idp]:=0C;
  Hash(idp,spix);
  IF spix<=keys THEN
    (*keyword*)
    klasse := KeyWord;
    val:=ORD(id[spix-1]);
  ELSE (*identifier*)
    klasse := Ident;
    val:=spix;
  END;
END ReadName;


(* ReadString     Read and hash a string
---------------------------------------------------------------------*)
PROCEDURE ReadString(VAR spix:CARDINAL);
VAR
  och: CHAR;
  idp: CARDINAL;
BEGIN
  idp:=idact; och:=ch;
  INC(idp); id[idp]:=och; NextCh;  (*store quote*)
  LOOP
    IF ch=och THEN NextCh; EXIT;
    ELSIF ch=EF THEN SemErr(24,line,col); EXIT;
    ELSIF ch=EOL THEN SemErr(23,line,col); EXIT;
    ELSE INC(idp); id[idp]:=ch; NextCh; END;
    END;
  INC(idp); id[idp]:=och;          (*store quote*)
  INC(idp); id[idp]:=0C;
  Hash(idp,spix)
  END ReadString;


(* RestartHash  Causes identifiers to be stored permanently
---------------------------------------------------------------------*)
PROCEDURE NARestartHash;
BEGIN storeid:=TRUE; END NARestartHash;


(* StopHash    Causes identifiers to be stored temporarily
---------------------------------------------------------------------*)
PROCEDURE NAStopHash;
BEGIN storeid:=FALSE; END NAStopHash;

VAR o:CARDINAL;
BEGIN  (*NAMES*)
  o := 32;
  FOR i:=0 TO htmax-1 DO ht[i]:=0; END;
  idact:=0;
  storeid:=TRUE;
  (* Schlsselworte fr Simpl mit ihrem Wert in tokenClass *)
  EnterKey(o+ 0,'ARRAY');         EnterKey(o+ 1,'BEGIN');
  EnterKey(o+ 2,'CASE');          EnterKey(o+ 3,'CLOSE');
  EnterKey(o+ 4,'CONST');         EnterKey(o+ 5,'DEFINITION');
  EnterKey(o+ 6,'DIV');           EnterKey(o+ 7,'DO');
  EnterKey(o+ 8,'ELSE');          EnterKey(o+ 9,'ELSIF');
  EnterKey(o+10,'END');           EnterKey(o+11,'EXIT');
  EnterKey(o+12,'FORWARD');       EnterKey(o+13,'IF');
  EnterKey(o+14,'IMPLEMENTATION');EnterKey(o+15,'IMPORT');
  EnterKey(o+16,'IN');            EnterKey(o+17,'IS');
  EnterKey(o+18,'LOOP');          EnterKey(o+19,'MOD');
  EnterKey(o+20,'NIL');           EnterKey(o+21,'OF');
  EnterKey(o+22,'OR');            EnterKey(o+23,'POINTER');
  EnterKey(o+24,'PROCEDURE');     EnterKey(o+25,'RECORD');
  EnterKey(o+26,'REPEAT');        EnterKey(o+27,'RETURN');
  EnterKey(o+28,'THEN');          EnterKey(o+29,'TO');
  EnterKey(o+30,'TYPE');          EnterKey(o+31,'UNTIL');
  EnterKey(o+32,'VAR');           EnterKey(o+33,'WHILE');
  EnterKey(o+34,'WITH');
  storeid := FALSE; (* nur die Schlsselworte kommen fest rein *)
  END NAMES;

PROCEDURE GetName(spix:CARDINAL;VAR name:ARRAY OF CHAR;VAR l:CARDINAL);
BEGIN NAGetName(spix, name, l); END GetName;

PROCEDURE RestartHash;
BEGIN NARestartHash; END RestartHash;

PROCEDURE StopHash;
BEGIN NAStopHash; END StopHash;



(* ReadNumber    Read and convert cardinal constant
---------------------------------------------------------------------*)
PROCEDURE ReadNumber(VAR val:CARDINAL);
BEGIN
  val:=0;
  WHILE class[ch]=digit DO
    IF (val>6553) OR ( (val=6553) AND (ch>'5') )
      THEN
        SemErr(22,line,col);
        WHILE class[ch]=digit DO NextCh; END;
      ELSE
        val:=10*val+(CARDINAL(ch)-CARDINAL('0'));
        NextCh;
        END;
    END;
  END ReadNumber;


(* GetSy     get next lexical symbol
---------------------------------------------------------------------*)
PROCEDURE GetSy(VAR token:CARDINAL);
VAR column,at,l: CARDINAL;
    typ: tokenClass;
    str: ARRAY[0..79] OF CHAR;
    fertig: BOOLEAN;
BEGIN
  (*WriteString(con, 'getsym');*)
  IF NOT EmptyQueue()
    THEN DeQueue(token);
    ELSE
      REPEAT
        WHILE ch=' ' DO NextCh; END;
        column:=col;
        fertig := TRUE;
        CASE class[ch] OF
          none:      (*typ:=nococosy; EnQueue(ORD(ch));*) NextCh;
        | letter:    ReadName(typ,at);
                     IF typ=Ident THEN EnQueue(at); END;
        | digit:     ReadNumber(at); typ:=Number; EnQueue(at);
        (*| chquote:   NextCh;
                     IF ch = '\' THEN
                       NextCh;
                       CASE ch OF
                         'n': ch := eol;
                         ELSE
                       END;
                     END;
                     EnQueue(CARDINAL(ORD(ch))); NextCh; NextCh;
                     typ := Character;*)
        | chquote,
          strquote:  ReadString(at); EnQueue(at);
                     typ:=String;
        | eql:       (*=*) typ:=Equal; NextCh;
        | period:    (*.*) NextCh;
                     IF class[ch] = period
                     THEN typ := DotDot; NextCh;
                     ELSE typ := Dot; END;
        | variant:   typ:=Variant; NextCh;
        | lpar:      (*( *) NextCh;
                     IF class[ch] # times THEN
                       (*unread := TRUE;*)
                       typ:=Lparen;
                     ELSE
                       (* Kommentar berlesen, nicht geschachtelt *)
                       LOOP
                         NextCh;
                         IF class[ch] = times THEN
                           NextCh;
                           IF class[ch] = rpar THEN
                             fertig := FALSE;
                             NextCh;
                             EXIT;
                           END;
                         END;
                       END;
                     END;
        | rpar:      (*)*) typ:=Rparen; NextCh;
        | lbrack:    (*[*) typ:=Lbracket; NextCh;
        | rbrack:    (*]*) typ:=Rbracket; NextCh;
        | lconbr:    (*{*) typ:=Lset; NextCh;
        | rconbr:    (*}*) typ:=Rset; NextCh;
        | latpar:    (*<*); NextCh;
                     IF class[ch] = eql
                     THEN typ := LessEqual; NextCh;
                     ELSE typ := Less; END;
        | ratpar:    (*>*) NextCh;
                     IF class[ch] = eql
                     THEN typ := GreaterEqual; NextCh;
                     ELSE typ := Greater; END;
        | semicolon: (*;*)typ:=SemiColon; NextCh;
        | colon:     (*:*) NextCh;
                     IF class[ch] = eql
                     THEN typ := Assignment; NextCh;
                     ELSE typ := Colon; END;
        | comma:     typ:=Comma;  NextCh;
        | endfile:   typ:=EndOfInput;
        | endline:   (*WriteLn(lst);
                     WriteLineNumber(lst);*)
                     typ:=EndOfLine;
                     col:=0; NextCh;
        | minus:     (*-*)typ:=Minus; NextCh;
        | times:     (***)typ:=Times; NextCh;
        | divide:    (*/*)typ:=Divide; NextCh;
        | plus:      (*+*)typ:=Plus; NextCh;
        | noteql:    (*#*)typ:=NotEqual; NextCh;
        (* zustzliche Dinge fr Oberon *)
        | arrow:     (*^*) typ:=Arrow; NextCh;
        | and:       (*&*) typ:=And; NextCh;
        | not:       (*~*) typ:=Not; NextCh;
        | attention: (*!*) typ:=Attention; NextCh;
        | atsign:    (*@*) typ:=At;  NextCh;
        ELSE
        END; (*CASE*)
        UNTIL fertig;
      IF typ = EndOfLine THEN
        token:=256*67+column;
      ELSIF typ = KeyWord THEN
        token:=256*at+column;
      ELSE
        token:=256*CARDINAL(ORD(typ))+column;
      END;
    END;
    (*InOut.WriteCard(token DIV 256, 5);*)
  END GetSy;

PROCEDURE CLOSE(); BEGIN Close(src); END CLOSE;

BEGIN (*cocolex*)
  zeile[0] := 0C;
  FOR c:=0C TO 377C DO class[c]:=none; END;
  FOR c:='a' TO 'z' DO class[c]:=letter; END;
  FOR c:='A' TO 'Z' DO class[c]:=letter; END;
  FOR c:='0' TO '9' DO class[c]:=digit; END;
  class[EF]:=endfile;    class[EOL]:=endline;
  class["'"]:=chquote;   class['$']:=dollar;   class['"']:=strquote;
  class['(']:=lpar;      class[')']:=rpar;     class[',']:=comma;
  class['-']:=minus;     class['.']:=period;   class[':']:=colon;
  class[';']:=semicolon; class['<']:=latpar;   class['=']:=eql;
  class['>']:=ratpar;    class['[']:=lbrack;   class[']']:=rbrack;
  class['{']:=lconbr;    class['|']:=variant;  class['}']:=rconbr;

  class['_']:=letter;                                      (* 24.11.86 La *)
  (* Ergnzungen fr Simpl *)
  class['*']:=times;    class['/']:=divide;
  class['+']:=plus;     class['#']:=noteql;
  (* Ergnzungen fr Oberon *)
  class['^']:=arrow;     class['&']:=and;      class['~']:=not;
  class['@']:=atsign;
  qf:=0; ql:=0;
  (*unread:= FALSE;*)
  InOut.WriteString('obn: ');
  InOut.ReadString(obnName);
  Lookup(src, obnName, 1024, FALSE);
  Assert(src.res=done, ADR('Datei nicht gefunden'));
  TermProcedure(CLOSE);
  END OberonLex.
