(* ------------------------------------------------------------------------
  :Program.     BeckerANSI.mod
  :Author.      Steffen Khler
  :Address.     Apenrader Str. 30  2870 Delmenhorst
  :Phone.       04221/23155
  :Copyright.   FreeWare
  :Language.    Modula-2
  :Translator.  M2Amiga 3.3d
  :History.     v1.0 Steffen Khler ??-Apr-90 initial
  :History.     v1.1 Kai Bolay      06-May-90 Complete Testing Deseaster
  :History.     v1.2 Steffen Khler 26-May-90 All known Bugs fixes
  :Imports.     IconSupport ([fbs] AMOK#5)
  :Contents.    Wandelt eine BeckerText- Textdatei in ASCII-Text mit
  :Contents.    ANSI- Steuersequenzen
------------------------------------------------------------------------ *)
MODULE BeckerANSI;

  FROM FileSystem  IMPORT Lookup,Response,Close,ReadChar,WriteChar,File;
  FROM Arguments   IMPORT GetArg,NumArgs;
  FROM Terminal    IMPORT WriteString,WriteLn;
  FROM Workbench   IMPORT WBObjectType;
  FROM Strings     IMPORT Length,Delete,Occurs,Compare,Insert;
  FROM SYSTEM      IMPORT ADR,CAST,BITSET;
  FROM Intuition   IMPORT Image;
  FROM IconSupport IMPORT CreateIcon;
  FROM Arts        IMPORT wbStarted;

  TYPE
    Style = SET OF (unterstrichen,fett,kursiv,hoch,tief,rot,Bit6,Bit7);

  VAR
    Zeichen    : CHAR;
    Argumente  : INTEGER;
    Input,
    Output     : File;
    Inputname,
    Outputname : ARRAY [0..78] OF CHAR;
    Buffer1,
    Buffer2    : CARDINAL;
    Laenge     : INTEGER;
    IconOK     : BOOLEAN;
    Icon       : Image;
    InLaenge,
    OutLaenge,
    LinkerRand : INTEGER;
    InZeile,
    OutZeile   : ARRAY [1..600] OF CHAR;
    OldStyle,
    NewStyle   : Style;
    Stil,
    Farbe,
    OldFarbe   : ARRAY [1..3] OF CHAR;

(* Image-Ed Image generation *)

VAR SKIconPalette : ARRAY[0..3] OF CARDINAL;
   SKIconImageData : ARRAY[0..228] OF CARDINAL;
   SKIconImage : Image;

PROCEDURE WriteEnd( VAR Output :File);

  BEGIN
    WriteChar(Output,CHR(0));
  END WriteEnd;


PROCEDURE ConvertLine;

  VAR
  InCounter,
  OutCounter,
  ZeilenLaenge,
  LineStyle,
  Counter    : INTEGER;
  a        : CHAR;


  PROCEDURE WriteOut(a:INTEGER);
    BEGIN
      OutZeile[OutCounter]:=CHR(a);
      INC(OutCounter);
    END WriteOut;


  PROCEDURE WriteStringOut(Code:ARRAY OF CHAR);

    VAR
      Counter : INTEGER;

    BEGIN
      Counter:=0;
      IF Code[Counter] # CHR(0) THEN
        WHILE Counter <= Length(Code)-1 DO
          WriteOut(ORD(Code[Counter]));
          INC(Counter);
        END;
      END;
    END WriteStringOut;


   PROCEDURE NormalStyle();
     BEGIN
      WriteOut(27);
      WriteOut(91);
      WriteStringOut('0');
      WriteOut(59);
      WriteStringOut('31');
      WriteOut(109);
      Farbe:='31';
      OldFarbe:='31';
     END NormalStyle;


   PROCEDURE CreateANSI();
     BEGIN
      IF NewStyle <> Style{} THEN
        IF (unterstrichen IN NewStyle) THEN
          WriteOut(27);
          WriteOut(91);
          WriteStringOut('4');
          WriteOut(109);
        END;
        IF (fett IN NewStyle) THEN
          WriteOut(27);
          WriteOut(91);
          WriteStringOut('1');
          WriteOut(109);
        END;
        IF kursiv IN NewStyle THEN
          WriteOut(27);
          WriteOut(91);
          WriteStringOut('3');
          WriteOut(109);
        END;
        IF rot IN NewStyle THEN
          WriteOut(27);
          WriteOut(91);
          WriteStringOut('7');
          WriteOut(109);
        END;
        IF hoch IN NewStyle THEN
          Farbe:='32';
        END;
        IF tief IN NewStyle THEN
          Farbe:='33';
        END;
        OldStyle:=NewStyle;
       END;
      IF Compare(OldFarbe,1,3,Farbe,TRUE)<>0 THEN
        WriteOut(27);
        WriteOut(91);
        WriteStringOut(Farbe);
        WriteOut(109);
        OldFarbe:=Farbe;
      END;
     END CreateANSI;


  BEGIN
    OutCounter:=1;
    InCounter:=7;
    IF ((LinkerRand>0) OR (ORD(InZeile[2])>0)) THEN
      NormalStyle();
      IF LinkerRand>0 THEN
        FOR Counter := 1 TO LinkerRand DO
          WriteOut(32);
        END;
      END;
      IF ORD(InZeile[2])>0 THEN
        FOR Counter := 1 TO ORD(InZeile[2]) DO
          WriteOut(32);
        END;
      END;
      CreateANSI();
    END;
    WHILE InCounter<=InLaenge DO
      IF ORD(InZeile[InCounter])=127 THEN
        INC(InCounter);
        NewStyle:=CAST(Style,(InZeile[InCounter]));
        INC(InCounter);
        NormalStyle();
        CreateANSI();
      END;
      CASE ORD(InZeile[InCounter]) OF
        13 :  INC(InCounter)   |
        0 : INC(InCounter);  |
        1 : WriteOut(32);
            INC(InCounter); |
        4 : INC(InCounter);
            IF ORD(InZeile[InCounter])=1 THEN
            WriteOut(ORD('-'));
            INC(InCounter);
            END;  |
      ELSE
        a:=InZeile[InCounter];
        WriteOut(ORD(a));
        INC(InCounter);
      END;
    END;
    OutLaenge:=OutCounter-1;
    IF InZeile[InLaenge-1]<>CHR(10) THEN
      OutZeile[OutLaenge]:=CHR(10);
    ELSE
      OldStyle:=Style{};
    END;
  END ConvertLine;


PROCEDURE Beckertext(VAR Input:File):BOOLEAN;

  BEGIN
    ReadChar(Input,Zeichen);
    RETURN (ORD(Zeichen)=1);
  END Beckertext;

PROCEDURE ReadLine(VAR Input:File): BOOLEAN;

  VAR
    a:CHAR;

  BEGIN
    InLaenge:=0;
    a:=CHR(0);
    WHILE NOT (Input.eof) AND ((ORD(a)#10) OR (InLaenge<7)) AND (InLaenge#199) DO
      INC(InLaenge);
      ReadChar(Input,a);
      InZeile[InLaenge]:=a;
    END;
    RETURN(ORD(a)=10);
  END ReadLine;

PROCEDURE WriteLine(VAR OutPut:File);

  VAR
    Counter:INTEGER;

  BEGIN
    Counter:=0;
    WHILE Counter<OutLaenge DO
      INC(Counter);
      WriteChar(Output,OutZeile[Counter]);
    END;
  END WriteLine;

PROCEDURE ReadHeader(VAR Input:File):BOOLEAN;

  VAR
    a      : CHAR;
    Counter,
    Laenge   : INTEGER;


PROCEDURE HeaderLaenge( Input:File):INTEGER;
       (* Das Input- File darf nicht ! als VAR- Parameter bergeben werden *)
  VAR
    a:CHAR;
    Counter:INTEGER;

  BEGIN
    Counter:=0;
    WHILE ((Counter<326766) AND (ORD(a)<>127) AND NOT(Input.eof)) DO
      INC(Counter);
      ReadChar(Input,a);
      CASE Counter OF
        5 :  LinkerRand:=ORD(a); |
        ELSE
        END;
    END;
    IF ORD(a)=127 THEN
      RETURN (Counter-7);
    ELSE
         RETURN(0);
    END;
  END HeaderLaenge;

  BEGIN
      Laenge:=HeaderLaenge(Input);
      IF Laenge= 0 THEN
        RETURN (FALSE);
      ELSE
        FOR Counter:= 1 TO Laenge DO
          ReadChar(Input,a);
        END;
      RETURN(TRUE);
      END;
  END ReadHeader;


BEGIN
   (* Image Palette *)
   SKIconPalette[0] := 000H;
   SKIconPalette[1] := 0ECAH;
   SKIconPalette[2] := 0C00H;
   SKIconPalette[3] := 0F60H;


   (* Image Data *)

   (* Plane 0 *)
   SKIconImageData[0] := 0000H;
   SKIconImageData[1] := 0000H;
   SKIconImageData[2] := 0000H;
   SKIconImageData[3] := 0000H;
   SKIconImageData[4] := 0000H;
   SKIconImageData[5] := 0000H;
   SKIconImageData[6] := 0000H;
   SKIconImageData[7] := 0000H;
   SKIconImageData[8] := 0000H;
   SKIconImageData[9] := 0000H;
   SKIconImageData[10] := 0000H;
   SKIconImageData[11] := 0000H;
   SKIconImageData[12] := 0000H;
   SKIconImageData[13] := 0000H;
   SKIconImageData[14] := 0000H;
   SKIconImageData[15] := 0000H;
   SKIconImageData[16] := 0000H;
   SKIconImageData[17] := 0000H;
   SKIconImageData[18] := 0000H;
   SKIconImageData[19] := 0000H;
   SKIconImageData[20] := 0000H;
   SKIconImageData[21] := 0000H;
   SKIconImageData[22] := 0000H;
   SKIconImageData[23] := 0000H;
   SKIconImageData[24] := 0000H;
   SKIconImageData[25] := 0000H;
   SKIconImageData[26] := 0000H;
   SKIconImageData[27] := 0000H;
   SKIconImageData[28] := 0000H;
   SKIconImageData[29] := 0000H;
   SKIconImageData[30] := 0000H;
   SKIconImageData[31] := 0000H;
   SKIconImageData[32] := 0000H;
   SKIconImageData[33] := 0000H;
   SKIconImageData[34] := 0000H;
   SKIconImageData[35] := 0000H;
   SKIconImageData[36] := 0000H;
   SKIconImageData[37] := 0000H;
   SKIconImageData[38] := 0000H;
   SKIconImageData[39] := 0000H;
   SKIconImageData[40] := 0000H;
   SKIconImageData[41] := 0000H;
   SKIconImageData[42] := 0000H;
   SKIconImageData[43] := 0000H;
   SKIconImageData[44] := 0000H;
   SKIconImageData[45] := 0000H;
   SKIconImageData[46] := 0000H;
   SKIconImageData[47] := 0000H;
   SKIconImageData[48] := 0000H;
   SKIconImageData[49] := 0000H;
   SKIconImageData[50] := 0000H;
   SKIconImageData[51] := 0000H;
   SKIconImageData[52] := 0000H;
   SKIconImageData[53] := 0000H;
   SKIconImageData[54] := 0000H;
   SKIconImageData[55] := 0000H;
   SKIconImageData[56] := 0000H;

   (* Plane 1 *)
   SKIconImageData[57] := 0003H;
   SKIconImageData[58] := 0F800H;
   SKIconImageData[59] := 0000H;
   SKIconImageData[60] := 007EH;
   SKIconImageData[61] := 0FC0H;
   SKIconImageData[62] := 0000H;
   SKIconImageData[63] := 03C0H;
   SKIconImageData[64] := 0070H;
   SKIconImageData[65] := 0000H;
   SKIconImageData[66] := 071FH;
   SKIconImageData[67] := 0C01CH;
   SKIconImageData[68] := 0000H;
   SKIconImageData[69] := 1C70H;
   SKIconImageData[70] := 4077H;
   SKIconImageData[71] := 0000H;
   SKIconImageData[72] := 30C0H;
   SKIconImageData[73] := 4ED1H;
   SKIconImageData[74] := 8000H;
   SKIconImageData[75] := 7189H;
   SKIconImageData[76] := 0B180H;
   SKIconImageData[77] := 0C000H;
   SKIconImageData[78] := 6180H;
   SKIconImageData[79] := 6300H;
   SKIconImageData[80] := 0C000H;
   SKIconImageData[81] := 0C0E0H;
   SKIconImageData[82] := 4600H;
   SKIconImageData[83] := 6000H;
   SKIconImageData[84] := 0C030H;
   SKIconImageData[85] := 0CC00H;
   SKIconImageData[86] := 6000H;
   SKIconImageData[87] := 0C658H;
   SKIconImageData[88] := 0F000H;
   SKIconImageData[89] := 6000H;
   SKIconImageData[90] := 448DH;
   SKIconImageData[91] := 98E0H;
   SKIconImageData[92] := 4000H;
   SKIconImageData[93] := 67B9H;
   SKIconImageData[94] := 9920H;
   SKIconImageData[95] := 0C000H;
   SKIconImageData[96] := 3103H;
   SKIconImageData[97] := 18C1H;
   SKIconImageData[98] := 8000H;
   SKIconImageData[99] := 1DCCH;
   SKIconImageData[100] := 0F07H;
   SKIconImageData[101] := 0000H;
   SKIconImageData[102] := 0700H;
   SKIconImageData[103] := 000CH;
   SKIconImageData[104] := 0000H;
   SKIconImageData[105] := 01C0H;
   SKIconImageData[106] := 00F0H;
   SKIconImageData[107] := 0000H;
   SKIconImageData[108] := 003FH;
   SKIconImageData[109] := 0FF80H;
   SKIconImageData[110] := 0000H;
   SKIconImageData[111] := 0000H;
   SKIconImageData[112] := 0000H;
   SKIconImageData[113] := 0000H;

   (* End of Image data *)


  Buffer1:=20000;
  Buffer2:=20000;
  OldStyle:=Style{};
  Argumente:=NumArgs();
  IF Argumente=1 THEN
    GetArg(1,Inputname,Laenge);
    Outputname:=Inputname;
    IF Occurs(Outputname,0,'.TXT',FALSE)<>-1 THEN
      Delete(Outputname,Occurs(Outputname,0,'.TXT',FALSE),4);
    ELSE
      Insert(Outputname,Length(Outputname),'.ANSI');
    END;
  END;
      IF (Argumente=2) AND (NOT wbStarted) THEN
    GetArg(1,Inputname,Laenge);
    GetArg(2,Outputname,Laenge);
  END;

  IF (Argumente=1) OR ((Argumente=2)AND (NOT wbStarted)) THEN
    Lookup(Input,Inputname,Buffer1,FALSE);
    IF Input.res=done THEN
      IF NOT Beckertext(Input) THEN
        WriteString(Inputname);
        WriteString(" : Kein BECKERText- Format. Konvertierung abgebrochen.. ");
        WriteLn;
      ELSE
        WriteString(Inputname);
        WriteString(" :als BeckerText erkannt");
        WriteLn;
        IF ReadHeader(Input) THEN
          Lookup(Output,Outputname,Buffer2,TRUE);
          IF Output.res=done THEN
            OldStyle:=Style{};
            OldFarbe:='31';
            Farbe:='31';
            WHILE ReadLine(Input) DO
              WriteString (".");
              ConvertLine;
              WriteLine(Output);
            END;
            WriteLn;
            WITH SKIconImage DO
              leftEdge := 0; topEdge := 0;
              width := 36; height := 19; depth := 2;
              imageData := ADR(SKIconImageData);
              planePick := 3H; planeOnOff := 0H;
              nextImage := NIL;
            END;

            (* End of Image-Ed Image generation *)
            IconOK:=CreateIcon(ADR(Outputname),ADR(SKIconImage),NIL,TRUE,
                       project,ADR(":c/MuchMore"));
            IF NOT IconOK THEN
              WriteString('Konnte leider kein ICON erstellen ?');
              WriteLn;
            END;
            Close(Output);
            WriteString("konvertierter Text gespeichert als: ");
            WriteString(Outputname);
            WriteLn;
          ELSE
            WriteString('AusgabeDatei lie sich nicht ffnen');
            WriteLn;
          END;
        ELSE
          WriteString('Dateiformat- Fehler');
          WriteLn;
          Close(Input);
        END;
      END;
    ELSE
      WriteString('Eingabedatei lie sich nicht ffnen');
      WriteLn;
      Close(Input);
    END;
  ELSE
    WriteString('                         Becker-ANSI  1.1  by Steffen Khler');
    WriteLn;
    WriteString('                   Konvertiert Beckertext-Dateien ins ANSI- Format');
    WriteLn;
    WriteLn;
    WriteString('   Aufruf : ber die Workbench durch Anklicken des TextIcons uund Doppelklick');
    WriteLn;
    WriteString('                         auf das Programm- Icon   oder');
    WriteLn;
    WriteString('              im CLI durch "  BeckerANSI Quelldatei <Zieldatei>"');
    WriteLn;
    WriteString('      wobei die Zieldatei, wenn nicht angegeben, auf Quelle - >.TXT<');
    WriteLn;
    WriteString('                               eingestellt ist.');
    WriteLn;
  END;
END BeckerANSI.


