(* ------------------------------------------------------------------------
  :Program.       Find
  :Contents.      CLI-Utility to search for a file
  :Author.        Kai Bolay [kai]
  :Address.       Hoffmannstrae 168
  :Address.       7250 Leonberg
  :Phone.         07152/22135
  :History.       v1.0 [kai] 16-Jun-90 initial
  :History.       v1.1 [kai] 07-Jul-90 recompiled => Pure !!!
  :Copyright.     Public Domain
  :Language.      OBERON
  :Translator.    OBERON v1.14
  :Imports.       DosHelp [kai], PatMatch [Bernd Preusing]
  :Remark.        Better than "LookFor", "WhereIs", "Dir >Datei OPT A"
  :Remark.        Compile/Link with -md to be reentrant!
------------------------------------------------------------------------ *)
MODULE Find;

IMPORT d: Dos,
       sd: SecureDos,
       io: io,
       a: Arguments,
       dh : DosHelp,
       p : PatMatch,
       Break,
       sys: SYSTEM;

CONST usage    = "Usage: Find [Dir]Pattern\n";
      oom      = "\x07Out of Memory!\n";
      lockErr  = "\x07Error: Can't Lock()!\n";
      exErr    = "\x07Error. Can't Examine()!\n";

VAR dir, pattern : ARRAY 80 OF CHAR;
    Error        : BOOLEAN;
    FLock        : d.FileLockPtr;
    Err          : LONGINT;
    found        : INTEGER;
    PathName     : ARRAY 200 OF CHAR;
    aux          : ARRAY 200 OF SHORTINT;


PROCEDURE LookDir (Dir : ARRAY OF CHAR);

VAR DirLock,
    OldDir : d.FileLockPtr;
    FIB    : d.FileInfoBlockPtr;

BEGIN
   NEW (FIB);
   IF FIB # NIL THEN
      DirLock := sd.Lock (sys.ADR (Dir), d.sharedLock);
      IF DirLock # NIL THEN
         OldDir := d.CurrentDir (DirLock);
         IF NOT (d.Examine (DirLock, FIB)) THEN
            Error := TRUE;
            io.WriteString (exErr);
         ELSE
            LOOP
               IF d.ExNext (DirLock, FIB) THEN END;
               Err := d.IoErr ();
               IF Err = d.noMoreEntries THEN EXIT END;
               IF Err # 0 THEN
                  Error := TRUE;
                  io.WriteString ("\x07IO-Error:"); io.WriteInt (Err, 5); io.WriteLn;
                  EXIT;
               END; (* IF *)
               IF p.Match (pattern, aux, FIB.fileName) THEN
                  FLock := d.Lock (sys.ADR (FIB.fileName), d.sharedLock);
                  IF FLock # NIL THEN
                     dh.GetPathFromLock (PathName, FLock);
                     d.UnLock (FLock);
                     io.WriteString (PathName); io.WriteLn;
                     INC (found);
                  ELSE
                     Error := TRUE;
                     io.WriteString (lockErr);
                  END; (* IF *)
               END; (* IF *)
               IF FIB.dirEntryType > 0 THEN LookDir (FIB.fileName) END;
            END; (* LOOP *)
         END; (* IF *)
         OldDir := d.CurrentDir (OldDir);
         sd.UnLock (DirLock);
      ELSE
         Error := TRUE;
         io.WriteString (lockErr);
      END; (* IF *)
      DISPOSE (FIB);
   ELSE
      Error := TRUE;
      io.WriteString (oom);
   END; (* IF *)
END LookDir;


BEGIN
   IF a.NumArgs() # 1 THEN
      io.WriteString (usage); HALT (20);
   END; (* IF *)
   a.GetArg (1, pattern);
   IF (pattern[0] = "?") AND (pattern[1] = 0X) THEN
      io.WriteString (usage); HALT (0);
   END; (* IF *)
   dh.SplitPath (pattern, dir, pattern);
   IF NOT (p.CmplPat (pattern, aux)) THEN
      io.WriteString ("\x07Syntax error in pattern!\n"); HALT (20);
   END; (* IF *)
   found := 0;
   Error := FALSE;
   LookDir (dir);
   IF NOT (Error) THEN
      IF found = 0 THEN
         io.WriteString ("No files matched!\n");
      ELSE
         io.WriteString ("Matched"); io.WriteInt (found, 4);
         io.WriteString (" files!\n");
      END; (* IF *)
   END; (* IF *)
END Find.
