(*--------------------------------------------------------------------------
 :Program.    DatenInOut.mod
 :Author.     Marcus Stratmann
 :Address.    Bttcherstrae 13, D-4250 Bottrop-Kirchhellen
 :Phone.      02045/5191
 :Copyright.  Public Domain
 :Language.   MODULA-II
 :Translator. M2Amiga v3.3d
 :Imports.    DiskInOut, typdefs, IntuiOps, TabellenOps;
 :History.    V1.0,  26-Sep-89
 :History.    V1.1,  16-May-90
 :Contents.   Manages the input and output of the used data
--------------------------------------------------------------------------*)

IMPLEMENTATION MODULE DatenInOut;  (* $S- $R- $V- $F- *)

FROM Arts        IMPORT TermProcedure;
FROM Conversions IMPORT ValToStr, StrToVal;
FROM DiskInOut   IMPORT SetInput, Done, ReadInt, ReadLn, CloseInput,
                        SetOutput, WriteInt, WriteString, WriteLn,
                        CloseOutput;
FROM Dos         IMPORT DeleteFile, Rename;
FROM IntuiOps    IMPORT Fenster, OeffneFenster, SchliesseFenster, WriteWin,
                        WriteZahl, StringEingabe, MyRequest,
                        InitVereinGadgets, WaehleZahl, KeineDaten,
                        MacheTitelzeile, Beep;
FROM Strings     IMPORT Insert, Length, last;
FROM SYSTEM      IMPORT ADR, ADDRESS;
FROM TabellenOps IMPORT BerechneTabellen;
FROM typdefs     IMPORT MaxMan, AnzMan, Daten, Namen, DatenAusgewertet,
                        FileName, TabellenBerechnet, DatenGesichert,
                        StatistikBerechnet, ArbeitsFile, Backup, NamenTyp,
                        Tabellen, Sortiert;

CONST
  DatenLadenStr = "Daten laden";
  DatenSpeichernStr = "Daten speichern";
  NeueSaisonStr = "Neue Saison";

VAR
  Eingabe: FileName;


PROCEDURE SicherheitsTest(Titel: ADDRESS);

  BEGIN
    IF NOT DatenGesichert THEN
      IF MyRequest(Titel, "Die alten Daten sind noch nicht gespeichert!",
                   "Speichern", "Egal", TRUE, FALSE) THEN
        Speichern;
      END;
    END;
  END SicherheitsTest;


PROCEDURE InitDaten;

  VAR
    i, j, k: CARDINAL;

  BEGIN
    FOR i := 1 TO MaxMan DO
      FOR j := 0 TO 1 DO
        FOR k := 1 TO MaxMan DO
          Daten[i,j,k] := -1;
        END;
      END;
      Namen[i] := "Mannschaft  ";
      Namen[i,11] := CHR(64 + i);
    END;
    DatenGesichert := TRUE;
    DatenAusgewertet := FALSE;
    TabellenBerechnet := FALSE;
    StatistikBerechnet := FALSE;
    ArbeitsFile := "";
  END InitDaten;


PROCEDURE NeueSaison;

  VAR
    TempNamen: NamenTyp;
    EingabeFenster, AusgabeFenster, AnzahlFenster: Fenster;
    antw, zeile: CARDINAL;
    temp, i: INTEGER;
    out: ARRAY[0..20] OF CHAR;
    err: BOOLEAN;

  BEGIN
    EingabeFenster := NIL;
    AusgabeFenster := NIL;
    AnzahlFenster := NIL;
    AusgabeFenster := OeffneFenster(206, 20, 228, 28, ADR("Neue Saison"));
    WriteWin(AusgabeFenster, 10, 20, "Wie viele Vereine (2-  ) ?");
    temp := MaxMan;
    WriteZahl(AusgabeFenster, temp, 2, 178, 20);
    REPEAT
      temp := WaehleZahl(AnzahlFenster, TRUE);
      IF (temp = 1) OR (temp > MaxMan) THEN
        Beep;
      END;
    UNTIL (temp < 1) OR ((temp > 1) AND (temp <= MaxMan));
    SchliesseFenster(AusgabeFenster);
    SchliesseFenster(AnzahlFenster);
    IF temp < 1 THEN
      RETURN;
    END;
    AusgabeFenster := OeffneFenster(20, 20, 220, 8 * temp + 20,
                                    ADR("Vereinsnamen"));
    TempNamen := Namen;
    i := 1;
    zeile := 20;
    REPEAT
      out := "";
      ValToStr(i, FALSE, out, 10, 2, " ", err);
      Insert(out, last, ". Vereinsname:");
      antw := StringEingabe(EingabeFenster, 250, 80, ADR(NeueSaisonStr),
                            out, TempNamen[i], TRUE, " OK ", "Abbruch");
      SchliesseFenster(EingabeFenster);
      WriteWin(AusgabeFenster, 10, zeile, TempNamen[i]);
      INC(i);
      INC(zeile, 8);
    UNTIL (i = temp + 1) OR (antw = 2);
    IF antw # 2 THEN
      IF MyRequest(ADR(NeueSaisonStr),
                   "Mit diesen Vereinen neue Saison starten?", " Ja ",
                   "Nein", FALSE, TRUE) THEN
        SicherheitsTest(ADR(NeueSaisonStr));
        AnzMan := temp;
        InitDaten;
        Namen := TempNamen;
        DatenGesichert := FALSE;
        InitVereinGadgets;
        MacheTitelzeile;
      END;
    END;
    SchliesseFenster(AusgabeFenster);
  END NeueSaison;


PROCEDURE LadeDaten(): BOOLEAN;

  VAR
    i, j, k: CARDINAL;
    len: INTEGER;
    win: Fenster;

  BEGIN
    win := NIL;
    SetInput(ArbeitsFile);
    IF NOT Done THEN
      RETURN FALSE;
    END;
    ReadInt(len);
    IF (len < 2) OR (len > MaxMan) THEN
      RETURN FALSE;
    END;
    AnzMan := len;
    k := (640 - (AnzMan + 2) * 9) DIV 2;
    win := OeffneFenster(k, 118, (AnzMan + 2) * 9, 20,
                         ADR(DatenLadenStr));
    FOR i := 1 TO AnzMan DO
      IF win # NIL THEN
        WriteWin(win, 9*i, 17, "*");
      END;
      ReadLn(Namen[i], len);
      IF NOT Done THEN
        CloseInput;
        SchliesseFenster(win);
        RETURN FALSE;
      END;
      Namen[i,25] := CHR(0);
      FOR j := 0 TO 1 DO
        FOR k := 1 TO AnzMan DO
          IF k # i THEN
            ReadInt(Daten[i,j,k]);
            IF NOT Done THEN
              CloseInput;
              SchliesseFenster(win);
              RETURN FALSE;
            END;
          ELSE
            Daten[i,j,k] := -1;
          END;
        END;
      END;
    END;
    CloseInput;
    SchliesseFenster(win);
    DatenAusgewertet := FALSE;
    TabellenBerechnet := FALSE;
    DatenGesichert := TRUE;
    StatistikBerechnet := FALSE;
    Eingabe := ArbeitsFile;
    RETURN TRUE;
  END LadeDaten;

PROCEDURE SpeicherDaten(): BOOLEAN;

  VAR
    i, j, k, m, n, len: INTEGER;
    win: Fenster;
    BackupName: FileName;
    Abbruch, gefunden, Erfolg: BOOLEAN;

  BEGIN
    win := NIL;
    IF Backup THEN
      BackupName := ArbeitsFile;
      len := Length(BackupName);
      i := len - 1;
      gefunden := FALSE;
      Abbruch := FALSE;
      REPEAT
        IF BackupName[i] = "." THEN
          gefunden := TRUE;
        ELSIF (BackupName[i] = "/") OR (BackupName[i] = ":") THEN
          Abbruch := TRUE;
        ELSE
          DEC(i);
        END;
      UNTIL Abbruch OR gefunden OR (i = -1);
      IF Abbruch THEN
        i := len;
      END;
      Insert(BackupName, i, ".BAK");
      BackupName[i+4] := CHR(0);
      Erfolg := DeleteFile(ADR(BackupName));
      Erfolg := Rename(ADR(ArbeitsFile), ADR(BackupName));
    END;
    SetOutput(ArbeitsFile);
    IF NOT Done THEN
      RETURN FALSE;
    END;
    IF Sortiert AND (NOT TabellenBerechnet) THEN
      BerechneTabellen;
    END;
    k := (640 - (AnzMan + 2) * 9) DIV 2;
    win := OeffneFenster(k, 118, (AnzMan+2) * 9, 20,
                         ADR(DatenSpeichernStr));
    WriteInt(AnzMan, 1);
    WriteLn;
    FOR i := 1 TO AnzMan DO
      IF win # NIL THEN
        WriteWin(win, 9*i, 17, "*");
      END;
      IF Sortiert THEN
        m := Tabellen.gsmt[i];
      ELSE
        m := i;
      END;
      WriteString(Namen[m]);
      IF NOT Done THEN
        CloseOutput;
        SchliesseFenster(win);
        RETURN FALSE;
      END;
      WriteLn;
      FOR j := 0 TO 1 DO
        FOR k := 1 TO AnzMan DO
          IF Sortiert THEN
            n := Tabellen.gsmt[k];
          ELSE
            n := k;
          END;
          IF n # m THEN
            WriteInt(Daten[m,j,n], 1);
            IF NOT Done THEN
              CloseOutput;
              SchliesseFenster(win);
              RETURN FALSE;
            END;
            IF (k = AnzMan) OR ((i = AnzMan) AND (k = AnzMan - 1)) THEN
              WriteLn;
            ELSE
              WriteString(" ");
            END;
            IF NOT Done THEN
              CloseOutput;
              SchliesseFenster(win);
              RETURN FALSE;
            END;
          END;
        END;
      END;
    END;
    CloseOutput;
    SchliesseFenster(win);
    DatenGesichert := TRUE;
    RETURN TRUE;
  END SpeicherDaten;


PROCEDURE Speichern;

  VAR
    win: Fenster;
    ok, Nochmal: BOOLEAN;
    OldFilename: FileName;

  BEGIN
    IF AnzMan = 0 THEN
      KeineDaten;
      RETURN;
    END;
    win := NIL;
    OldFilename := ArbeitsFile;
    IF StringEingabe(win, 152, 90, ADR(DatenSpeichernStr), "Filename?",
                     Eingabe, TRUE, "Speichern", "Abbruch") = 2 THEN
      SchliesseFenster(win);
      ArbeitsFile := OldFilename;
      Eingabe := OldFilename;
      RETURN;
    END;
    ArbeitsFile := Eingabe;
    SchliesseFenster(win);
    REPEAT
      ok := SpeicherDaten();
      IF NOT ok THEN
        Nochmal := MyRequest(ADR(DatenSpeichernStr),
                            "Fehler beim Speichern! Nochmal versuchen?",
                            " Ja ", "Nein", TRUE, FALSE);
      END;
    UNTIL ok OR NOT Nochmal;
    IF NOT ok THEN
      ArbeitsFile := OldFilename;
    END;
    MacheTitelzeile;
  END Speichern;


PROCEDURE Laden(FileEingabe: BOOLEAN);

  VAR
    win: Fenster;
    ok, Nochmal: BOOLEAN;
    OldFilename: FileName;

  BEGIN
    SicherheitsTest(ADR(DatenLadenStr));
    win := NIL;
    OldFilename := ArbeitsFile;
    IF FileEingabe THEN
      IF StringEingabe(win, 152, 90, ADR(DatenLadenStr), "Filename?",
                       Eingabe, TRUE, "Laden", "Abbruch") = 2 THEN
        SchliesseFenster(win);
        ArbeitsFile := OldFilename;
        Eingabe := OldFilename;
        RETURN;
      END;
      ArbeitsFile := Eingabe;
      SchliesseFenster(win);
    END;
    REPEAT
      ok := LadeDaten();
      IF NOT ok THEN
        Nochmal := MyRequest(ADR(DatenLadenStr),
                            "Fehler beim Laden! Nochmal versuchen?", " Ja ",
                            "Nein", TRUE, FALSE);
      END;
    UNTIL ok OR NOT Nochmal;
    IF NOT ok THEN
      AnzMan := 0;
      InitDaten;
    END;
    InitVereinGadgets;
    MacheTitelzeile;
  END Laden;


BEGIN
  Eingabe := "";
END DatenInOut.
