(*--------------------------------------------------------------------------
 :Program.    IntuiOps.mod
 :Author.     Marcus Stratmann
 :Address.    Bttcherstrae 13, D-4250 Bottrop-Kirchhellen
 :Phone.      02045/5191
 :Copyright.  Public Domain
 :Language.   MODULA-II
 :Translator. M2Amiga v3.3d
 :Imports.    typdefs
 :History.    V1.0,  26-Sep-89
 :History.    V1.1,  16-May-90
 :Contents.   Manages the screen, windows, gadgets and everything having
 :Contents.   to do with Intuition
--------------------------------------------------------------------------*)

IMPLEMENTATION MODULE IntuiOps;  (* $S- $R- $V- $F- *)

FROM Arts        IMPORT Assert, TermProcedure;
FROM Conversions IMPORT ValToStr;
FROM Exec        IMPORT WaitPort, GetMsg, ReplyMsg, UByte;
FROM Graphics    IMPORT ViewModes, ViewModeSet, Move, Text, SetAPen,
                        SetBPen, SetDrMd, DrawModeSet, jam1, jam2,
                        ClearScreen, Draw;
FROM Intuition   IMPORT ScreenPtr, NewScreen, WindowPtr, NewWindow,
                        OpenScreen, CloseScreen, customScreen, WindowFlags,
                        WindowFlagSet, IDCMPFlagSet, IDCMPFlags, OpenWindow,
                        CloseWindow, Gadget, IntuiText, GadgetFlags,
                        GadgetFlagSet, ActivationFlags, ActivationFlagSet,
                        Border, boolGadget, GadgetPtr, IntuiMessagePtr,
                        RefreshWindowFrame, StringInfo, strGadget,
                        DisplayBeep, RemoveGadget, AddGadget, RefreshGList,
                        gadgHNone, IntuiTextPtr, Menu, MenuItem, MenuPtr,
                        MenuItemPtr, SetMenuStrip, MenuItemFlags,
                        MenuItemFlagSet, menuEnabled, menuNull, reqGadget,
                        Requester, InitRequester, Request, ActivateGadget;
FROM RealConversions IMPORT RealToStr;
FROM Strings     IMPORT Length, Copy, Insert, last;
FROM SYSTEM      IMPORT ADR, ADDRESS, INLINE, BITSET;
FROM typdefs     IMPORT MaxMan, AnzMan, Namen, ArbeitsFile, DatenGesichert,
                        SicherheitsAbfrage, Backup, Sortiert, ScreenTitel;

CONST
  MenuePunkte = 8;
  StandardTitel = "LigaManager V1.1";
  Ueberschrift = "LigaManager V1.1  by  Marcus Stratmann";

TYPE
  xyBorderType = ARRAY[0..9] OF INTEGER;
  DataPtrTyp = POINTER TO DataTyp;
  DataTyp = ARRAY[-1..MenuePunkte],[0..5] OF INTEGER;

VAR
  i: INTEGER;
  NameGadgets: ARRAY[1..MaxMan+1] OF Gadget;
  NameTexts: ARRAY[1..MaxMan+1] OF IntuiText;
  NameBorder: Border;
  xyNameBorder: xyBorderType;
  TorGadgets: ARRAY[0..11] OF Gadget;
  TorTexts: ARRAY[0..11] OF IntuiText;
  TorBorder: Border;
  xyTorBorder: xyBorderType;
  TorStrings: ARRAY[0..11],[0..3] OF CHAR;
  ReqGadgets: ARRAY[1..2] OF Gadget;
  ReqTexts: ARRAY[1..2] OF IntuiText;
  ReqBorder: ARRAY[1..2] OF Border;
  xyReqBorder: ARRAY[1..2] OF xyBorderType;
  EingabeGadget: Gadget;
  EingabeBorder: Border;
  xyEingabeBorder: xyBorderType;
  EingabeInfo: StringInfo;
  TabellenGadgets: ARRAY[-1..3] OF Gadget;
  TabellenTexts: ARRAY[-1..3] OF IntuiText;
  TabellenBorder: Border;
  xyTabellenBorder: xyBorderType;
  MenueGadgets: ARRAY[-1..MenuePunkte] OF Gadget;
  MenueTexts: ARRAY[-1..MenuePunkte] OF IntuiText;
  MenueBorder: ARRAY[-1..MenuePunkte] OF Border;
  xyMenueBorder: ARRAY[-1..MenuePunkte] OF xyBorderType;
  InfoGadget: Gadget;
  InfoGadText: IntuiText;
  InfoGadBorder: Border;
  xyInfoGadBorder: xyBorderType;
  InfoRequester: Requester;
  InfoReqTexts: ARRAY[0..5] OF IntuiText;
  InfoReqBorder: Border;
  xyInfoReqBorder: xyBorderType;
  ProjektMenue: Menu;
  ProjektMenueItems: ARRAY[0..4] OF MenuItem;
  ProjektMenueTexts: ARRAY[0..4] OF IntuiText;
  DatenMenue: Menu;
  DatenMenueItems: ARRAY[0..4] OF MenuItem;
  DatenMenueTexts: ARRAY[0..4] OF IntuiText;
  EinstellungMenue: Menu;
  EinstellungMenueItems: ARRAY[0..2] OF MenuItem;
  EinstellungMenueTexts: ARRAY[0..2] OF IntuiText;
  err: BOOLEAN;
  Top, Left, ID: INTEGER;
  Next, Render: ADDRESS;
  TextAddress: IntuiTextPtr;
  Activat: ActivationFlagSet;
  Flags: GadgetFlagSet;
  PosData: DataTyp;
  PosDataPtr: DataPtrTyp;


PROCEDURE PositionsDaten;   (* $E- *)

  (* Position und Gre der MenueGadgets und Position des Textes. Format: *)
  (* Gadget.leftEdge, Gadget.topEdge, Gadget.width, Gadget.height,        *)
  (* IntuiText.leftEdge, ItuiText.topEdge                                 *)

  BEGIN
    INLINE(009AH, 0016H, 014CH, 000CH, 000EH, 0002H,  (* Ueberschrift     *)
           009AH, 00B6H, 014CH, 000CH, 0096H, 0002H,  (* Ende             *)
           009AH, 0038H, 00A0H, 000CH, 003CH, 0002H,  (* Laden            *)
           0146H, 0038H, 00A0H, 000CH, 002CH, 0002H,  (* Speichern        *)
           009AH, 004CH, 00A0H, 000CH, 0004H, 0002H,  (* Ergebnisse eing. *)
           0146H, 004CH, 00A0H, 000CH, 0004H, 0002H,  (* Ergebnisse ausg. *)
           009AH, 0060H, 014CH, 000CH, 0086H, 0002H,  (* Tabellen         *)
           009AH, 0088H, 014CH, 000CH, 0086H, 0002H,  (* Spieltag         *)
           009AH, 0074H, 014CH, 000CH, 0082H, 0002H,  (* Statistik        *)
           009AH, 009CH, 014CH, 000CH, 007AH, 0002H); (* Neue Saison      *)
  END PositionsDaten;


PROCEDURE OeffneScreen;

  VAR
    ns: NewScreen;

  BEGIN
    WITH ns DO
      leftEdge := 0;
      topEdge := 0;
      width := 640;
      height := 256;
      depth :=  2;
      detailPen := 0;
      blockPen := 1;
      viewModes := ViewModeSet{hires};
      type :=  customScreen;
      font := NIL;
      defaultTitle := ADR(ScreenTitel);
      gadgets := NIL;
      customBitMap := NIL;
    END;
    myScreen := OpenScreen(ns);
    Assert(myScreen # NIL, ADR("Screen nicht zu ffnen"));
  END OeffneScreen;


PROCEDURE SchliesseScreen;

  BEGIN
    IF myScreen # NIL THEN
      WHILE myScreen^.firstWindow # NIL DO   (*  hat da etwa jemand ein  *)
        CloseWindow(myScreen^.firstWindow);  (*  Fenster aufgelassen ?   *)
      END;
      CloseScreen(myScreen);
      myScreen := NIL;
    END;
  END SchliesseScreen;


PROCEDURE Beep;

  BEGIN
    DisplayBeep(myScreen);
  END Beep;


PROCEDURE OeffneSpezFenster(l, t, w, h: INTEGER; IDCMP: IDCMPFlagSet;
                            Flags: WindowFlagSet; FirstGad: GadgetPtr;
                            Name: ADDRESS): Fenster;

  VAR
    nw: NewWindow;
    win: Fenster;

  BEGIN
    IF (t + h) > myScreen^.height THEN
      t := myScreen^.height - h;
      IF t < 0 THEN
        t := 0;
        h := myScreen^.height;
      END;
    END;
    IF (l + w) > myScreen^.width THEN
      l := myScreen^.width - w;
      IF l < 0 THEN
        l := 0;
        w := myScreen^.width;
      END;
    END;
    IF l < 0 THEN
      l := 0;
    END;
    IF t < 0 THEN
      t := 0;
    END;
    WITH nw DO
      leftEdge := l;
      topEdge := t;
      width := w;
      height := h;
      detailPen := 0;
      blockPen := 1;
      idcmpFlags := IDCMP;
      flags := Flags;
      firstGadget := FirstGad;
      checkMark := NIL;
      title := Name;
      screen := myScreen;
      bitMap := NIL;
      minWidth := 20;
      maxWidth := myScreen^.width;
      minHeight := 10;
      maxHeight := myScreen^.height;
      type := customScreen;
    END;
    win := OpenWindow(nw);
    IF win # NIL THEN
      SetAPen(win^.rPort, 1);
      SetBPen(win^.rPort, 0);
      SetDrMd(win^.rPort, jam2);
    ELSE
      Beep;
    END;
    RETURN win;
  END OeffneSpezFenster;


PROCEDURE OeffneFenster(l, t, w, h: INTEGER; name: ADDRESS): Fenster;

  BEGIN
    RETURN OeffneSpezFenster(l, t, w, h, IDCMPFlagSet{},
                             WindowFlagSet{windowActive}, NIL, name);
  END OeffneFenster;


PROCEDURE SchliesseFenster(VAR win: Fenster);

  BEGIN
    IF win # NIL THEN
      CloseWindow(win);
      win := NIL;
    END;
  END SchliesseFenster;


PROCEDURE MacheTitelzeile;

  BEGIN
    IF AnzMan = 0 THEN
      ScreenTitel := StandardTitel;
    ELSE
      IF Length(ArbeitsFile) = 0 THEN
        ScreenTitel := "(unbenannt)";
      ELSE
        ScreenTitel := ArbeitsFile;
      END;
      IF NOT DatenGesichert THEN
        Insert(ScreenTitel, last, "  (gendert)");
      END;
    END;
  END MacheTitelzeile;


PROCEDURE WriteWin(win: Fenster; x, y: INTEGER; text: ARRAY OF CHAR);

  (* ACHTUNG: Es wird NICHT win # NIL berprft! *)
  BEGIN
    Move(win^.rPort, x, y);
    Text(win^.rPort, ADR(text), Length(text));
  END WriteWin;


PROCEDURE WriteZahl(win: Fenster; zahl: INTEGER; laenge: INTEGER;
                    x, y: INTEGER);

  VAR
    out: ARRAY[0..9] OF CHAR;
    err: BOOLEAN;

  BEGIN
    ValToStr(zahl, FALSE, out, 10, laenge, " ", err);
    WriteWin(win, x, y, out);
  END WriteZahl;


PROCEDURE WriteRealZahl(win: Fenster; zahl: REAL; laenge1, laenge2: INTEGER;
                        x, y: INTEGER);

  VAR
    out: ARRAY[0..9] OF CHAR;
    err: BOOLEAN;

  BEGIN
    RealToStr(zahl, out, laenge1, laenge2, FALSE, err);
    WriteWin(win, x, y, out);
  END WriteRealZahl;


PROCEDURE Linie(win: Fenster; x, y, laenge: INTEGER; horizontal: BOOLEAN);

  BEGIN
    Move(win^.rPort, x, y);
    IF horizontal THEN
      Draw(win^.rPort, x + laenge, y);
    ELSE
      Draw(win^.rPort, x, y + laenge);
      Move(win^.rPort, x + 1, y);
      Draw(win^.rPort, x + 1, y + laenge);
    END;
  END Linie;


PROCEDURE LoescheFenster(win: Fenster; x, y: INTEGER);

  BEGIN
    IF win # NIL THEN
      Move(win^.rPort, x, y);
      ClearScreen(win^.rPort);
      RefreshWindowFrame(win);
    END;
  END LoescheFenster;


PROCEDURE ZeigeInfo(win: Fenster);

  BEGIN
    IF Request(ADR(InfoRequester), win) THEN END;
  END ZeigeInfo;


PROCEDURE WaehleVerein(VAR win: Fenster; x, y: INTEGER): CARDINAL;

  VAR
    Message: IntuiMessagePtr;
    calledGad: GadgetPtr;
    h: INTEGER;

  BEGIN
    IF win = NIL THEN
      h := ((AnzMan + 1) * 11) + 5;
      win := OeffneSpezFenster(x, y, 210, h, IDCMPFlagSet{gadgetUp},
                               WindowFlagSet{borderless},
                               ADR(NameGadgets[1]), NIL);
    END;
    IF win # NIL THEN
      WaitPort(win^.userPort);
      Message := GetMsg(win^.userPort);
      calledGad := Message^.iAddress;
      ReplyMsg(Message);
      RETURN calledGad^.gadgetID;
    ELSE
      RETURN 0
    END;
  END WaehleVerein;


PROCEDURE WaehleZahl(VAR win: Fenster; Mitte: BOOLEAN): INTEGER;

  VAR
    Message: IntuiMessagePtr;
    calledGad: GadgetPtr;
    Tore, Wahl: CARDINAL;
    Offset: INTEGER;

  BEGIN
    IF win = NIL THEN
      IF Mitte THEN
        Offset := 22;
      ELSE
        Offset := 11;
      END;
      win := OeffneSpezFenster(myScreen^.mouseX - 48,
                               myScreen^.mouseY - Offset, 96, 46,
                               IDCMPFlagSet{gadgetUp},
                               WindowFlagSet{borderless},
                               ADR(TorGadgets[0]), NIL);
    END;
    IF win # NIL THEN
      Tore := 0;
      LOOP
        WaitPort(win^.userPort);
        Message := GetMsg(win^.userPort);
        calledGad := Message^.iAddress;
        ReplyMsg(Message);
        Wahl := calledGad^.gadgetID;
        IF Wahl = 11 THEN
          RETURN -1;
        END;
        Tore := Tore + Wahl;
        IF Wahl < 10 THEN EXIT END;
      END;
      RETURN Tore;
    ELSE
      RETURN -1;
    END;
  END WaehleZahl;


PROCEDURE WaehleTabelle(VAR win: Fenster; x, y: INTEGER): INTEGER;

  VAR
    Message: IntuiMessagePtr;
    calledGad: GadgetPtr;

  BEGIN
    IF win = NIL THEN
      TabellenTexts[-1].iText := ADR(DruckerText[FALSE]);
      win := OeffneSpezFenster(x, y, 342, 34, IDCMPFlagSet{gadgetUp,
                               gadgetDown}, WindowFlagSet{borderless},
                               ADR(TabellenGadgets[-1]), NIL);
    END;
    IF win # NIL THEN
      WaitPort(win^.userPort);
      Message := GetMsg(win^.userPort);
      calledGad := Message^.iAddress;
      ReplyMsg(Message);
      RETURN calledGad^.gadgetID;
    ELSE
      RETURN 0
    END;
  END WaehleTabelle;


PROCEDURE MenueAuswahl(): CARDINAL;

  VAR
    win: Fenster;
    Message: IntuiMessagePtr;
    calledGad: GadgetPtr;
    Class: IDCMPFlagSet;
    Code, MenuNum, ItemNum: CARDINAL;
    pos: INTEGER;

  BEGIN
    win := OeffneSpezFenster(0, 0, 640, 256, IDCMPFlagSet{gadgetUp,
                             gadgetDown, menuPick}, WindowFlagSet{backDrop,
                             borderless, simpleRefresh, gimmeZeroZero,
                             activate}, ADR(MenueGadgets[-1]), NIL);
    IF win # NIL THEN
      IF SetMenuStrip(win, ADR(ProjektMenue)) THEN END;
      LOOP
        WaitPort(win^.userPort);
        Message := GetMsg(win^.userPort);
        calledGad := Message^.iAddress;
        Class := Message^.class;
        Code := Message^.code;
        ReplyMsg(Message);
        IF gadgetDown IN Class THEN
          ZeigeInfo(win);
        ELSIF (menuPick IN Class) AND (Code # menuNull) THEN
          MenuNum := Code MOD 32;
          ItemNum := (Code DIV 32) MOD 32;
          CASE MenuNum OF
          | 0: CASE ItemNum OF   (* Projekt *)
               | 0: SchliesseFenster(win);
                    RETURN 1;
               | 1: SchliesseFenster(win);
                    RETURN 2;
               | 2: SchliesseFenster(win);
                    RETURN 8;
               | 3: ZeigeInfo(win);
               | 4: SchliesseFenster(win);
                    RETURN 0;
               END;
          | 1: SchliesseFenster(win);  (* Daten *)
               CASE ItemNum OF
               | 0: RETURN 3;
               | 1: RETURN 4;
               | 2: RETURN 5;
               | 3: RETURN 7;
               | 4: RETURN 6;
               END;
          | 2: CASE ItemNum OF
               | 0: Backup := NOT Backup;
                    IF Backup THEN
                      INCL(EinstellungMenueItems[0].flags, checked);
                    ELSE
                      EXCL(EinstellungMenueItems[0].flags, checked);
                    END;
               | 1: SicherheitsAbfrage := NOT SicherheitsAbfrage;
                    IF SicherheitsAbfrage THEN
                      INCL(EinstellungMenueItems[1].flags, checked);
                    ELSE
                      EXCL(EinstellungMenueItems[1].flags, checked);
                    END;
               | 2: Sortiert := NOT Sortiert;
                    IF Sortiert THEN
                      INCL(EinstellungMenueItems[2].flags, checked);
                    ELSE
                      EXCL(EinstellungMenueItems[2].flags, checked);
                    END;
               END;
          END;
        ELSIF gadgetUp IN Class THEN
          IF calledGad^.gadgetID >= 0 THEN
            SchliesseFenster(win);
            RETURN calledGad^.gadgetID;
          END;
        END;
      END (* LOOP *);
    ELSE (* win = NIL *)
      RETURN 0
    END;
  END MenueAuswahl;


PROCEDURE MyRequest(titel: ADDRESS; text, posText, negText: ARRAY OF CHAR;
                    AufJedenFall, AufPosText: BOOLEAN): BOOLEAN;

  VAR
    Message: IntuiMessagePtr;
    calledGad: GadgetPtr;
    w, x, y, mouseX, mouseY, gad1Length, gad2Length: INTEGER;
    win: Fenster;
    negativ, positiv: ARRAY[0..40] OF CHAR;

  BEGIN
    IF AufJedenFall OR SicherheitsAbfrage THEN
      Copy(positiv, posText, 0, 40);
      Copy(negativ, negText, 0, 40);
      w := (Length(text) * 8) + 16;
      gad1Length := Length(positiv) * 8;
      gad2Length := Length(negativ) * 8;
      IF (gad1Length + gad2Length + 42) > w THEN
        w := gad1Length + gad2Length + 42;
      END;
      ReqGadgets[1].width := gad1Length + 6;
      ReqGadgets[1].topEdge := 26;
      ReqGadgets[2].topEdge := 26;
      ReqGadgets[2].leftEdge := (w - gad2Length) - 18;
      ReqGadgets[2].width := gad2Length + 6;
      ReqTexts[1].iText := ADR(positiv);
      ReqTexts[2].iText := ADR(negativ);
      xyReqBorder[1,2] := gad1Length + 8;
      xyReqBorder[1,4] := gad1Length + 8;
      xyReqBorder[2,2] := gad2Length + 8;
      xyReqBorder[2,4] := gad2Length + 8;
      mouseX := myScreen^.mouseX;
      mouseY := myScreen^.mouseY;
      IF AufPosText THEN
        x := mouseX - (10 + (ReqGadgets[1].width / 2));
      ELSE
        x := mouseX - (w / 2);
      END;
      IF x < 0 THEN
        x := 0;
      END;
      y := mouseY - 30;
      IF y < 0 THEN
        y := 0;
      END;
      win := OeffneSpezFenster(x, y, w, 42, IDCMPFlagSet{gadgetUp},
                               WindowFlagSet{windowDrag, windowDepth,
                               activate}, ADR(ReqGadgets[1]), titel);
      IF win # NIL THEN
        WriteWin(win, 8, 20, text);
        WaitPort(win^.userPort);
        Message := GetMsg(win^.userPort);
         calledGad := Message^.iAddress;
        ReplyMsg(Message);
        SchliesseFenster(win);
        IF calledGad^.gadgetID = 1 THEN
          RETURN TRUE;
        ELSE
          RETURN FALSE;
        END;
      ELSE
        RETURN FALSE
      END;
    ELSE
      RETURN TRUE;
    END;
  END MyRequest;


PROCEDURE KeineDaten;

  BEGIN
    IF MyRequest(ADR("Daten?!"), "Keine Daten geladen!", " OK ", " OK ",
                 TRUE, TRUE) THEN END;
  END KeineDaten;


PROCEDURE StringEingabe(VAR win: Fenster; x, y: INTEGER; titel: ADDRESS;
                        text: ARRAY OF CHAR; VAR str: ARRAY OF CHAR;
                        request: BOOLEAN; posText, negText:
                        ARRAY OF CHAR): CARDINAL;

  VAR
    Message: IntuiMessagePtr;
    calledGad: GadgetPtr;
    UndoBuffer: ARRAY[0..80] OF CHAR;
    len, zusatz, w, h, gad1Length, gad2Length: INTEGER;
    negativ, positiv: ARRAY[0..40] OF CHAR;
    active: BOOLEAN;

  BEGIN
    UndoBuffer := " ";
    WITH EingabeInfo DO
      buffer := ADR(str);
      undoBuffer := ADR(UndoBuffer);
      bufferPos := Length(str);
      maxChars := HIGH(str) + 1;
      dispPos := 0;
    END;
    len := (HIGH(str) + 2) * 8;
    zusatz := 0;
    IF len > 320 THEN
      len := 320;
      zusatz := 8;
    END;
    EingabeGadget.width := len;
    len := len + zusatz;
    xyEingabeBorder[2] := len - 2;
    xyEingabeBorder[4] := len - 2;
    w := (Length(text) * 8) + 16;
    IF w < len + 18 THEN
      w := len + 18;
    ELSE
      EingabeGadget.leftEdge := ((w - EingabeGadget.width) DIV 2) + 2;
    END;
    EingabeGadget.nextGadget := NIL;
    h := 42;
    IF request THEN
      EingabeGadget.nextGadget := ADR(ReqGadgets[1]);
      Copy(positiv, posText, 0, 40);
      Copy(negativ, negText, 0, 40);
      gad1Length := Length(positiv) * 8;
      gad2Length := Length(negativ) * 8;
      ReqGadgets[1].width := gad1Length + 6;
      ReqGadgets[1].topEdge := 42;
      ReqGadgets[2].topEdge := 42;
      ReqGadgets[2].leftEdge := (w - gad2Length) - 18;
      ReqGadgets[2].width := gad2Length + 6;
      ReqTexts[1].iText := ADR(positiv);
      ReqTexts[2].iText := ADR(negativ);
      xyReqBorder[1,2] := gad1Length + 8;
      xyReqBorder[1,4] := gad1Length + 8;
      xyReqBorder[2,2] := gad2Length + 8;
      xyReqBorder[2,4] := gad2Length + 8;
      h := 58;
    END;
    win := OeffneSpezFenster(x, y, w, h, IDCMPFlagSet{gadgetUp},
                             WindowFlagSet{windowDrag, windowDepth,
                             activate}, ADR(EingabeGadget), titel);
    IF win # NIL THEN
      WriteWin(win, 8, 20, text);
      active := ActivateGadget(ADR(EingabeGadget), win, NIL);
      WaitPort(win^.userPort);
      Message := GetMsg(win^.userPort);
      calledGad := Message^.iAddress;
      ReplyMsg(Message);
      SchliesseFenster(win);
      RETURN calledGad^.gadgetID;
    END;
  END StringEingabe;


PROCEDURE InitGadget(VAR Gad: Gadget; l, t, w, h: INTEGER;
                     fl: GadgetFlagSet; act: ActivationFlagSet;
                     type: CARDINAL; render: ADDRESS; text: IntuiTextPtr;
                     info: ADDRESS; id: INTEGER; next: GadgetPtr);

  BEGIN
    WITH Gad DO
      nextGadget := next;
      leftEdge := l;
      topEdge := t;
      width := w;
      height := h;
      flags := fl;
      activation := act;
      gadgetType := type;
      gadgetRender := render;
      selectRender := NIL;
      gadgetText := text;
      specialInfo := info;
      gadgetID := id;
    END;
  END InitGadget;


PROCEDURE InitIntuiText(VAR IText: IntuiText; l, t: INTEGER; text: ADDRESS;
                        fPen, bPen: UByte; dm: DrawModeSet;
                        next: IntuiTextPtr);

  BEGIN
    WITH IText DO
      frontPen := fPen;
      backPen := bPen;
      drawMode := dm;
      leftEdge := l;
      topEdge := t;
      iTextFont := NIL;
      iText := text;
      nextText := next;
    END;
  END InitIntuiText;


PROCEDURE InitBorder(VAR Bord: Border; l, t: INTEGER; fPen, bPen:UByte;
                     XY: ADDRESS);

  BEGIN
    WITH Bord DO
      leftEdge := l;
      topEdge := t;
      frontPen := fPen;
      backPen := bPen;
      drawMode := jam2;
      count := 5;
      xy := XY;
      nextBorder := NIL;
    END;
  END InitBorder;


PROCEDURE InitxyBorder(VAR xy: xyBorderType; w, h: INTEGER);

  BEGIN
    xy[0] := 0;  xy[1] := 0;
    xy[2] := w;  xy[3] := 0;
    xy[4] := w;  xy[5] := h;
    xy[6] := 0;  xy[7] := h;
    xy[8] := 0;  xy[9] := 0;
  END InitxyBorder;


PROCEDURE InitMenu(VAR menu: Menu; l, w: INTEGER; name: ADDRESS;
                   first: MenuItemPtr; next: MenuPtr);

  BEGIN
    WITH menu DO
      leftEdge := l;
      topEdge := 0;
      width := w;
      height := 0;
      flags := BITSET{menuEnabled};
      menuName := name;
      firstItem := first;
      nextMenu := next;
    END;
  END InitMenu;


PROCEDURE InitMenuItem(VAR item: MenuItem; l, t, w, h: INTEGER;
                       text: ADDRESS; next: MenuItemPtr);

  BEGIN
    WITH item DO
      leftEdge := l;
      topEdge := t;
      width := w;
      height := h;
      flags := MenuItemFlagSet{itemEnabled, itemText, highComp};
      itemFill := text;
      selectFill := NIL;
      subItem := NIL;
      nextItem := next;
    END;
  END InitMenuItem;


PROCEDURE InitVereinGadgets;

  VAR
    i, Top, ID: INTEGER;
    Next: ADDRESS;

  BEGIN
    FOR i := 1 TO AnzMan + 1 DO
      ID := i;
      Top := 11 * i - 10;
      IF i = AnzMan + 1 THEN
        Next := NIL;
        ID := 0;
        INC(Top, 4);
      ELSE
        Next := ADR(NameGadgets[i+1]);
      END;
      InitGadget(NameGadgets[i], 2, Top, 206, 10, GadgetFlagSet{},
                 ActivationFlagSet{relVerify}, boolGadget, ADR(NameBorder),
                 ADR(NameTexts[i]), NIL, ID, Next);
      IF i = AnzMan + 1 THEN
        TextAddress := ADR("Ende / Zurck");
      ELSE
        TextAddress := ADR(Namen[i]);
      END;
      InitIntuiText(NameTexts[i], 3, 1, TextAddress, 1, 0, jam2, NIL);
    END;
    InitxyBorder(xyNameBorder, 207, 11);
    InitBorder(NameBorder, -1, -1, 1, 0, ADR(xyNameBorder));
  END InitVereinGadgets;


BEGIN  (* IntuiOps *)
  OeffneScreen;
  TermProcedure(SchliesseScreen);
  InitVereinGadgets;

  (*  --- initialisiere Gadgets fr Torauswahl -------------------------  *)

  FOR i := 0 TO 11 DO
    IF i < 11 THEN
      Next := ADR(TorGadgets[i+1]);
    ELSE
      Next := NIL;
    END;
    InitGadget(TorGadgets[i], (i MOD 3) * 31 + 2, (i DIV 3) * 11 + 1, 30,
               10, GadgetFlagSet{}, ActivationFlagSet{relVerify},
               boolGadget, ADR(TorBorder), ADR(TorTexts[i]), NIL, i, Next);
    InitIntuiText(TorTexts[i], 3, 1, ADR(TorStrings[i]), 1, 0, jam2, NIL);
    IF i < 10 THEN
      ValToStr(i, FALSE, TorStrings[i], 10, 2, " ", err);
    END;
  END;
  TorStrings[10] := "+10";
  TorStrings[11] := " * ";
  InitxyBorder(xyTorBorder, 31, 11);
  InitBorder(TorBorder, -1, -1, 1, 0, ADR(xyTorBorder));

  (* --- initialisiere Gadgets fr Requester ---------------------------- *)

  FOR i := 1 TO 2 DO
    IF i = 1 THEN
      Next := ADR(ReqGadgets[2]);
    ELSE
      Next := NIL;
    END;
    InitGadget(ReqGadgets[i], 10, 0, 0, 10, GadgetFlagSet{},
               ActivationFlagSet{relVerify}, boolGadget, ADR(ReqBorder[i]),
               ADR(ReqTexts[i]), NIL, i, Next);
    InitIntuiText(ReqTexts[i], 3, 1, NIL, 1, 0, jam2, NIL);
    InitxyBorder(xyReqBorder[i], 0, 11);
    InitBorder(ReqBorder[i], -1, -1, 1, 0, ADR(xyReqBorder[i]));
  END;

  (* --- Initialisiere Gadget fr TextEingabe --------------------------- *)

  InitGadget(EingabeGadget, 12, 27, 0, 10, GadgetFlagSet{},
             ActivationFlagSet{relVerify}, strGadget, ADR(EingabeBorder),
             NIL, ADR(EingabeInfo), 0, NIL);
  InitxyBorder(xyEingabeBorder, 0, 11);
  InitBorder(EingabeBorder, -3, -2, 1, 0, ADR(xyEingabeBorder));

  (* --- initialisiere Gadgets fr Auswahl der Tabellen ----------------- *)

  FOR i := -1 TO 3 DO
    Activat := ActivationFlagSet{relVerify};
    Flags := GadgetFlagSet{};
    Top := 11 * i - 10;
    Left := 2;
    IF i = -1 THEN
      Activat := ActivationFlagSet{gadgImmediate};
      Flags := gadgHNone;
      Top := 1;
      Left := 174;
    END;
    IF i = 0 THEN
      Top := 23;
      Left := 174;
    END;
    IF i # 3 THEN
      Next := ADR(TabellenGadgets[i+1]);
    ELSE
      Next := NIL;
    END;
    InitGadget(TabellenGadgets[i], Left, Top, 166, 10, Flags, Activat,
               boolGadget, ADR(TabellenBorder), ADR(TabellenTexts[i]), NIL,
               i, Next);
    CASE i OF
    | -1: TextAddress := ADR(DruckerText[FALSE]);
    |  0: TextAddress := ADR("Zurck");
    |  1: TextAddress := ADR("Gesamtbilanz");
    |  2: TextAddress := ADR("Heimbilanz");
    |  3: TextAddress := ADR("Auswrtsbilanz");
    END;
    InitIntuiText(TabellenTexts[i], 3, 1, TextAddress, 1, 0, jam2, NIL);
  END;
  InitxyBorder(xyTabellenBorder, 167, 11);
  InitBorder(TabellenBorder, -1, -1, 1, 0, ADR(xyTabellenBorder));

  (* --- initialisiere Gadgets & Text fr Menueauswahl ------------------ *)

  PosDataPtr := ADR(PositionsDaten);
  PosData := PosDataPtr^;
  FOR i := -1 TO MenuePunkte DO
    Render := ADR(MenueBorder[i]);
    IF i < 0 THEN
      Activat := ActivationFlagSet{gadgImmediate};
      Flags := gadgHNone;
    ELSE
      Activat := ActivationFlagSet{relVerify};
      Flags := GadgetFlagSet{};
    END;
    IF i # MenuePunkte THEN
      Next := ADR(MenueGadgets[i+1]);
    ELSE
      Next := NIL;
    END;
    InitGadget(MenueGadgets[i], PosData[i,0], PosData[i,1], PosData[i,2],
               PosData[i,3], Flags, Activat, boolGadget, Render,
               ADR(MenueTexts[i]), NIL, i, Next);
    CASE i OF
    | -1: TextAddress := ADR(Ueberschrift);
    |  0: TextAddress := ADR("Ende");
    |  1: TextAddress := ADR("Laden");
    |  2: TextAddress := ADR("Speichern");
    |  3: TextAddress := ADR("Ergebnisse eingeben");
    |  4: TextAddress := ADR("Ergebnisse ausgeben");
    |  5: TextAddress := ADR("Tabellen");
    |  6: TextAddress := ADR("Spieltag");
    |  7: TextAddress := ADR("Statistik");
    |  8: TextAddress := ADR("Neue Saison");
    END;
    InitIntuiText(MenueTexts[i], PosData[i,4], PosData[i,5], TextAddress, 1,
                  0, jam2, NIL);
    InitxyBorder(xyMenueBorder[i], PosData[i,2] + 1, PosData[i,3] + 1);
    InitBorder(MenueBorder[i], -1, -1, 1, 0, ADR(xyMenueBorder[i]));
  END;

  (* --- initialisiere Projektmenue ------------------------------------- *)

  InitMenu(ProjektMenue, 0, 80, ADR("Projekt"), ADR(ProjektMenueItems[0]),
           ADR(DatenMenue));
  FOR i := 0 TO 4 DO
    IF i # 4 THEN
      Next := ADR(ProjektMenueItems[i+1]);
    END;
    Top := i * 11;
    CASE i OF
    | 0: TextAddress := ADR("Laden");
    | 1: TextAddress := ADR("Speichern");
    | 2: TextAddress := ADR("Neu");
    | 3: TextAddress := ADR("Info");
    | 4: TextAddress := ADR("Ende");
         Next := NIL;
    END;
    InitMenuItem(ProjektMenueItems[i], 0, Top, 78, 10,
                 ADR(ProjektMenueTexts[i]), Next);
    InitIntuiText(ProjektMenueTexts[i], 4, 1, TextAddress, 2, 0, jam1, NIL);
  END;

  (* --- initialisiere Datenmenue --------------------------------------- *)

  InitMenu(DatenMenue, 80, 64, ADR("Daten"), ADR(DatenMenueItems[0]),
           ADR(EinstellungMenue));
  FOR i := 0 TO 4 DO
    IF i # 4 THEN
      Next := ADR(DatenMenueItems[i+1]);
    END;
    Top := i * 11;
    CASE i OF
    | 0: TextAddress := ADR("Eingeben");
    | 1: TextAddress := ADR("Ausgeben");
    | 2: TextAddress := ADR("Tabellen");
    | 3: TextAddress := ADR("Statistik");
    | 4: TextAddress := ADR("Spieltag");
         Next := NIL;
    END;
    InitMenuItem(DatenMenueItems[i], 0, Top, 78, 10,
                 ADR(DatenMenueTexts[i]), Next);
    InitIntuiText(DatenMenueTexts[i], 4, 1, TextAddress, 2, 0, jam1, NIL);
  END;

  (* --- initialisiere Einstellungsmenue -------------------------------- *)

  InitMenu(EinstellungMenue, 144, 128, ADR("Einstellungen"),
           ADR(EinstellungMenueItems[0]), NIL);
  FOR i := 0 TO 2 DO
    Top := i * 11;
    CASE i OF
    | 0: TextAddress := ADR("Backup");
         Next := ADR(EinstellungMenueItems[1]);
    | 1: TextAddress := ADR("Sicherheitsabfragen");
         Next := ADR(EinstellungMenueItems[2]);
    | 2: TextAddress := ADR("Sortiert speichern");
         Next := NIL;
    END;
    InitMenuItem(EinstellungMenueItems[i], 0, Top, 174, 10,
                 ADR(EinstellungMenueTexts[i]), Next);
    INCL(EinstellungMenueItems[i].flags, checkIt);
    InitIntuiText(EinstellungMenueTexts[i], 20, 1, TextAddress, 2, 0, jam1,
                  NIL);
  END;
  IF Backup THEN
    INCL(EinstellungMenueItems[0].flags, checked);
  END;
  IF SicherheitsAbfrage THEN
    INCL(EinstellungMenueItems[1].flags, checked);
  END;
  IF Sortiert THEN
    INCL(EinstellungMenueItems[2].flags, checked);
  END;

  (* --- initialisiere InfoRequester ------------------------------------ *)

  InitGadget(InfoGadget, 83, 95, 54, 10, GadgetFlagSet{},
             ActivationFlagSet{endGadget, relVerify},
             boolGadget + reqGadget, ADR(InfoGadBorder), ADR(InfoGadText),
             NIL, -255, NIL);
  InitIntuiText(InfoGadText, 3, 1, ADR("Weiter"), 2, 1, jam2, NIL);
  InitxyBorder(xyInfoGadBorder, 55, 11);
  InitBorder(InfoGadBorder, -1, -1, 3, 0, ADR(xyInfoGadBorder));

  InitRequester(ADR(InfoRequester));
  WITH InfoRequester DO
    leftEdge := 210;
    topEdge := 64;
    width := 220;
    height := 110;
    reqGadget := ADR(InfoGadget);
    reqText := ADR(InfoReqTexts[0]);
    reqBorder := ADR(InfoReqBorder);
    backFill := 1;
  END;
  InitIntuiText(InfoReqTexts[0], 58, 10, ADR(" LigaManager "), 2, 1,
                jam2, ADR(InfoReqTexts[1]));
  InitIntuiText(InfoReqTexts[1], 46, 25, ADR(" Copyright 1990"), 2, 1,
                jam2, ADR(InfoReqTexts[2]));
  InitIntuiText(InfoReqTexts[2], 34, 35, ADR("by Marcus Stratmann"), 2, 1,
                jam2, ADR(InfoReqTexts[3]));
  InitIntuiText(InfoReqTexts[3], 26, 50, ADR("Weitere Informationen"), 2, 1,
                jam2, ADR(InfoReqTexts[4]));
  InitIntuiText(InfoReqTexts[4], 18, 60, ADR("im File LigaManager.dok"), 2, 1,
                jam2, ADR(InfoReqTexts[5]));
  InitIntuiText(InfoReqTexts[5], 22, 75, ADR("Version 1.1, 16-May-90"),
                2, 1, jam2, NIL);
  InitxyBorder(xyInfoReqBorder, 217, 107);
  InitBorder(InfoReqBorder, 1, 1, 3, 0, ADR(xyInfoReqBorder));

  (* --- Sonstiges ------------------------------------------------------ *)

  DruckerText[TRUE]  := "Ausgabe auf: Drucker";
  DruckerText[FALSE] := "Ausgabe auf: Screen ";

END IntuiOps.
