(*--------------------------------------------------------------------------
 :Program.    Spieltag.mod
 :Author.     Marcus Stratmann
 :Address.    Bttcherstrae 13, D-4250 Bottrop-Kirchhellen
 :Phone.      02045/5191
 :Copyright.  Public Domain
 :Language.   MODULA-II
 :Translator. M2Amiga v3.3d
 :Imports.    typdefs, IntuiOps, TabellenOps
 :History.    V1.0,  26-Sep-89
 :History.    V1.1,  16-May-90
 :Contents.   Control over currently running matches
--------------------------------------------------------------------------*)

IMPLEMENTATION MODULE Spieltag;  (* $S- $R- $V- $F- *)

FROM Conversions IMPORT ValToStr;
FROM Exec        IMPORT WaitPort, GetMsg, ReplyMsg;
FROM Graphics    IMPORT jam1;
FROM IntuiOps    IMPORT Fenster, OeffneFenster, SchliesseFenster, WriteWin,
                        WriteZahl, WaehleZahl, WaehleVerein, LoescheFenster,
                        OeffneSpezFenster, MyRequest, WriteRealZahl,
                        InitGadget, InitIntuiText, KeineDaten,
                        MacheTitelzeile, Beep;
FROM Intuition   IMPORT Gadget, IntuiText, GadgetFlags, GadgetFlagSet,
                        ActivationFlags, ActivationFlagSet, boolGadget,
                        IDCMPFlags, IDCMPFlagSet, WindowFlags,
                        WindowFlagSet, GadgetPtr, IntuiMessagePtr,
                        MoveWindow;
FROM Strings     IMPORT Insert, last;
FROM SYSTEM      IMPORT ADR;
FROM TabellenOps IMPORT TabelleAusgeben, DatenAuswertung;
FROM typdefs     IMPORT MaxMan, AnzMan, Daten, Namen, TabellenBerechnet,
                        TorePunkte, DatenAusgewertet, StatistikBerechnet,
                        DatenGesichert;

CONST
  MaxSpiele = MaxMan DIV 2;

VAR
  GegnerGadgets, TorGadgets: ARRAY[1..MaxSpiele],[0..1] OF Gadget;
  GegnerTexts: ARRAY[1..MaxSpiele],[0..1] OF IntuiText;
  HalbzeitGadgets: ARRAY[1..MaxSpiele] OF Gadget;
  HalbzeitText: IntuiText;
  EndeGadget: Gadget;
  Message: IntuiMessagePtr;
  calledGad: GadgetPtr;
  class: IDCMPFlagSet;
  i, j, Code: INTEGER;


PROCEDURE AendereStatus;

  BEGIN
    TabellenBerechnet := FALSE;
    DatenAusgewertet := FALSE;
    DatenGesichert := FALSE;
    StatistikBerechnet := FALSE;
  END AendereStatus;


PROCEDURE SpieltagDurchfuehren;

  VAR
    AusgabeFenster, ZahlFenster, VereinFenster, SpielFenster,
     TabellenFenster, TorFenster, EndeFenster: Fenster;
    Gegner, Tore, HalbzeitTore, AltTore: ARRAY[1..MaxSpiele],[0..1]
                                         OF INTEGER;
    Heim, Gast, SpieltagTore, SaisonTore,
      SaisonSpiele: INTEGER;
    i, j, Top, AnzahlSpiele: INTEGER;
    AlterStatus, err, TortextGeschrieben: BOOLEAN;
    out: ARRAY[0..80] OF CHAR;


  PROCEDURE SchreibeSpielstand(SpielNr: CARDINAL);

    VAR
      zeile: CARDINAL;

    BEGIN
      IF SpielFenster # NIL THEN
        zeile := 8 * SpielNr + 2;
        WriteZahl(SpielFenster, Tore[SpielNr,0], 2, 450, zeile);
        WriteWin(SpielFenster, 466, zeile, ":");
        WriteZahl(SpielFenster, Tore[SpielNr,1], -2, 474, zeile);
        IF HalbzeitTore[SpielNr,0] > -1 THEN
          WriteWin(SpielFenster, 498, zeile, "(");
          WriteZahl(SpielFenster, HalbzeitTore[SpielNr,0], 1, 506, zeile);
          WriteWin(SpielFenster, 514, zeile, ":");
          WriteZahl(SpielFenster, HalbzeitTore[SpielNr,1], 1, 522, zeile);
          WriteWin(SpielFenster, 530, zeile, ")");
        ELSE
          WriteWin(SpielFenster, 498, zeile, "     ");
        END;
      END;
    END SchreibeSpielstand;


  PROCEDURE SchreibeTore;

    BEGIN
      IF TorFenster # NIL THEN
        IF NOT TortextGeschrieben THEN
          WriteWin(TorFenster, 10, 10, "Tore gesamt:");
          WriteWin(TorFenster, 10, 20, "Spieltag:");
          WriteWin(TorFenster, 10, 28, "Saison:");
          WriteWin(TorFenster, 10, 40, "Tore pro Spiel:");
          WriteWin(TorFenster, 10, 50, "Spieltag:");
          WriteWin(TorFenster, 10, 58, "Saison:");
          TortextGeschrieben := TRUE;
        END;
        WriteZahl(TorFenster, SpieltagTore, -2, 100, 20);
        WriteZahl(TorFenster, SaisonTore, -4, 100, 28);
        WriteRealZahl(TorFenster, FLOAT(SpieltagTore) / FLOAT(AnzahlSpiele),
                      -3, 1, 100, 50);
        WriteRealZahl(TorFenster, FLOAT(SaisonTore) / FLOAT(SaisonSpiele),
                      -3, 1, 100, 58);
      END;
    END SchreibeTore;


  PROCEDURE Tor(code: INTEGER);

    VAR
      HeimAusw, Verein, SpielNr: INTEGER;
      Erhoehe, Aenderung: BOOLEAN;

    BEGIN
      SpielNr := code DIV 2;
      HeimAusw := code MOD 2;
      IF SpielNr > MaxSpiele THEN
        DEC(SpielNr, MaxSpiele);
        Erhoehe := FALSE;
      ELSE
        Erhoehe := TRUE;
      END;
      Verein := Gegner[SpielNr, HeimAusw];
      Aenderung := TRUE;
      IF Erhoehe THEN
        INC(Tore[SpielNr,HeimAusw]);
        INC(Daten[Gegner[SpielNr,HeimAusw],HeimAusw,
                           Gegner[SpielNr,1-HeimAusw]]);
        INC(SpieltagTore);
        INC(SaisonTore);
      ELSIF Tore[SpielNr,HeimAusw] > 0 THEN
        DEC(Tore[SpielNr,HeimAusw]);
        DEC(Daten[Gegner[SpielNr,HeimAusw],HeimAusw,
                           Gegner[SpielNr,1-HeimAusw]]);
        DEC(SpieltagTore);
        DEC(SaisonTore);
      ELSE
        Aenderung := FALSE;
      END;
      IF Aenderung THEN
        AendereStatus;
        SchreibeSpielstand(SpielNr);
        IF TabellenFenster # NIL THEN
          TabelleAusgeben(1, TabellenFenster, FALSE, FALSE);
        END;
        SchreibeTore;
      END;
    END Tor;


  PROCEDURE Halbzeit(SpielNr: INTEGER);

    VAR
      i: CARDINAL;

    BEGIN
      IF HalbzeitTore[SpielNr,0] < 0 THEN
        HalbzeitTore[SpielNr,0] := Tore[SpielNr,0];
        HalbzeitTore[SpielNr,1] := Tore[SpielNr,1];
      ELSE
        HalbzeitTore[SpielNr,0] := -1;
        HalbzeitTore[SpielNr,1] := -1;
      END;
    SchreibeSpielstand(SpielNr);
    END Halbzeit;


  PROCEDURE WriteAusgabe(x, y: INTEGER; str: ARRAY OF CHAR);

    BEGIN
      IF AusgabeFenster # NIL THEN
        WriteWin(AusgabeFenster, x, y, str);
      END;
    END WriteAusgabe;


  BEGIN  (* SpieltagDurchfuehren *)
    IF AnzMan = 0 THEN
      KeineDaten;
      RETURN;
    END;
    AusgabeFenster := OeffneFenster(210, 15, 220, 64, ADR("Spieltag"));
    WriteAusgabe(10, 20, "Wie viele Spiele (1-  ) ?");
    WriteZahl(AusgabeFenster, AnzMan DIV 2, 2, 170, 20);
    ZahlFenster := NIL;
    REPEAT
      AnzahlSpiele := WaehleZahl(ZahlFenster, TRUE);
      IF AnzahlSpiele < 1 THEN
        SchliesseFenster(ZahlFenster);
        SchliesseFenster(AusgabeFenster);
        RETURN
      ELSIF AnzahlSpiele > (AnzMan DIV 2) THEN
        Beep;
      END;
    UNTIL AnzahlSpiele <= (AnzMan DIV 2);
    SchliesseFenster(ZahlFenster);
    MoveWindow(AusgabeFenster, -210, 0);
    VereinFenster := NIL;
    FOR i := 1 TO AnzahlSpiele DO
      REPEAT
        LoescheFenster(AusgabeFenster, 10, 15);
        ValToStr(i, FALSE, out, 10, 2, " ", err);
        WriteAusgabe(10, 20, out);
        WriteAusgabe(26, 20, ". Spiel:");
        Insert(out, last, ". Spiel: ");
        WriteAusgabe(10, 30, "Heimmannschaft?");
        Heim := WaehleVerein(VereinFenster, 250, 15);
        IF Heim < 1 THEN
          SchliesseFenster(VereinFenster);
          SchliesseFenster(AusgabeFenster);
          RETURN
        END;
        WriteAusgabe(10, 38, Namen[Heim]);
        Insert(out, last, Namen[Heim]);
        Insert(out, last, " - ");
        WriteAusgabe(10, 48, "Gastmannschaft?");
        REPEAT
          Gast := WaehleVerein(VereinFenster, -1, -1);
        UNTIL Gast # Heim;
        IF Gast < 1 THEN
          SchliesseFenster(VereinFenster);
          SchliesseFenster(AusgabeFenster);
          RETURN
        END;
        WriteAusgabe(10, 56, Namen[Gast]);
        Insert(out, last, Namen[Gast]);
      UNTIL MyRequest(ADR("Richtig?"), out, "Richtig", "Falsch", FALSE,
                      TRUE);
      Gegner[i,0] := Heim;
      Gegner[i,1] := Gast;
    END;
    SchliesseFenster(VereinFenster);
    AlterStatus := DatenGesichert;
    FOR i := 1 TO AnzahlSpiele DO
      FOR j := 0 TO 1 DO
        AltTore[i,j] := Daten[Gegner[i,j],j,Gegner[i,1-j]];
        Tore[i,j] := 0;
        HalbzeitTore[i,j] := -1;
        GegnerTexts[i,j].iText := ADR(Namen[Gegner[i,j]]);
        Daten[Gegner[i,j],j,Gegner[i,1-j]] := 0;
        AendereStatus;
      END;
      IF i # AnzahlSpiele THEN
        GegnerGadgets[i,1].nextGadget := ADR(GegnerGadgets[i+1,0]);
        TorGadgets[i,1].nextGadget    := ADR(TorGadgets[i+1,0]);
        HalbzeitGadgets[i].nextGadget := ADR(HalbzeitGadgets[i+1]);
      ELSE
        GegnerGadgets[i,1].nextGadget := ADR(TorGadgets[1,0]);
        TorGadgets[i,1].nextGadget    := ADR(HalbzeitGadgets[1]);
        HalbzeitGadgets[i].nextGadget := NIL;
      END;
    END;
    SchliesseFenster(AusgabeFenster);
    IF AnzahlSpiele <= 9 THEN
      Top := 12;
    ELSE
      Top := 0;
    END;
    SpielFenster := OeffneSpezFenster(0, Top, 546, AnzahlSpiele * 8 + 7,
                                      IDCMPFlagSet{gadgetUp},
                                      WindowFlagSet{},
                                      ADR(GegnerGadgets[1,0]), NIL);
    INC(Top, AnzahlSpiele * 8 + 9);
    IF AnzahlSpiele = 10 THEN
      DEC(Top, 1);
    END;
    EndeFenster := OeffneSpezFenster(550, 12, 90, 15,
                                     IDCMPFlagSet{gadgetUp},
                                     WindowFlagSet{}, ADR(EndeGadget), NIL);
    IF (SpielFenster = NIL) OR (EndeFenster = NIL) THEN
      SchliesseFenster(SpielFenster);
      SchliesseFenster(EndeFenster);
      RETURN;   (* Aussichtslos! *)
    END;
    WriteWin(EndeFenster, 29, 10, "ENDE");
    FOR i := 1 TO AnzahlSpiele DO
      SchreibeSpielstand(i);
    END;
    TabellenFenster := OeffneSpezFenster(0, Top, 496, AnzMan * 8 + 8,
                                         IDCMPFlagSet{}, WindowFlagSet{},
                                         NIL, NIL);
    IF TabellenFenster # NIL THEN
      TabelleAusgeben(1, TabellenFenster, FALSE, FALSE);
    END;
    SpieltagTore := 0;
    IF NOT DatenAusgewertet THEN
      DatenAuswertung;
    END;
    SaisonTore := 0;
    SaisonSpiele := 0;
    FOR i := 1 TO AnzMan DO
      INC(SaisonTore, TorePunkte.gsmt[i,1]);
      INC(SaisonSpiele, TorePunkte.gsmt[i,3]);
    END;
    SaisonSpiele := SaisonSpiele DIV 2;
    TorFenster := OeffneSpezFenster(500, Top, 140, 64, IDCMPFlagSet{},
                                    WindowFlagSet{}, NIL, NIL);
    TortextGeschrieben := FALSE;
    SchreibeTore;
    LOOP
      Message := GetMsg(SpielFenster^.userPort);
      IF Message # NIL THEN
        class := Message^.class;
        calledGad := Message^.iAddress;
        ReplyMsg(Message);
        Code := calledGad^.gadgetID;
        IF Code < 0 THEN
          Halbzeit(-Code);
        ELSE
          Tor(Code);
        END;
      END;
      Message := GetMsg(EndeFenster^.userPort);
      IF Message # NIL THEN
        ReplyMsg(Message);
        IF MyRequest(ADR("Spieltag"), "Spieltag beenden?", " Ja ", "Nein",
                     FALSE, TRUE) THEN
          EXIT;
        END;
      END;
    END;
    IF NOT MyRequest(ADR("Spieltag beenden"), "Ergebnisse bernehmen?",
                     " Ja ", "Nein", TRUE, TRUE) THEN
      FOR i := 1 TO AnzahlSpiele DO
        FOR j := 0 TO 1 DO
          Daten[Gegner[i,j],j,Gegner[i,1-j]] := AltTore[i,j];
        END;
      END;
      AendereStatus;
      DatenGesichert := AlterStatus;
    END;
    MacheTitelzeile;
    SchliesseFenster(TabellenFenster);
    SchliesseFenster(SpielFenster);
    SchliesseFenster(TorFenster);
    SchliesseFenster(EndeFenster);
  END SpieltagDurchfuehren;


BEGIN (* Spieltag *)

  (*  --- initialisiere Gadgets ----------------------------------------  *)

  FOR i := 1 TO MaxSpiele DO
    FOR j := 0 TO 1 DO
      InitGadget(GegnerGadgets[i,j], 230 * j + 10, 8 * i - 4, 206, 8,
                 GadgetFlagSet{}, ActivationFlagSet{relVerify}, boolGadget,
                 NIL, ADR(GegnerTexts[i,j]), NIL, i * 2 + j,
                 ADR(GegnerGadgets[i,1]));
      InitGadget(TorGadgets[i,j], 24 * j + 450, 8 * i - 4, 16, 8,
                 GadgetFlagSet{}, ActivationFlagSet{relVerify}, boolGadget,
                 NIL, NIL, NIL, i * 2 + j + (MaxSpiele * 2),
                 ADR(TorGadgets[i,1]));
      InitIntuiText(GegnerTexts[i,j], 3, 0, NIL, 1, 0, jam1, NIL);
    END;  (* FOR j := 0 TO 1 *)
    InitGadget(HalbzeitGadgets[i], 216, 8 * i - 4, 24, 8, GadgetFlagSet{},
               ActivationFlagSet{relVerify}, boolGadget, NIL,
               ADR(HalbzeitText), NIL, -i, NIL);
  END;  (* FOR i := 1 TO MaxSpiele *)

  InitIntuiText(HalbzeitText, 0, 0, ADR(" - "), 1, 0, jam1, NIL);

  InitGadget(EndeGadget, 2, 1, 86, 13, GadgetFlagSet{},
             ActivationFlagSet{relVerify}, boolGadget, NIL, NIL, NIL, 0,
             NIL);


END Spieltag.
