(* --------------------------------------------------------------------
  :Program.       mCD.mod
  :Contents.      mCD: ermglicht Directorywechsel mit Wildcards
  :Contents.      und ohne Angabe des gesamten Pfadnamens
  :Contents.      Funktion hnlich wie Norton-cd auf IBM-PC's
  :Usage.         Try "mCD ?"
  :Author.        Kai Bolay [kai]
  :Address.       Hoffmannstrae 168, 7250 Leonberg 1
  :Phone.         07152/22135
  :Support.       Idea: Ludwig Geromiller
  :History.       v1.0 [kai] 30-May-90 completely rewritten and enhanced
  :History.       v1.1 [kai] 07-Jul-90 enhanced, recompiled => Pure!!!
  :History.       v1.2 [kai] 10-Jul-90 logical bug (splitted too soon) fixed
  :Copyright.     Public Domain
  :Language.      Oberon
  :Translator.    OBERON v1.14
  :Imports.       DosHelp [kai], PatMatch [Bernd Preusing]
  :Remark.        Die Datei ":Tree.mcd" enthlt die Directorystruktur und
  :Remark.        wird bei Bedarf angelegt
  :Remark.        This is fully reentrant (pure); Compile/Link with -md
-------------------------------------------------------------------- *)
MODULE mCD;

IMPORT io,
       Break,
       d   : Dos,
       sd  : SecureDos,
       e   : Exec,
       f   : FileSystem,
       so  : StringOps,
       a   : Arguments,
       dh  : DosHelp,
       p   : PatMatch,
       sys : SYSTEM;

CONST TreeName = ":Tree.mcd";
      oom      = "\x07Out of Memory!\n";
      lockErr  = "\x07Error: Can't Lock()!\n";
      exErr    = "\x07Error. Can't Examine()!\n";
      readErr  = "\x07Error while reading!\n";
      writeErr = "\x07Error while writing!\n";
      notFound = "\x07Directory not found!\n";
      usage    = "Usage: mCD [-s|-t|[Device]Pattern]\n\n       -s:      Scan Device (updates Tree.mcd)\n       -t:      Print tree structure\n       Pattern: Wildcards of Directory\n       Device:  Search on this Device\n";

TYPE DirEntryPtr = POINTER TO DirEntry;
     DirEntry    = RECORD
                      next   : DirEntryPtr;
                      sub    : DirEntryPtr;
                      parent : DirEntryPtr;
                      name   : ARRAY 31 OF CHAR;
                   END; (* RECORD *)
     PathPtr     = POINTER TO Path;
     Path        = RECORD
                      next : PathPtr;
                      name : ARRAY 200 OF CHAR;
                   END; (* RECORD *)

VAR Root    : DirEntryPtr;
    Found   : PathPtr;
    pattern : ARRAY 100 OF CHAR;
    device  : ARRAY  30 OF CHAR;
    aux     : ARRAY 100 OF SHORTINT;
    Me      : d.ProcessPtr;


PROCEDURE FreeTree;

   PROCEDURE FreeDir (Entry : DirEntryPtr);

   BEGIN
     IF Entry # NIL THEN
        FreeDir (Entry^.sub);
        FreeDir (Entry^.next);
        DISPOSE (Entry);
     END; (* IF *)
   END FreeDir;

BEGIN
   FreeDir (Root);
   Root := NIL;
END FreeTree;


PROCEDURE GetTree;

VAR RootLock : d.FileLockPtr;
    Err      : BOOLEAN;
    yn       : ARRAY 4 OF CHAR;

   PROCEDURE ScanDir (Entry : DirEntryPtr);

   VAR DirLock, OldDir    : d.FileLockPtr;
       FIB                : d.FileInfoBlockPtr;
       NewEntry, OldEntry : DirEntryPtr;

   BEGIN
      NEW (FIB);
      IF FIB # NIL THEN
         DirLock := d.Lock (sys.ADR (Entry^.name), d.sharedLock);
         IF DirLock # NIL THEN
            OldDir := d.CurrentDir (DirLock);
            IF (d.Examine (DirLock, FIB)) AND (FIB^.dirEntryType > 0) THEN
               NewEntry := NIL;
               WHILE d.ExNext (DirLock, FIB) DO
                  IF FIB^.dirEntryType > 0 THEN
                     OldEntry := NewEntry;
                     NEW (NewEntry);
                     IF NewEntry # NIL THEN
                        IF OldEntry = NIL THEN
                           Entry^.sub       := NewEntry;
                           NewEntry^.parent := Entry;
                        ELSE
                           OldEntry^.next   := NewEntry;
                           NewEntry^.parent := OldEntry^.parent;
                        END; (* IF *)
                        NewEntry^.next  := NIL;
                        NewEntry^.sub   := NIL;
                        so.Assign (FIB^.fileName, NewEntry^.name);
                        ScanDir (NewEntry); (* rekursiver Aufruf *)
                     ELSE
                        Err := TRUE; io.WriteString (oom);
                     END; (* IF *)
                  END; (* IF *)
               END; (* WHILE *)
            ELSE
               Err := TRUE; io.WriteString (exErr);
            END; (* IF *)
            DirLock := d.CurrentDir (OldDir);
            d.UnLock (DirLock);
         ELSE
            Err := TRUE; io.WriteString (lockErr);
         END; (* IF *)
         DISPOSE (FIB);
      ELSE
         Err := TRUE; io.WriteString (oom);
      END; (* IF *)
   END ScanDir;

BEGIN
   io.WriteString ("Scan Device (y/n)? "); io.ReadString (yn);
   IF CAP (yn[0]) = "Y" THEN
      Err := FALSE;
      NEW (Root);
      IF Root # NIL THEN
         Root^.next   := NIL;
         Root^.sub    := NIL;
         Root^.parent := NIL;
         RootLock := d.Lock (sys.ADR (":"), d.sharedLock);
         IF RootLock # NIL THEN
            dh.GetPathFromLock (Root^.name, RootLock);
            d.UnLock (RootLock);
            ScanDir (Root);
         ELSE
            Err := TRUE; io.WriteString (lockErr);
         END; (* IF *)
      ELSE
         Err := TRUE; io.WriteString (oom);
      END; (* IF *)
      IF Err THEN HALT (20) END;
   ELSE
      HALT (0);
   END; (* IF *)
END GetTree;


PROCEDURE WriteTree;

VAR file : f.File;
    Err  : BOOLEAN;

   PROCEDURE WriteDir (Entry : DirEntryPtr);

      PROCEDURE WriteID (true : BOOLEAN);

      BEGIN
         IF NOT (Err) THEN
            IF true THEN Err := NOT (f.WriteChar (file, "*"))
                    ELSE Err := NOT (f.WriteChar (file, "-")) END;
         END; (* IF *)
      END WriteID;

   BEGIN
      IF Entry # NIL THEN
         WriteID (Entry^.sub  # NIL); WriteID (Entry^.next # NIL);
         IF NOT (Err) THEN
            Err := NOT (f.WriteString (file, Entry^.name));
         END; (* IF *)
         IF Err THEN
            io.WriteString (writeErr);
         ELSE
            WriteDir (Entry^.sub);
            WriteDir (Entry^.next);
         END; (* IF *)
      END; (* IF *)
   END WriteDir;


BEGIN
   IF Root # NIL THEN
      IF f.Open (file, TreeName, TRUE) THEN
         Err := FALSE;
         WriteDir (Root);
         IF NOT (f.Close (file)) OR Err THEN HALT (20) END;
      ELSE
         io.WriteString (writeErr); HALT (20);
      END; (* IF *)
   END; (* IF *)
END WriteTree;


PROCEDURE ReadTree;

VAR file : f.File;
    Err  : BOOLEAN;
    line : ARRAY 100 OF CHAR;

   PROCEDURE ReadDir (VAR Entry : DirEntryPtr; parent : DirEntryPtr);

   VAR ID : ARRAY 2 OF CHAR;

   BEGIN
      IF NOT (Err) AND f.ReadString (file, line) THEN
         NEW (Entry);
         IF Entry # NIL THEN
            Entry^.parent := parent;
            so.SubString (line, Entry^.name, 2, so.Length (line)-2);
            ID[0] := line[0]; ID[1] := line[1];
            IF ID[0] = "*" THEN
               ReadDir (Entry^.sub, Entry);
            ELSE
               Entry^.sub := NIL;
               IF ID[0] # "-" THEN Err:= TRUE; io.WriteString (readErr) END;
            END;
            IF ID[1] = "*" THEN
               ReadDir (Entry^.next, Entry^.parent);
            ELSE
               Entry^.next := NIL;
               IF ID[1] # "-" THEN Err:= TRUE; io.WriteString (readErr) END;
            END;
         ELSE
            Err := TRUE; io.WriteString (oom);
         END; (* IF *)
      ELSE
         IF file.status # f.eof THEN
            Err := TRUE; io.WriteString (readErr);
         END; (* IF *)
         Entry := NIL;
      END; (* IF *)
   END ReadDir;

BEGIN
   Err := FALSE;
   IF NOT (f.Open (file, TreeName, FALSE)) THEN
      io.WriteString ("\x07Can't find Tree file!\n");
      GetTree;
      WriteTree;
   ELSE
      ReadDir (Root, NIL);
      IF NOT (f.Close (file)) OR Err THEN HALT (20) END;
   END; (* IF *)
END ReadTree;


PROCEDURE SearchTree;

VAR last : CHAR;
    new  : PathPtr;

   PROCEDURE SearchDir (Entry : DirEntryPtr);

      PROCEDURE MakePath (Entry : DirEntryPtr);

      BEGIN
         IF Entry^.parent # NIL THEN
            MakePath (Entry^.parent);
            last := new^.name[so.Length (new^.name)-1];
            IF (last # "/") AND (last # ":") THEN so.AppendChar (new^.name, "/") END;
            so.Append (new^.name, Entry^.name);
         ELSE
            so.Assign (Entry^.name, new^.name);
         END; (* IF *)
      END MakePath;

   BEGIN
      IF Entry # NIL THEN
         IF p.Match (pattern, aux, Entry^.name) THEN
            NEW (new);
            IF new # NIL THEN
               MakePath (Entry);
               new^.next := Found;
               Found := new;
            ELSE
               io.WriteString (oom);
            END; (* IF *)
         END; (* IF *)
         SearchDir (Entry^.sub);
         SearchDir (Entry^.next);
      END; (* IF *)
   END SearchDir;

BEGIN
   SearchDir (Root);
END SearchTree;


PROCEDURE PrintTree;

VAR Depth, i : INTEGER;

   PROCEDURE PrintDir (Entry : DirEntryPtr);

   BEGIN
      IF Entry # NIL THEN
         i := Depth; WHILE i > 0 DO io.WriteString ("   "); DEC (i) END;
         io.WriteString (Entry^.name); io.WriteLn;
         INC (Depth);
         PrintDir (Entry^.sub);
         DEC (Depth);
         PrintDir (Entry^.next);
      END; (* IF *)
   END PrintDir;

BEGIN
   Depth := 0;
   PrintDir (Root);
END PrintTree;


PROCEDURE FreeFound;

   PROCEDURE FreePath (p : PathPtr);
   BEGIN IF p # NIL THEN FreePath (p^.next); DISPOSE (p) END; END FreePath;

BEGIN
   FreePath (Found); Found := NIL;
END FreeFound;


PROCEDURE CD (dir : ARRAY OF CHAR);

VAR lock : d.FileLockPtr;
    i    : SHORTINT;

BEGIN
   lock := d.Lock (sys.ADR (dir), d.accessRead);
   IF lock # NIL THEN
      d.UnLock (sd.oldCurrentDir);
      sd.oldCurrentDir := lock;
      IF d.CurrentDir (sd.oldCurrentDir) = NIL THEN END;
      i := 0;
      REPEAT
         INC (i);
         Me^.cli^.setName^[i] := dir[i-1];
      UNTIL (dir[i] = 0X) OR (i = 60); (* I guess! *)
      Me^.cli^.setName^[0] := sys.VAL (CHAR, i);
   ELSE
      io.WriteString (lockErr); HALT (20);
   END; (* IF *)
END CD;


PROCEDURE GoNextCD;

VAR cur : PathPtr;
    cd  : ARRAY 200 OF CHAR;

BEGIN
   dh.GetPathFromLock (cd, sd.oldCurrentDir);
   cur := Found;
   LOOP
      IF cur = NIL THEN EXIT END;
      IF so.Compare (cd, cur^.name) = 0 THEN EXIT END;
      cur := cur^.next;
   END; (* LOOP *)
   IF (cur = NIL) OR (cur^.next = NIL) THEN cur := Found ELSE cur := cur^.next END;
   IF cur # NIL THEN CD (cur^.name) END;
END GoNextCD;


BEGIN
   Me := sys.VAL (d.ProcessPtr, e.FindTask (NIL));
   IF (a.NumArgs () > 1) THEN
      io.WriteString (usage); HALT (20);
   END;
   IF a.NumArgs () = 0 THEN
      IF (Me^.cli # NIL) AND (Me^.cli^.setName # NIL) THEN
         IF d.Write (d.Output(), sys.ADR (Me^.cli^.setName^[1]),
                     ORD (Me^.cli^.setName^[0]))=0 THEN END; io.WriteLn;
      ELSE
         io.WriteString ("\x07Use mCD just from CLI!\n");
      END; (* IF *)
   ELSE
      a.GetArg (1, pattern);
      IF so.Compare ("?", pattern) = 0 THEN
         io.WriteString (usage); HALT (0);
      ELSIF so.Compare ("-s", pattern) = 0 THEN
         GetTree;
         WriteTree;
      ELSIF so.Compare ("-t", pattern) = 0 THEN
         ReadTree;
         PrintTree;
      ELSE
         IF dh.DirExists (pattern) THEN
            IF NOT (dh.GetFullPath (pattern)) THEN
               io.WriteString (notFound); HALT (20);
            END; (* IF *)
            CD (pattern);
         ELSE
            dh.SplitPath (pattern, device, pattern);
            IF device[0] # 0X THEN
               IF device[so.Length(device)-1] # ":" THEN
                  io.WriteString ("\x07Is this a device??\n"); HALT (20);
               END; (* IF *)
               IF NOT (dh.GetFullPath (device)) THEN
                  io.WriteString ("\x07Device not found!\n"); HALT (20);
               END; (* IF *)
               CD (device);
            END; (* IF *)
            IF p.CmplPat (pattern, aux) THEN
               ReadTree;
               SearchTree;
               IF Found # NIL THEN
                  GoNextCD;
               ELSE
                  io.WriteString (notFound);
               END; (* IF *)
            ELSE
               io.WriteString ("\x07Invalid pattern!\n"); HALT (20);
            END; (* IF *)
         END; (* IF *)
      END; (* IF *)
   END; (* IF *)
CLOSE
   FreeTree;
   FreeFound;
END mCD.
