(**********************************************************************

    :Program.    ShowHunks.mod
    :Contents.   CLI Command - lists all hunks of a load file
    :Author.     Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Public Domain
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    DosSupport1.2 [bne], DisAss [bep]
    :History.    V1.0b [bne] 13.Mar.1989
    :History.    V1.1a [bne] 14.Mar.1989 (+ WriteCode)
    :History.    V1.1b [fbs] 22-Jun-89   removed bug with hunkext
    :History.    V1.1c [fbs] 26-Jun-90   ExtDRef16 support
    :History.    V1.1d [fbs] 26-Jun-90   Supports ChipMem-Hunks
    :Usage.      SHOWHUNKS FILE/A

**********************************************************************)

MODULE ShowHunks;

FROM Arguments  IMPORT NumArgs,GetArg;
FROM Arts       IMPORT Terminate,DetectCtrlC;
FROM DisAss     IMPORT Disass,Line;
FROM DosSupport IMPORT HUNKUNIT,HUNKNAME,HUNKCODE,HUNKDATA,HUNKBSS,
                HUNKRELOC32,HUNKRELOC16,HUNKRELOC8,HUNKEXT,HUNKSYMBOL,
                HUNKDEBUG,HUNKEND,HUNKHEADER,HUNKCHIPMEM,HUNKFASTMEM;
FROM FileSystem IMPORT File,Response,Lookup,Close,ReadBytes,ReadByteBlock;
FROM InOut      IMPORT WriteString,WriteInt,WriteHex,WriteLn;
FROM MemSystem  IMPORT NoCareAllocate,Deallocate;
FROM SYSTEM     IMPORT ADDRESS,ADR,CAST,LONGSET,SHIFT;

CONST   MaxLen=79;
        HUNKDRELOC16=3F8H;

TYPE    StringLen=[0..MaxLen];
        String=ARRAY StringLen OF CHAR;

VAR     ObjFile:File;
        FileName:String;
        Len:INTEGER;
        HunkNum:INTEGER;

(**)
(* Input procedures                                                     *)
(**)
PROCEDURE ReadLong(VAR LongWord:LONGINT);
BEGIN
  ReadByteBlock(ObjFile,LongWord);
END ReadLong;

PROCEDURE ReadName(Size:StringLen;VAR Name:String);
VAR     ActSize:LONGINT;
BEGIN
  Size:=Size*4;
  ReadBytes(ObjFile,ADR(Name),Size,ActSize);
  IF Size<=MaxLen THEN
    Name[Size]:=CHAR(0);
  END;
END ReadName;

(**)
(* Output procedures                                                    *)
(**)
PROCEDURE Write(Text:ARRAY OF CHAR);
BEGIN
  WriteString(Text);
  WriteLn;
END Write;

PROCEDURE WriteTab(Text:ARRAY OF CHAR;Val:LONGINT);

BEGIN
  WriteString(Text);
  WriteHex(Val,8); WriteString("  ("); WriteInt(Val,1); WriteString(")");
  WriteLn;
END WriteTab;

PROCEDURE WriteData(Size:LONGINT);
VAR     Row,Data:LONGINT;
        Column,CharNum:INTEGER;
        Char:CHAR;
        Ascii:ARRAY [0..16] OF CHAR;
BEGIN
  Size:=Size*4;
  FOR Row:=0 TO Size-1 BY 16 DO
    WriteHex(Row,6);
    WriteString(": ");
    Column:=0;
    REPEAT
      ReadLong(Data);
      WriteHex(Data,8);
      WriteString(" ");
      FOR CharNum:=0 TO 3 DO
        Char:=CHAR(SHIFT(Data,-8*(3-CharNum))MOD 256);
        IF (Char<" ") OR (Char>CHAR(127))AND(Char<CHR(160)) THEN
          Char:=".";
        END;
        Ascii[Column+CharNum]:=Char;
      END;
      INC(Column,4);
    UNTIL (Column=16) OR (Row+Column=Size);
    Ascii[Column]:=CHAR(0);
    Write(Ascii);
  END;
END WriteData;

PROCEDURE WriteCode(Size:LONGINT);
VAR     Segment,CodeAdr,CodeEnd:ADDRESS;
        ActSize:LONGINT;
        Command:Line;
BEGIN
  IF Size=0 THEN RETURN END;
  Size:=Size*4;
  NoCareAllocate(Segment,Size);
  ReadBytes(ObjFile,Segment,Size,ActSize);
  CodeAdr:=Segment;
  CodeEnd:=LONGINT(Segment)+ActSize;
  WHILE CodeAdr<CodeEnd DO
    CodeAdr:=Disass(ADR(Command),CodeAdr,-LONGINT(Segment));
    Write(Command.str);
  END;
  Deallocate(Segment);
END WriteCode;

PROCEDURE WriteName;
VAR     Size:LONGINT;
        Name:String;
BEGIN
  ReadLong(Size);
  ReadName(Size,Name);
  Write(Name);
END WriteName;

PROCEDURE WriteSymbols;
CONST   symb    = 0;
        def     = 100H;
        abs     = 200H;
        res     = 300H;
        ref32   = 8100H;
        common  = 8200H;
        ref16   = 8300H;
        ref8    = 8400H;
        dref16  = 8600H;
VAR     Type: CARDINAL;
        Size:INTEGER;
        Value,Count:LONGINT;
        Name:String;
BEGIN
  LOOP
    ReadByteBlock(ObjFile,Type);
    ReadByteBlock(ObjFile,Size);
    IF (Type=0) AND (Size=0) THEN EXIT END;
    ReadName(Size,Name);
    IF Type<ref32 THEN
      ReadLong(Value);
      CASE Type OF
        |symb: WriteString("symbol definition     : ");
        |def:  WriteString("relocatable definition: ");
        |abs:  WriteString("absolute definition   : ");
        |res:  WriteString("library definition    : ");
      END;
      WriteString(Name); WriteTab(" = ",Value);
    ELSIF Type#common THEN
      CASE Type OF
        |ref32: WriteString("32");
        |ref16,dref16: WriteString("16");
        |ref8:  WriteString(" 8")
      END;
      WriteString(" bit ");
      IF Type=dref16 THEN WriteString("data-") END;
      WriteString("references on: ");
      Write(Name);
      ReadLong(Count);
      WriteData(Count);
    ELSE
      WriteString("common block: ");
      Write(Name);
      ReadLong(Count);
      WriteTab("size: ",Count);
      Write("references:");
      ReadLong(Count);
      WriteData(Count);
    END; (* if *)
  END; (* loop *)
END WriteSymbols;

(**)
(* Hunks                                                                *)
(**)
PROCEDURE HunkHeader;
VAR     Size,First,Last,Hunk:LONGINT;
        Name:String;
BEGIN
  Write("HUNK_HEADER:");
  Write("resident libraries:");
  LOOP
    ReadLong(Size);
    IF Size=0 THEN EXIT END;
    ReadName(Size,Name);
    Write(Name);
  END;
  ReadLong(Size);
  WriteTab("Number of hunks: ",Size);
  ReadLong(First);
  WriteTab("First hunk to load: ",First);
  ReadLong(Last);
  WriteTab("Last hunk to load: ",Last);
  Write("Size of hunks:");
  FOR Hunk:=First TO Last DO
    ReadLong(Size);
    WriteInt(Hunk,2);
    WriteString(": $");
    WriteHex(Size MOD 40000000H * 4,8);
    WriteString("  ("); WriteInt(Size MOD 40000000H * 4,1);
    WriteString(")");
    IF ODD(Size DIV 40000000H) THEN WriteString(" CHIPMEM") END;
    WriteLn;
  END;
END HunkHeader;

PROCEDURE HunkUnit;
BEGIN
  Write("HUNK_UNIT");
  WriteName;
END HunkUnit;

PROCEDURE HunkName;
BEGIN
  Write("HUNK_NAME");
  WriteName;
END HunkName;

PROCEDURE HunkCode;
VAR     Size:LONGINT;
BEGIN
  WriteTab("HUNK_CODE ",HunkNum);
  ReadLong(Size);
  WriteCode(Size);
END HunkCode;

PROCEDURE HunkData;
VAR     Size:LONGINT;
BEGIN
  WriteTab("HUNK_DATA ",HunkNum);
  ReadLong(Size);
  WriteData(Size);
END HunkData;

PROCEDURE HunkBss;
VAR     Size:LONGINT;
BEGIN
  WriteTab("HUNK_BSS ",HunkNum);
  ReadLong(Size);
  WriteTab("Size: ",Size);
END HunkBss;

PROCEDURE HunkReloc(n: INTEGER);
VAR     Offsets,Num:LONGINT;
BEGIN
  WriteString("HUNK_RELOC"); WriteInt(n,2); WriteLn;
  LOOP
    ReadLong(Offsets);
    IF Offsets=0 THEN EXIT END;
    ReadLong(Num);
    WriteTab("Offsets in Hunk no: ",Num);
    WriteData(Offsets);
  END;
END HunkReloc;

PROCEDURE HunkDReloc(n: INTEGER);
VAR     Offsets,Num:LONGINT;
BEGIN
  WriteString("HUNK_DRELOC"); WriteInt(n,2); WriteLn;
  LOOP
    ReadLong(Offsets);
    IF Offsets=0 THEN EXIT END;
    ReadLong(Num);
    WriteTab("Offsets in Hunk no: ",Num);
    WriteData(Offsets);
  END;
END HunkDReloc;

PROCEDURE HunkExt;
BEGIN
  Write("HUNK_EXT");
  WriteSymbols;
END HunkExt;

PROCEDURE HunkSymbol;
BEGIN
  Write("HUNK_SYMBOL");
  WriteSymbols;
END HunkSymbol;

PROCEDURE HunkDebug;
VAR     Size:LONGINT;
BEGIN
  Write("HUNK_DEBUG");
  ReadLong(Size);
  WriteData(Size);
END HunkDebug;

PROCEDURE HunkEnd;
BEGIN
  Write("HUNK_END");
  INC(HunkNum);
END HunkEnd;

(**)
(* Main                                                                 *)
(**)
PROCEDURE ListHunks;
VAR     Hunk:LONGINT;
BEGIN
  HunkNum:=0;
  WHILE NOT ObjFile.eof DO
    ReadLong(Hunk);
    IF 30 IN CAST(LONGSET,Hunk) THEN
      WriteString("CHIPMEM-");
      DEC(Hunk,40000000H);
    ELSIF 31 IN CAST(LONGSET,Hunk) THEN
      WriteString("FASTMEM-");
      INC(Hunk,40000000H);
      INC(Hunk,40000000H);
    END;
    CASE Hunk OF
      |HUNKHEADER: HunkHeader;
      |HUNKUNIT: HunkUnit;
      |HUNKNAME: HunkName;
      |HUNKCODE: HunkCode;
      |HUNKDATA: HunkData;
      |HUNKBSS: HunkBss;
      |HUNKRELOC32: HunkReloc(32);
      |HUNKRELOC16: HunkReloc(16);
      |HUNKDRELOC16: HunkDReloc(16);
      |HUNKRELOC8:  HunkReloc(8);
      |HUNKEXT: HunkExt;
      |HUNKSYMBOL: HunkSymbol;
      |HUNKDEBUG: HunkDebug;
      |HUNKEND: HunkEnd;
    ELSE
      WriteString("unknown hunk type: ");
      WriteHex(Hunk,4);
      WriteLn;
      Terminate(0);
    END;
  END;
END ListHunks;

BEGIN
  DetectCtrlC(TRUE);
  IF NumArgs()=1 THEN
    GetArg(1,FileName,Len);
    IF (FileName[0]#"?") OR (Len#1) THEN
      Lookup(ObjFile,FileName,512,FALSE);
      IF ObjFile.res=done THEN
        ListHunks;
        Close(ObjFile);
      ELSE
        Write("File not found");
      END;
      RETURN
    END;
  END;
  Write("USAGE: SHOWHUNKS FILE/A");
END ShowHunks.

