(**********************************************************************

    :Program.    StringForm.mod
    :Contents.   string formatting procedures
    :Author.     Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Public Domain
    :Language.   Oberon
    :Translator. OBERON v1.01
    :History.    V1.0 [bne] 11.May.1989
    :History.    V1.1 [bne] 30.Dec.1989 (bug fixes)
    :History.    V2.0 [kai] 09-May-1990 Ported to OBERON

**********************************************************************)

MODULE StringForm;

IMPORT so : StringOps;

(* All procedures will truncate the string if it was too long to
   fit into the array boundaries. *)

CONST
  nul=CHR(0);
  space=" ";

PROCEDURE AppendBlanks * (VAR String:ARRAY OF CHAR;
                           Width:INTEGER);
(*:Semantic.  Appends blanks to <String> until it has the length <Width>.
*)

  VAR
    Pos,Max:INTEGER;
  BEGIN
    IF Width<=LEN(String) THEN
      Max:=Width-1;
    ELSE
      Max:=LEN(String);
    END;
    Pos:=so.Length(String);
    WHILE Pos<=Max DO
      String[Pos]:=space;
      INC(Pos);
    END;
    IF Pos<=LEN(String) THEN
      String[Pos]:=nul;
    END;
  END AppendBlanks;

PROCEDURE LeftInsertChar (VAR String: ARRAY OF CHAR;
                             Char: CHAR;
                             Num: INTEGER);
  VAR
    Pos,Max:INTEGER;
  BEGIN
    IF Num <= LEN (String) THEN
      Max:= so.Length(String) + Num;
      IF Max > LEN (String) THEN
        Max:= LEN (String);
      END;
      Pos:= Max - Num;
      WHILE Pos >= 0 DO
        String[Max]:= String[Pos];
        DEC (Max);
        DEC (Pos);
      END;
      DEC (Num);
    ELSE
      Num:= LEN (String);
    END;
    Pos := 0;
    WHILE Pos <= Num DO
      String[Pos]:= Char;
      INC (Pos);
    END;
  END LeftInsertChar;

PROCEDURE Indent * (VAR String:ARRAY OF CHAR;
                     Margin:INTEGER);
(*:Semantic.  Inserts <Margin> blanks on the left side of <String>.
*)
  BEGIN
    LeftInsertChar(String,space,Margin);
  END Indent;

PROCEDURE CenterAdjust * (VAR String:ARRAY OF CHAR;
                           Width:INTEGER);
(*:Semantic.  Centers <String> within a field <Width> chars wide.
*)
  VAR
    Len:INTEGER;
  BEGIN
    Len:=so.Length(String);
    IF Len<Width THEN
      Indent(String,(Width-so.Length(String)) DIV 2);
      AppendBlanks(String,Width);
    END;
  END CenterAdjust;

PROCEDURE CutBlanks * (VAR String:ARRAY OF CHAR);

(*:Semantic.  Cuts leading and trailing blanks.
*)

  VAR
    Pos:INTEGER;
  BEGIN
    Pos:=0;
    WHILE (Pos<=LEN (String)) AND (String[Pos]=space) DO
      INC(Pos);
    END;
    so.DeleteSubString(String,0,Pos);
    Pos:=so.Length(String)-1;
    WHILE (Pos>=0) AND (String[Pos]=space) DO
      DEC(Pos);
    END;
    INC(Pos);
    IF Pos<=LEN (String) THEN
      String[Pos]:=nul;
    END;
  END CutBlanks;

PROCEDURE RightAdjust * (VAR String:ARRAY OF CHAR;
                          Width:INTEGER);
(*:Semantic.  Right-adjusts <String> within a field <Width> chars wide.
*)
  VAR
    Len:INTEGER;
  BEGIN
    CutBlanks(String);
    Len:=so.Length(String);
    IF Len<Width THEN
      Indent(String,Width-Len);
    END;
  END RightAdjust;

PROCEDURE EmptyString * (String:ARRAY OF CHAR):BOOLEAN;
(*:Result.    TRUE, if <String> has the length 0 or consists of blanks only.
*)
  VAR
    Pos:INTEGER;
  BEGIN
    Pos := 0;
    WHILE Pos <= LEN(String) DO
      IF String[Pos]=nul THEN
        RETURN TRUE;
      ELSIF String[Pos]#space THEN
        RETURN FALSE;
      END;
      INC (Pos);
    END;
    RETURN TRUE;
  END EmptyString;

PROCEDURE LeftInsertZeroes * (VAR String:ARRAY OF CHAR;
                               Width:INTEGER);
(*:Semantic.  Inserts zeroes until the length of <String> is <Width>.
*)

  VAR
    Len:INTEGER;
  BEGIN
    CutBlanks(String);
    Len:=so.Length(String);
    IF Len<Width THEN
      LeftInsertChar(String,"0",Width-Len);
    END;
  END LeftInsertZeroes;

END StringForm.

