MODULE tridemo; (* jr/7sep87 *)

FROM SYSTEM IMPORT
 ADDRESS, ADR, BYTE, SHIFT;
FROM Arts IMPORT
 Assert;
FROM Graphics IMPORT
 AreaInfo, RastPortPtr, TmpRas, ViewPortPtr, ViewModeSet,
 AllocRaster, AreaMove, AreaDraw, AreaEnd, FreeRaster, InitArea, InitTmpRas,
 SetAPen, SetRGB4;
FROM Intuition IMPORT
 stdScreenHeight, customScreen, ScreenPtr, NewScreen,
 OpenScreen, CloseScreen;
FROM RandomNumber IMPORT
 RND;

CONST
 WIDTH=320;
 DEPTH=5;
 POINTS=3;

VAR
 scr: ScreenPtr;
 rp: RastPortPtr;
 trp: ADDRESS;
 creg: CARDINAL;
 MaxHeight: INTEGER;

TYPE
 VPoint=RECORD
  x, y, xv, yv: INTEGER; (* position and velocity of point *)
 END;


PROCEDURE CreateScreen(w, h, d: INTEGER; title: ADDRESS): ScreenPtr;
 VAR ns: NewScreen;
 BEGIN
  ns.leftEdge:=0; ns.topEdge:=0; ns.width:=w; ns.height:=h; ns.depth:=d;
  ns.detailPen:=0; ns.blockPen:=1;
  ns.defaultTitle:=title;
  ns.viewModes:=ViewModeSet{}; ns.type:=customScreen;
  ns.font:=NIL; ns.gadgets:=NIL; ns.customBitMap:=NIL;
  RETURN OpenScreen(ns)
 END CreateScreen;


PROCEDURE MovePoint(VAR p: VPoint);
 VAR n: INTEGER;
 BEGIN
  WITH p DO
   n:=x+xv;
   IF n>=WIDTH THEN                      (* Bounce from right side *)
    xv:=-xv; x:=2*WIDTH-1-n
   ELSIF n<0 THEN                        (* Bounce from left side *)
    xv:=-xv; x:=-n
   ELSE
    x:=n
   END;

   n:=y+yv;
   IF n>=MaxHeight THEN                  (* Bounce from bottom side *)
    yv:=-yv; y:=2*MaxHeight-1-n
   ELSIF n<11 THEN                       (* Bounce from top side *)
    yv:=-yv; y:=21-n
   ELSE
    y:=n
   END
  END
 END MovePoint;


PROCEDURE Show(VAR p: ARRAY OF VPoint);
 VAR i: INTEGER; r:BOOLEAN;
 BEGIN
  creg:=creg MOD 31+1; SetAPen(rp, creg);
  r:=AreaMove(rp, p[0].x, p[0].y);
  FOR i:=1 TO POINTS-1 DO
   r:=AreaDraw(rp, p[i].x, p[i].y)
  END;
  r:=AreaEnd(rp)
 END Show;


PROCEDURE Demo;
 VAR
  i: INTEGER;
  pts: ARRAY [0..POINTS-1] OF VPoint;
  frames: CARDINAL;
 BEGIN
  FOR i:=0 TO POINTS-1 DO
   WITH pts[i] DO
    x:=RND(WIDTH); y:=RND(MaxHeight);
    REPEAT xv:=RND(16)-8 UNTIL xv#0;
    REPEAT yv:=RND(16)-8 UNTIL yv#0
   END
  END;

  FOR frames:=0 TO 1000 DO
   Show(pts);
   FOR i:=0 TO POINTS-1 DO MovePoint(pts[i]) END
  END
 END Demo;


PROCEDURE InitColors;
 VAR
  i: INTEGER;
  vp: ViewPortPtr;
 BEGIN
  vp:=ADR(scr^.viewPort);
  FOR i:=0 TO 7 DO
   SetRGB4(vp, i,    8,    0, 8+i );
   SetRGB4(vp, i+8,  8+i,  0, 15  );
   SetRGB4(vp, i+16, 15,   0, 15-i);
   SetRGB4(vp, i+24, 15-i, 0, 8   )
  END;
  SetRGB4(vp, 0, 0, 0, 0)
 END InitColors;


PROCEDURE Cleanup;
 BEGIN
  IF trp#NIL THEN FreeRaster(trp, WIDTH, MaxHeight) END;
  IF scr#NIL THEN CloseScreen(scr) END
 END Cleanup;

VAR
 ai: AreaInfo;
 tr: TmpRas;
 abuf: ARRAY [0..POINTS*5] OF CARDINAL;
BEGIN
 trp:=NIL; scr:=NIL;
 scr:=CreateScreen(WIDTH, stdScreenHeight, DEPTH, ADR("Triangle Demo"));
 Assert(scr#NIL, ADR('cannot open screen'));

 MaxHeight:=scr^.height;
 rp:=ADR(scr^.rastPort);
 InitColors;

 InitArea(ai, ADR(abuf), SIZE(abuf) DIV 5);
 trp:=AllocRaster(WIDTH, MaxHeight);
 Assert(trp#NIL, ADR('cannot alloc raster'));
 InitTmpRas(tr, trp, ((WIDTH+15) DIV 16) * MaxHeight);
 rp^.areaInfo:=ADR(ai);
 rp^.tmpRas:=ADR(tr);

 Demo
CLOSE
 Cleanup
END tridemo.mod
