(*---------------------------------------------------------------------------
:Program.        SwapIconCols.mod
:Contents.       Swap the colors 1 and 2 of an icon
:Author.         Christian Stiens
:Address.        Heustiege 2, W-4710 Ldinghausen
:Copyright.      PD
:Language.       Oberon
:Translator.     Amiga Oberon V1.17 A+L
:History.        V1.0, 23-Feb-91
:Usage.          SwapIconCols <wildcards> [...]
:Remark.         needs arp.library
---------------------------------------------------------------------------*)

MODULE SwapIconCols;

  (* $NilChk- $RangeChk- $OvflChk- $CaseChk- $ReturnChk- *)

  IMPORT
    a  : Arguments,
    d  : Dos,
    ic : Icon,
    I  : Intuition,
    s  : SYSTEM,
    st : Strings,
    wb : Workbench,
    io,ARP;

  CONST
    usage  = "Usage: SwapIconCols <wildcards> [...]\n";

  VAR
    args : INTEGER;
    nr   : INTEGER;
    ap   : ARP.AnchorPathPtr;
    buf  : POINTER TO ARRAY 256 OF CHAR;
    pat  : ARRAY 256 OF CHAR;

  PROCEDURE SwapCols(im: I.ImagePtr);
    VAR p1,p2: POINTER TO INTEGER;
        i,temp,words: INTEGER;
  BEGIN
    IF (im # NIL) & (im.imageData # NIL) & (im.depth >= 2) THEN
      words := im.height * ((im.width+15) DIV 16);
      p1 := im.imageData;
      p2 := s.VAL(LONGINT,p1) + LONG(words) * 2;
      i := 0; WHILE i < words DO
        temp := p1^; p1^ := p2^; p2^ := temp;
        INC(p1,2); INC(p2,2); INC(i);
      END;
    END;
  END SwapCols;

  PROCEDURE SwapIcon(name: ARRAY OF CHAR); (* $CopyArrays- *)
    VAR icon : wb.DiskObjectPtr;
        im   : I.ImagePtr;
        i    : INTEGER;
        info : ARRAY 256 OF CHAR;
        lock : d.FileLockPtr;
  BEGIN
    IF name[0] # 0X THEN
      COPY(name,info);
      st.Append(info,".info");
      lock := ARP.Lock(info,d.sharedLock);
      IF lock # NIL THEN
        ARP.UnLock(lock);
        io.WriteString(name); io.WriteString(".info..");
        icon := ic.GetDiskObject(name);
        IF icon # NIL THEN
          IF I.gadgImage IN icon.gadget.flags THEN
            im := icon.gadget.gadgetRender;
            SwapCols(im);
          END;
          IF I.gadgHImage IN icon.gadget.flags THEN
            im := icon.gadget.selectRender;
            SwapCols(im);
          END;
          IF NOT ic.PutDiskObject(name,icon) THEN
            io.WriteString("write error\n");
          ELSE
            io.WriteString("colors swapped\n");
          END;
          ic.FreeDiskObject(icon);
        ELSE
          io.WriteString("not found\n");
        END; (* IF icon # NIL *)
      END;
    END; (* IF name[0] # 0X *)
  END SwapIcon;

BEGIN
  args := a.NumArgs();
  a.GetArg(1,pat);
  IF (args = 0) OR (pat[0] = "?") THEN
    io.WriteString(usage);
    HALT(0)
  END;

  nr := 1;
  WHILE nr <= args DO
    a.GetArg(nr,pat);

    ap := ARP.ArpAlloc(s.SIZE(ap^) + s.SIZE(buf^));
    IF ap = NIL THEN io.WriteString("No memory!\n"); HALT(0) END;
    ap.strLen := s.SIZE(buf^); ap.breakBits := LONGSET{d.ctrlC,d.ctrlD};
    buf := s.VAL(LONGINT,ap) + s.SIZE(ap^);

    IF ARP.FindFirst(pat,ap)=0 THEN
      REPEAT
        SwapIcon(buf^);
      UNTIL ARP.FindNext(ap)#0;
    END;
    ARP.FreeAnchorChain(ap);

    INC(nr);
  END; (* WHILE *)
END SwapIconCols.

