(*----------------------------------------------------------------------
  :Program.    CreateModule.mod
  :Contents.   Erstellt Modula-Quellcode zum Aufbau einer Gadget-Struktur.
  :Author.     Hubert Bildstein
  :Copyright.  Public Domain
  :Language.   Modula-2
  :Translator. M2Amiga V3.3d
  :History.    V1.0   5.12.1990
----------------------------------------------------------------------*)

IMPLEMENTATION MODULE CreateModule;
(* Erstellen des MODULA-Quellcodes *)

FROM SYSTEM      IMPORT ADR;
FROM ASCII       IMPORT eol, nul;
FROM Str         IMPORT Length, Concat, Copy;
FROM FileSystem  IMPORT File, Response, Lookup, Close, WriteBytes, WriteChar;
FROM FileNames   IMPORT GetPath;
FROM FileNames1  IMPORT GetExtension;
FROM Conversions IMPORT ValToStr;
FROM Intuition   IMPORT strGadget, propGadget, boolGadget, ActivationFlags;
FROM Gadgets     IMPORT PropType, PropTypeSet;
FROM DataStruct  IMPORT BorderType, GadgDefType, WindowDefType,
                        Bord1x, Bord1y, Bord2x, Bord2y;

VAR f   : File;
    act : LONGINT;

(*--------------------------------------------------------------------------*)
(* interne Prozeduren *)
(*--------------------------------------------------------------------------*)

PROCEDURE wr (Text : ARRAY OF CHAR);
(* Schreiben eines Strings *)

BEGIN
 WriteBytes (f,ADR(Text),Length(Text),act);
END wr;

(*--------------------------------------------------------------------------*)

PROCEDURE wi (Zahl : INTEGER);
(* Schreiben einer Zahl *)

VAR String : ARRAY [1..10] OF CHAR;
    err    : BOOLEAN;

BEGIN
 ValToStr (Zahl,TRUE,String,10,1," ",err);
 wr (String);
END wi;

(*--------------------------------------------------------------------------*)

PROCEDURE cr (Anzahl : INTEGER);
(* Zeilenende "Anzahl" mal schreiben *)
VAR i : INTEGER;
BEGIN

 FOR i:=1 TO Anzahl DO
   WriteChar (f,eol);
 END; (*FOR*)

END cr;
(*--------------------------------------------------------------------------*)
PROCEDURE cr1;  (* Ein Zeilenende *)
BEGIN
 cr(1);
END cr1;
(*--------------------------------------------------------------------------*)
PROCEDURE cr2;  (* Zwei Zeilenende *)
BEGIN
 cr(2);
END cr2;

(*--------------------------------------------------------------------------*)

PROCEDURE WriteAFlags (flag : ActivationFlags);
(* Schreibt ein Element von ActivationFlags als Klartext *)
BEGIN
 CASE flag OF
 |  relVerify     : wr ("relVerify");
 |  gadgImmediate : wr ("gadgImmediate");
 |  toggleSelect  : wr ("toggleSelect");
 |  stringCenter  : wr ("stringCenter");
 |  stringRight   : wr ("stringRight");
 |  longint       : wr ("longint");
 ELSE
 END; (*CASE*)
END WriteAFlags;

(*--------------------------------------------------------------------------*)

PROCEDURE WriteType (Typ : CARDINAL);
(* Schreibt gadgetType im Klartext *)
BEGIN
 CASE Typ OF
 |  strGadget  : wr ("strGadget ");
 |  boolGadget : wr ("boolGadget");
 |  propGadget : wr ("propGadget");
 ELSE
 END; (*CASE*)
END WriteType;

(*--------------------------------------------------------------------------*)
(* Erstellen der Module *)
(*--------------------------------------------------------------------------*)

PROCEDURE Create (Filename : ARRAY OF CHAR;
                  NumbGadg : INTEGER;
                  GadgDefs : ARRAY OF GadgDefType;
                  WinDef   : WindowDefType) : Response;
(* Erstellen eines ausfhrlichen MODULA-Quellcodes, der das Window samt
   Gadget-Struktur erstellt. Es wird ein Definition- und ein Implementation-
   Module unter dem angegebenen Namen erstellt. *)

CONST FileBuffer = 512;

TYPE FileNameTyp = ARRAY [1..60] OF CHAR;

VAR NameCopy   : FileNameTyp;
    RawName  : FileNameTyp;
    ext, path  : FileNameTyp;
    len        : INTEGER;
    NumbStr    : INTEGER;   (* Anzahl String-Gadgets *)
    NumbProp   : INTEGER;   (* Anzahl Prop-Gadgets *)
    NumbBorder : INTEGER;   (* Anzahl Rahmen (insges.) *)
    NumbText   : INTEGER;   (* Anzahl Gadget-Texte (insges.) *)
    NumbXY     : INTEGER;   (* Anzahl xy-Arrays fr Rahmen *)
    Numb1Char  : INTEGER;   (* Anzahl 1-buchstabiger Gadget-Texte *)
    i, iB, iT  : INTEGER;
    iS, iP, iX : INTEGER;
    iC         : INTEGER;
    first      : BOOLEAN;
    actFlags   : ActivationFlags;

BEGIN

(* Anzahl der Prop- und String-Gadgets, der Rahmen und Texte ermitteln *)

 iB := 0; iT := 0; iS := 0; iP := 0; iX := 0; iC := 0;
 NumbStr := 0; NumbProp := 0; NumbBorder := 0; NumbText := 0; NumbXY := 0;
 Numb1Char := 0;

 FOR i:=0 TO NumbGadg-1 DO
    IF (GadgDefs[i].type = strGadget) THEN
       INC (NumbStr);
    ELSIF (GadgDefs[i].type = propGadget) THEN
       INC (NumbProp);
    END; (*IF*)

    IF (GadgDefs[i].type = propGadget) THEN
       INC (NumbBorder);
    ELSE
       IF (GadgDefs[i].border = Single) THEN
          INC (NumbBorder); INC (NumbXY);
       ELSIF (GadgDefs[i].border = Double) THEN
          INC (NumbBorder,2); INC (NumbXY,2);
       END; (*IF*)
    END; (*IF*)

    IF (GadgDefs[i].text[1] # nul) THEN
       INC (NumbText);
    END; (*IF*)

    IF (Length(GadgDefs[i].text) = 1) THEN  (* Sonderbehandlung fr *)
       INC (Numb1Char);                     (* 1-buchstabigen Text, da *)
    END; (*IF*)                             (* ADR("x") nicht geht. (s.u.) *)
 END; (*FOR*)

(* DEFINITION-MODULE erstellen *)
(*-----------------------------*)

(* File erffnen *)
 Copy (NameCopy, Filename);
 Copy (RawName, Filename);
 Concat (Filename,".def");
 GetExtension (RawName,ext,len);     (* Extension abtrennen *)
 GetPath (RawName,path,len);         (* Pfad abtrennen *)

 Lookup (f,Filename,FileBuffer,TRUE);
 IF (f.res # done) THEN Close (f); RETURN f.res END;

 wr ("DEFINITION MODULE "); wr (RawName); wr(";"); cr1;
 wr ("(* Aufbau einer Gadget-Struktur in einem Window *)"); cr1;
 wr ("(* Modul erstellt mit GadgetEd V1.0 *)");cr2;

 wr ("FROM SYSTEM IMPORT ADDRESS;"); cr2;

 wr ("CONST NumbGadg"); wr(RawName); wr (" = "); wi(NumbGadg);
 wr (";   (* Anzahl der Gadgets *)"); cr2;

 wr ("PROCEDURE Init"); wr(RawName); cr1;
 wr ("                        (VAR wptr   : ADDRESS;"); cr1;
 wr ("                             Screen : ADDRESS;"); cr1;
 wr ("                             Title  : ADDRESS);"); cr1;
 wr ("(* Aufbauen der Gadget-Struktur. "); cr1;
 wr ("   wptr  : Zeiger auf das Window, in das die Gadget-Struktur ein-");cr1;
 wr ("           gebaut werden soll."); cr1;
 wr ("           Falls wptr = NIL wird ein Standard-Window, mit Gre");cr1;
 wr ("           und Position wie im Editor definiert, erffnet.");cr2;

 wr ("   Diese Parameter haben nur fr wptr = NIL eine Wirkung:");cr1;
 wr ("   Screen: Zeiger auf Screen, in dem das Window erscheinen soll."); cr1;
 wr ("           NIL fr WorkbenchScreen."); cr1;
 wr ("   Title : Zeiger auf Namen des Windows *)"); cr2;

 wr ("PROCEDURE Close"); wr(RawName); wr (";"); cr1;
 wr ("(* Schlieen des Gadget-Windows. *)"); cr2;

 wr ("PROCEDURE Get"); wr(RawName); wr("GPtr (ID : INTEGER) : ADDRESS;");cr1;
 wr ("(* Liefert den GadgetPtr des Gadgets mit der angegebenen ID. *)"); cr2;

 wr ("(*"); cr1;
 wr ("Liste der Gadgets mit IDs:"); cr1;
 wr ("--------------------------"); cr2;

 wr ("Bereich der IDs: 0 .. NumbGadg-1"); cr2;

 wr ("ID  Typ         Text des Gadgets"); cr1;
 wr ("----------------------------------------------------------"); cr1;
 FOR i:=0 TO NumbGadg-1 DO
     IF (i <= 9) THEN wi(0) END;
     wi (i); wr ("  ");                          (* ID *)
     WriteType (GadgDefs[i].type); wr ("   ");   (* Typ *)
     wr (GadgDefs[i].text); cr1;                 (* Text *)
 END; (*FOR*)
 wr ("*)"); cr2;

 wr ("END "); wr (RawName); wr("."); cr1;

 IF (f.res # done) THEN Close (f); RETURN f.res END;

(* Datei schlieen *)
 Close (f);

(* IMPLEMENTATION-MODULE erstellen *)
(*---------------------------------*)

 Copy (Filename, NameCopy);
 Concat (Filename,".mod");
 Lookup (f,Filename,FileBuffer,TRUE);
 IF (f.res # done) THEN Close (f); RETURN f.res END;

 wr ("IMPLEMENTATION MODULE "); wr (RawName); wr(";"); cr1;
 wr ("(* Erstellen einer Gadget-Struktur *)"); cr2;

 wr ("FROM SYSTEM    IMPORT ADR,ADDRESS;"); cr1;
 wr ("FROM Intuition IMPORT NewWindow, WindowPtr, IDCMPFlags, IDCMPFlagSet,");
 cr1;
 wr ("                      WindowFlags, WindowFlagSet, ScreenFlags,"); cr1;
 wr ("                      ScreenFlagSet, OpenWindow, CloseWindow,"); cr1;
 wr ("                      customScreen, Gadget, IntuiText, Border,"); cr1;
 wr ("                      StringInfo, PropInfo, GadgetFlagSet, ");cr1;
 wr ("                      GadgetFlags, ActivationFlags, AddGList,");cr1;
 wr ("                      ActivationFlagSet, boolGadget, strGadget,"); cr1;
 wr ("                      propGadget, PropInfoFlagSet, PropInfoFlags,");cr1;
 wr ("                      RefreshGList;"); cr1;
 wr ("FROM Graphics  IMPORT jam1, jam2;"); cr(3);


 wr ("TYPE XYTyp = ARRAY [0..9] OF INTEGER;"); cr1;
 wr ("     BufTyp = ARRAY [0..255] OF CHAR;"); cr2;

 wr ("VAR GadgWindow : NewWindow;"); cr1;
 wr ("    WPtr       : WindowPtr;"); cr1;
 wr ("    Pos        : INTEGER;"); cr1;
 wr ("    Gadgets : ARRAY [0.."); wi(NumbGadg-1); wr("] OF Gadget;"); cr1;
 IF (NumbBorder > 0) THEN
 wr ("    Borders : ARRAY [0.."); wi(NumbBorder-1); wr("] OF Border;"); cr1;
 END; (*IF*)
 IF (NumbXY > 0) THEN
 wr ("    XY      : ARRAY [0.."); wi(NumbXY-1); wr("] OF XYTyp;"); cr1;
 END; (*IF*)
 IF (NumbText > 0) THEN
 wr ("    GText   : ARRAY [0.."); wi(NumbText-1); wr("] OF IntuiText;"); cr1;
 END; (*IF*)
 IF (NumbStr > 0) THEN
 wr ("    SInfo   : ARRAY [0.."); wi(NumbStr-1); wr("] OF StringInfo;"); cr1;
 wr ("    Buf     : ARRAY [0.."); wi(NumbStr-1); wr("] OF BufTyp;"); cr1;
 wr ("    UBuf    : BufTyp;"); cr1;
 END; (*IF*)
 IF (NumbProp > 0) THEN
 wr ("    PInfo   : ARRAY [0.."); wi(NumbProp-1); wr("] OF PropInfo;"); cr1;
 END; (*IF*)
 IF (Numb1Char > 0) THEN
 wr ("    TChar   : ARRAY [0.."); wi(2*Numb1Char-1); wr("] OF CHAR;"); cr1;
 END; (*IF*)
 cr2;

 wr ("PROCEDURE Init"); wr(RawName); cr1;
 wr ("                        (VAR wptr   : ADDRESS;"); cr1;
 wr ("                             Screen : ADDRESS;"); cr1;
 wr ("                             Title  : ADDRESS);"); cr1;
 wr ("(* Aufbau der Gadget-Struktur im Window *)"); cr2;

 wr ("BEGIN"); cr2;

 wr ("(* Window aufbauen *)"); cr1;
 wr ("(*-----------------*)"); cr2;

 wr (" IF (wptr = NIL) THEN"); cr2;

 wr ("   WITH GadgWindow DO"); cr1;
 wr ("     leftEdge := "); wi (WinDef.x); wr ("; topEdge := ");
 wi (WinDef.y); wr (";"); cr1;
 wr ("     width := "); wi (WinDef.width); wr ("; height := ");
 wi (WinDef.height); wr (";"); cr1;
 wr ("     detailPen := 0; blockPen := 1;"); cr1;
 wr ("     idcmpFlags := IDCMPFlagSet{gadgetDown,gadgetUp,closeWindow};");
 cr1;
 wr ("     flags := WindowFlagSet{windowDrag,windowDepth,"); cr1;
 wr ("                            windowClose,activate};"); cr1;
 wr ("     firstGadget := ADR(Gadgets[0]);"); cr1;
 wr ("     checkMark := NIL;"); cr1;
 wr ("     title := Title;"); cr1;
 wr ("     screen := Screen;"); cr1;
 wr ("     bitMap := NIL;"); cr1;
 wr ("     minWidth := 30; minHeight := 20; maxWidth := 640;");
 wr (" maxHeight := 256;"); cr1;
 wr ("     IF (Screen # NIL) THEN"); cr1;
 wr ("       type := customScreen;"); cr1;
 wr ("     ELSE"); cr1;
 wr ("       type := ScreenFlagSet{wbenchScreen};"); cr1;
 wr ("     END; (*IF*)"); cr1;
 wr ("   END; (*WITH*)"); cr2;

 wr (" END; (*IF*)"); cr2;

 wr ("(*-------------------------------------");
 wr ("-------------------------------------*)"); cr2;

 wr ("(* Aufbau der Gadget-Strukturen *)"); cr1;
 wr ("(*------------------------------*)"); cr2;

(* Gadget-Definitionen erstellen *)
(*-------------------------------*)

 FOR i:=0 TO NumbGadg-1 DO

    wr (" WITH Gadgets["); wi (i); wr ("] DO"); cr1;
    wr ("    nextGadget := ");
    IF (i = NumbGadg-1) THEN
        wr ("NIL;")
    ELSE
        wr ("ADR(Gadgets["); wi(i+1); wr("]);")
    END; (*IF*)
    cr1;
    wr ("    leftEdge := "); wi(GadgDefs[i].lEdge); wr (";"); cr1;
    wr ("    topEdge := "); wi(GadgDefs[i].tEdge); wr(";"); cr1;
    wr ("    width := "); wi(GadgDefs[i].width); wr(";"); cr1;
    wr ("    height := "); wi(GadgDefs[i].height); wr(";"); cr1;
    wr ("    flags := GadgetFlagSet{};"); cr1;
    wr ("    activation := ActivationFlagSet{");
    first := TRUE;
    FOR actFlags:=MIN(ActivationFlags) TO MAX(ActivationFlags) DO
       IF (actFlags IN GadgDefs[i].aFlags) THEN
          IF (NOT first) THEN  (* erstes Element? *)
             wr (",");
          ELSE
             first := FALSE;
          END; (*IF*)
          WriteAFlags (actFlags);
       END; (*IF*)
    END; (*FOR*)
    wr ("};"); cr1;
    wr ("    gadgetType := "); WriteType (GadgDefs[i].type); wr (";"); cr1;
    wr ("    gadgetRender := ");
    IF (GadgDefs[i].border = No) AND (GadgDefs[i].type # propGadget) THEN
       wr ("NIL;");
    ELSE
       wr ("ADR(Borders["); wi(iB); wr ("]);");
    END; (*IF*)
    cr1;
    wr ("    selectRender := NIL;"); cr1;
    wr ("    gadgetText := ");
    IF (GadgDefs[i].text[1] = nul) THEN
       wr ("NIL;");
    ELSE
       wr ("ADR(GText["); wi(iT); wr ("]);");
    END; (*IF*)
    cr1;
    wr ("    specialInfo := ");
    IF (GadgDefs[i].type = strGadget) THEN
       wr ("ADR(SInfo["); wi(iS); wr ("]);");
    ELSIF (GadgDefs[i].type = propGadget) THEN
       wr ("ADR(PInfo["); wi(iP); wr ("]);");
    ELSE
       wr ("NIL;");
    END; (*IF*)
    cr1;
    wr ("    gadgetID := "); wi(i); wr (";"); cr1;
    wr ("    userData := NIL;"); cr1;
    wr (" END; (*WITH*)"); cr2;

(* Text-Records erstellen *)
(*------------------------*)

    IF (GadgDefs[i].text[1] # nul) THEN
       wr ("(* Gadget-Text *)"); cr1;
       wr (" WITH GText["); wi(iT); wr ("] DO"); cr1;
       wr ("    frontPen := "); wi (GadgDefs[i].fPen); wr ("; ");
       wr (    "backPen := "); wi (GadgDefs[i].bPen); wr (";"); cr1;
       wr ("    drawMode := jam2;"); cr1;
       wr ("    leftEdge := "); wi (GadgDefs[i].xText); wr ("; ");
       wr (    "topEdge := "); wi(GadgDefs[i].yText); wr (";"); cr1;
       wr ("    iTextFont := NIL;"); cr1;
       IF (Length(GadgDefs[i].text) = 1) THEN
          wr ("    TChar["); wi(iC); wr("] := '"); wr (GadgDefs[i].text);
          wr ("';"); cr1;
          wr ("    TChar["); wi(iC+1); wr("] := 00C;"); cr1;
          wr ("    iText := ADR(TChar["); wi(iC); wr ("]);"); cr1;
          INC (iC,2);
       ELSE
          wr ("    iText := ADR('"); wr (GadgDefs[i].text); wr ("');"); cr1;
       END; (*IF*)
       wr ("    nextText := NIL;"); cr1;
       wr (" END; (*WITH*)"); cr2;
       INC (iT);
    END; (*IF*)

(* Border-Records erstellen *)
(*--------------------------*)

    IF (GadgDefs[i].border # No) OR (GadgDefs[i].type = propGadget) THEN
       wr ("(* Gadget-Border(s) *)"); cr1;
       wr (" WITH Borders["); wi(iB); wr ("] DO"); cr1;
       wr ("    leftEdge := "); wi(-Bord1x); wr ("; ");
       wr (    "topEdge := "); wi(-Bord1y); wr (";"); cr1;
       wr ("    frontPen := 1; ");
       wr (    "backPen := 0;"); cr1;
       wr ("    drawMode := jam2;"); cr1;
       wr ("    count := 5;"); cr1;
       IF (GadgDefs[i].type # propGadget) THEN
          wr ("    xy := ADR(XY["); wi(iX); wr ("]);");
       ELSE
          wr ("    xy := NIL;");
       END; (*IF*)
       cr1;
       wr ("    nextBorder := ");
       IF (GadgDefs[i].border=Double) AND (GadgDefs[i].type#propGadget) THEN
          wr ("ADR(Borders["); wi (iB+1); wr ("]);");
       ELSE
          wr ("NIL;");
       END; (*IF*)
       cr1;
       wr (" END; (*WITH*)"); cr2;
       INC (iB);

       IF (GadgDefs[i].type # propGadget) THEN
          wr (" XY["); wi(iX); wr(",0] := 0; ");
          wr (" XY["); wi(iX); wr(",1] := 0; ");
          cr1;
          wr (" XY["); wi(iX); wr(",2] := ");
          wi(GadgDefs[i].width+2*Bord1x-1); wr("; ");
          wr (" XY["); wi(iX); wr(",3] := 0; ");
          cr1;
          wr (" XY["); wi(iX); wr(",4] := ");
          wi(GadgDefs[i].width+2*Bord1x-1); wr("; ");
          wr (" XY["); wi(iX); wr(",5] := ");
          wi(GadgDefs[i].height+2*Bord1y-1); wr("; "); cr1;
          wr (" XY["); wi(iX); wr(",6] := 0; ");
          wr (" XY["); wi(iX); wr(",7] := ");
          wi(GadgDefs[i].height+2*Bord1y-1); wr("; "); cr1;
          wr (" XY["); wi(iX); wr(",8] := 0; ");
          wr (" XY["); wi(iX); wr(",9] := 0; ");
          cr2;
          INC (iX);
       END; (*IF*)

       IF (GadgDefs[i].border=Double) AND (GadgDefs[i].type#propGadget) THEN
          wr (" WITH Borders["); wi(iB); wr ("] DO"); cr1;
          wr ("    leftEdge := "); wi(-Bord1x-Bord2x); wr ("; ");
          wr (    "topEdge := "); wi(-Bord1y-Bord2y); wr (";"); cr1;
          wr ("    frontPen := 1; ");
          wr (    "backPen := 0;"); cr1;
          wr ("    drawMode := jam2;"); cr1;
          wr ("    count := 5;"); cr1;
          wr ("    xy := ADR(XY["); wi(iX); wr ("]);"); cr1;
          wr ("    nextBorder := NIL;"); cr1;
          wr (" END; (*WITH*)"); cr2;
          INC (iB);

          wr (" XY["); wi(iX); wr(",0] := 0; ");
          wr (" XY["); wi(iX); wr(",1] := 0; ");
          cr1;
          wr (" XY["); wi(iX); wr(",2] := ");
          wi(GadgDefs[i].width+2*Bord1x+2*Bord2x-1); wr("; ");
          wr (" XY["); wi(iX); wr(",3] := 0; ");
          cr1;
          wr (" XY["); wi(iX); wr(",4] := ");
          wi(GadgDefs[i].width+2*Bord1x+2*Bord2x-1); wr("; ");
          wr (" XY["); wi(iX); wr(",5] := ");
          wi(GadgDefs[i].height+2*Bord1y+2*Bord2y-1);
          wr("; ");
          cr1;
          wr (" XY["); wi(iX); wr(",6] := 0; ");
          wr (" XY["); wi(iX); wr(",7] := ");
          wi(GadgDefs[i].height+2*Bord1y+2*Bord2y-1); wr("; ");
          cr1;
          wr (" XY["); wi(iX); wr(",8] := 0; ");
          wr (" XY["); wi(iX); wr(",9] := 0; ");
          cr2;
          INC (iX);

       END; (*IF*)
    END; (*IF*)

(* Info-Strukturen erstellen *)
(*---------------------------*)

    IF (GadgDefs[i].type = strGadget) THEN       (* String-Gadget *)

       wr ("(* String-Info *)"); cr1;
       wr (" WITH SInfo["); wi(iS); wr("] DO"); cr1;
       wr ("    buffer := ADR(Buf["); wi(iS); wr("]);"); cr1;
       wr ("    undoBuffer := ADR(UBuf);"); cr1;
       wr ("    bufferPos := 0; ");
       wr (    "dispPos := 0;"); cr1;
       wr ("    maxChars := "); wi(GadgDefs[i].maxChars); wr(";"); cr1;
       wr ("    longInt := 0;"); cr1;
       wr (" END; (*WITH*)"); cr2;
       INC (iS);

    ELSIF (GadgDefs[i].type = propGadget) THEN    (* Proportional-Gadget *)

       wr ("(* Prop-Info *)"); cr1;
       wr (" WITH PInfo["); wi(iP); wr("] DO"); cr1;
       wr ("    flags := PropInfoFlagSet{autoKnob");
       IF (Horiz IN GadgDefs[i].pType) THEN
          wr (",freeHoriz");
       END; (*IF*)
       IF (Vert IN GadgDefs[i].pType) THEN
          wr (",freeVert");
       END; (*IF*)
       wr ("};"); cr1;
       wr ("    horizPot := 0; ");
       wr (    "vertPot := 0;"); cr1;
       wr ("    horizBody := 65535 DIV "); wi(GadgDefs[i].hSteps);
             wr(";"); cr1;
       wr ("    vertBody := 65535 DIV "); wi(GadgDefs[i].vSteps);
             wr(";"); cr1;
       wr (" END; (*WITH*)"); cr2;
       INC (iP);

    END; (*IF*)

 wr ("(*-------------------------------------");
 wr ("------------------*)"); cr2;

 END; (*FOR*)

 wr ("(* Fenster ffnen: *)"); cr1;
 wr (" IF (wptr = NIL) THEN"); cr1;
 wr ("    WPtr := OpenWindow (GadgWindow);"); cr1;
 wr ("    wptr := WPtr;"); cr1;
 wr (" ELSE"); cr1;
 wr ("    Pos := AddGList (wptr,ADR(Gadgets[0]),0,"); wi(NumbGadg);
 wr (",NIL);"); cr1;
 wr ("    RefreshGList (ADR(Gadgets[0]),wptr,NIL,"); wi(NumbGadg);
 wr (");"); cr1;
 wr ("    WPtr := wptr;"); cr1;
 wr (" END; (*IF*)"); cr2;

 wr ("END Init"); wr (RawName); wr (";"); cr2;

 wr ("(*-------------------------------------");
 wr ("-------------------------------------*)"); cr2;

 wr ("PROCEDURE Close"); wr(RawName); wr (";"); cr1;
 wr ("(* Schlieen des Gadget-Windows. *)"); cr2;

 wr ("BEGIN"); cr1;
 wr (" IF (WPtr#NIL) THEN CloseWindow (WPtr) END;"); cr1;
 wr ("END Close"); wr (RawName); wr (";"); cr2;

 wr ("(*-------------------------------------");
 wr ("-------------------------------------*)"); cr2;

 wr ("PROCEDURE Get"); wr(RawName); wr("GPtr (ID : INTEGER) : ADDRESS;");cr1;
 wr ("(* Liefert den GadgetPtr des Gadgets mit der angegebenen ID. *)"); cr2;

 wr ("BEGIN"); cr1;
 wr (" RETURN ADR(Gadgets[ID]);"); cr1;
 wr ("END Get"); wr(RawName); wr ("GPtr;"); cr2;

 wr ("(*-------------------------------------");
 wr ("-------------------------------------*)"); cr2;

 wr ("BEGIN"); cr1;
 wr (" WPtr := NIL;"); cr1;
 wr ("END "); wr (RawName); wr(".");

 IF (f.res # done) THEN Close (f); RETURN f.res END;

(* Datei schlieen *)
 Close (f);
 RETURN f.res;

END Create;

(*--------------------------------------------------------------------------*)

END CreateModule.
