(*********************************************************************
  :Program.    Kruemel.mod
  :Contents.   Program, showing a cellular automata machine
  :Author.     Thomas Berndt; idea by David Griffeath
  :Copyright.  PD
  :Language.   Modula-2
  :Translator. M2Amiga AMSoft V3.3d
  :History.    02-91 Thomas Berndt
  :Address.    Neckarhauserstr. 94 6800 Mannheim-71
  :Imports.    IOModule (own I/O-module)
*********************************************************************)

MODULE Kruemel ;
(* $R- $V- $S- $F- *)
FROM Arts         IMPORT AllLevelTermProc ;
FROM SYSTEM       IMPORT ADR,ADDRESS,FFP ;
FROM Intuition    IMPORT ScreenFlags,ScreenFlagSet,NewScreen,ScreenPtr,
                         NewWindow,WindowPtr,WindowFlags,WindowFlagSet,
                         IDCMPFlags,IDCMPFlagSet,OpenScreen,CloseScreen,
                         OpenWindow,CloseWindow,IntuiMessagePtr,
                         customScreen ;
FROM Graphics     IMPORT ViewModes,ViewModeSet,RastPortPtr,WritePixel,
                         SetAPen,SetRGB4,RectFill ;
FROM IOModule     IMPORT InitIO,GotoXY,Cls,WriteString,WriteCard,
                         ReadCard,ReadChar,GetChar ;
FROM RandomNumber IMPORT PutSeed,RND ;

VAR
  NuScr                 : NewScreen ;
  NuWin                 : NewWindow ;
  Scr                   : ScreenPtr ;
  Win                   : WindowPtr ;
  rpp                   : RastPortPtr ;
  newCell,oldCell       : ARRAY [0..149],[0..149] OF [0..31] ;
  i,j,k1,k2,l,Colors,
  Width,Height,xmin,
  ymin                  : CARDINAL ;
  Cycle                 : LONGCARD ;
  ok                    : BOOLEAN ;
  ch                    : CHAR ;

PROCEDURE Close ;
BEGIN
  IF Win#NIL THEN
    CloseWindow(Win) ;
  END ; (* IF *)
  IF Scr#NIL THEN
    CloseScreen(Scr) ;
  END ; (* IF *)
END Close ;

PROCEDURE BuildUp ;
BEGIN
  WITH NuScr DO
    leftEdge := 0 ;             topEdge := 0 ;
    width := 320 ;              height := 256 ;
    depth := 5 ;                detailPen := 0 ;
    blockPen := 1 ;             viewModes := ViewModeSet{} ;
    type := customScreen ;      font := NIL ;
    defaultTitle := ADR("Krmelmonster") ;
    gadgets := NIL ;            customBitMap := NIL ;
  END ; (* WITH *)
  Scr := OpenScreen(NuScr) ;
  WITH NuWin DO
    leftEdge    := 0 ;           topEdge := 11 ;
    width       := 320 ;         height := 245 ;
    detailPen   := 0 ;           blockPen := 1 ;
    idcmpFlags  := IDCMPFlagSet{vanillaKey} ;
    flags       := WindowFlagSet{gimmeZeroZero,activate,borderless} ;
    firstGadget := NIL ;         checkMark := NIL ;
    title       := NIL ;
    screen := Scr ;
    bitMap      := NIL ;         type := customScreen ;
    minWidth   := 100 ;          maxWidth := 320 ;
    minHeight  := 20 ;           maxHeight := 256 ;
  END ; (* WITH *)
  Win := OpenWindow(NuWin) ;
  rpp := Win^.rPort ;
  InitIO(Win) ;
  FOR i := 2 TO 17 DO
    SetRGB4(ADR(Scr^.viewPort),i,(i-1)*4,i-2,(i-1)*2) ;
    SetAPen(rpp,i) ;
    RectFill(rpp,80+(i-2)*10,210,90+(i-2)*10,220) ;
  END ; (* FOR *)
  SetAPen(rpp,1) ;
  GotoXY(10,1) ;
  WriteString("Krmelmonster") ;
  GotoXY(1,3) ;
  WriteString("Farbtiefe (1-16) :") ;
  GotoXY(1,4) ;
  WriteString("Width (10-150) :") ;
  GotoXY(1,5) ;
  WriteString("Hhe (10-150) :") ;
END BuildUp ;

PROCEDURE EnterDatas ;
BEGIN
  SetAPen(rpp,0) ;
  RectFill(rpp,xmin,ymin,xmin+Width,ymin+Height) ;
  SetAPen(rpp,1) ;
  REPEAT
    GotoXY(20,3) ;
    ReadCard(Colors) ;
  UNTIL (Colors>0) AND (Colors<=16) ;
  REPEAT
    GotoXY(19,4) ;
    ReadCard(Width) ;
  UNTIL (Width>9) AND (Width<=150) ;
  REPEAT
    GotoXY(17,5) ;
    ReadCard(Height) ;
  UNTIL (Height>9) AND (Height<=150) ;
  GotoXY(5,29) ;
  WriteString("Cycle : 0            ") ;
  xmin := 160-(Width DIV 2) ;
  ymin := 128-(Height DIV 2) ;
END EnterDatas ;

PROCEDURE SetCells ;
BEGIN
  FOR i := 0 TO Width-1 DO
    FOR j := 0 TO Height-1 DO
      newCell[i,j] := RND(Colors) ;
    END ; (* FOR *)
  END ; (* FOR *)
  oldCell := newCell ;
END SetCells ;

PROCEDURE DrawCells ;
BEGIN
  SetAPen(rpp,1) ;
  GotoXY(20,29) ;
  WriteString("Zeichne") ;
  FOR i := 0 TO Width-1 DO
    FOR j := 0 TO Height-1 DO
      SetAPen(rpp,newCell[i,j]+2) ;
      ok := WritePixel(rpp,xmin+i,ymin+j) ;
    END ; (* FOR *)
  END (* FOR *)
END DrawCells ;

PROCEDURE CompareCells ;
BEGIN
  Cycle := 0 ;
  REPEAT
    SetAPen(rpp,1) ;
    GotoXY(20,29) ;
    WriteString("Rechne ") ;
    FOR i := 0 TO Width-1 DO
      FOR j := 0 TO Height-1 DO
        IF oldCell[(i-1) MOD Width,j]=(oldCell[i,j]+1) MOD Colors THEN
          newCell[i,j] := oldCell[(i-1) MOD Width,j] ;
        END ; (* IF *)
        IF oldCell[(i+1) MOD Width,j]=(oldCell[i,j]+1) MOD Colors THEN
          newCell[i,j] := oldCell[(i+1) MOD Width,j] ;
        END ; (* IF *)
        IF oldCell[i,(j-1) MOD Height]=(oldCell[i,j]+1) MOD Colors THEN
          newCell[i,j] := oldCell[i,(j-1) MOD Height] ;
        END ; (* IF *)
        IF oldCell[i,(j+1) MOD Height]=(oldCell[i,j]+1) MOD Colors THEN
          newCell[i,j] := oldCell[i,(j+1) MOD Height] ;
        END ; (* IF *)
      END ; (* FOR *)
    END ; (* FOR *)
    DrawCells ;
    oldCell := newCell ;
    INC(Cycle) ;
    SetAPen(rpp,1) ;
    GotoXY(14,29) ;
    WriteCard(Cycle,1) ;
  UNTIL GetChar()#0C ;
END CompareCells ;

BEGIN
  AllLevelTermProc(Close) ;
  PutSeed(ADR(i)) ;
  BuildUp ;
  REPEAT
    EnterDatas ;
    SetCells ;
    DrawCells ;
    CompareCells ;
    GotoXY(5,29) ;
    WriteString("Ende (j/n)               ") ;
    REPEAT
      GotoXY(15,29) ;
      ReadChar(ch) ;
    UNTIL (ch="j") OR (ch="n") ;
  UNTIL ch="j" ;
END Kruemel .
