(*---------------------------------------------------------------------------
    :Program.    OnLine
    :Author.     Thomas Stolze,
    :Address.    Goslarsche Str. 32, 3000 Hannover 21, Germany
    :Version.    1.0
    :Date.       30-Sep-90
    :Copyright.  PD
    :Language.   Modula-II
    :Translator. M2Amiga V3.32d
    :Imports.    FileRequester.
    :Contents.   Print Utility.
    :LastChange. 20-Dec-90
---------------------------------------------------------------------------*)
(* $R- *) (* $V- *) (* $S- *) (* $N- *) (* $F- *)
IMPLEMENTATION MODULE PrintOut;

IMPORT FileSystem;

FROM Arts            IMPORT AllLevelTermProc,Assert,Requester,BreakPoint;
FROM Conversions     IMPORT StrToVal,ValToStr;
FROM DateConversions IMPORT DateInfo,DateToStr,FromDos,month3;
FROM Dos             IMPORT Delay,Date,DateStamp;
FROM Exec            IMPORT UByte,MsgPortPtr,GetMsg,ReplyMsg,Forbid,Permit;
FROM ExecSupport     IMPORT CreatePort,DeletePort;
FROM Graphics        IMPORT jam2;
FROM Intuition       IMPORT GetPrefs,Preferences,PreferencesPtr;
FROM Routines        IMPORT Block,PrintInfo,PrintModes,PrintModeSet,PrintNews,
                            PrintNewsPtr,prtInfo,Queued,QueuePrintOrder,window;
FROM Storage         IMPORT ALLOCATE,Available,DEALLOCATE;
FROM SYSTEM          IMPORT ADR;

(* FROM InOut IMPORT WriteLn,WriteInt,WriteString;*)

CONST printerDevice = "PRT:";

VAR guard : MsgPortPtr;

PROCEDURE PrintIt(pInfo : PrintInfo; graph : BOOLEAN; VAR prtTrouble : BOOLEAN);
TYPE PrefsPtr = POINTER TO Prefs;
     Prefs    = RECORD
                  firstrow,
                  lastrow,
                  top,
                  bottom,
                  lmargin,
                  fileLength  : LONGINT;
                  paperLength,
                  paperWidth  : CARDINAL;
                END;

VAR file,
    prt    : FileSystem.File;
    actual,
    rows,
    bytes  : LONGINT;
    path   : ARRAY [0..80] OF CHAR;
    chr    : CHAR;
    prefs  : Prefs;

    PROCEDURE Length(str : ARRAY OF CHAR): CARDINAL;
    VAR i : CARDINAL;
    BEGIN
      i:=0; WHILE str[i] # 00C DO INC(i) END; RETURN i;
    END Length;

    PROCEDURE SendHeadLine;
    VAR ddate : Date;
        dinfo : DateInfo;
        dstr,
        tstr,
        nstr,
        pstr  : ARRAY [0..25] OF CHAR;
        err   : BOOLEAN;
        len1,
        len2,
        i     : CARDINAL;
    BEGIN
      DateStamp(ADR(ddate)); FromDos(ddate,dinfo); len1:=0; len2:=0;
      dstr:=(""); tstr:=(""); nstr:=("");
      IF (time IN pInfo.printMode) THEN
         DateToStr(dinfo,pInfo.timeFormat,pInfo.monthName,tstr);
         len1:=Length(tstr);
         FileSystem.WriteBytes(prt,ADR(tstr),len1,actual);
      END;
      IF (page IN pInfo.printMode) THEN
         ValToStr(pInfo.pageCounter,FALSE,nstr,10,4," ",err);
         len2:=(prefs.paperWidth DIV 2) - 4 - len1;
         FOR i:=0 TO len2 DO FileSystem.WriteChar(prt," ") END;
         FileSystem.WriteBytes(prt,ADR("- "),2,actual);
         FileSystem.WriteBytes(prt,ADR(nstr),4,actual);
         FileSystem.WriteBytes(prt,ADR(" -"),2,actual);
         len1:= (prefs.paperWidth DIV 2) + 8;
      END;
      IF (date IN pInfo.printMode) THEN
         DateToStr(dinfo,pInfo.dateFormat,pInfo.monthName,dstr);
         FOR i:=0 TO prefs.paperWidth - len1 - Length(dstr) DO
             FileSystem.WriteChar(prt," ");
         END;
         FileSystem.WriteBytes(prt,ADR(dstr),Length(dstr),actual);
      END;
    END SendHeadLine;

    PROCEDURE ShowGraphics;
    VAR pro : LONGINT;
        val : INTEGER;
    BEGIN
      IF graph THEN
         val:= prefs.fileLength DIV 100; IF val = 0 THEN INC(val) END;
         pro:=(bytes DIV val) * 4;
         IF (pro > 416) THEN pro:=416 END; IF (pro < 25) THEN pro:=25 END;
         Block(window^.rPort,1,24,27,pro,41,jam2);
      END;
    END ShowGraphics;

    PROCEDURE GetAllInfos;
    VAR iprefPtr : PreferencesPtr;
        signed,
        err      : BOOLEAN;
    BEGIN
     IF Available(SIZE(Preferences)) THEN
        ALLOCATE(iprefPtr,SIZE(Preferences));
          GetPrefs(iprefPtr,SIZE(Preferences));
          prefs.paperLength:=iprefPtr^.paperLength-1;
          prefs.paperWidth:=iprefPtr^.printRightMargin;
        DEALLOCATE(iprefPtr,SIZE(Preferences));
     END;

     WITH prefs DO
        StrToVal(pInfo.numArray[3],firstrow,signed,10,err);
        StrToVal(pInfo.numArray[4],lastrow,signed,10,err);
        StrToVal(pInfo.numArray[0],top,signed,10,err);
        StrToVal(pInfo.numArray[1],bottom,signed,10,err);
        StrToVal(pInfo.numArray[2],lmargin,signed,10,err);
      END;
    END GetAllInfos;

    PROCEDURE SendCommand(com : ARRAY OF CHAR);
    VAR actual : LONGINT;
    BEGIN
      FileSystem.WriteChar(prt,CHAR(33C));
      FileSystem.WriteBytes(prt,ADR(com),Length(com),actual);
    END SendCommand;

    PROCEDURE PreparePrint;
    VAR i : LONGINT;
    BEGIN
      IF graph THEN Block(window^.rPort,0,24,27,416,41,jam2) END;

      SendCommand("#1");
      IF (nlq IN pInfo.printMode) THEN
                SendCommand('[2"z') ELSE SendCommand('[1"z') END;

      i:=0; rows:=0; bytes:=0;
      WHILE i < prefs.firstrow DO
        FileSystem.ReadChar(file,chr);  INC(bytes);
        IF chr = CHAR(12C) THEN INC(i); INC(rows); END;
      END;

    END PreparePrint;

    PROCEDURE PrepareLMargin;
    VAR i : INTEGER;
    BEGIN
      FOR i:=1 TO prefs.lmargin DO FileSystem.WriteChar(prt," ") END;
    END PrepareLMargin;

    PROCEDURE PrepareTop;
    VAR i : INTEGER;
    BEGIN
      FOR i:=0 TO prefs.top DO FileSystem.WriteChar(prt,CHAR(12C)) END;
      INC(pInfo.rowCounter,prefs.top);
      PrepareLMargin;
    END PrepareTop;

    PROCEDURE ManageCrt(): BOOLEAN;
    VAR i : INTEGER;
    BEGIN
      INC(pInfo.rowCounter); INC(rows);  FileSystem.WriteChar(prt,chr);
      ShowGraphics;

      IF rows > prefs.lastrow THEN RETURN FALSE END;

      IF (format IN pInfo.printMode) THEN
         IF pInfo.rowCounter >
                      INTEGER(prefs.paperLength)-prefs.bottom-prefs.top THEN

         FOR i:=0 TO prefs.bottom DO FileSystem.WriteChar(prt,CHAR(12C)) END;
            pInfo.rowCounter:=0; INC(pInfo.pageCounter);
            SendHeadLine; PrepareTop;
         END;
         PrepareLMargin;
      END;
      RETURN TRUE;
    END ManageCrt;

    PROCEDURE FeedPaper;
    VAR i : INTEGER;
    BEGIN
      IF (format IN pInfo.printMode) THEN
         FOR i:=pInfo.rowCounter+1 TO prefs.paperLength DO
           FileSystem.WriteChar(prt,CHAR(12C));
         END;
      END;
    END FeedPaper;

    PROCEDURE DatasToPrinter;
    VAR exit : BOOLEAN;

        PROCEDURE CheckPort;
        VAR msg : PrintNewsPtr;
            que : Queued;
        BEGIN
          msg:=GetMsg(guard);
          IF msg # NIL THEN
             que.pI:=msg^.print; ReplyMsg(msg); QueuePrintOrder(que);
          END;
        END CheckPort;

    BEGIN
      pInfo.rowCounter:=0; pInfo.pageCounter:=1; exit:=FALSE;
      IF (format IN pInfo.printMode) THEN
         PreparePrint; SendHeadLine; PrepareLMargin; PrepareTop;
      END;
      REPEAT
        FileSystem.ReadChar(file,chr); INC(bytes);
        CASE chr OF
          12C : CheckPort; IF NOT ManageCrt() THEN exit:=TRUE END;
        ELSE
          FileSystem.WriteChar(prt,chr);
        END;
      UNTIL file.eof OR exit;
      FeedPaper;
      IF graph THEN Block(window^.rPort,0,24,27,416,41,jam2) END;
    END DatasToPrinter;

    PROCEDURE CheckPrinter(): BOOLEAN;
    VAR reg[12570624] : UByte;
        bool          : BOOLEAN;
    BEGIN
      LOOP
        CASE reg OF
          255 : bool:=Requester(ADR("Printer Trouble:"),
                                      ADR("Printer Off!"),
                                      ADR("CONTINUE"),ADR("CANCEL"));
                IF NOT bool THEN prtTrouble:=TRUE; RETURN FALSE END;
        | 252 : prtTrouble:=FALSE; RETURN TRUE;
        ELSE
           bool:=Requester(ADR("Printer Trouble:"),ADR("Printer Offline!"),
                           ADR("CONTINUE"),ADR("CANCEL"));
           IF NOT bool THEN prtTrouble:= TRUE; RETURN FALSE END;
        END;
      END;
    END CheckPrinter;
BEGIN
  guard:=CreatePort(ADR("OnLine_PRT_Port"),0);
  Assert(guard # NIL,ADR("MsgPort not Created"));

  GetAllInfos;

  IF CheckPrinter() THEN
     FileSystem.Lookup(file,pInfo.strPath,10240,FALSE);
     IF (file.res = FileSystem.done) THEN
        FileSystem.Length(file,prefs.fileLength);
        FileSystem.Lookup(prt,printerDevice,4096,TRUE);
        IF (prt.res = FileSystem.done) THEN
           DatasToPrinter;
           FileSystem.Close(file);
        END;
        FileSystem.Close(prt);
     END;
  END;
  DeletePort(guard); guard:=NIL;
END PrintIt;

PROCEDURE Cleanup;
BEGIN
  IF guard # NIL THEN DeletePort(guard); guard:=NIL; END;
END Cleanup;

BEGIN
  AllLevelTermProc(ADR(Cleanup));
  prtInfo.pI.monthName:=month3;
END PrintOut.
