(**********************************************************************

    :Program.    FileRequester.mod
    :Contents.   Einfaches Interface zum FileRequester der "req.library"
    :Author.     Jrgen Zimmermann [JnZ]
    :Address.    Ringstrae 6, W-6719 Altleiningen, Germany
    :Phone.      06356/1456
    :Copyright.  Public Domain (but donation is always welcome!)
    :Language.   Modula-2
    :Translator. M2Amiga AMSoft V4.096d
    :Imports.    "TaskMemory" [bne]
    :Imports.    Die Prozedur "GetPathFromLock" habe ich aus dem Modul
    :Imports.    "Disky" von Kai Bolay entnommen und an die Anforderungen
    :Imports.    in diesem Interface angepat.
    :Imports.    Die "req.library" mu im Verzeichnis "LIBS:" stehen!
    :History.    V1.0  [JnZ] 25.May.1991 first internal version which works
**********************************************************************)


IMPLEMENTATION MODULE FileRequester;

IMPORT Arts, DosD, DosL, DosSupport, ExecL, IntuitionD, ReqD, ReqL,
       String, SYSTEM, TaskMemory, WorkbenchD;


PROCEDURE GetPathFromLock(VAR Path   : ARRAY OF CHAR;
                          ThisLockPtr: DosD.FileLockPtr);
(* von irgendeiner PD-Disk aus 'C'in Modula-II bersetzt (Autor ???) [kai]*)

   VAR CurDirPtr : DosD.FileLockPtr;
       OldDirPtr : DosD.FileLockPtr;
       VolumeLen : INTEGER;
       FIBPtr    : DosD.FileInfoBlockPtr;

   BEGIN
      Path[0]:=0C;
      CurDirPtr:=DosSupport.DupLock(ThisLockPtr);
      IF (CurDirPtr = NIL)
         THEN
            RETURN;
      END; (* IF *)
      TaskMemory.Allocate(FIBPtr,SIZE(FIBPtr^));
      IF (FIBPtr # NIL)
         THEN
            ExecL.Forbid;
            String.Copy(Path,CurDirPtr^.volume^.name^);
            ExecL.Permit;
            String.BStrToStr(Path);
            String.Concat(Path,":");
            VolumeLen:=String.Length(Path);
            WHILE (CurDirPtr # NIL) DO
               IF NOT(DosL.Examine(CurDirPtr,FIBPtr))
                  THEN
                     Path[0]:=0C;
                     DosSupport.UnLock(CurDirPtr);
                     CurDirPtr:=NIL;
                  ELSE
                     OldDirPtr:=CurDirPtr;
                     CurDirPtr:=DosSupport.ParentDir(OldDirPtr);
                     DosSupport.UnLock(OldDirPtr);
                     IF (CurDirPtr # NIL)
                        THEN
                           IF (String.Length(Path) # VolumeLen)
                              THEN
                                 String.Insert(Path,VolumeLen,"/");
                           END; (* IF *)
                           String.Insert(Path,VolumeLen,FIBPtr^.fileName);
                     END; (* IF *)
               END; (* IF *)
            END; (* WHILE *)
            TaskMemory.Deallocate(FIBPtr);
      END; (* IF *)
   END GetPathFromLock;


PROCEDURE FileRequest(RequesterWindow : IntuitionD.WindowPtr;
                      load            : BOOLEAN; (* FALSE means saving *)
                      Title           : ARRAY OF CHAR;
                      VAR FileNamePath: ARRAY OF CHAR): BOOLEAN;

   VAR  freq      : ReqD.FileRequester;
        dirstring : ReqD.DirString;
        filestring: ReqD.FileString;
        wholefile : ReqD.PathString;
        pathPos   : LONGINT;
        lock      : DosD.FileLockPtr;
        msg       : WorkbenchD.WBStartupPtr;


   BEGIN
      IF (String.Length(FileNamePath) # 0)
         THEN
            pathPos:=String.LastPos(FileNamePath,MAX(LONGCARD),"/");
            IF (pathPos = String.noOccur)
               THEN
                  pathPos:=String.LastPos(FileNamePath,MAX(LONGCARD),":");
                  IF (pathPos # String.noOccur)
                     THEN
                        String.CopyPart(dirstring,FileNamePath,0,pathPos);
                        String.CopyPart(filestring,FileNamePath,(pathPos + 1),(String.Length(FileNamePath) - pathPos));
                     ELSE
                        dirstring:="";
                        filestring:="";
                  END; (* IF *)
               ELSE
                  String.CopyPart(dirstring,FileNamePath,0,pathPos);
                  String.CopyPart(filestring,FileNamePath,(pathPos + 1),(String.Length(FileNamePath) - pathPos));
            END; (* IF *)
         ELSE (* Arts unterstzt mich ab jetzt, d.h. ich setze den
                 Pfad im Path-Gadget genau auf den Pfad, der beim Start des
                 Programms vorgegeben ist: Dadurch kann man sich per "Parent"
                 bis zur obersten Ebene hindurchhangeln, nicht wie bei dem
                 Requester in "m2emacs"! *)
            IF (Arts.wbStarted)
               THEN
                  msg:=Arts.startupMsg;
                  lock:=msg^.argList^[0].lock;
                  IF (lock # NIL)
                     THEN
                        GetPathFromLock(dirstring,lock);
                  END; (* IF *)
               ELSE
                  GetPathFromLock(dirstring,Arts.oldCurrentDir);
            END; (* IF *)
      END; (* IF *)


      WITH freq DO
         versionNumber    :=0;
         title            :=SYSTEM.ADR(Title);
         dir              :=SYSTEM.ADR(dirstring);
         file             :=SYSTEM.ADR(filestring);
         pathName         :=SYSTEM.ADR(wholefile);
         flags            :=SYSTEM.LONGSET{ReqD.infogadget,ReqD.caching};
         window           :=RequesterWindow;
         maxExtendedSelect:=0;
         numlines         :=15;
         numcolumns       :=30; (* Anzahl der angezeigten Zeichen der Files! *)
         devcolumns       :=15;
         flags            :=SYSTEM.LONGSET{};
         IF load
            THEN
               INCL(flags,ReqD.loading);
            ELSE
               INCL(flags,ReqD.saving);
         END; (* IF *)
         (* Farben knnen den eigenen Wnschen angepat werden! *)
         dirnamescolor    :=3;
         devicenamescolor :=2;
         detailcolor      :=0;
         blockcolor       :=1;
         gadgettextcolor  :=1;
         textmessagecolor :=3;
         stringnamecolor  :=1;
         stringgadgetcolor:=2;
         boxbordercolor   :=3;
         gadgetboxcolor   :=3;
         windowLeftEdge   :=0;
         windowTopEdge    :=0;
         show             :="*";
         hide             :="";
      END; (* WITH *)

      IF ReqL.FileRequest(SYSTEM.ADR(freq))
         THEN
            String.Copy(FileNamePath,wholefile);
            RETURN(TRUE);
         ELSE
            RETURN(FALSE);
      END; (* IF *)
   END FileRequest;
 END FileRequester.
