(*---------------------------------------------------------------------------
  :Program.    FastLister
  :Author.     Thomas Stolze
  :Address.    Goslarsche Str. 32, 3000 Hannover 21, Germany
  :Phone.      (0)511 / 75 10 77
  :Version.    V2.5
  :Date.       05-Sep-90
  :Copyright.  Shareware
  :Language.   Modula-2
  :Translator. M2Amiga V4.0d
  :Contents.   Programming Utility.
  :Remark.     Needs "MuchMore" in C:-Directory  by F. Siebert
  :Imports.    QText  by F. Siebert AMOK#19
  :LastUpdate. 29-JUN-91
---------------------------------------------------------------------------*)
(*$ LargeVars:=FALSE NilChk:= FALSE OverflowChk:=FALSE RangeChk:=FALSE
    StackChk:=FALSE *)
IMPLEMENTATION MODULE DirSorter;

FROM Arts          IMPORT Assert;
FROM DosD          IMPORT accessRead,FileInfoBlock,FileInfoBlockPtr,FileLockPtr;
FROM DosL          IMPORT Examine,ExNext,Lock,UnLock;
FROM Storage       IMPORT ALLOCATE,Available,DEALLOCATE;
FROM String        IMPORT CapString,Compare,Copy,Occurs;
FROM SYSTEM        IMPORT ADDRESS,ADR;

VAR Directory : FileLockPtr;

PROCEDURE AllocEntry(VAR entryPtr: DirPtr): BOOLEAN;
BEGIN
  IF entryPtr = NIL THEN
     IF Available(SIZE(DirName)) THEN
        ALLOCATE(entryPtr,SIZE(DirName));
     ELSE
        RETURN FALSE;
     END;
  END;
  WITH entryPtr^ DO fileName := (""); typ := notdefined; END;
  RETURN TRUE;
END AllocEntry;

PROCEDURE DeallocEntry(VAR entryPtr : DirPtr);
BEGIN
  DEALLOCATE(entryPtr,SIZE(DirName)); entryPtr:=NIL;
END DeallocEntry;

PROCEDURE ReadDirectory(lock : FileLockPtr; infos : BOOLEAN): BOOLEAN;
VAR good,alloc  : BOOLEAN;
    fileInfoPtr : FileInfoBlockPtr;

  PROCEDURE CheckforInfo(ptr : FileInfoBlockPtr): BOOLEAN;
  VAR from : INTEGER;
  BEGIN
    IF (Occurs(ptr^.fileName,0,".info",FALSE) = -1) THEN
       RETURN TRUE;
    END;
    RETURN FALSE;
  END CheckforInfo;

BEGIN
  DirEntries[DirColumn]:=0;
  IF Available(SIZE(FileInfoBlock)) THEN
     ALLOCATE(fileInfoPtr,SIZE(FileInfoBlock));
     IF Examine(lock,fileInfoPtr) THEN
        REPEAT
          IF infos THEN alloc:=TRUE ELSE alloc:=CheckforInfo(fileInfoPtr) END;
          IF alloc THEN
             IF AllocEntry(DirArray[DirColumn,DirEntries[DirColumn]]) THEN
                WITH fileInfoPtr^ DO
                  IF (fileInfoPtr^.dirEntryType < 0) THEN
                     DirArray[DirColumn,DirEntries[DirColumn]]^.typ:=file;
                     Copy(DirArray[DirColumn,
                                   DirEntries[DirColumn]]^.fileName,fileName);
                     INC(DirEntries[DirColumn]);
                  ELSE
                   IF DirEntries[DirColumn] = 0 THEN
                      DirArray[DirColumn,DirEntries[DirColumn]]^.typ:=directory;
                        Copy(DirArray[DirColumn,
                             DirEntries[DirColumn]]^.fileName,fileName);
                      INC(DirEntries[DirColumn]);
                   END;
                  END;
                END;
             END;
          END;
        UNTIL (ExNext(lock,fileInfoPtr) = FALSE) OR
              (DirEntries[DirColumn] > maxMax);
        good := TRUE;
        DEC(DirEntries[DirColumn]);
     ELSE
       good := FALSE;
     END;
     DEALLOCATE(fileInfoPtr,SIZE(FileInfoBlock));
     RETURN good;
  END;
  RETURN FALSE;
END ReadDirectory;

PROCEDURE QSort(beginning : CARDINAL);
VAR i,j  : CARDINAL;
  PROCEDURE FirstHigher(low,high : ARRAY OF CHAR): BOOLEAN;
  VAR i : CARDINAL;
  BEGIN
    CapString(low); CapString(high);
    IF Compare(low,high) > 0 THEN
       RETURN TRUE;
    END;
    RETURN FALSE;
  END FirstHigher;
  PROCEDURE Sort(low,high : CARDINAL);
  VAR i,j : CARDINAL;
      x,w : DirPtr;
  BEGIN
    i := low; j := high;
    x := DirArray[DirColumn,(low + high) DIV 2];
    REPEAT
      WHILE FirstHigher(x^.fileName,DirArray[DirColumn,i]^.fileName) DO INC(i) END;
      WHILE FirstHigher(DirArray[DirColumn,j]^.fileName,x^.fileName) DO DEC(j) END;
      IF (i <= j) THEN
        w := DirArray[DirColumn,i];
        DirArray[DirColumn,i] := DirArray[DirColumn,j];
        DirArray[DirColumn,j] := w;
        INC(i);
        DEC(j);
      END;
    UNTIL (i > j);
    IF (low < j) THEN Sort(low,j) END;
    IF (i < high)  THEN Sort(i,high) END;
  END Sort;
BEGIN
  Sort(beginning,DirEntries[DirColumn]);
END QSort;

PROCEDURE StoreDirectory(fileName : ADDRESS): BOOLEAN;
BEGIN
  Directory:=Lock(fileName,accessRead);
  IF (Directory # NIL) THEN
     IF ReadDirectory(Directory,FALSE) THEN
        UnLock(Directory); Directory:=NIL; QSort(1);
        RETURN TRUE;
     END;
  END;
  UnLock(Directory); Directory:=NIL;
  RETURN FALSE;
END StoreDirectory;

BEGIN

CLOSE
  IF (Directory # NIL) THEN UnLock(Directory) END;
END DirSorter.
