(***************************************************************************

:Program.    Strings.mod
:Author.     Fridtjof Siebert, Nicolas Benezan, Hartmut Goebel [hG]
:Copyright.   Copyright Original Module 1991 Fridtjof Siebert
:Copyright.  new V1.1 Procedures are Public Domain
:Language.   Oberon
:Translator. Oberon Compiler V2.00
:History.    V1.0, 10-Apr-91 Fridtjof Siebert
:History.    V1.1, 13 Oct 1991 Hartmut Goebel + Concat, Overwrite
:History.          + OccursChar, OccursCharPos, NCStrCmpN, StrCmpN, NCStrCmp
:Support.          some V1.1 Procedures taken from N.Benezan's StringOps
:Date.       18 Oct 1991 18:59:24

***************************************************************************)

MODULE Strings;

IMPORT s: SYSTEM;

PROCEDURE Length*(str: ARRAY OF CHAR): INTEGER; (* $EntryExitCode- *)
BEGIN
s.INLINE(02C5FH,          (*      move.l  (sp)+,a6  *)
         0301FH,          (*      move.w  (sp)+,d0  *)
         0205FH,          (*      move.l  (sp)+,a0  *)
         05340H,          (*      subq    #1,d0     *)
         03200H,          (*      move.w  d0,d1     *)
         04A18H,          (* l:   tst.b   (a0)+     *)
         057C9H, 0FFFCH,  (*      dbeq    d1,l      *)
         09041H,          (*      sub.w   d1,d0     *)
         04ED6H);         (*      jmp     (a6)      *)
END Length;


PROCEDURE Append*(VAR s1: ARRAY OF CHAR; s2: ARRAY OF CHAR); (* $CopyArrays- *)
(* hngt s2 an s1 an *)

VAR l,p,q: INTEGER;
BEGIN
  p := Length(s1);
  l := Length(s2);
  q := 0;
  WHILE (p<LEN(s1)) AND (q<l) DO
    s1[p] := s2[q];
    INC(p); INC(q);
  END;
  IF p<LEN(s1) THEN s1[p] := 0X END;
END Append;


PROCEDURE Occurs*(VAR  s: ARRAY OF CHAR;
                  search: ARRAY OF CHAR): INTEGER;  (* $CopyArrays- *)
(* prft, ob search in s vorkommt und gibt die Position oder -1 zurck *)

VAR l,L,i,j,start: INTEGER;

BEGIN
  l := Length(s); L := Length(search); start := 0;
  WHILE start<l DO
    j := 0; i := start;
    WHILE (j<L) AND (search[j]=s[i]) DO
      INC(j); INC(i);
    END;
    IF j=L THEN RETURN i-L END;
    INC(start);
  END;
  RETURN -1;
END Occurs;


PROCEDURE OccursPos*(VAR  s: ARRAY OF CHAR;
                     search: ARRAY OF CHAR;
                      start: INTEGER): INTEGER;  (* $CopyArrays- *)
(* prft, ob search ab Zeichen Nummer start in s vorkommt und gibt die
Position oder -1 zurck *)

VAR l,L,i,j: INTEGER;

BEGIN
  l := Length(s); L := Length(search);
  WHILE start<l DO
    j := 0; i := start;
    WHILE (j<L) AND (search[j]=s[i]) DO
      INC(j); INC(i);
    END;
    IF j=L THEN RETURN i-L END;
    INC(start);
  END;
  RETURN -1;
END OccursPos;


PROCEDURE Cut*(VAR  s: ARRAY OF CHAR; from,cnt: INTEGER;
               VAR to: ARRAY OF CHAR);
(* kopiert von Zeichen from ausgehend cnt Zeichen nach to *)

VAR i: INTEGER;

BEGIN
  i := 0;
  WHILE (i<cnt) AND (from<LEN(s)) AND (i<LEN(to)) DO
    to[i] := s[from];
    INC(i);
    INC(from);
  END;
  IF i<LEN(to) THEN to[i] := 0X END;
END Cut;


PROCEDURE Upper*(VAR s: ARRAY OF CHAR);
(* wandelt alle Buchstaben in Grobuchstaben um. Ist fr Caseinsensives
   Vergleichen ntzlich *)

VAR i: INTEGER;
BEGIN
  i := 0;
  WHILE (i<LEN(s)) AND (s[i]#0X) DO s[i] := CAP(s[i]); INC(i) END;
END Upper;


PROCEDURE Insert*(VAR s: ARRAY OF CHAR;   (* $CopyArrays- *)
                     at: INTEGER;
                    str: ARRAY OF CHAR);
(* fgt str in s an der Stelle at eine. Beispiel:
   s = "Hallo"; Insert(s,3,"liHal");
     dies ergibt "HalliHallo". *)

VAR l1,l2,i: INTEGER;
BEGIN
  l1 := Length(s);
  l2 := Length(str);
  i := l1; WHILE (i>=at) DO         s[i+l2] := s  [i]; DEC(i) END;
  i := l2; WHILE (i>  0) DO DEC(i); s[at+i] := str[i]         END;
END Insert;


PROCEDURE Delete*(VAR s: ARRAY OF CHAR; at, cnt: INTEGER);
(* lscht n Zeichen aus s ab der Stelle at.
   Beispiel Delete("abcdefg",3,3) ergibt "abcg". *)
VAR l: INTEGER;
BEGIN
  l := Length(s)-cnt;
  WHILE at<l DO
    s[at] := s[at+cnt];
    INC(at);
  END;
  IF at<LEN(s) THEN s[at] := 0X END;
END Delete;


PROCEDURE AppendChar*(VAR s: ARRAY OF CHAR; c: CHAR);
(* hngt c an s1 an *)

VAR l: INTEGER;
BEGIN
  l := Length(s);
  s[l] := c; INC(l);
  IF l<LEN(s) THEN s[l] := 0X END;
END AppendChar;


PROCEDURE InsertChar*(VAR s: ARRAY OF CHAR;
                         at: INTEGER;
                          c: CHAR);
(* fgt c in s an der Stelle at eine. Beispiel:
   s = "Halo"; Insert(s,3,"l");
     dies ergibt "Hallo". *)

VAR i: INTEGER;
BEGIN
  i := Length(s);
  WHILE (i>=at) DO s[i+1] := s[i]; DEC(i) END;
  s[at] := c;
END InsertChar;

(*---- end original module -------*)
(*---- new V1.1 procedures -------*)

PROCEDURE OccursChar*(VAR s: ARRAY OF CHAR;
                          c: CHAR): INTEGER;
VAR
  i: INTEGER;
BEGIN
  i := 0;
  WHILE (i <= LEN(s)) AND (s[i] # 0X) DO
    IF s[i] = c THEN
      RETURN i; END;
    INC(i);
  END;
  RETURN -1;
END OccursChar;


PROCEDURE OccursCharPos*(VAR s: ARRAY OF CHAR;
                             c: CHAR;
                         start: INTEGER): INTEGER;
VAR
  step: INTEGER;
BEGIN
  IF start < 0 THEN
    start := -start;
    step := -1;
  ELSE
    step := 1;
  END;
  WHILE (start <= LEN(s)) AND (start >= 0) AND (s[start] # 0X) DO
    IF s[start] = c THEN
      RETURN start; END;
    INC(start,step);
  END;
  RETURN -1;
END OccursCharPos;


PROCEDURE Concat*(Head, Tail: ARRAY OF CHAR;  (* $CopyArrays- *)
                  VAR String: ARRAY OF CHAR);
BEGIN
  COPY(Head,String);
  Append(String,Tail);
END Concat;


PROCEDURE OverWrite*(VAR string: ARRAY OF CHAR; (* $CopyArrays- *)
                        overlay: ARRAY OF CHAR;
                            pos: INTEGER);
VAR
  i, len: INTEGER;
BEGIN
  len := Length(overlay)-1;
  IF pos+len > LEN(string) THEN
    len := LEN(string)-pos;
  END;
  i := 0;
  WHILE i <= len DO
    string[pos] := overlay[i];
    INC(pos); INC(i);
  END;
END OverWrite;


PROCEDURE StrCmpN*(s1, s2: ARRAY OF CHAR;      (* $CopyArrays- *)
                   maxlen: INTEGER): INTEGER;
VAR
  i :INTEGER;
  Diff, Res, Max:INTEGER;
BEGIN
  Res := LEN(s1)-LEN(s2);
  IF Res < 0 THEN
    Max := LEN(s1);
  ELSE
    Max := LEN(s2);
  END;
  IF Max > maxlen THEN
    Max := maxlen; END;

  i := 0;
  WHILE i <= Max DO
    Diff := ORD(s1[i])-ORD(s2[i]);
    IF (Diff # 0) OR (s1[i] = 0X) THEN
      RETURN Diff
    END;
    INC(i);
  END;

  IF i = maxlen THEN
    RETURN 0; END;

  (* LEN(s1) = Length(s2)? und umgekehrt *)
  IF (Res < 0) AND (s2[i] # 0X) THEN
  ELSIF (Res > 0) AND (s1[i] # 0X) THEN
  ELSE
    RETURN 0;
  END;
  RETURN Res;
END StrCmpN;


PROCEDURE NCStrCmpN*(s1, s2: ARRAY OF CHAR;     (* $CopyArrays- *)
                   maxlen: INTEGER): INTEGER;
VAR
  i :INTEGER;
  Diff, Res, Max:INTEGER;
BEGIN
  Res := LEN(s1)-LEN(s2);
  IF Res < 0 THEN
    Max := LEN(s1);
  ELSE
    Max := LEN(s2);
  END;
  IF Max > maxlen THEN
    Max := maxlen; END;

  i := 0;
  WHILE i <= Max DO
    Diff := ORD(CAP(s1[i]))-ORD(CAP(s2[i]));
    IF (Diff # 0) OR (s1[i] = 0X) THEN
      RETURN Diff
    END;
    INC(i);
  END;

  IF i = maxlen THEN
    RETURN 0; END;

  (* LEN(s1) = Length(s2)? und umgekehrt *)
  IF (Res < 0) AND (s2[i] # 0X) THEN
  ELSIF (Res > 0) AND (s1[i] # 0X) THEN
  ELSE
    RETURN 0;
  END;
  RETURN Res;
END NCStrCmpN;


PROCEDURE NCStrCmp*(s1, s2: ARRAY OF CHAR): INTEGER; (* $CopyArrays- *)
VAR
  i :INTEGER;
  Diff, Res, Max: INTEGER;
BEGIN
  Res := LEN(s1)-LEN(s2);
  IF Res < 0 THEN
    Max := LEN(s1);
  ELSE
    Max := LEN(s2);
  END;

  i := 0;
  WHILE i <= Max DO
    Diff := ORD(CAP(s1[i]))-ORD(CAP(s2[i]));
    IF (Diff#0) OR (s1[i] = 0X) THEN
      RETURN Diff
    END;
    INC(i);
  END;

  (* LEN(s1) = Length(s2)? und umgekehrt *)
  IF (Res < 0) AND (s2[i] # 0X) THEN
  ELSIF (Res > 0) AND (s1[i] # 0X) THEN
  ELSE
    RETURN 0;
  END;
  RETURN Res;
END NCStrCmp;

END Strings.

