(**********************************************************************

:Program.    GetOptions.mod
:Contens.    Ein Modul fr M2m, das die Optionen verarbeitet.
:Author.     Bernd Braun
:Address.    Lippestr. 11, D-3300 Braunschweig
:Phone.      0531/845498
:Copyright.  Public Domain
:Language.   Modula-2
:Translator. M2Amiga A+L V4.096d
:Imports.    DynStr, NewInOut
:History.    V1.0 21.Jun.1991
:History.    V1.1 19.Jul.1991 SyncRun aus ARP eingebaut.
:History.    V1.2 26.Jul.1991 Programm lt sich jetzt unterbrechen,
:History.                     allerdings nicht Compiler oder Linker.
:History.    V1.3 27.Jul.1991 Hrtetest mit A+L Sourcen berstanden.

***********************************************************************)

IMPLEMENTATION MODULE GetOptions;

   FROM ASCII IMPORT
      ht, nul, sp;
   FROM Arts IMPORT
      Exit, wbStarted, startupMsg;
   FROM DynStr IMPORT
      DynString, MakeDynString, ForgetDynString, StatString,
      MakeStatString;
   FROM Header IMPORT
      option, optionen, optionenset, compileropt, linkeropt, SysPath,
      prozessor, M2mName, M2Path, EnvName, true, false, anztool,
      MakePath;
   FROM IconL IMPORT
      GetDiskObject, FreeDiskObject, FindToolType;
   FROM NewInOut IMPORT
      FILE, Open, ModeOld, WriteString, WriteLn, Write, WriteInt, done,
      ReadStringFile, Close;
   FROM String IMPORT
      Concat, ConcatChar, Compare, Length, Copy, noOccur, Occurs,
      FirstPos, Delete;
   FROM WorkbenchD IMPORT
      WBStartupPtr, DiskObjectPtr;

   CONST
      rcIllOpt       = 8;

   VAR
      tooltypes      : ARRAY [ 1 .. 13 ] OF DynString;
      stringptr      : DynString;
      diskobjectptr  : DiskObjectPtr;
      toolarrayptr   : POINTER TO DynString;
      wbstartupptr   : WBStartupPtr;

   (*
      Ausgabe des Hilfstextes bei Eingabe von ?.
   *)
   PROCEDURE Gebrauch;
   BEGIN
      WriteString ( M2mName );
      WriteLn;
      WriteString ( 'Aufruf:\n' +
                    ' m2m {+-012348dfilnawq ' +
                    '-ySysPath -cCompO -oLinkO ? ?? Modul}\n' );
   END Gebrauch;

   (*
      Gibt, wenn Option warnings gesetzt ist einen Fehlertext aus
      und beendet das Programm.
   *)
   PROCEDURE IllOption;
   BEGIN
      IF warnings IN option THEN
         WriteString ( 'ungltige Option!\n' );
         Gebrauch;
      END;
      Exit ( rcIllOpt );
   END IllOption;

   (*
      Ausgabe des Hilfstextes bei Eingabe von ??.
   *)
   PROCEDURE ZeigeOptionen ( opt : optionenset );

      (* Schreibt je nachdem ob o IN os ist ein + oder - auf den Schirm. *)
      PROCEDURE WriteOpt ( o  : optionen;
                           os : optionenset );
      BEGIN
         IF o IN os THEN
            Write ( '+' );
         ELSE
            Write ( '-' );
         END;
      END WriteOpt;
   BEGIN
      WriteString ( 'Status:\n  ' );
      WriteOpt ( dependencies, opt );
      WriteString ( 'd (useDpd)\t' );
      WriteOpt ( forcecpu, opt );
      WriteString ( 'f (forceCPU)\t' );
      WriteOpt ( icons, opt );
      WriteString ( 'i (Icons)\n  ' );
      WriteOpt ( make, opt );
      WriteString ( 'n (doIt)\t' );
      WriteOpt ( makeall, opt );
      WriteString ( 'a (doAll)\t' );
      WriteOpt ( warnings, opt );
      WriteString ( 'w (warnings)\n  ' );
      WriteOpt ( verbose, opt );
      WriteString ( 'q (verbose)\n  ' );

      WriteString ( 'CPU: ' );
      WriteInt ( prozessor, 1 );
      WriteString ( '\n  Compiler-Optionen (c):"' );
      WriteString ( compileropt^ );
      WriteString ( '"\n  Linker-Optionen   (o):"' );
      WriteString ( linkeropt^ );
      WriteString ( '"\n  SysPath           (y):"' );
      WriteString ( SysPath^ );
      WriteString ( '"\n' );
   END ZeigeOptionen;

   (*
      Optionen und dynamische Strings werden voreingestellt.
   *)
   PROCEDURE Voreinstellung;
   BEGIN
      option    := optionenset { link, warnings, make, verbose };
      prozessor := 0;
      IF wbStarted THEN
         INCL ( option, icons );
      END;
      MakeDynString ( '', compileropt );
      MakeDynString ( '', linkeropt );
      MakeDynString ( M2Path, SysPath );

      (* ToolType-Namen eintragen. *)
      MakeDynString ( 'PROZESSOR',     tooltypes [ 1 ] );
      MakeDynString ( 'DEPENDENCIES',  tooltypes [ 2 ] );
      MakeDynString ( 'FORCECPU',      tooltypes [ 3 ] );
      MakeDynString ( 'ICONS',         tooltypes [ 4 ] );
      MakeDynString ( 'LINK',          tooltypes [ 5 ] );
      MakeDynString ( 'MAKE',          tooltypes [ 6 ] );
      MakeDynString ( 'MAKEALL',       tooltypes [ 7 ] );
      MakeDynString ( 'WARNINGS',      tooltypes [ 8 ] );
      MakeDynString ( 'COMPOPTS',      tooltypes [ 9 ] );
      MakeDynString ( 'LINKOPTS',      tooltypes [ 10 ] );
      MakeDynString ( 'SYSPATH',       tooltypes [ 11 ] );
      MakeDynString ( 'VERBOSE',       tooltypes [ 12 ] );
   END Voreinstellung;

   (*
      Compiler- und Linkeroptionen anpassen.
   *)
   PROCEDURE Anpassen;
      VAR
         Str,
         Str2 : StatString;
         i,
         pos  : INTEGER;
         c, d : CHAR;
   BEGIN
      (* Compiler *)
      MakeStatString ( Str, compileropt );
      ForgetDynString ( compileropt );

      (* Icons-Flag suchen und lschen. *)
      pos := Occurs ( Str, 0, 'i', FALSE );
      IF pos # noOccur THEN
         Delete ( Str, pos - 1, 2 );
      END;
      (* Prozessor-Flag suchen und lschen. *)
      FOR i := 0 TO 9 DO
         pos := FirstPos ( Str, 0, CHR ( i + ORD ( '0' ) ) );
         IF pos # noOccur THEN
            Delete ( Str, pos - 1, 2 );
         END;
      END;
      (* Prozessor und Icons-Flag wieder einfgen. *)
      IF icons IN option THEN
         Concat ( Str, '+i' );
      END;
      ConcatChar ( Str, '+' );
      ConcatChar ( Str, CHR ( prozessor + ORD ( '0' ) ) );
      MakeDynString ( Str, compileropt );

      (* Linker *)
      MakeStatString ( Str, linkeropt );
      ForgetDynString ( linkeropt );

      (* Icons-Flag suchen und lschen. *)
      pos := Occurs ( Str, 0, 'i', FALSE );
      IF pos # noOccur THEN
         Delete ( Str, pos - 1, 2 );
      END;

      (* Prozessor-Flag suchen und lschen. *)
      i := 1;
      WHILE i < Length ( Str ) DO
         c := Str [ i - 1 ];
         d := Str [ i ];
         IF ( ORD ( '0' ) <= ORD (  d  ) ) AND
            ( ORD (  d  ) <= ORD ( '9' ) ) THEN
            CASE c OF
               '0' .. '9', 's' :
            ELSE
               Delete ( Str, i - 1, 2 );
            END;
         END;
         INC ( i );
      END;
      Copy ( Str2, Str );
      Copy ( Str, '' );
      (* Prozessor und Icons-Flag wieder einfgen. *)
      IF icons IN option THEN
         Concat ( Str, '+i' );
      END;
      ConcatChar ( Str, '+' );
      ConcatChar ( Str, CHR ( prozessor + ORD ( '0' ) ) );
      Concat ( Str, Str2 );
      MakeDynString ( Str, linkeropt );
   END Anpassen;

   (*
      Extrahiert aus einem String Optionen und setzt sie in den
      option-SET ein.
   *)
   PROCEDURE Liesoptionen ( optstr : ARRAY OF CHAR );
      VAR
         c         : CHAR;
         i, j      : INTEGER;
         plusopt   : BOOLEAN;
         tempstr   : StatString;

      (* Included o in op wenn rein TRUE ist, sonst excluden. *)
      PROCEDURE IncOpt (     rein : BOOLEAN;
                         VAR o    : optionenset;
                             op   : optionen );
      BEGIN
         IF rein THEN
            INCL ( o, op );
         ELSE
            EXCL ( o, op );
         END;
      END IncOpt;

      (* Bearbeitet aktuellen Optionenstring optstr und interpretiert ihn
         je nach o als Compileroption, Linkeroption oder Systempfad. *)
      PROCEDURE SetOpt ( o : INTEGER );
      BEGIN
         CASE o OF
            0 : ForgetDynString ( compileropt );
         |  1 : ForgetDynString ( linkeropt );
         |  2 : ForgetDynString ( SysPath );
         END;
         j := 0;
         INC ( i );
         c := optstr [ i ];
         WHILE ( c # sp ) AND ( c # nul ) DO
            tempstr [ j ] := c;
            INC ( i );
            INC ( j );
            c := optstr [ i ];
         END;
         tempstr [ j ] := nul;
         CASE o OF
            0 : MakeDynString ( tempstr, compileropt );
         |  1 : MakeDynString ( tempstr, linkeropt );
         |  2 : MakeDynString ( tempstr, SysPath );
         END;
      END SetOpt;
   BEGIN
      (* Optionen herauslesen. *)
      i := -1;
      LOOP
         INC ( i );
         c := optstr [ i ];
         IF c = nul THEN
            RETURN;
         END;
         IF ( c = '+' ) OR ( c = '-' ) THEN
            plusopt := c = '+';
            LOOP
               INC ( i );
               c := optstr [ i ];
               IF c = nul THEN
                  RETURN;
               END;
               IF ( c = '+' ) OR ( c = '-' ) THEN
                  DEC ( i );
                  EXIT;
               ELSE
                  CASE c OF
                     sp    :  EXIT;
                  |  '0' .. '9' :
                              prozessor := ORD ( c ) - ORD ( '0' );
                  |  'd'   :  IncOpt ( plusopt, option, dependencies );
                  |  'f'   :  IncOpt ( plusopt, option, forcecpu );
                  |  'i'   :  IncOpt ( plusopt, option, icons );
                  |  'l'   :  IncOpt ( plusopt, option, link );
                  |  'n'   :  IncOpt ( plusopt, option, make );
                  |  'a'   :  IncOpt ( plusopt, option, makeall );
                  |  'w'   :  IncOpt ( plusopt, option, warnings );
                  |  'c'   :  SetOpt ( 0 );
                              EXIT;
                  |  'o'   :  SetOpt ( 1 );
                              EXIT;
                  |  'y'   :  SetOpt ( 2 );
                              EXIT;
                  |  'q'   :  IF plusopt THEN
                                 INCL ( option, verbose );
                              ELSE
                                 EXCL ( option, verbose );
                              END;
                  ELSE
                     IllOption;
                  END (* OF CASE *);
               END (* OF IF *);
            END (* OF LOOP *);
         ELSIF c = '?' THEN
            Gebrauch;
            INC ( i );
            c := optstr [ i ];
            IF c = '?' THEN
               ZeigeOptionen ( option );
            END;
         END (* OF IF *);
      END (* OF LOOP *);
   END Liesoptionen;

   (*
      Lesen der Optionen aus EnvName, falls vorhanden.
   *)
   PROCEDURE GetEnvironment;
      VAR
         optstring : StatString;
         file      : FILE;
   BEGIN
      file := Open ( EnvName, ModeOld );
      IF done THEN
         (* Environment gefunden. *)
         ReadStringFile ( file, optstring );
         Liesoptionen ( optstring );
         Close ( file );
      END;
   END GetEnvironment;

   (*
      Liest Optionen aus dem Info-File.
   *)
   PROCEDURE GetToolTypes;
      VAR
         i   : INTEGER;
         Str : StatString;
   BEGIN
      (* Tooltype holen, falls vorhanden. *)
      wbstartupptr := startupMsg;
      diskobjectptr := GetDiskObject( wbstartupptr^.argList^[ 0 ].name );
      IF diskobjectptr # NIL THEN
         toolarrayptr := diskobjectptr^.toolTypes;
         IF toolarrayptr = NIL THEN
            stringptr := NIL;
         ELSE
            stringptr := toolarrayptr^;
         END;
      END;
      IF stringptr # NIL THEN
         (* ToolType-Eintrge suchen. *)
         FOR i := 1 TO anztool DO
            stringptr := FindToolType
                         ( diskobjectptr^.toolTypes, tooltypes [ i ] );
            IF stringptr # NIL THEN
               (* Optionen setzen. *)
               CASE i OF
                  2, 3, 4, 5, 6, 7, 8, 12 :
                        IF Compare ( stringptr^, false ) = 0 THEN
                           EXCL ( option, VAL ( optionen, i - 2 ) );
                        ELSIF Compare ( stringptr^, true ) = 0 THEN
                           INCL ( option, VAL ( optionen, i - 2 ) );
                        ELSE
                           IllOption;
                        END;
               |  1  :  (* PROZESSOR *)
                        prozessor := ORD ( stringptr^[ 0 ] ) - ORD ( '0' );
               |  9  :  (* COMPOPTS *)
                        ForgetDynString ( compileropt );
                        MakeDynString ( stringptr^, compileropt );
               |  10 :  (* LINKOPTS *)
                        ForgetDynString ( linkeropt );
                        MakeDynString ( stringptr^, linkeropt );
               |  11 :  (* SYSPATH *)
                        ForgetDynString ( SysPath );
                        Copy ( Str, stringptr^ );
                        MakePath ( Str );
                        MakeDynString ( Str, SysPath );
               ELSE
                  IllOption;
               END;
            END (* OF IF *);
         END (* OF FOR *);
      END (* OF IF *);
   END GetToolTypes;

   (*
      Lscht allozierte Strings.
   *)
   PROCEDURE Cleanup;
      VAR
         i : INTEGER;
   BEGIN
      ForgetDynString ( compileropt );
      ForgetDynString ( linkeropt );
      ForgetDynString ( SysPath );
      FOR i := 1 TO anztool DO
         ForgetDynString ( tooltypes [ i ] );
      END;
   END Cleanup;

BEGIN
CLOSE
   Cleanup;
END GetOptions.
