(**********************************************************************

:Program.    Graph.mod
:Contens.    Ein Modul fr M2m, das einen Graph mit Adjazenzlisten
:Contens.    zur Verfgung stellt.
:Author.     Bernd Braun
:Address.    Lippestr. 11, D-3300 Braunschweig
:Phone.      0531/845498
:Copyright.  Public Domain
:Language.   Modula-2
:Translator. M2Amiga A+L V4.096d
:Imports.    DynStr, NewInOut
:History.    V1.0 21.Jun.1991
:History.    V1.1 19.Jul.1991 SyncRun aus ARP eingebaut.
:History.    V1.2 26.Jul.1991 Programm lt sich jetzt unterbrechen,
:History.                     allerdings nicht Compiler oder Linker.
:History.    V1.3 27.Jul.1991 Hrtetest mit A+L Sourcen berstanden.

***********************************************************************)

IMPLEMENTATION MODULE Graph;

   FROM Arts IMPORT
      Assert;
   FROM DosD IMPORT
      Date;
   FROM DynStr IMPORT
      DynString, StatString, MakeDynString, ForgetDynString;
   FROM Header IMPORT
      Typen, PfadPtr, PfadElem, NameandExt, optionen, option,
      prozessor, CompareDate, NullDate, MaxDate,
      GetDatefromFile;
   FROM Storage IMPORT
      ALLOCATE, DEALLOCATE;
   FROM NewInOut IMPORT
      WriteLn, WriteString, FILE, WriteStringFile, WriteIntFile,
      WriteLnFile, Open, Close, done, ModeOld, ModeNew, ReadIntFile,
      ReadStringFile;
   FROM String IMPORT
      Compare, Concat, Copy, ConcatChar, Length;
   FROM SymScanner IMPORT
      ScanSymObj;
   FROM SYSTEM IMPORT
      ADR;

   CONST
      formerr     = 'Format-Fehler im .m2m File!';

   VAR
      GlobalGraph : Graph;

   (*
      Fgt eine neue Ecke in den Graphen ein.
      Fr SourceName gilt: Endet er mit:
         '.def' ist er ein Definition Module
         '.mod' ist er ein Implementation Module
         '' ( keine Endung ) ist er ein Modul ( Hauptprogramm ),
         deshalb mu das Hauptmodul ohne Endung bergeben werden.
   *)
   PROCEDURE NeueEcke ( SourceName : DynString ) : EckenPtr;
      VAR
         e     : EckenPtr;
         Name,
         Ext   : StatString;
   BEGIN
      (* Neue Ecke erzeugen. *)
      ALLOCATE ( e, SIZE ( Ecke ) );
      WITH e^ DO
         ModulName := SourceName;
         ModulPfad := NIL;
         NameandExt ( Name, Ext, SourceName^ );
         IF Compare ( Ext, 'def' ) = 0 THEN
            Typ := definition;
         ELSIF Compare ( Ext, 'mod' ) = 0 THEN
            Typ := implementation;
         ELSE
            Typ := modul;
         END;
         symobjDatum := NullDate;
         ErstellungsDatum := NullDate;
         AdjazenzListe := NIL;
         mussComp := FALSE;
         markiert := FALSE;
         next := NIL;
      END; (* OF WITH *)

      (* Und in den Graphen am Schwanz einfgen. *)
      WITH GlobalGraph DO
         IF Kopf = NIL THEN
            Kopf    := e;
            Schwanz := e;
         ELSE
            Schwanz^.next := e;
            Schwanz := e;
         END;
      END;

      (* Ecke zurckgeben. *)
      RETURN e;
   END NeueEcke;

   (*
      Neue Kante erzeugen und in AdjazenzListe von VonEcke am
      Anfang einfgen.
   *)
   PROCEDURE NeueKante ( VonEcke, NachEcke : EckenPtr );
      VAR
         k : KantenPtr;
   BEGIN
      ALLOCATE ( k, SIZE ( Kante ) );
      WITH k^ DO
         Adjazent := NachEcke;
         next     := VonEcke^.AdjazenzListe;
      END;
      VonEcke^.AdjazenzListe := k;
   END NeueKante;

   (*
      Sucht die Ecke mit dem Namen SourceName im Grphen,
      wird sie nicht gefunden wird NIL zurckgegeben.
   *)
   PROCEDURE SucheEcke ( SourceName : DynString ) : EckenPtr;
      VAR
         e : EckenPtr;
   BEGIN
      e := GlobalGraph.Kopf;
      WHILE e # NIL DO
         IF Compare ( e^.ModulName^, SourceName^ ) = 0 THEN
            RETURN e;
         ELSE
            e := e^.next;
         END;
      END;
      RETURN NIL;
   END SucheEcke;

   (*
      Nacht eine Tiefensuche durch den Graphen und gibt dabei
      eine Liste mit Ecken zurck.
      Wenn makeall TRUE ist, werden die Erstellungzeiten nicht
      bercksichtigt und eine vollstndige Liste zurckgegeben.
      Wenn man diese Liste von vorne bis hinten durchgeht hat
      man die richtige Compilationsreihenfolge fr alle Module,
      die das Hauptmodulimportiert.
   *)
   PROCEDURE TiefenSuche (     HauptModul : EckenPtr;
                           VAR Progs      : PfadPtr );
      VAR
         kante       : KantenPtr;
         p           : PfadPtr;
         compilieren : BOOLEAN;
         Name,
         Ext         : StatString;
         DynStr      : DynString;
         latesymDate,
         Datum       : Date;
         found       : BOOLEAN;
         cpu,
         DateDiff    : INTEGER;
   BEGIN
      IF makeall IN option THEN
         compilieren := TRUE;
      ELSE
         compilieren := FALSE;
      END;
      latesymDate := NullDate;
      WITH HauptModul^ DO
         markiert := TRUE;
         kante := AdjazenzListe;
         WHILE kante # NIL DO
            WITH kante^ DO
               IF NOT Adjazent^.markiert THEN
                  TiefenSuche ( Adjazent, Progs );
               END;
               IF Adjazent^.mussComp AND ( Adjazent^.Typ = definition ) THEN
                  (* Abhngige Module werden nur compiliert, wenn
                     ein importiertes Definitionsmodul compiliert
                     werden mu. *)
                  compilieren := TRUE;
               END;
               IF Adjazent^.Typ = definition THEN
                  MaxDate ( latesymDate, latesymDate, Adjazent^.symobjDatum );
               END;
            END;
            kante := kante^.next;
         END;
         IF gefunden AND NOT compilieren THEN
            (* Prfen ob '.sym' oder '.obj' jngeren Datums sind. *)
            NameandExt ( Name, Ext, ModulName^ );
            IF ( Typ = implementation ) OR ( Typ = modul ) THEN
               Copy ( Ext, Name );
               IF forcecpu IN option THEN
                  (* Nur Objectfile dieses Prozessors prfen. *)
                  CASE prozessor OF
                     0  :  Concat ( Name, '.obj' );
                  ELSE
                     Concat ( Name, '.ob' );
                     ConcatChar ( Name, CHR ( prozessor + ORD ( '0' ) ) );
                  END;
                  MakeDynString ( Name, DynStr );
                  ScanSymObj ( DynStr, ModulPfad, FALSE,
                               found, Datum );
                  symobjDatum := Datum;
                  ForgetDynString ( DynStr );
               ELSE
                  (* Das erste Objectfile ab diesem Prozessor prfen. *)
                  cpu := prozessor;
                  found := FALSE;
                  WHILE ( cpu >= 0 ) AND NOT found DO
                     NameandExt ( Name, Ext, ModulName^ );
                     CASE cpu OF
                        0  :  Concat ( Name, '.obj' );
                     ELSE
                        Concat ( Name, '.ob' );
                        ConcatChar ( Name, CHR ( cpu + ORD ( '0' ) ) );
                     END;
                     MakeDynString ( Name, DynStr );
                     ScanSymObj ( DynStr, ModulPfad, FALSE,
                                  found, Datum );
                     symobjDatum := Datum;
                     ForgetDynString ( DynStr );
                     DEC ( cpu );
                  END;
               END;
               DateDiff := CompareDate ( Datum, ErstellungsDatum );
               IF verbose IN option THEN
                  NameandExt  ( Name, Ext, ModulName^ );
                  IF NOT found THEN
                     WriteString ( 'Fehlendes  ObjFile fr ' );
                     WriteString ( Name );
                     WriteLn;
                  ELSIF DateDiff < 0 THEN
                     WriteString ( 'Ungltiges ObjFile fr ' );
                     WriteString ( Name );
                     WriteString ( ' obj>mod\n' );
                  END;
               END;
            ELSIF Typ = definition THEN
               NameandExt ( Name, Ext, ModulName^ );
               Copy ( Ext, Name );
               Concat ( Name, '.sym' );
               MakeDynString ( Name, DynStr );
               ScanSymObj ( DynStr, ModulPfad, TRUE,
                            found, Datum );
               symobjDatum := Datum;
               DateDiff := CompareDate ( Datum, ErstellungsDatum );
               IF verbose IN option THEN
                  IF NOT found THEN
                     WriteString ( 'Fehlendes  SymFile fr ' );
                     WriteString ( Ext );
                     WriteLn;
                  ELSIF DateDiff < 0 THEN
                     WriteString ( 'Ungltiges SymFile fr ' );
                     WriteString ( Ext );
                     WriteString ( ' sym>mod\n' );
                  END;
               END;
            END;
            IF found THEN
               compilieren := ( DateDiff < 0 ) OR
                  ( CompareDate ( latesymDate, symobjDatum ) > 0 );
            ELSE
               compilieren := TRUE;
            END;
         END;
         (* Diese Ecke in Programmliste Progs einfgen. *)
         IF gefunden AND compilieren THEN
            ALLOCATE ( p, SIZE ( PfadElem ) );
            WITH p^ DO
               ProgName := ModulName;
               PfadName := ModulPfad;
               (* Zeiger auf String wird hier nur kopiert!
                  Daher darf der Inhalt erst bei Programmschluss zerstrt
                  werden. *)
               next := Progs;
            END;
            Progs := p;
         END;
         mussComp := compilieren;
      END;
   END TiefenSuche;

   (*
      Lscht den Graphen.
   *)
   PROCEDURE KillGraph;
      VAR
         e, f : EckenPtr;
         k, l : KantenPtr;
   BEGIN
      e := GlobalGraph.Kopf;
      WHILE e # NIL DO
         ForgetDynString ( e^.ModulName );
         ForgetDynString ( e^.ModulPfad );
         k := e^.AdjazenzListe;
         WHILE k # NIL DO
            l := k^.next;
            DEALLOCATE ( k, SIZE ( Kante ) );
            k := l;
         END;
         f := e^.next;
         DEALLOCATE ( e, SIZE ( Ecke ) );
         e := f;
      END;
      WITH GlobalGraph DO
         Kopf := NIL;
         Schwanz := NIL;
      END;
   END KillGraph;

   (*
      Gibt das jungste Erstellungsdatum einer Ecke im Graphen zurck.
   *)
   PROCEDURE LatestDate ( VAR Datum : Date );
      VAR
         Eck : EckenPtr;
   BEGIN
      Datum := NullDate;
      Eck := GlobalGraph.Kopf;
      WHILE Eck # NIL DO
         WITH Eck^ DO
            MaxDate ( Datum, ErstellungsDatum, Datum );
         END;
         Eck := Eck^.next;
      END;
   END LatestDate;

   (*
      Liefert die Ecke mit der Nummer Nr zurck.
   *)
   PROCEDURE GetEcke ( Nr : INTEGER ) : EckenPtr;
      VAR
         Eck : EckenPtr;
   BEGIN
      Eck := GlobalGraph.Kopf;
      WHILE Eck # NIL DO
         IF Eck^.EckenNr = Nr THEN
            RETURN Eck;
         ELSE
            Eck := Eck^.next;
         END;
      END;
   END GetEcke;

   (*
      Speichert den Graphen im Pfad Pfad unter dem Namen Name mit
      der Endung .m2m
   *)
   PROCEDURE SaveGraph ( Name, Pfad : ARRAY OF CHAR );
      VAR
         file     : FILE;
         Ext,
         FileName : StatString;
         Eck      : EckenPtr;
         Kant     : KantenPtr;
         Num      : INTEGER;
   BEGIN
      NameandExt ( Name, Ext, Name );
      Copy ( FileName, Pfad );
      Concat ( FileName, Name );
      Concat ( FileName, '.m2m' );

      IF NOT GlobalGraph.Kopf^.gefunden THEN
         RETURN;
      END;

      (* Den Ecken im Graphen Nummern geben. *)
      Num := 1;
      Eck := GlobalGraph.Kopf;
      WHILE Eck # NIL DO
         Eck^.EckenNr := Num;
         INC ( Num );
         Eck := Eck^.next;
      END;

      file := Open ( FileName, ModeNew );
      IF NOT done AND ( warnings IN option ) THEN
         WriteString ( 'Warnung: Kann Graphen nicht in ' );
         WriteString ( FileName ) ;
         WriteString ( ' abspeichern!' );
         WriteLn;
      ELSE

         (* Graphen auf File FileName schreiben. *)
         (* Ecken speichern. *)
         Eck := GlobalGraph.Kopf;
         WHILE Eck # NIL DO
            WITH Eck^ DO
               WriteIntFile ( file, EckenNr, 1 );
               WriteLnFile  ( file );
               WriteStringFile ( file, ModulName^ );
               WriteLnFile  ( file );
               IF ModulPfad # NIL THEN
                  WriteStringFile ( file, ModulPfad^ );
               END;
               WriteLnFile  ( file );
               WriteIntFile ( file, ORD ( Typ ), 1 );
               WriteLnFile  ( file );
               WriteIntFile ( file, ORD ( gefunden ), 1 );
               WriteLnFile  ( file );
            END; (* OF WITH *)
            Eck := Eck^.next;
         END; (* OF WHILE *)
         WriteStringFile ( file, '0\n' );

         (* Kanten speichern. *)
         Eck := GlobalGraph.Kopf;
         WHILE Eck # NIL DO
            WITH Eck^ DO
               WriteIntFile ( file, EckenNr, 1 );
               WriteLnFile  ( file );
               Kant := AdjazenzListe;
               WHILE Kant # NIL DO
                  WriteIntFile ( file, Kant^.Adjazent^.EckenNr, 1 );
                  WriteLnFile  ( file );
                  Kant := Kant^.next;
               END;
               WriteStringFile ( file, '0\n' );
            END; (* OF WITH *)
            Eck := Eck^.next;
         END; (* OF WHILE *)
         WriteStringFile ( file, '0\n' );
         Close ( file );
      END; (* OF IF *)
   END SaveGraph;

   (*
      Ld einen Abhngigkeitsgraphen aus Pfad/Name.m2m
   *)
   PROCEDURE LoadGraph ( Name, Pfad : ARRAY OF CHAR ) : BOOLEAN;
      VAR
         Nr,
         Int         : INTEGER;
         file        : FILE;
         Str,
         QuellName,
         ProgName,
         PfadName    : StatString;
         DynStr      : DynString;
         Eck         : EckenPtr;
         Kant        : KantenPtr;
   BEGIN
      Copy ( QuellName, Pfad );
      Concat ( QuellName, Name );
      Concat ( QuellName, '.m2m' );
      file := Open ( QuellName, ModeOld );
      IF NOT done AND ( warnings IN option ) THEN
         WriteString ( 'Warnung: Kann Graphen nicht laden!' );
         WriteLn;
         RETURN FALSE;
      ELSE
         (* Ecken einladen. *)
         ReadIntFile ( file, Nr );
         WHILE done AND ( Nr # 0 ) DO
            ReadStringFile ( file, Str );
            Assert ( done, ADR ( formerr ) );
            MakeDynString ( Str, DynStr );
            Eck := NeueEcke ( DynStr );
            ReadStringFile ( file, Str );
            WITH Eck^ DO
               MakeDynString ( Str, ModulPfad );
               EckenNr := Nr;
               ReadIntFile ( file, Int );
               Assert ( done, ADR ( formerr ) );
               Typ := VAL ( Typen, Int );
               ReadIntFile ( file, Int );
               Assert ( done, ADR ( formerr ) );
               gefunden := VAL ( BOOLEAN, Int );
            END;
            ReadIntFile ( file, Nr );
            Assert ( done, ADR ( formerr ) );
            Eck := Eck^.next;
         END; (* OF WHILE *)

         (* Kanten einladen. *)
         ReadIntFile ( file, Nr );
         WHILE done AND ( Nr # 0 ) DO
            Eck := GetEcke ( Nr );
            ReadIntFile ( file, Int );
            WHILE done AND ( Int # 0 ) DO
               NeueKante ( Eck, GetEcke ( Int ) );
               ReadIntFile ( file, Int );
            END;
            ReadIntFile ( file, Nr );
         END;
      END; (* OF IF *)
      RETURN TRUE;
   END LoadGraph;

   (*
      Liet die Erstellungszeiten der Module im Graphen neu ein.
   *)
   PROCEDURE UpdateDates;
      VAR
         Eck      : EckenPtr;
         Pfad,
         Pfad2,
         Pfad3    : StatString;
         found    : BOOLEAN;
   BEGIN
      Eck := GlobalGraph.Kopf;
      WHILE Eck # NIL DO
         WITH Eck^ DO
            IF gefunden THEN
               IF Length ( ModulPfad^ ) = 0 THEN
                  (* Modul wurde im aktuellen Verzeichnis gefunden. *)
                  Copy ( Pfad,  '' );
                  Copy ( Pfad2, '' );
                  Copy ( Pfad3, '/' );
               ELSE
                  (* Modulpfad ist angegeben. *)
                  Copy ( Pfad,  ModulPfad^ );
                  Copy ( Pfad2, ModulPfad^ );
                  Copy ( Pfad3, ModulPfad^ );
               END;
               Concat ( Pfad2, 'txt/' );
               Concat ( Pfad3, 'txt/' );

               Concat ( Pfad,  ModulName^ );
               Concat ( Pfad2, ModulName^ );
               Concat ( Pfad3, ModulName^ );

               GetDatefromFile ( Pfad, ErstellungsDatum, found );
               IF NOT found THEN
                  GetDatefromFile ( Pfad2, ErstellungsDatum, found );
               END;
               IF NOT found THEN
                  GetDatefromFile ( Pfad3, ErstellungsDatum, found );
               END;
            END; (* OF IF *)
         END; (* OF WITH *)
         Eck := Eck^.next;
      END; (* OF WHILE *)
   END UpdateDates;

BEGIN
   (* Graph initialisieren. *)
   WITH GlobalGraph DO
      Kopf    := NIL;
      Schwanz := NIL;
   END;
END Graph.
