(**********************************************************************

:Program.    M2m.mod
:Contens.    Ein Makeutility fr das Modula-2 Entwicklungssystem von A+L.
:Author.     Bernd Braun
:Address.    Lippestr. 11, D-3300 Braunschweig
:Phone.      0531/845498
:Copyright.  Public Domain
:Language.   Modula-2
:Translator. M2Amiga A+L V4.096d
:Imports.    DynStr, NewInOut, ARP
:History.    V1.0 21.Jun.1991
:History.    V1.1 19.Jul.1991 SyncRun aus ARP eingebaut.
:History.    V1.2 26.Jul.1991 Programm lt sich jetzt unterbrechen,
:History.                     allerdings nicht Compiler oder Linker.
:History.    V1.3 27.Jul.1991 Hrtetest mit A+L Sourcen berstanden.
:Remark.     arp.library mu in libs: sein.

***********************************************************************)

(*
   Bugs:
   - Programm lt sich nicht anhalten, wenn Compiler oder Linker
     am arbeiten sind.
   - DosL.Execute gibt keine Fehler von Compiler und Linker zurck.
     Das passiert, wenn M2m von Workbench aufgerufen wird.
     Bei CLI-Aufruf wird ARP.SyncRun benutzt.
*)

MODULE m2m;

   FROM Arguments IMPORT
      NumArgs, GetArg, GetLock;
   FROM ARP IMPORT
      SyncRun;
   FROM Arts IMPORT
      Assert, Exit, wbStarted;
   FROM Break IMPORT
      InstallException;
   FROM DosL IMPORT
      Execute;
   FROM DynStr IMPORT
      DynString, StatString, MakeDynString, ForgetDynString;
   FROM GetOptions IMPORT
      Voreinstellung, ZeigeOptionen, Anpassen, Liesoptionen, GetEnvironment,
      GetToolTypes;
   FROM Graph IMPORT
      SucheEcke, TiefenSuche, KillGraph, EckenPtr, SaveGraph,
      UpdateDates, LoadGraph, NeueKante;
   FROM Header IMPORT
      optionen, compileropt, linkeropt, SysPath, M2mName,
      PfadPtr, PfadElem, Typen, PfadListe, option, PathName, rcOk,
      LongString, CompName, LinkName, PathandProg, NameandExt, MakePath;
   FROM Storage IMPORT
      ALLOCATE, DEALLOCATE;
   FROM NewInOut IMPORT
      FILE, Open, Close, ReadStringFile, ModeOld, stdout, GetHandle,
      WriteString, WriteLn, WriteInt, ReadString, done, Write;
   FROM SourceScanner IMPORT
      Eintragen;
   FROM SymScanner IMPORT
      ScanBin;
   FROM String IMPORT
      Copy, Length, Concat, ConcatChar, FirstPos, noOccur,CopyPart,
      Compare;
   FROM SYSTEM IMPORT
      ADR;

   VAR
      i              : INTEGER;

   (*
      Dreht die PfadListe List um.
   *)
   PROCEDURE FlipPfadListe ( VAR List : PfadPtr );
      VAR
         q, p : PfadPtr;
   BEGIN
      (* Umdrehen der Liste Kanaele. *)
      q := NIL;
      WHILE List # NIL DO
         p       := List;
         List    := p^.next;
         p^.next := q;
         q       := p;
      END;
      List := q;
   END FlipPfadListe;

   (*
      Lscht die PfadListe.
   *)
   PROCEDURE KillPfadListe ( VAR Liste : PfadPtr );
      VAR
         p, q : PfadPtr;
   BEGIN
      q := Liste;
      WHILE q # NIL DO
         p := q^.next;
         ForgetDynString ( q^.ProgName );
         ForgetDynString ( q^.PfadName );
         DEALLOCATE ( q, SIZE ( PfadElem ) );
         q := p;
      END;
   END KillPfadListe;

   (*
      Liet aus SysPath:path die PfadListe.
   *)
   PROCEDURE GetPfadListe (     Pfad : ARRAY OF CHAR;
                            VAR List : PfadPtr );
      VAR
         Str     : StatString;
         file,
         neufile : FILE;
         p       : PfadPtr;
         NeuPath,
         NeuPfad : StatString;
         found   : BOOLEAN;
   BEGIN
      Copy ( Str, Pfad );
      Concat ( Str, PathName );
      file := Open ( Str, ModeOld );
      Assert ( done, ADR ( 'Kann path nicht ffnen!' ) );

      ReadStringFile ( file, Str );
      WHILE done DO
         MakePath ( Str );
         (* Prfen, ob Str schon in PfadListe vorhanden. *)
         p := List;
         found := FALSE;
         WHILE NOT found AND ( p # NIL ) DO
            IF Compare ( p^.PfadName^, Str ) = 0 THEN
               found := TRUE;
            ELSE
               p := p^.next;
            END;
         END;
         IF NOT found THEN
            (* Neues PfadElement erzeugen. *)
            ALLOCATE ( p, SIZE ( PfadElem ) );
            WITH p^ DO
               next := List;
               ProgName := NIL;
               MakeDynString ( Str, PfadName );
            END;
            List := p;
            Copy ( NeuPfad, Str );
            Copy ( NeuPath, Str );
            Concat ( NeuPath, PathName );
            (* Prfen, ob in den angegebenen Pfaden path-Files stehen. *)
            neufile := Open ( NeuPath, ModeOld );
            IF neufile # NIL THEN
               Close ( neufile );
               GetPfadListe ( NeuPfad, List );
            END;
         END;
         ReadStringFile ( file, Str );
      END;
      FlipPfadListe ( List );
      Close ( file );
   END GetPfadListe;

   (* Trgt Standartmodule in den Graphen ein. *)
   PROCEDURE StandartEintragen ( Eck : EckenPtr );
      PROCEDURE New ( Str : ARRAY OF CHAR );
         VAR
            DynS : DynString;
      BEGIN
         MakeDynString ( Str, DynS );
         Eintragen ( DynS );
         NeueKante ( Eck, SucheEcke ( DynS ) );
      END New;
   BEGIN
      New ( "Profiler" );
      New ( "MathREAL" );
      New ( "MathFFP" );
      New ( "MathIEEEDoubBas" );
   END StandartEintragen;

   (*
      Macht das Modul Prog zu einem lauffhigen Programm.
   *)
   PROCEDURE DoIt ( Prog : ARRAY OF CHAR );
      VAR
         CompArg     : LongString;
         PrgName     : DynString;
         CompNameRun,               (* Compilername fr SyncRun. *)
         LinkNameRun,               (* Linkername fr SyncRun. *)
         Str,
         LastPfad,
         LinkArg,
         NeuPfad,
         NewPath,
         Pfad,
         Ext,
         Name        : StatString;
         errnum,
         pos,
         len         : INTEGER;
         ProgNames,
         p           : PfadPtr;
         error,
         linken,
         compilieren,
         fertig      : BOOLEAN;
         Eck         : EckenPtr;
         neufile     : FILE;
   BEGIN
      error := FALSE;
      PathandProg ( Pfad, Name, Prog );
      (* In Pfad steht der Pfad in dem das Modul Name steht. *)
      Copy ( NeuPfad, Pfad );
      Copy ( NewPath, Pfad );
      Concat ( NewPath, PathName );
      (* Prfen, ob in den angegebenen Pfaden path-Files stehen. *)
      neufile := Open ( NewPath, ModeOld );
      IF neufile # NIL THEN
         Close ( neufile );
         GetPfadListe ( NeuPfad, PfadListe );
      END;

      (* Aktuellen Pfadnamen als erstes einfgen. *)
      ALLOCATE ( p, SIZE ( PfadElem ) );
      WITH p^ DO
         next := PfadListe;
         ProgName := NIL;
         MakeDynString ( Pfad, PfadName );
      END;
      PfadListe := p;

      Copy ( Str, Name );
      IF FirstPos ( Name, 0, '.' ) = noOccur THEN
         Concat ( Str, '.mod' );
      END;
      MakeDynString ( Str, PrgName );

      IF ( dependencies IN option ) THEN
         (* Graphen laden. *)
         NameandExt ( Name, Ext, Name );
         IF verbose IN option THEN
            WriteString ( ' - ' );
            WriteString ( Name );
            WriteString ( '.m2m' );
            WriteLn;
         END;
         IF LoadGraph ( Name, Pfad ) THEN
            UpdateDates;
         ELSE
            (* Rekursiv alle Module einlesen. *)
            Eintragen ( PrgName );
            Eck := SucheEcke ( PrgName );
            IF Eck^.gefunden THEN
               StandartEintragen ( Eck );
               (* Graphen speichern. *)
               NameandExt ( Name, Ext, Name );
               IF verbose IN option THEN
                  WriteString ( ' + ' );
                  WriteString ( Name );
                  WriteString ( '.m2m' );
                  WriteLn;
               END;
               SaveGraph ( Name, Pfad );
            END;
         END;
      ELSE
         (* Rekursiv alle Module einlesen. *)
         Eintragen ( PrgName );
         Eck := SucheEcke ( PrgName );
         IF Eck^.gefunden THEN
            StandartEintragen ( Eck );
            (* Graphen speichern. *)
            NameandExt ( Name, Ext, Name );
            IF verbose IN option THEN
               WriteString ( ' + ' );
               WriteString ( Name );
               WriteString ( '.m2m' );
               WriteLn;
            END;
            SaveGraph ( Name, Pfad );
         END;
      END;

      (* berhaupt gefunden? *)
      Eck := SucheEcke ( PrgName );
      IF ( Eck # NIL ) AND Eck^.gefunden THEN
         TiefenSuche ( SucheEcke ( PrgName ), ProgNames );
         IF verbose IN option THEN
            WriteLn;
         END;
         FlipPfadListe ( ProgNames );
         fertig := FALSE;
         p := ProgNames;
         compilieren := p # NIL;
         WHILE compilieren AND NOT fertig DO
            (* String fr Compileraufruf generieren. *)
            Copy ( LastPfad, '' );
            IF wbStarted THEN
               (* Fr Execute. *)
               Copy ( CompArg, SysPath^ );
               Concat ( CompArg, CompName );
               ConcatChar ( CompArg, ' ' );
            ELSE
               (* Fr SyncRun. *)
               Copy ( CompArg, "" );
               Copy ( CompNameRun, SysPath^ );
               Concat ( CompNameRun, CompName );
            END;
            Concat ( CompArg, compileropt^ );
            WHILE ( p # NIL ) AND ( Length ( CompArg ) < 200 ) DO
               ConcatChar ( CompArg, ' ' );
               IF Compare ( LastPfad, p^.PfadName^ ) # 0 THEN
                  Copy ( LastPfad, p^.PfadName^ );
                  Concat ( CompArg, '-g' );
                  Concat ( CompArg, LastPfad );
                  ConcatChar ( CompArg, ' ' );
               END;
               Concat ( CompArg, p^.ProgName^ );
               p := p^.next;
            END;
            IF p = NIL THEN
               fertig := TRUE;
            END;
            IF verbose IN option THEN
               IF NOT wbStarted THEN
                  WriteString ( CompNameRun );
                  Write ( " " );
               END;
               WriteString ( CompArg );
               WriteLn;
            END;
            IF make IN option THEN
               IF verbose IN option THEN
                  WriteLn;
               END;
               IF wbStarted THEN
                  errnum := Execute ( ADR ( CompArg ), NIL,
                                      GetHandle ( stdout ) );
                  (* Verweigert die Arbeit bei langen Strings! *)
                  IF errnum # -1 THEN
                     error := TRUE;
                  END;
               ELSE
                  errnum := SyncRun ( ADR ( CompNameRun ), ADR ( CompArg ),
                                      NIL, GetHandle ( stdout ) );
                  IF errnum # 0 THEN
                     error := TRUE;
                  END;
               END;
               IF error AND ( warnings IN option ) THEN
                  WriteString ( '!! Fehler aufgetreten: Rckgabewert=' );
                  WriteInt    ( errnum, 1 );
                  WriteLn;
               END;
            END;
         END;
         IF NOT error THEN
            (* String fr Linkeraufruf generieren. *)
            IF wbStarted THEN
               (* Fr Execute. *)
               Copy ( LinkArg, SysPath^ );
               Concat ( LinkArg, LinkName );
               ConcatChar ( LinkArg, ' ' );
            ELSE
               (* Fr SyncRun. *)
               Copy ( LinkArg, "" );
               Copy ( LinkNameRun, SysPath^ );
               Concat ( LinkNameRun, LinkName );
            END;
            Concat ( LinkArg, linkeropt^ );
            ConcatChar ( LinkArg, ' ' );
            IF Length ( Pfad ) # 0 THEN
               Concat ( LinkArg, '-g' );
               Concat ( LinkArg, Pfad );
               ConcatChar ( LinkArg, ' ' );
            END;
            pos := FirstPos ( Name, 0, '.' );
            IF pos # noOccur THEN
               CopyPart ( Name, Name, 0, pos );
            END;
            linken := compilieren AND ( link IN option );
            IF NOT linken AND ( link IN option ) THEN
               linken := ScanBin ( Name, Pfad );
            END;
            IF linken THEN
               Concat ( LinkArg, Name );
               IF verbose IN option THEN
                  WriteLn;
                  IF NOT wbStarted THEN
                     WriteString ( LinkNameRun );
                     Write ( " " );
                  END;
                  WriteString ( LinkArg );
                  WriteLn;
               END;
            END;
            IF linken AND ( make IN option ) THEN
               IF verbose IN option THEN
                  WriteLn;
               END;
               IF wbStarted THEN
                  errnum := Execute ( ADR ( LinkArg ), NIL,
                                      GetHandle ( stdout ) );
                  IF errnum # -1 THEN
                     error := TRUE;
                  END;
               ELSE
                  errnum := SyncRun ( ADR ( LinkNameRun ), ADR ( LinkArg ),
                                     NIL, GetHandle ( stdout ) );
                  IF errnum # 0 THEN
                     error := TRUE;
                  END;
               END;
               IF error AND ( warnings IN option ) THEN
                  WriteString ( '!! Fehler aufgetreten: Rckgabewert=' );
                  WriteInt    ( errnum, 1 );
                  WriteLn;
               ELSIF verbose IN option THEN
                  WriteString ( '\nMake erfolgreich durchgefhrt.' );
                  WriteLn;
               END;
            END;
         END;
         IF NOT compilieren AND NOT linken THEN
            IF verbose IN option THEN
               PathandProg ( Pfad, Name, Prog );
               NameandExt  ( Prog, Pfad, Name );
               WriteString ( Prog );
               WriteString ( ' ist auf dem neuesten Stand.' );
               WriteLn;
            END;
         ELSIF error AND ( verbose IN option ) THEN
            WriteString ( 'Make gescheitert.' );
            WriteLn;
         END;
      ELSIF verbose IN option THEN
         WriteLn;
         WriteString ( Name );
         WriteString ( ' nicht gefunden.' );
      END;
      IF verbose IN option THEN
         WriteLn;
      END;
   END DoIt;

   (*
      Fragt ineraktiv nach Optionen und Modulnamen.
   *)
   PROCEDURE Interaktion;
      VAR
         command : StatString;
         c       : CHAR;
   BEGIN
      INCL ( option, verbose );
      LOOP
         WriteString ( 'm2m> ' );
         ReadString  ( command );
         IF Length ( command ) = 0 THEN
            Exit ( rcOk );
         ELSE
            c := command [ 0 ];
            IF ( c = '+' ) OR ( c = '-' ) OR ( c = '?' ) THEN
               Liesoptionen ( command );
               Anpassen;
            ELSE
               (* Bearbeite Hauptmodul command. *)
               DoIt ( command );
               KillGraph;
            END;
         END;
      END;
   END Interaktion;

   (*
      Wertet die Kommandozeile nach Optionen und Modulen aus.
   *)
   PROCEDURE Kommandozeile;
      VAR
         i,
         j        : INTEGER;
         arg      : StatString;
         c        : CHAR;
   BEGIN
      i := 1;
      WHILE i <= NumArgs () DO
         GetArg ( i, arg, j );
         c := arg [ 0 ];
         IF ( c = '+' ) OR ( c = '-' ) OR ( c = '?' ) THEN
            Liesoptionen ( arg );
            Anpassen;
         ELSE
            IF verbose IN option THEN
               WriteString ( 'm2m> ' );
               WriteString ( arg );
               WriteLn;
            END;
            (* Bearbeite Hauptmodul arg. *)
            DoIt ( arg );
         END;
         INC ( i );
      END;
   END Kommandozeile;

BEGIN
   InstallException;
   Voreinstellung;
   IF verbose IN option THEN
      WriteString ( 'Amiga Modula-2 Make, ' );
      WriteString ( M2mName );
      WriteLn;
   END;
   GetEnvironment;
   IF wbStarted THEN
      GetToolTypes;
   END;
   Anpassen;
   GetPfadListe ( SysPath^, PfadListe );
   (* Argumente holen. *)
   IF NumArgs () = 0 THEN
      Interaktion;
   ELSE
      Kommandozeile;
   END;
CLOSE
   KillPfadListe ( PfadListe );
END m2m.
