(**********************************************************************

:Program.    SymScanner.mod
:Contens.    Ein Modul fr M2m, das .sym .obj und gelinke Programme nach
:Contens.    Erstellungszeiten abfragt.
:Author.     Bernd Braun
:Address.    Lippestr. 11, D-3300 Braunschweig
:Phone.      0531/845498
:Copyright.  Public Domain
:Language.   Modula-2
:Translator. M2Amiga A+L V4.096d
:Imports.    DynStr
:History.    V1.0 21.Jun.1991
:History.    V1.1 19.Jul.1991 SyncRun aus ARP eingebaut.
:History.    V1.2 26.Jul.1991 Programm lt sich jetzt unterbrechen,
:History.                     allerdings nicht Compiler oder Linker.
:History.    V1.3 27.Jul.1991 Hrtetest mit A+L Sourcen berstanden.

***********************************************************************)

IMPLEMENTATION MODULE SymScanner;

   FROM DosD IMPORT
      Date;
   FROM DynStr IMPORT
      DynString, StatString;
   FROM Header IMPORT
      PfadPtr, CompareDate, MaxDate, latestDate, NullDate,
      GetDatefromFile;
   FROM String IMPORT
      Copy, Concat, Length;

   (*
      Sucht ein .sym oder .obj Modul Namens ModulN im Pfad ModulP.
      Gibt in gefunden TRUE zurck falls gefunden
      und in Datum das Erstellungsdatum.
   *)
   PROCEDURE ScanSymObj (     ModulN,
                              ModulP      : DynString;
                              symbol      : BOOLEAN;
                          VAR gefunden    : BOOLEAN;
                          VAR Datum       : Date    );
      VAR
         Pfad,
         Pfad2,
         Pfad3 : StatString;
   BEGIN
      (* Voreinstellungen. *)
      gefunden := FALSE;
      Datum := NullDate;

      IF ModulP = NIL THEN
         (* Kein Sourceprogramm vorhanden. *)
         RETURN;
      ELSIF Length ( ModulP^ ) = 0 THEN
         (* Modul wurde im aktuellen Verzeichnis gefunden. *)
         Copy ( Pfad,  '' );
         Copy ( Pfad2, '' );
         Copy ( Pfad3, '/' );
      ELSE
         (* Modulpfad ist angegeben. *)
         Copy ( Pfad,  ModulP^ );
         Copy ( Pfad2, ModulP^ );
         Copy ( Pfad3, ModulP^ );
      END;

      IF symbol THEN
         Concat ( Pfad2, 'sym/' );
         Concat ( Pfad3, 'sym/' );
      ELSE
         Concat ( Pfad2, 'obj/' );
         Concat ( Pfad3, 'obj/' );
      END;
      Concat ( Pfad, ModulN^ );
      Concat ( Pfad2, ModulN^ );
      Concat ( Pfad3, ModulN^ );

      GetDatefromFile ( Pfad2, Datum, gefunden );
      IF NOT gefunden THEN
         GetDatefromFile ( Pfad, Datum, gefunden );
      END;
      IF NOT gefunden THEN
         GetDatefromFile ( Pfad3, Datum, gefunden );
      END;
      MaxDate ( latestDate, Datum, latestDate );
   END ScanSymObj;

   (*
      Sucht in Pfad ModPfad ein gelinktes Programm Namens ModName.
      Gibt TRUE zurck, wenn es gelinkt werden mu.
   *)
   PROCEDURE ScanBin ( ModName, ModPfad : ARRAY OF CHAR ) : BOOLEAN;
      VAR
         Pfad,
         Pfad2,
         Pfad3    : StatString;
         gefunden : BOOLEAN;
         Datum    : Date;
   BEGIN
      (* Voreinstellungen. *)
      gefunden := FALSE;
      Datum := NullDate;

      IF Length ( ModPfad ) = 0 THEN
         (* Modul wurde im aktuellen Verzeichnis gefunden. *)
         Copy ( Pfad,  '' );
         Copy ( Pfad2, '' );
         Copy ( Pfad3, '/' );
      ELSE
         (* Modulpfad ist angegeben. *)
         Copy ( Pfad,  ModPfad );
         Copy ( Pfad2, ModPfad );
         Copy ( Pfad3, ModPfad );
      END;
      Concat ( Pfad2, 'bin/' );
      Concat ( Pfad3, 'bin/' );

      Concat ( Pfad,  ModName );
      Concat ( Pfad2, ModName );
      Concat ( Pfad3, ModName );

      GetDatefromFile ( Pfad2, Datum, gefunden );
      IF NOT gefunden THEN
         GetDatefromFile ( Pfad, Datum, gefunden );
      END;
      IF NOT gefunden THEN
         GetDatefromFile ( Pfad3, Datum, gefunden );
      END;
      IF NOT gefunden THEN
         RETURN TRUE;
      ELSE
         IF CompareDate ( latestDate, Datum ) > 0 THEN
            RETURN TRUE;
         ELSE
            RETURN FALSE;
         END;
      END;
   END ScanBin;

END SymScanner.
