MODULE ums;

(***
   ums.library interface for Amiga-Oberon,
   official documentation for ums.library,  91 by Martin Horneffer

   History:
   13.12.91: started history; small modifications to V8; English comments
***)

IMPORT Utility, Exec, Intuition, SYSTEM;

CONST
  libName = "ums.library";

  libVersion = 9;

VAR
  base * : Exec.LibraryPtr;

TYPE
  (* msgs are identified by the following type *)
  NUM * 	= LONGINT;

  (* general types *)

CONST
  MaxMsgs *	= MAX(NUM);

TYPE
  APTR *	= Exec.APTR;
  STRPTR *	= POINTER TO ARRAY MAX(INTEGER)-1 OF CHAR;

  oldMsg *	= ARRAY 16 OF STRPTR;
  oldMsgPtr *	= POINTER TO oldMsg;


PROCEDURE Login		*{base,-30}( user{2}	: ARRAY OF CHAR;
                                     passwd{3}	: ARRAY OF CHAR): LONGINT;

PROCEDURE Logout	*{base,-36}( account{2}	: LONGINT);

PROCEDURE PutUMSMsg	*{base, -42}( account{2}: LONGINT;
                               	      Msg{3}	: oldMsgPtr): INTEGER;

PROCEDURE GetMsgHdr	*{base, -48}( account{2}: LONGINT;
                                      MsgNum{3}	: NUM): oldMsgPtr;

PROCEDURE GetMsgTxt	*{base, -54}( account{2} : LONGINT;
                                      Msg{3}     : oldMsgPtr): INTEGER;

PROCEDURE FreeMsg	*{base, -60}( account{2}: LONGINT;
                                      Msg{3}	: oldMsgPtr);

PROCEDURE NextNew	*{base, -66}( account{2}: LONGINT;
                                      MsgNum{3}	: NUM): NUM;

PROCEDURE SearchID	*{base, -72}( account{2}: LONGINT;
                                      id{3}	: ARRAY OF CHAR): NUM;

PROCEDURE ChangeID	*{base, -78}( account{2}: LONGINT;
                                      MsgNum{3}	: NUM;
                                      id{4}	: ARRAY OF CHAR): BOOLEAN;

PROCEDURE ChainUp	*{base, -84}( account{2}: LONGINT;
                                      MsgNum{3}	: NUM): NUM;

PROCEDURE ChainDn	*{base, -90}( account{2}: LONGINT;
                                      MsgNum{3}	: NUM): NUM;

PROCEDURE ChainRt	*{base, -96}( account{2}: LONGINT;
                                      MsgNum{3} : NUM): NUM;

PROCEDURE ChainLt	*{base,-102}( account{2}: LONGINT;
                                      MsgNum{3}	: NUM): NUM;

PROCEDURE GetGlobal	*{base,-108}( account{2}: LONGINT;
                                      name{3}	: ARRAY OF CHAR;
				      data{4}	: ARRAY OF CHAR);

PROCEDURE DumpConfig	*{base,-114}( account{2}: LONGINT) ;

PROCEDURE ErrNum	*{base,-120}( account{2}: LONGINT): INTEGER;

PROCEDURE ErrTxt	*{base,-126}( account{2}: LONGINT): STRPTR;

PROCEDURE DeleteMsg	*{base,-132}( account{2}: LONGINT;
                                      MsgNum{3}	: NUM): BOOLEAN;

PROCEDURE ChangeAttributes*{base,-138}( account{2}: LONGINT;
                                      MsgNum{3} : NUM;
                                      att{4}	: ARRAY OF CHAR): BOOLEAN;

PROCEDURE NextStatus	*{base,-144}( account{2}: LONGINT;
                                      from{3}	: NUM;
                                      mask{4}	: SET;
                                      status{5}	: SET): NUM;

(*** definetly obsolete
PROCEDURE SelectStatus	*{base,-150}( account{2}: LONGINT;
                                      set{3}	: SET;
                                      mask{4}	: SET;
                                      status{5}	: SET): NUM;

PROCEDURE SelectGroup	*{base,-156}( account{2}: LONGINT;
                                      set{3}	: SET;
                                      group{4}	: ARRAY OF CHAR): NUM;
***)

PROCEDURE SetStatus	*{base,-162}( acc{2}	: LONGINT;
                                      MsgNum{3}	: NUM;
                                      unset{4}	: SET;
                                      set{5}: SET);

PROCEDURE PrevStatus	*{base,-168}( acc{2}	: LONGINT;
                                      from{3}	: NUM;
                                      mask{4}	: SET;
                                      status{5}	: SET): NUM;

PROCEDURE SelStatus	*{base,-174}( acc{2}	: LONGINT;
                                      set{3}	: SET;
                                      unset{4}	: SET;
                                      mask{5}	: SET;
                                      status{6}	: SET): NUM;

PROCEDURE SelGroup	*{base,-180}( acc{2}	: LONGINT;
                                      set{3}	: SET;
                                      unset{4}	: SET;
                                      group{5}	: ARRAY OF CHAR): NUM;

PROCEDURE GetStatus	*{base,-186}( acc{2}	: LONGINT;
                                      MsgNum{3}	: LONGINT): SET;

PROCEDURE FlushUMS	*{base,-192};

PROCEDURE CleanMB	*{base,-198}(): INTEGER;

PROCEDURE QuitUMS	*{base,-204};


(*** V8: ***)

PROCEDURE UMSARexxQuery	*{base,-210}();

(*** V9: ***)

PROCEDURE AllocUMSPattern*{base,-216}( pattern{2}: ARRAY OF CHAR;
                                       case{3}	: BOOLEAN): APTR;

PROCEDURE FreeUMSPattern*{base,-222}( pHandle{2}: APTR );

PROCEDURE MatchUMSPattern*{base,-228}( pHandle{2}: APTR;
                                      string{3}	: ARRAY OF CHAR): BOOLEAN;

PROCEDURE ExportedMsg	*{base,-234}( acc{2}	: LONGINT;
                                      num{3}	: LONGINT);

PROCEDURE CannotExport	*{base,-240}( acc{2}	: LONGINT;
                                      num{3}	: LONGINT;
                                      error{4}	: ARRAY OF CHAR): BOOLEAN;

PROCEDURE LogUMS	*{base,-246}( acc{2}	: LONGINT;
                                      level{4}	: LONGINT;
                                      format{5}	: ARRAY OF CHAR;
                                      args{6} 	: ARRAY OF SYSTEM.ADDRESS);
PROCEDURE LogUms	*{base,-246}( acc{2}	: LONGINT;
                                      level{4}	: LONGINT;
                                      format{5}	: ARRAY OF CHAR;
                                      args{6}.. : LONGINT);

PROCEDURE UMSRLogin	*{base,-252}( server{2}	: ARRAY OF CHAR;
                                      user{3}	: ARRAY OF CHAR;
                                      passwd{4}	: ARRAY OF CHAR): LONGINT;

PROCEDURE WriteUMSMsg		*{base,-258}( acc{2}	: LONGINT;
                       		              tagItems{3}: ARRAY OF Utility.TagItem): LONGINT;
PROCEDURE WriteUMSMsgTags	*{base,-258}( acc{2}	: LONGINT;
                                	      tagItems{3}..: Utility.Tag): LONGINT;

PROCEDURE ReadUMSMsg		*{base,-264}( acc{2}	: LONGINT;
                           	              tagItems{3}: ARRAY OF Utility.TagItem): BOOLEAN;
PROCEDURE ReadUMSMsgTags	*{base,-264}( acc{2}	: LONGINT;
                                   	      tagItems{3}..: Utility.Tag): BOOLEAN;

PROCEDURE FreeUMSMsg		*{base,-270}( acc{2}	: LONGINT;
	                                      msgNum{3}	: LONGINT);

PROCEDURE UMSSelect		*{base,-266}( acc{2}	: LONGINT;
        	                              tagItems{3}: ARRAY OF Utility.TagItem): LONGINT;
PROCEDURE UMSSelectTags		*{base,-276}( acc{2}	: LONGINT;
                	                      tagItems{3}..: Utility.Tag): LONGINT;

PROCEDURE UMSSearch		*{base,-282}( acc{2}	: LONGINT;
                        	              tagItems{3}: ARRAY OF Utility.TagItem): LONGINT;
PROCEDURE UMSSearchTags		*{base,-282}( acc{2}	: LONGINT;
                                	      tagItems{3}..: Utility.Tag): LONGINT;

PROCEDURE ReadUMSConfig		*{base,-288}( acc{2}: LONGINT;
                                	      tagItems{3}: ARRAY OF Utility.TagItem): STRPTR;
PROCEDURE ReadUMSConfigTags	*{base,-288}( acc{2}: LONGINT;
                                              tagItems{3}..: Utility.Tag): STRPTR;

PROCEDURE FreeUMSConfig		*{base,-294}( acc{2}	: LONGINT;
                                              str{3}	: STRPTR);

PROCEDURE WriteUMSConfig	*{base,-300}( acc{2}: LONGINT;
                                	      tagItems{3}: ARRAY OF Utility.TagItem): BOOLEAN;
PROCEDURE WriteUMSConfigTags	*{base,-300}( acc{2}: LONGINT;
                                              tagItems{3}..: Utility.Tag): BOOLEAN;

PROCEDURE PrivateServerCall	*{base,-306}( command{2}: LONGINT;
                                              arg{3}	: APTR): LONGINT;

BEGIN
  base := Exec.OpenLibrary( libName, libVersion);
  IF base=NIL THEN
    SYSTEM.SETREG(0,Intuition.DisplayAlert(0,"\x00\x64\x14missing ums.library\o\o",50));
    HALT(0)
  END;

CLOSE
  IF base#NIL THEN Exec.CloseLibrary(base) END;
END ums.
