(*---------------------------------------------------------------------------
  :Program.    CxSupport.mod
  :Author.     Thomas Igracki
  :Address.    Obstallee 45, 1000 Berlin 20, W-Germany
  :E-Mail.     T.IGRACKI@BAMP.ZER
  :Version.    V1.1
  :Date.       Aug 1992
  :Copyright.  Me
  :Language.   Oberon
  :Translator. Amiga Oberon 2.14d
  :Contents.   Commodities.library support functions
  :Usage.      IMPORT CxSupport;
  :Remark.     OS2.0 Only!
  :Remark.     If you use a custom function then Compile & link _WITHOUT_
  :Remark.     SmallData (-d)! But you can link with option s (SMALL) and
  :Remark.     use (* $StackChk- $SaveAllRegs+ $ClearVars- $DeallocPars- *)
  :Remark.     for this Procedure!!
  :History.    V1.1: Added HotFunc()!
---------------------------------------------------------------------------*)
MODULE CxSupport;
IMPORT
   s: SYSTEM, e: Exec, c*: Commodities, OberonLib;
TYPE
   CustomProcType* = PROCEDURE (co: c.CxObjPtr; cxm: c.CxMsgPtr);
   (* define your custom function like this *)
VAR
   broker    : c.CxObjPtr; (* Only 1 Broker is supported! *)
   ExChSig*  : SHORTINT;   (* IF this Signal arrives then call HandleExCh() *)
   SendPort,
   ComPort   : e.MsgPortPtr;
   cxo       : c.CxObjPtr; (* Global CxObjPtr, get it with GetLastCxObj() *)

PROCEDURE InitBroker* (name, title, descr: ARRAY OF CHAR; showHide: BOOLEAN): SHORTINT;
VAR nb: c.NewBroker; err: LONGINT; (* $CopyArrays- *)
BEGIN
     IF broker # NIL THEN RETURN -1 END; (* Only 1! *)
     ComPort := e.CreateMsgPort(); IF ComPort = NIL THEN RETURN -1 END;
     SendPort:= e.CreateMsgPort(); IF SendPort= NIL THEN e.DeleteMsgPort(ComPort); RETURN -1 END;

     nb := c.NewBroker (c.nbVersion, NIL,NIL,NIL, c.notify, {},0,NIL,0);
     nb.name := s.ADR(name); nb.title := s.ADR(title); nb.descr := s.ADR(descr);
     nb.port := ComPort; ExChSig := ComPort.sigBit;
     IF showHide THEN nb.flags := {c.showHide} END;
     broker := c.CxBroker (nb,err);

     IF err # c.ErrOk THEN
        e.DeleteMsgPort (ComPort); e.DeleteMsgPort (SendPort); RETURN -1
     ELSE
        RETURN SendPort.sigBit
     END;
END InitBroker;

PROCEDURE Activate* (on: BOOLEAN);
BEGIN
     IF broker # NIL THEN
        s.SETREG (0,c.ActivateCxObj (broker,s.VAL(SHORTINT,on)))
     END;
END Activate;

PROCEDURE HotKeyID* (descr: ARRAY OF CHAR; ID: INTEGER); (* $CopyArrays- *)
BEGIN
     IF broker = NIL THEN RETURN END; (* First InitBroker() *)
     cxo := c.CxFilter (s.ADR(descr)); IF cxo = NIL THEN RETURN END;
     c.AttachCxObj (cxo, c.CxSender (SendPort, ID));
     c.AttachCxObj (cxo, c.CxTranslate (NIL));
     c.AttachCxObj (broker, cxo);
END HotKeyID;

PROCEDURE HotKeySig* (descr: ARRAY OF CHAR; sig: SHORTINT); (* $CopyArrays- *)
BEGIN
     IF broker = NIL THEN RETURN END; (* First InitBroker() *)
     cxo := c.CxFilter (s.ADR(descr)); IF cxo = NIL THEN RETURN END;
     c.AttachCxObj (cxo, c.CxSignal (OberonLib.Me, sig));
     c.AttachCxObj (cxo, c.CxTranslate (NIL));
     c.AttachCxObj (broker, cxo);
END HotKeySig;

PROCEDURE CxFuncID* (func: CustomProcType; ID: INTEGER);
BEGIN
     IF broker = NIL THEN RETURN END; (* First InitBroker() *)
     cxo := c.CxCustom (s.VAL(e.APTR,func),0); IF cxo = NIL THEN RETURN END;
     c.AttachCxObj (cxo, c.CxSender (SendPort, ID));
     c.AttachCxObj (cxo, c.CxTranslate (NIL));
     c.AttachCxObj (broker, cxo);
END CxFuncID;

PROCEDURE CxFunc* (func: CustomProcType);
BEGIN
     IF broker = NIL THEN RETURN END; (* First InitBroker()!! *)
     cxo := c.CxCustom (s.VAL(e.APTR,func),0); IF cxo = NIL THEN RETURN END;
     c.AttachCxObj (cxo, c.CxTranslate (NIL));
     c.AttachCxObj (broker, cxo);
END CxFunc;

PROCEDURE HotFunc* (descr: ARRAY OF CHAR; func: CustomProcType); (* $CopyArrays- *)
BEGIN
     IF broker = NIL THEN RETURN END; (* First InitBroker() *)
     cxo := c.CxFilter (s.ADR(descr)); IF cxo = NIL THEN RETURN END;
     c.AttachCxObj (cxo, c.CxCustom (s.VAL(e.APTR,func), 0));
     c.AttachCxObj (cxo, c.CxTranslate (NIL));
     c.AttachCxObj (broker, cxo);
END HotFunc;

PROCEDURE GetID*(): INTEGER;
BEGIN
     IF broker = NIL THEN RETURN -1 END;
     RETURN SHORT(c.CxMsgID (s.VAL(e.APTR,e.GetMsg (SendPort))))
END GetID;

PROCEDURE HandleExCh* (VAR show, hide, kill: BOOLEAN);
BEGIN
     show := FALSE; hide := FALSE; kill := FALSE;
     CASE c.CxMsgID (s.VAL(e.APTR,e.GetMsg (ComPort))) OF
           c.cmdDisable   : Activate(FALSE)
          |c.cmdEnable    : Activate(TRUE)
          |c.cmdAppear    : show := TRUE
          |c.cmdDisappear : hide := TRUE
          |c.cmdUnique,
           c.cmdKill      : kill := TRUE
     ELSE
     END
END HandleExCh;

PROCEDURE InformBroker* (co: c.CxObjPtr; cxm: c.CxMsgPtr);
BEGIN     c.DivertCxMsg (cxm,co,co);
END InformBroker;

PROCEDURE RemBroker*;
BEGIN
     IF broker # NIL THEN c.DeleteCxObjAll (broker); broker := NIL END;
     e.DeleteMsgPort (ComPort); e.DeleteMsgPort (SendPort);
     ComPort := NIL; SendPort := NIL;
END RemBroker;

BEGIN
    (* $IFNOT ClearVars *) broker := NIL (* $END *)
CLOSE
    RemBroker
END CxSupport.
