(*------------------------------------------

  :Program.     LockWindow.mod
  :Author.      Albert Weinert [awn]
  :Address.     Feldblumenweg 2, wD-5000 Kln 71
  :EMail.       ZNet: A.WEINERT@DARKNESS.ZER; Fido: Albert Weinert 2:241/4033
  :EMail.       InterNet: aweinert@darkness.gun.de
  :Version.     V1.1
  :Date.        19-Aug-1992
  :Copyright.   Albert Weinert 1992
  :Language.    Oberon
  :Translator.  Amiga Oberon 2.14d
  :Contents.    Funktionen um ein Window zu "verschliessen" so das keine Eingaben mehr in
  :Contents.    dieses Window gemacht werden kann.
  :Imports.     Nichts.
  :Remarks.     Nur unter AmigaOS 2.04 getestet. Msste aber auch unter 1.3 laufen.
  :Remarks.     Basiert auf SourceCodes von David Junod, Franz Josef Reichert und einem
  :Remarks.     Beispiel aus dem RKM:Libraries 3rd Edition.
  :Bugs.        keine bekannten
  :History.     V0.0   [awn] 02-Aug-1992 : Erstellt
  :History.     V1.0   [awn] 02-Aug-1992 : Funktioniert
  :History.     V1.1   [awn] 19-Aug-1992 : Das WaitPointer setzten in eine eigene Prozedur
  :History.            gepackt: Damit man den WaitPointer auch setzen kann ohne das Window zu
  :History.            "verschliessen".
  :History.     V1.1.1 [kai] 03-Nov-1992 : Unter 3.0 wird der Prefs-Waitpointer genommen

--------------------------------------------*)
MODULE LockWindow;
IMPORT I:Intuition,u:Utility;
CONST waBusyPointer  = I.waDummy + 035H; (* v39 *)
VAR ibase: I.IntuitionBasePtr;
PROCEDURE SetWindowPointer {ibase,-816} (win{8}: I.WindowPtr; tag1{9}..: u.Tag);
TYPE  sprite = ARRAY 36 OF INTEGER;
(* $DataChip+ *)
CONST waitPointer = sprite(
        00000H, 00000U,
        00400H, 007C0U,
        00000H, 007C0U,
        00100H, 00380U,
        00000H, 007E0U,
        007C0H, 01FF8U,
        01FF0H, 03FECU,
        03FF8H, 07FDEU,
        03FF8H, 07FBEU,
        07FFCH, 0FF7FU,
        07EFCH, 0FFFFU,
        07FFCH, 0FFFFU,
        03FF8H, 07FFEU,
        03FF8H, 07FFEU,
        01FF0H, 03FFCU,
        007C0H, 01FF8U,
        00000H, 007E0U,
        00000H, 00000H);             (* reserved, must be NULL *)
(* $DataChip- *)
(*------------------------------------------
  :Procedure.   SetWaitPointer
  :Date.        19-Aug-1992
  :Contents.    Setzt fr ein Window den WaitPointer der WB 2.04
  :Input.       window = Zeiger auf das Window fr den der WaitPointer ist
  :Result.      nichts
--------------------------------------------*)
PROCEDURE SetWaitPointer(window:I.WindowPtr);
BEGIN
  IF ibase.libNode.version < 39 THEN
    I.SetPointer(window,waitPointer,16,16,0,0)
  ELSE
    SetWindowPointer (window, waBusyPointer, I.LTRUE, u.done);
  END;
END SetWaitPointer;

(*------------------------------------------
  :Procedure.   ClearWaitPointer
  :Date.        19-Aug-1992
  :Contents.    Lscht den WaitPointer wieder und setzt den Pointer wieder auf den
  :Contents.    DefaultPointer
  :Input.       window = Zeiger auf das Window in der Pointer zurckgesetzt werden soll
  :Result.      nichts
--------------------------------------------*)
PROCEDURE ClearWaitPointer(window:I.WindowPtr);
BEGIN
  IF ibase.libNode.version < 39 THEN
    I.ClearPointer(window)
  ELSE
    SetWindowPointer (window, u.done);
  END;
END ClearWaitPointer;

(*------------------------------------------
  :Procedure.   LockWindow
  :Date.        19-Aug-1992
  :Contents.    "Verschliesst" ein Window damit keine eingaben mehr mglich sind und
  :Contents.    setzt zustzlich noch den WaitPointer damit der User was das er nicht
  :Contents.    eingeben darg (falls er nicht merkt :-)
  :Input.       window = Zeiger auf das Window der "verschlossen" werden soll
  :Result.      Zeiger auf einen Requester: Dieser MUSS an UnlockWindow() bergeben werden
  :Result.      sonst kann das Window nicht wieder "aufgeschlossen" werden.
--------------------------------------------*)
PROCEDURE LockWindow*(window:I.WindowPtr):I.RequesterPtr;
VAR req: I.RequesterPtr;
BEGIN
  IF window = NIL THEN RETURN NIL END;
  NEW(req);
  IF req # NIL THEN
    IF I.Request(req,window) THEN
      SetWaitPointer(window);
    ELSE
      DISPOSE(req);
    END;
  END;
  RETURN req;
END LockWindow;

(*------------------------------------------
  :Procedure.   UnlockWindow
  :Date.        02-Aug-1992
  :Contents.    Schliesst ein Window wieder auf und lscht den WaitPointer so das wieder
  :Contents.    eingaben in das Window mglich sind.
  :Input.       req = Zeiger auf den Requester den man von LockWindow bekommt.
  :Result.      nichts
--------------------------------------------*)
PROCEDURE UnlockWindow*(req:I.RequesterPtr);
BEGIN
  IF req # NIL THEN
    ClearWaitPointer(req.rWindow);
    I.EndRequest(req,req.rWindow);
    DISPOSE(req);
  END;
END UnlockWindow;

BEGIN
  ibase := I.int;
END LockWindow.
