(* ==================================================================== *)
(* === NEPHTHYS Module Library ======================================== *)
(* ==================================================================== *)
(*

  :Program.     Palette.MOD
  :Contents.    simple but useful, font-sensitive palette requester

  :Author.      Peter Frhlich [phf]
  :Copyright.   Copyright  1992 by NEPHTHYS Software, P.Frhlich.
  :Copyright.   Read the documentation for details.

  :Language.    Oberon (revised)
  :Translator.  Amiga Oberon V2.14

  :History.     V0.0 [phf] 10-Jun-1992 [created]
  :History.     V0.5 [phf] 10-Jun-1992 layout complete
  :History.     V0.6 [phf] 11-Jun-1992 colors work
  :History.     V0.9 [phf] 12-Jun-1992 swap, copy and spread
  :History.     V0.9 [phf] 13-Jun-1992 small cleanups
  :History.     V0.9 [phf] 26-Jun-1992 myLock/UnlockPubScreen(NIL)
  :History.     V1.0 [phf] 05-Jul-1992 req. resized, spread works
  :History.     V1.0 [phf] 05-Jul-1992 removed PubScreen handling
  :History.     V1.1 [phf] 09-Jul-1992 fixed help requester bug
  :History.     V1.1 [phf] 14-Jul-1992 [last edit]

  :Support.     Matthias Zepf "Palette.c" AmigaPLUS 4/92
  :Imports.     -
  :Bugs.        none known

  :Address.     Peter Frhlich, Ebenbckstrae 19, D-8000 Mnchen 60
  :Address.     Z-NET:P.FROEHLICH@AMC.ZER

  :Update.      -
  :Remark.      -

*)
(* ==================================================================== *)

MODULE Palette;

(* ==================================================================== *)

IMPORT
  i := Intuition, gt := GadTools, e := Exec,
  u := Utility, g := Graphics,
  SYSTEM;

(* ==================================================================== *)

(*
 * Screen and display stuff
 *)
VAR
  vi: gt.VisualInfo;
  screen : i.ScreenPtr;
  vp: g.ViewPortPtr;

(*
 * Window stuff
 *)
CONST
  ScreenTitle = "Palette Requester V1.1  1992 P.Frhlich";

VAR
  window: i.WindowPtr;

(*
 * Gadget stuff
 *)
CONST
  addBoolWidth = 8;
  addBoolHeight = 4;

CONST
  ContextID =  0;
  PaletteID =  1;
  RedID     =  2;
  GreenID   =  3;
  BlueID    =  4;
  UseID     =  5;
  CancelID  =  6;
  SwapID    =  7;
  CopyID    =  8;
  UndoID    =  9;
  SpreadID  = 10;

  RedName = "_R:     ";
  GreenName = "_G:     ";
  BlueName = "_B:     ";

  UseName = "_Use";
  UndoName = "U_ndo";
  CancelName = "_Cancel";

  SwapName = "Swap";
  CopyName = "Copy";
  SpreadName = "Spread";

  MaxBoolText = "XXXXXX"; (* 6 characters *)

VAR
  gadgetList, aGadget: i.GadgetPtr;
  redSlider, greenSlider, blueSlider : i.GadgetPtr;

(* MISC *)
CONST
  maxPaletteName* = 64;

VAR
  lastColor, currentColor: INTEGER;

  numColors : INTEGER;
  oldColors : ARRAY 256 OF INTEGER;

  title : ARRAY maxPaletteName OF CHAR;

  (* color values *)
  color, red, green, blue: INTEGER;
  (* saved colors *)
  sred, sgreen, sblue: INTEGER;

(*
 * Help_EasyRequester
 *)
CONST
  helpText = '    Palette Requester V1.1\n\
    1992 by Peter Frhlich\n\
\n\
Select  the color you want to\n\
change in the upper field and\n\
use the sliders to change the\n\
red,  green  and blue portion\n\
respectively.\n\
   "Use"    keeps   the   new\n\
palette,  "Undo"  takes  back\n\
the  last  slider changes and\n\
"Cancel"  restores  the  ori-\n\
ginal palette.\n\
   Please  note  that "Swap",\n\
"Copy" and "Spread" can\'t be\n\
undone.';

  helpEasy = i.EasyStruct(
                           SIZE(i.EasyStruct),
                           LONGSET{},
                           SYSTEM.ADR("Help"),
                           SYSTEM.ADR(helpText),
                           SYSTEM.ADR("Continue")
                         );

(* === stub routines ================================================== *)

VAR
  in : i.IntuitionBasePtr;

PROCEDURE myLockPubScreen  {in,-510}(name{8}          : SYSTEM.ADDRESS): i.ScreenPtr;
PROCEDURE myUnlockPubScreen{in,-516}(name{8}          : SYSTEM.ADDRESS;
                                     screen{9}        : i.ScreenPtr);

(* ==================================================================== *)

(* === exported utility routines ====================================== *)

PROCEDURE RGB*(c: INTEGER; VAR r,g,b: INTEGER);
(*
 * RGB - split the color "c" in it's red, green
 *       and blue parts.
 *)
BEGIN
  b := SYSTEM.VAL(INTEGER,SYSTEM.VAL(SET,c)*{0..3});
  c := SYSTEM.ROT(c,-4);
  g := SYSTEM.VAL(INTEGER,SYSTEM.VAL(SET,c)*{0..3});
  c := SYSTEM.ROT(c,-4);
  r := SYSTEM.VAL(INTEGER,SYSTEM.VAL(SET,c)*{0..3});
END RGB;

PROCEDURE COLOR*(r,g,b: INTEGER; VAR c: INTEGER);
(*
 * COLOR - create a color "c" from the given red, green
 *         and blue parts.
 *)
BEGIN
  c := 0;

  g := SYSTEM.ROT(g,4);
  r := SYSTEM.ROT(r,8);

  c := r+g+b;
END COLOR;

(* === private routines =============================================== *)

PROCEDURE OpenPalette(underMouse: BOOLEAN; x,y: INTEGER): BOOLEAN;
VAR
  ng: gt.NewGadget;
  boolWidth, boolHeight : INTEGER;
  w,h: INTEGER; (* window width and height *)
  xmin,ymin,xmax,ymax: INTEGER;

BEGIN
  boolWidth  := g.TextLength(SYSTEM.ADR(screen.rastPort),MaxBoolText,6) + addBoolWidth;
  boolHeight := screen.rastPort.font.ySize + addBoolHeight;

  w := 3 * boolWidth + 2 * gt.interWidth + 2 * gt.interWidth;
  h := 5 * boolHeight + screen.bitMap.depth * boolHeight + 5 * gt.interHeight + 2 * gt.interHeight;

  xmin := screen.wBorLeft + gt.interWidth;
  ymin := screen.wBorTop + screen.rastPort.font.ySize + gt.interHeight;
  xmax := w - screen.wBorRight - gt.interWidth;
  ymax := h - screen.wBorBottom - gt.interHeight;

  IF underMouse THEN
    x := screen.mouseX - w + boolWidth DIV 2;
    y := screen.mouseY - h - boolHeight DIV 2 + 2;
  END;
  IF x < 0 THEN x := 0; END; IF y < 0 THEN y := 0; END;

  vi := gt.GetVisualInfo(screen,u.done);

  (* create context for gadtools *)
  aGadget := gt.CreateContext(gadgetList);

  (* Palette gadget *)
  ng.leftEdge   := xmin;
  ng.topEdge    := ymin;
  ng.width      := xmax-xmin+gt.interWidth;
  ng.height     := boolHeight * screen.bitMap.depth;
  ng.gadgetText := NIL;
  ng.flags      := LONGSET{};
  ng.textAttr   := screen.font;
  ng.visualInfo := vi;
  ng.gadgetID   := PaletteID;
  aGadget := gt.CreateGadget(gt.paletteKind, aGadget, ng,
                             gt.paDepth,screen.bitMap.depth,
                             gt.paColor,0,
                             gt.paIndicatorWidth,boolWidth DIV 2,
                             gt.paIndicatorHeight,boolHeight * 2,
                             u.done);


  (* Copy gadget *)
  ng.topEdge    := ng.topEdge+ng.height+gt.interHeight;
  ng.width      := boolWidth;
  ng.height     := boolHeight;
  ng.gadgetText := SYSTEM.ADR(CopyName);
  ng.gadgetID   := CopyID;
  aGadget := gt.CreateGadget(gt.buttonKind, aGadget, ng,
                             u.done);



  (* Swap gadget *)
  ng.leftEdge   := ng.leftEdge + ng.width + gt.interWidth;
  ng.gadgetText := SYSTEM.ADR(SwapName);
  ng.gadgetID   := SwapID;
  aGadget := gt.CreateGadget(gt.buttonKind, aGadget, ng,
                             u.done);


  (* Spread gadget *)
  ng.leftEdge   := ng.leftEdge + ng.width + gt.interWidth;
  ng.gadgetText := SYSTEM.ADR(SpreadName);
  ng.gadgetID   := SpreadID;
  aGadget := gt.CreateGadget(gt.buttonKind,
                             aGadget,
                             ng,
                             u.done);



  (* Red slider *)
  ng.leftEdge   := xmin+boolWidth+gt.interWidth;
  ng.topEdge    := ng.topEdge + ng.height +gt.interHeight;
  ng.width      := boolWidth*2+gt.interWidth;
  ng.gadgetText := SYSTEM.ADR(RedName);
  ng.gadgetID   := RedID;
  aGadget := gt.CreateGadget(gt.sliderKind, aGadget, ng,
                             gt.underscore,ORD("_"),
                             gt.slMin,0,
                             gt.slMax,15,
                             gt.slLevel,0,
                             gt.slMaxLevelLen,2,
                             gt.slLevelFormat,SYSTEM.ADR("%2.2ld "),
                             i.gaImmediate,i.LTRUE,
                             i.gaFollowMouse,i.LTRUE,
                             i.gaRelVerify,i.LTRUE,
                             u.done);

  redSlider := aGadget;

  (* Green slider *)
  ng.topEdge    := ng.topEdge + ng.height + gt.interHeight;
  ng.gadgetText := SYSTEM.ADR(GreenName);
  ng.gadgetID   := GreenID;
  aGadget := gt.CreateGadget(gt.sliderKind, aGadget, ng,
                             gt.underscore,ORD("_"),
                             gt.slMin,0,
                             gt.slMax,15,
                             gt.slLevel,0,
                             gt.slMaxLevelLen,2,
                             gt.slLevelFormat,SYSTEM.ADR("%2.2ld "),
                             i.gaImmediate,i.LTRUE,
                             i.gaFollowMouse,i.LTRUE,
                             i.gaRelVerify,i.LTRUE,
                             u.done);

  greenSlider := aGadget;

  (* Blue slider *)
  ng.topEdge    := ng.topEdge + ng.height + gt.interHeight;
  ng.gadgetText := SYSTEM.ADR(BlueName);
  ng.gadgetID   := BlueID;
  aGadget := gt.CreateGadget(gt.sliderKind, aGadget, ng,
                             gt.underscore,ORD("_"),
                             gt.slMin,0,
                             gt.slMax,15,
                             gt.slLevel,0,
                             gt.slMaxLevelLen,2,
                             gt.slLevelFormat,SYSTEM.ADR("%2.2ld "),
                             i.gaImmediate,i.LTRUE,
                             i.gaFollowMouse,i.LTRUE,
                             i.gaRelVerify,i.LTRUE,
                             u.done);

  blueSlider := aGadget;

  (* Use gadget *)
  ng.leftEdge   := xmin;
  ng.topEdge    := ng.topEdge + ng.height + gt.interHeight;
  ng.width      := boolWidth;
  ng.gadgetText := SYSTEM.ADR(UseName);
  ng.gadgetID   := UseID;
  aGadget := gt.CreateGadget(gt.buttonKind, aGadget, ng,
                             gt.underscore,ORD("_"),
                             u.done);



  (* Undo gadget *)
  ng.leftEdge   := ng.leftEdge + ng.width + gt.interWidth;
  ng.gadgetText := SYSTEM.ADR(UndoName);
  ng.gadgetID   := UndoID;
  aGadget := gt.CreateGadget(gt.buttonKind, aGadget, ng,
                             gt.underscore,ORD("_"),
                             u.done);


  (* Cancel gadget *)
  ng.leftEdge   := ng.leftEdge + ng.width + gt.interWidth;
  ng.gadgetText := SYSTEM.ADR(CancelName);
  ng.gadgetID   := CancelID;
  aGadget := gt.CreateGadget(gt.buttonKind, aGadget, ng,
                             gt.underscore,ORD("_"),
                             u.done);


  (* open the window *)
  window := i.OpenWindowTagsA(NIL,
    i.waLeft, x,
    i.waTop, y,
    i.waInnerWidth, w,
    i.waInnerHeight, h,
    i.waIDCMP,SYSTEM.VAL(LONGINT,LONGSET{i.closeWindow,i.rawKey,i.vanillaKey}+gt.buttonIDCMP+gt.paletteIDCMP+gt.sliderIDCMP),
    i.waTitle, SYSTEM.ADR(title),
    i.waScreenTitle, SYSTEM.ADR(ScreenTitle),
    i.waCustomScreen, screen, (* ??? *)
    i.waDragBar, i.LTRUE,
    i.waDepthGadget, i.LTRUE,
    i.waCloseGadget, i.LTRUE,
    i.waActivate, i.LTRUE,
    i.waSmartRefresh, i.LTRUE,
    i.waAutoAdjust, i.LTRUE,
    i.waRMBTrap, i.LTRUE,
    u.done
  );

  SYSTEM.SETREG(0,i.AddGList(window,gadgetList,-1,-1,NIL));
  i.RefreshGList(gadgetList,window,NIL,-1);
  gt.RefreshWindow(window,NIL);

  RETURN TRUE;
END OpenPalette;


PROCEDURE ClosePalette();
BEGIN
  IF window # NIL THEN
    i.CloseWindow(window); window := NIL;
  END;

  IF gadgetList # NIL THEN
    gt.FreeGadgets(gadgetList); gadgetList := NIL;
  END;

  IF vi # NIL THEN
    gt.FreeVisualInfo(vi); vi := NIL;
  END;
END ClosePalette;


(* ==================================================================== *)


PROCEDURE SaveColors();
VAR
  i : INTEGER;
BEGIN
  numColors := vp.colorMap.count;

  i := 0;
  WHILE (i < numColors) DO
    oldColors[i] := g.GetRGB4(vp.colorMap,i);
    INC(i);
  END;
END SaveColors;

PROCEDURE RestoreColors();
VAR
  i : INTEGER;
BEGIN
  g.LoadRGB4(vp,oldColors,numColors);
END RestoreColors;


PROCEDURE SaveColor(i: INTEGER);
VAR
  c: INTEGER;
BEGIN
  c := g.GetRGB4(vp.colorMap,i);
  RGB(c,sred,sgreen,sblue);
END SaveColor;

PROCEDURE RestoreColor(i: INTEGER);
BEGIN
  g.SetRGB4(vp,i,sred,sgreen,sblue);
END RestoreColor;


PROCEDURE SwapColors(i,j: INTEGER);
VAR
  ci, cj, r, gr, b: INTEGER;
BEGIN
  ci := g.GetRGB4(vp.colorMap,i);
  cj := g.GetRGB4(vp.colorMap,j);

  RGB(ci,r,gr,b);
  g.SetRGB4(vp,j,r,gr,b);

  RGB(cj,r,gr,b);
  g.SetRGB4(vp,i,r,gr,b);
END SwapColors;

PROCEDURE CopyColors(i, (*TO*) j: INTEGER);
VAR
  c, r, gr, b: INTEGER;
BEGIN
  c := g.GetRGB4(vp.colorMap,i);

  RGB(c,r,gr,b);
  g.SetRGB4(vp,j,r,gr,b);
END CopyColors;

PROCEDURE SpreadColors(i,j: INTEGER);
VAR
  n, steps,ri,gi,bi,rj,gj,bj,ir,ig,ib,
  ci,cj,h: INTEGER;
BEGIN
  IF (i > j) THEN h := i; i := j; j := h; END;
  (* ensure i <= j *)

  ci := g.GetRGB4(vp.colorMap,i);
  cj := g.GetRGB4(vp.colorMap,j);

  steps := j - i;
  IF ( steps > 1 ) THEN
    (* i and j are not neighbours *)

    RGB(ci,ri,gi,bi); RGB(cj,rj,gj,bj);

    ri := ri * 128; rj := rj * 128;
    gi := gi * 128; gj := gj * 128;
    bi := bi * 128; bj := bj * 128;

    ir := (rj - ri) DIV steps;
    ig := (gj - gi) DIV steps;
    ib := (bj - bi) DIV steps;

    n := 1;
    WHILE (n < steps) DO

      ri := ri + ir; gi := gi + ig; bi := bi + ib;
      g.SetRGB4(vp,i+n,ri DIV 128,gi DIV 128,bi DIV 128);

      INC(n);
    END;

  END;

END SpreadColors;

(* ==================================================================== *)

PROCEDURE SetSliders();
BEGIN
  color := g.GetRGB4(vp.colorMap,currentColor);
  blue  := SYSTEM.VAL(INTEGER,SYSTEM.VAL(SET,color)*{0..3});
  color := SYSTEM.ROT(color,-4);
  green := SYSTEM.VAL(INTEGER,SYSTEM.VAL(SET,color)*{0..3});
  color := SYSTEM.ROT(color,-4);
  red   := SYSTEM.VAL(INTEGER,SYSTEM.VAL(SET,color)*{0..3});

  gt.SetGadgetAttrs(redSlider^,window,NIL,gt.slLevel,red,u.done);
  gt.SetGadgetAttrs(greenSlider^,window,NIL,gt.slLevel,green,u.done);
  gt.SetGadgetAttrs(blueSlider^,window,NIL,gt.slLevel,blue,u.done);
END SetSliders;


PROCEDURE SetColor();
BEGIN
  g.SetRGB4(vp,currentColor,red,green,blue);
END SetColor;


PROCEDURE More(VAR c: INTEGER);
BEGIN
  IF (c < 15) THEN INC(c); END;
END More;

PROCEDURE Less(VAR c: INTEGER);
BEGIN
  IF (c > 0) THEN DEC(c); END;
END Less;

(* ==================================================================== *)

PROCEDURE Palette*( scr: i.ScreenPtr; underMouse: BOOLEAN; x,y: INTEGER;
                    tit: ARRAY OF CHAR ): INTEGER;

(*
 *
 *
 *)

VAR
  message : i.IntuiMessagePtr;
  gadget : i.GadgetPtr;
  class : LONGSET;
  code : INTEGER;
  qual : SET;
  key : CHAR;
  stringInfo : i.StringInfoPtr;

  (* modes *)
  swap, copy, spread : BOOLEAN;

  (* lock for intuition base *)
  lock : LONGINT;

BEGIN
  swap := FALSE; copy := FALSE; spread := FALSE;
  COPY(tit,title);
  screen := scr;
  currentColor := -1; (* error condition *)

  IF screen # NIL THEN
    vp := SYSTEM.ADR(screen.viewPort);

    SaveColors();
    IF OpenPalette(underMouse,x,y) THEN END;

    currentColor := 0; SaveColor(currentColor);
    SetSliders();

    (* IDCMP *)
    LOOP (* FOREVER *)

      e.WaitPort(window.userPort);
      message := gt.GetIMsg(window.userPort);
      WHILE (message # NIL) DO

        class := message.class;
        code  := message.code;
        gadget := message.iAddress;
        qual := message.qualifier;
        gt.ReplyIMsg(message);

        IF i.closeWindow IN class THEN
          RestoreColors; EXIT;
        ELSIF i.vanillaKey IN class THEN
          key := CHR(code);
          CASE key OF
            "c","C" : RestoreColors; EXIT;|
            "u","U" : EXIT;|
            "n","N" : RestoreColor(currentColor); SetSliders();|

            "r" : More(red);   SetColor; SetSliders; |
            "g" : More(green); SetColor; SetSliders; |
            "b" : More(blue);  SetColor; SetSliders; |

            "R" : Less(red);   SetColor; SetSliders; |
            "G" : Less(green); SetColor; SetSliders; |
            "B" : Less(blue);  SetColor; SetSliders; |
          ELSE
            (* all other keys do nothing *)
          END;
        ELSIF i.rawKey IN class THEN
          CASE code OF
            05FH: IF i.EasyRequest(window,SYSTEM.ADR(helpEasy),NIL,u.done) = 0 THEN END;|
          ELSE
            (* all other keys do nothing *)
          END;
        ELSIF (i.gadgetUp IN class) OR (i.gadgetDown IN class) THEN
          CASE gadget.gadgetID OF
             PaletteID: currentColor := code;

                        IF swap THEN
                          SwapColors(lastColor,currentColor); swap := FALSE;
                        ELSIF copy THEN
                          CopyColors(lastColor,currentColor); copy := FALSE;
                        ELSIF spread THEN
                          SpreadColors(lastColor,currentColor); spread := FALSE;
                        END;

                        SaveColor(currentColor); SetSliders(); |

             RedID  : red   := code; SetColor(); |
             GreenID: green := code; SetColor(); |
             BlueID : blue  := code; SetColor(); |

             UseID   : EXIT; |
             UndoID  : RestoreColor(currentColor); SetSliders();|
             CancelID: RestoreColors; EXIT;|

             SwapID  : swap := TRUE; copy := FALSE; spread := FALSE;
                       lastColor := currentColor;|
             CopyID  : copy := TRUE; swap := FALSE; spread := FALSE;
                       lastColor := currentColor;|
             SpreadID: spread := TRUE; copy := FALSE; swap := FALSE;
                       lastColor := currentColor;|
          ELSE
            (*i.DisplayBeep(NIL);*)
          END;
        ELSIF (i.mouseMove IN class) THEN
          CASE gadget.gadgetID OF
             RedID  : red   := code; SetColor(); |
             GreenID: green := code; SetColor(); |
             BlueID : blue  := code; SetColor(); |
          ELSE
            (*i.DisplayBeep(NIL);*)
          END;
        ELSIF i.refreshWindow IN class THEN
          (*IF NOT RefreshPanel() THEN EXIT; END;*)
          gt.BeginRefresh(window); gt.EndRefresh(window,i.LTRUE);
        ELSE
          (*i.DisplayBeep(NIL);*) (* we received an IDCMP we didn't ask for *)
        END;

        message := gt.GetIMsg(window.userPort);
      END;

    END;

    ClosePalette();
  END;

  RETURN currentColor;
END Palette;

(* ==================================================================== *)
BEGIN
  in := i.int;
  gadgetList := NIL; window := NIL; vi := NIL;
CLOSE
  ClosePalette();
END Palette.
(* ==================================================================== *)
