(*------------------------------------------

  :Program.     TextStreams
  :Author.      Dieter Bogdoll [db]
  :Address.     SMail: Gudrunstr. 7, D-8000 Mnchen 19
  :Address.     EMail: landon@amc.zer
  :Address.     EMail: 2:246/221.3@fidonet
  :Address.     EMail: bogdoll@informatik.tu-muenchen.de
  :Version.     V1.0
  :Date.        19-Feb-1993
  :Copyright.   Public Domain
  :Language.    Oberon
  :Translator.  Amiga Oberon 3.00d
  :Contents.    Erweiterung von Streams fuer Ein/Ausgabe ueber CON:
  :Imports.     Streams [db], Format [db]
  :Bugs.        nicht ausfuehrlich getestet
  :History.     V0.0   [db] 15-Feb-1993 : Erstellt
  :History.     V1.0   [db] 19-Feb-1993 : Gebrauchsfertig

--------------------------------------------*)
MODULE TextStreams;

(* $SET Fast <- dadurch werden einige Routinen (fuer den Amiga ) optimiert *)

IMPORT e:=Exec, y:=SYSTEM, d:=Dos, Streams, f:=Format;

CONST
  eof = 1CX;

PROCEDURE Length (str: ARRAY OF CHAR): LONGINT;
  (* $EntryExitCode- *)
  BEGIN
    y.INLINE( 0225FH, 0201FH, 0205FH, 05380H, 02200H, 04A18H, 057C9H, 0FFFCH,
              06708H, 00481H, 00001H, 00000H, 06AF0H, 09081H, 04ED1H);
END Length;

TYPE
  TextStream* = POINTER TO TextStreamDesc;
  TextStreamDesc* = RECORD (Streams.StreamDesc)
                      x,y,w,h: INTEGER;
                      in,out: d.FileHandlePtr;
                    END; (* abstract *)

  PROCEDURE (s: TextStream) Open* (name: ARRAY OF CHAR; new: BOOLEAN): BOOLEAN;
    (* new hier nur ein Dummy *)
    VAR helpstr : ARRAY 255 OF CHAR;
    BEGIN
      IF Length(name)>0 THEN
          f.Format5(helpstr, "CON:%ld/%ld/%ld/%ld/%s",s.x,s.y,s.w,s.h,y.ADR(name));
          s.in := d.Open(helpstr,d.newFile);
          s.out:= s.in;
        ELSE
          s.in := d.Input();
          s.out:= d.Output();
      END;
      RETURN TRUE
  END Open;

  PROCEDURE (s: TextStream) Close* (): BOOLEAN;
    BEGIN
      IF (s.in#d.Input()) THEN
          RETURN d.Close(s.in)
        ELSE
          RETURN TRUE
      END;
  END Close;

  PROCEDURE (s: TextStream) Write* (ch: CHAR);
    BEGIN y.SETREG(0,d.Write(s.out,ch,1));
  END Write;

  PROCEDURE (s: TextStream) WriteLn*;
    BEGIN
      (* $IF Fast *)
        y.SETREG(0,d.Write(s.out,"\n",1));
      (* $ELSE *)
        s.Write("\n");
      (* $END *)
  END WriteLn;

(* $IF Fast *)
  PROCEDURE (s: TextStream) WriteString*(a: ARRAY OF CHAR);
    BEGIN
      y.SETREG(0,d.Write(s.out,a,Length(a)));
  END WriteString;
(* $END *)

  PROCEDURE (s: TextStream) Read* (VAR ch: CHAR);
    BEGIN IF d.Read(s.in,ch,1)#1 THEN ch:=eof END;
  END Read;

PROCEDURE Create*(x,y,w,h: INTEGER): TextStream;
(*------------------------------------------
  :Input.     x,y : Koordinaten des Textfensters
  :Input.     w,h : Breite und Hoehe des Fensters
  :Output.    liefert einen TextStream, der aber
  :Output.    noch geoeffnet werden muss
  :Update.    19-Feb-1993  [db] - Erstellt
--------------------------------------------*)
  VAR t: TextStream;
  BEGIN
    NEW(t);
    t.x := x; t.y := y;
    t.w := w; t.h := h;
    RETURN t
END Create;


END TextStreams.
