(*----------------------------------------------------------------------------
 :Program.     FreqCount.mod
 :Contents.    Frequenz-Messer
 :Author.      Christian Stiens
 :Address.     Snail-Mail:           E-Mail:
 :Address.     Heustiege 2           FIDO: 2:245/5802.25
 :Address.     W-4710 Ldinghausen   UUCP: Christian_Stiens@ouzonix.bo.open.de
 :Copyright.   Freeware,  93 by me
 :Language.    Oberon-2
 :Translator.  Amiga Oberon 3.01
 :History.     V1.0, 25-Feb-93
 :Imports.     DeviceSupport37, SegNum
----------------------------------------------------------------------------*)

MODULE FreqCount;

  IMPORT
    au := Audio,
    d  := Dos,
    e  := Exec,
    ds := DeviceSupport37,
    g  := Graphics,
    hw := Hardware,
       (* io,
    trans:=MathTrans  *)
          Misc,
    I  := Intuition,
    sn := SegNum,
    ol := OberonLib,
    rq := Requests,
    SYS:= SYSTEM;

  CONST
    version = "\o$VER: freqcount 1.0 (25.2.93)";
    n = 512;
    sampfreq = 23646.0;
    per = 75;

  VAR
    ioa      : au.IOAudioPtr;
    win      : I.WindowPtr;
    chan     : SHORTINT;
    notes    : ARRAY 12,2 OF CHAR;
    msg      : I.IntuiMessagePtr;
    h        : LONGINT;
    close    : BOOLEAN;
    pp,pb    : e.APTR;
    f        : REAL;
    wave     : ARRAY n OF INTEGER;

(*---------------------------------------------------------------------*)

  PROCEDURE Record;
    VAR i: INTEGER;
        audci: SHORTINT;
        prb: ARRAY n OF SHORTINT;
  BEGIN
    (* $RangeChk- $OvflChk- $NilChk- *)

    audci := hw.aud0i + chan;

    (* use audio channel for timing: *)

    hw.custom.intena := {audci};    (* disable audio interrupt *)

    hw.custom.aud[chan].ptr := NIL;
    hw.custom.aud[chan].len := 1;   (* one word *)
    hw.custom.aud[chan].per := per;
    hw.custom.aud[chan].vol := 0;

    hw.custom.dmacon := {hw.aud0+chan}; (* Audio DMA off *)

    e.Disable;

    hw.custom.intreq := {audci};
    hw.custom.aud[chan].dat := 0;

    prb[0] := SYS.VAL(SHORTINT,hw.ciaa.prb);

    FOR i := 0 TO n-1 DO
      REPEAT UNTIL audci IN hw.custom.intreqr; (* Wait on audio interrupt *)
      hw.custom.intreq := {audci};             (* reset interrupt bit *)
      hw.custom.aud[chan].dat := 0;
      prb[i] := SYS.VAL(SHORTINT,hw.ciaa.prb); (* read the parallel port *)
    END;

    e.Enable;

    FOR i := 0 TO n-1 DO
      wave[i] := LONG(prb[i]+SYS.VAL(SHORTINT,-128));
    END;

    (* $RangeChk= $OvflChk= $NilChk= *)

  END Record;

(*---------------------------------------------------------------------*)

  PROCEDURE Measure(): REAL;
    VAR i,lasty,y,zerocnt:INTEGER;
        lastzero,x,firstzero: REAL;
        ysum: LONGINT; ycnt: INTEGER;
        notgotfirst: BOOLEAN;
  BEGIN
    zerocnt := 0;
    ysum := 0;
    ycnt := 0;
    notgotfirst := TRUE;
    lasty := wave[0];
    FOR i := 1 TO n-1 DO
      y := wave[i];
      INC(ysum,ABS(y)); INC(ycnt);
      IF (lasty <= 0) & (y > 0) THEN             (* Nulldurchgang? *)
        x := i - y/(y-lasty);
        IF notgotfirst THEN firstzero := x; notgotfirst := FALSE END;
        lastzero := x;
        INC(zerocnt);
      END;
      lasty := y;
    END;
    IF (ysum DIV ycnt > 7) & (zerocnt > 1) THEN
      RETURN (sampfreq / (lastzero-firstzero)) * (zerocnt-1);
    END;
    RETURN 0
  END Measure;

(*---------------------------------------------------------------------*)

  PROCEDURE IoInit (ioa: e.MessagePtr);
    CONST allocMap = "\x01\x02\x04\x08";
  BEGIN
    WITH ioa: au.IOAudioPtr DO
      ioa.data := SYS.ADR(allocMap);
      ioa.length := 4;
      ioa.request.message.node.pri := 127;
    END;
  END IoInit;

(*---------------------------------------------------------------------*)

BEGIN
  pp := -1; pb := -1;
  IF (I.base.libNode.version<37) OR (ol.OldSP.stackSize<4000) THEN HALT(d.error) END;
  IF version="" THEN END;

  e.CopyMem("C C#D D#E F F#G G#A A#H ",notes,SIZE(notes));

  Misc.base := e.OpenResource(Misc.miscName);
  IF Misc.base=NIL THEN HALT(0) END;

  pp := Misc.AllocMiscResource(Misc.parallelPort,"FreqCount");
  pb := Misc.AllocMiscResource(Misc.parallelBits,"FreqCount");
  rq.Assert((pp=NIL) & (pb=NIL),"Can't alloc parallel port");

  ioa := ds.OpenDev(au.audioName,0,LONGSET{},SIZE(au.IOAudio),IoInit);
  rq.Assert(ioa # NIL,"Can't open audio.device");

  CASE SYS.VAL(LONGINT,ioa.request.unit) OF
    1: chan := 0 |
    2: chan := 1 |
    4: chan := 2 |
    8: chan := 3     ELSE HALT(0)
  END;

  win := I.OpenWindowTagsA(NIL, I.waLeft,100,
                                I.waTop, 50,
                                I.waInnerWidth, 330,
                                I.waInnerHeight, 55,
                                I.waFlags, LONGSET{I.windowDrag,I.windowDepth,I.windowClose,I.activate,I.noCareRefresh},
                                I.waIDCMP, LONGSET{I.closeWindow},
                                I.waTitle, SYS.ADR("FreqCount"));
  rq.Assert(win#NIL,"Can't open window");

  g.SetAPen(win.rPort,2);
  g.SetOPen(win.rPort,1);

  hw.ciaa.ddrb := SHORTSET{};                   (* All Bits are input *)
  hw.ciab.ddra := hw.ciab.ddra + SHORTSET{1,2}; (* Bit 1 & 2 are output *)
  INCL(hw.ciab.pra,1); EXCL(hw.ciab.pra,2);     (* choose left channel *)

  close := FALSE;

  WHILE ~close DO

    msg := e.GetMsg(win.userPort);
    IF msg#NIL THEN
      close := I.closeWindow IN msg.class;
      e.ReplyMsg(msg);
    END;

    Record;
    f := Measure();

    (*- Diese Kommentarklammer lschen, wenn Notenausgabe erwnscht

    IF f>=128 THEN
      h := ENTIER((trans.Log(f) - 4.873767) * 17.31234 + 0.5);
      io.WriteString(notes[h MOD 12]); io.WriteInt(h DIV 12,1); io.WriteLn;
    END;

    -*)

    sn.DrawSegNum(win.rPort,win.borderLeft+8,win.borderTop+4, 40,40, ENTIER(f), 5);

    d.Delay(10);

  END;

CLOSE

  IF win#NIL THEN I.CloseWindow(win) END;
  IF (pp=NIL)&(pb=NIL) THEN hw.ciab.ddra := hw.ciab.ddra-SHORTSET{1,2} END;
  ds.CloseDev(ioa);
  IF pp=NIL THEN Misc.FreeMiscResource(Misc.parallelPort) END;
  IF pb=NIL THEN Misc.FreeMiscResource(Misc.parallelBits) END;

END FreqCount.


