MODULE Stacks;

(* ------------------------------------------------------------------------ *)
(* Module fuer den Umgang mit der dyn. Datenstruktur 'Stack'                *)
(* ------------------------------------------------------------------------ *)
(* NewStack      : Neuer Stack anlegen                                      *)
(* DisposeStack  : Stack loeschen                                           *)
(* Get           : Element vom Stack nehmen                                 *)
(* Put           : Element auf den Stack legen                              *)
(* GetNodeNum    : Zeiger auf das N-tes Element anfordern                   *)
(* IsEmpty       : Pruefen, ob der Stack leer ist                           *)
(* NumberOfNodes : Wieviele Elemente sind auf dem Stack?                    *)
(* ------------------------------------------------------------------------ *)
(* (c) by Roger Meyer, 1992                                                 *)
(* ------------------------------------------------------------------------ *)

TYPE
  StackPtr*= POINTER TO Stack;
  NodePtr *= POINTER TO Node;

  Node    *= RECORD
               next  : NodePtr;
             END;

  Stack   *= RECORD
               size  : INTEGER;
               first : NodePtr;
             END;


PROCEDURE NewStack*():StackPtr;

VAR
  Result : StackPtr;

BEGIN
  NEW(Result);
  Result.size  := 0;
  Result.first := NIL;

  RETURN Result;

END NewStack;


PROCEDURE DisposeStack*(VAR SP: StackPtr);

VAR
  Temp,Next : NodePtr;

BEGIN
  Temp := SP.first;

  WHILE Temp#NIL DO
    Next := Temp.next;
    DISPOSE(Temp);
    Temp := Next;
  END;

  DISPOSE(SP);

END DisposeStack;


PROCEDURE Get*(SP : StackPtr):NodePtr;

VAR
  Result : NodePtr;

BEGIN
  IF SP.size#0 THEN
    Result      := SP.first;
    SP.first    := SP.first.next;
    DEC(SP.size);
    RETURN Result;

  ELSE
    RETURN NIL;

  END;

END Get;


PROCEDURE GetNodeNum*(SP : StackPtr;C : INTEGER):NodePtr;

VAR
  Result : NodePtr;
  Cnt    : INTEGER;

BEGIN
  Result := SP.first;
  Cnt    := 0;

  WHILE (Cnt<C) AND (Result#NIL) DO
    Result := Result.next;
    INC(Cnt);
  END;

  RETURN Result;

END GetNodeNum;


PROCEDURE Put*(SP : StackPtr; NP : NodePtr);

BEGIN
  NP.next   := SP.first;
  SP.first  := NP;
  INC(SP.size);

END Put;


PROCEDURE IsEmpty*(SP : StackPtr):BOOLEAN;

BEGIN
  RETURN SP.size=0;

END IsEmpty;


PROCEDURE NumberOfNodes*(SP : StackPtr):INTEGER;

BEGIN
  RETURN SP.size;

END NumberOfNodes;


END Stacks.
