(*----------------------------------------------------------------------------
 :Program.     DHT.mod
 :Contents.    Hartley Transform
 :Author.      Christian Stiens
 :Address.     Snail-Mail:           E-Mail:
 :Address.     Heustiege 2           FIDO: 2:245/5802.25
 :Address.     W-4710 Ldinghausen   UUCP: Christian_Stiens@ouzonix.bo.open.de
 :Copyright.   public domain
 :Language.    Oberon-2, 68000 Assembler
 :Translator.  Amiga Oberon 3.01
 :History.     V1.0, 30-Mar-93
 :Support.     Daniel Wesemann
----------------------------------------------------------------------------*)

MODULE DHT;

  (* $JOIN sintab_1024x16.o $JOIN DHT.o *)

  IMPORT
    SYS := SYSTEM;


  CONST
    sintablen = 1024;


  VAR
    sintab ["_sintab_1024x16"] : ARRAY sintablen+sintablen DIV 4 OF INTEGER;


  PROCEDURE Sqr (i{0}: INTEGER): LONGINT; (* $EntryExitCode- *)
  BEGIN
    SYS.INLINE(0C1C0H,04E75H);   (* MULS D0,D0 *)
  END Sqr;


  PROCEDURE Mulu (i{0},j{1}: INTEGER): LONGINT; (* $EntryExitCode- *)
  BEGIN
    SYS.INLINE(0C0C1H,04E75H);   (* MULU D1,D0 *)
  END Mulu;


  PROCEDURE Int2Long (i{1}: INTEGER): LONGINT; (* $EntryExitCode- *)
  BEGIN
    SYS.INLINE(07000H,03001H,04E75H);
  END Int2Long;


  PROCEDURE Sqrt (i{1}: LONGINT): INTEGER; (* $EntryExitCode- *)
  BEGIN
    SYS.INLINE(07000H,            (*      MOVEQ  #0,D0     *)
               02201H,            (*      MOVE.L D1,D1     *)
               0671CH,            (*      BEQ.S  ende      *)
               0303CH,08000H,     (*      MOVE.W #32768,D0 *)
               03400H,            (*      MOVE.W D0,D2     *)
               0E24AH,            (*      LSR.W  #1,D2     *)
               03600H,            (* loop MOVE.W D0,D3     *)
               0C6C3H,            (*      MULU   D3,D3     *)
               0B283H,            (*      CMP.L  D3,D1     *)
               06708H,            (*      BEQ.S  \2        *)
               06204H,            (*      BHI.S  \1        *)
               09042H,            (*      SUB.W  D2,D0     *)
               06002H,            (*      BRA.S  \2        *)
               0D042H,            (* \1   ADD.W  D2,D0     *)
               0E24AH,            (* \2   LSR.W  #1,D2     *)
               066ECH,            (*      BNE.S  loop      *)
               04E75H)            (* ende RTS              *)
  END Sqrt;



  (* $RangeChk- $OvflChk- $NilChk- $StackChk- *)


  PROCEDURE DHTASM {"_DHT"} (in{8},out{9}: SYS.PTR; n{0},log2n{1}: INTEGER; sintab{10}: SYS.PTR);


  PROCEDURE DHT* (VAR in,out: ARRAY OF INTEGER; n: INTEGER);

    VAR log2n: INTEGER;

  BEGIN
    log2n := 0; WHILE ASH(LONG(1),log2n) < n DO INC(log2n) END;
    IF (LEN(in)<n) OR (LEN(out)<n) OR (n#ASH(LONG(1),log2n)) THEN RETURN END;
    DHTASM(SYS.ADR(in),SYS.ADR(out),n,log2n,SYS.ADR(sintab));
  END DHT;


  PROCEDURE Abs* (VAR arr: ARRAY OF INTEGER; n: INTEGER);
    VAR i: INTEGER;
  BEGIN
    IF LEN(arr) >= n THEN
      i := n DIV 2;
      REPEAT
        arr[i] := SHORT(SYS.LSH(Mulu(Sqrt(Sqr(arr[i])+Sqr(arr[n-i])),181),-8));
        arr[n-i] := arr[i];
        DEC(i);
      UNTIL i=0;
    END;
  END Abs;


  PROCEDURE Abs2* (VAR arr: ARRAY OF INTEGER; n: INTEGER);
    VAR i: INTEGER;
  BEGIN
    IF LEN(arr) >= n THEN
      i := n DIV 2;
      REPEAT
        arr[i] := Sqrt(Int2Long(Sqrt(Sqr(arr[i])+Sqr(arr[n-i])))*16384);
        arr[n-i] := arr[i];
        DEC(i);
      UNTIL i=0;
    END;
  END Abs2;


END DHT.

