(*---------------------------------------------------------------------------
 :Program.     Spectroscope.mod
 :Contents.    Realtime frequency analysis with sound samplers
 :Author.      Christian Stiens
 :Address.     Snail-Mail:           E-Mail:
 :Address.     Heustiege 2           FIDO: 2:245/5802.25
 :Address.     W-4710 Ldinghausen   UUCP: Christian_Stiens@ouzonix.bo.open.de
 :Copyright.   Freeware,  1993 by cs-soft, All Rights Reserved
 :Language.    Oberon-2
 :Translator.  Amiga Oberon 3.01
 :History.     V1.0, 03-Mar-92, first release
 :History.     V1.1, 07-Sep-92, small changes
 :History.     V2.0, 30-Mar-93, uses Hartley Transform now
 :History.           05-Apr-93, dont pokes in screen bitmap directly
 :Imports.     DeviceSupport37, DHT
---------------------------------------------------------------------------*)

MODULE Spectroscope;

  IMPORT
    au := Audio,
          Dos,
    ds := DeviceSupport37,  (* StopWatch, *)
    e  := Exec,
          DHT,
    g  := Graphics,
    gt := GadTools,
    hw := Hardware,
    I  := Intuition,
          Misc,
    rq := Requests,
    str:= Strings,
    tr := MathTrans,
    u  := Utility,
    SYS:= SYSTEM;


  CONST
    version = "\o$VER: spectroscope 2.0 (6.4.93)";


  CONST
    width = 640;
    height = 200;
    depth = 2;

    n = 128;          (* 128 Point FFT *)
    m = n DIV 2;      (* Number levelmeters *)
    d = 10;           (* sinking rate *)

    defaultSampFreq = 16000;
    factorSF = 20;
    defaultLevel = 128;


  VAR
    sintab ["_sintab_1024x16"] : ARRAY 1024 OF INTEGER;


  VAR
    ioa       : au.IOAudioPtr;
    msg       : I.IntuiMessagePtr;
    chan      : SHORTINT;
    per       : INTEGER;
    pp,pb     : e.APTR;
    level     : INTEGER;
    rp        : g.RastPortPtr;
    vp        : g.ViewPortPtr;
    helpBM    : g.BitMapPtr;
    scrBM     : g.BitMapPtr;
    oldpri    : SHORTINT;
    sync      : BOOLEAN;
    i,j       : INTEGER;
    hold,last : ARRAY m+1 OF INTEGER;
    ein, aus  : ARRAY n OF INTEGER;


(*---------------------------------------------------------------------*)

  PROCEDURE RFProc; (* $EntryExitCode- *)
  BEGIN
    SYS.INLINE(016C0U, 04E75U);
  END RFProc;

(*---------------------------------------------------------------------*)

  TYPE
    NewMenuType = ARRAY 4 OF gt.NewMenu;

  VAR
    menu    : I.MenuPtr;
    newMenu : NewMenuType;

  CONST
    NewMenu = NewMenuType(
      gt.title, SYS.ADR("Project"),     NIL,         {},  LONGSET{}, NIL,
      gt.item , SYS.ADR("About..."),    NIL,         {},  LONGSET{}, NIL,
      gt.item , SYS.ADR("Quit"),        SYS.ADR("Q"),{},  LONGSET{}, NIL,
      gt.end  , NIL,                    NIL,         {},  LONGSET{}, NIL);

(*---------------------------------------------------------------------*)
(*---------------------------------------------------------------------*)

(*
 *  Source generated with GadToolsBox V1.4
 *  which is (c) Copyright 1991,92 Jaba Development
 *  Oberon-Sourcecode-Generator by Kai Bolay (AMOK)
 *)


CONST
  GDGadget00  = 0;
  GDGadget10  = 1;
  GDGadget20  = 2;
  GDGadget30  = 3;
  GDGadget40  = 4;

CONST
  Project0CNT = 5;
  Project0Left = 0;
  Project0Top = 11;
  Project0Width = 640;
  Project0Height = 200-11;
VAR
  Scr: I.ScreenPtr;
  VisualInfo: e.APTR;
  Project0Wnd: I.WindowPtr;
  Project0GList: I.GadgetPtr;
  Project0Gadgets: ARRAY Project0CNT OF I.GadgetPtr;

TYPE
  Gadget000LArray = ARRAY 3 OF e.STRPTR;
CONST
  Gadget000Labels = Gadget000LArray (
    SYS.ADR ("Left"),
    SYS.ADR ("Right"),
    NIL );

CONST
  topaz8 = g.TextAttr (SYS.ADR ("topaz.font"), 8, SHORTSET{}, SHORTSET{});

TYPE
  Project0GTypesArray = ARRAY Project0CNT OF INTEGER;
CONST
  Project0GTypes = Project0GTypesArray (
    gt.cycleKind,
    gt.checkBoxKind,
    gt.checkBoxKind,
    gt.sliderKind,
    gt.sliderKind);

TYPE
  Project0NGadArray = ARRAY Project0CNT OF gt.NewGadget;
CONST
  Project0NGad = Project0NGadArray (
    370, 14, 100, 12, SYS.ADR ("Channel"),    NIL, GDGadget00, LONGSET {gt.placeTextAbove,gt.highLabel} ,NIL, NIL,
    568,  2,  26, 11, SYS.ADR ("Low Pri"),    NIL, GDGadget10, LONGSET {gt.placeTextLeft,gt.highLabel} ,NIL, NIL,
    568, 16,  26, 11, SYS.ADR ("Sync"),       NIL, GDGadget20, LONGSET {gt.placeTextLeft,gt.highLabel} ,NIL, NIL,
     30, 14, 150, 12, SYS.ADR ("Freq Range"), NIL, GDGadget30, LONGSET {gt.placeTextAbove,gt.highLabel} ,NIL, NIL,
    200, 14, 150, 12, SYS.ADR ("Level"),      NIL, GDGadget40, LONGSET {gt.placeTextAbove,gt.highLabel} ,NIL, NIL);

TYPE
  Project0GTagsArray = ARRAY 23 OF u.Tag;
CONST
  Project0GTags = Project0GTagsArray (
    gt.cyLabels, SYS.ADR (Gadget000Labels[0]), u.done,
    gt.cbChecked,I.LFALSE,u.done,
    gt.cbChecked,I.LTRUE,u.done,
    gt.slMin, 2000 DIV factorSF, gt.slMax, 32000 DIV factorSF, gt.slLevel, defaultSampFreq DIV factorSF, u.done,
    gt.slMin, 0, gt.slMax, 255, gt.slLevel, defaultLevel, u.done);


PROCEDURE SetupScreen (): INTEGER;
VAR
  rect: g.Rectangle;
  left: INTEGER;
BEGIN
  IF I.QueryOverscan(g.defaultMonitorID+g.hiresKey,rect,I.oScanText)#0 THEN
    left := (rect.maxX-rect.minX+1 - 640) DIV 2;
  ELSE left := 0 END;
  Scr := I.OpenScreenTagsA (NIL,
            I.saLeft,          left,
            I.saWidth,         640,
            I.saHeight,        200,
            I.saDepth,         2,
            I.saFont,          SYS.ADR (topaz8),
            I.saDisplayID,     g.defaultMonitorID+g.hiresKey,
            I.saPens,          SYS.ADR ("\xFF\xFF"),
            I.saTitle,         SYS.ADR (version[7]),
            u.done);
  IF Scr = NIL THEN RETURN 1 END;

  VisualInfo := gt.GetVisualInfo (Scr, u.done);
  IF VisualInfo = NIL THEN RETURN 2 END;

  RETURN 0;
END SetupScreen;

PROCEDURE CloseDownScreen;
BEGIN
  IF VisualInfo # NIL THEN
    gt.FreeVisualInfo (VisualInfo);
    VisualInfo := NIL;
  END;
  IF Scr # NIL THEN
    IF I.CloseScreen (Scr) THEN END;
    Scr := NIL;
  END;
END CloseDownScreen;


PROCEDURE Project0Render;
BEGIN
  gt.DrawBevelBox (Project0Wnd.rPort, 61, 42, 518, 132, gt.visualInfo, VisualInfo, gt.bbRecessed, I.LTRUE, u.done);
END Project0Render;


PROCEDURE OpenProject0Window (): INTEGER;
TYPE
  TagArrayPtr = UNTRACED POINTER TO ARRAY MAX (INTEGER) OF u.TagItem;
VAR
  ng: gt.NewGadget;
  gad: I.GadgetPtr;
  help: TagArrayPtr;
  lc, tc, lvc, offx, offy: INTEGER;
BEGIN
  offx := 0; offy := 2;
  gad := gt.CreateContext (Project0GList);
  IF gad = NIL THEN RETURN 1 END;
  lc := 0; tc := 0; lvc := 0;
  WHILE lc < Project0CNT DO
    ng := Project0NGad[lc];
    ng.visualInfo := VisualInfo;
    ng.textAttr   := SYS.ADR (topaz8);
    INC (ng.leftEdge, offx);
    INC (ng.topEdge, offy);
    help := u.CloneTagItems (SYS.VAL (TagArrayPtr, SYS.ADR (Project0GTags[tc]))^);
    IF help = NIL THEN RETURN 8 END;
    gad := gt.CreateGadgetA (Project0GTypes[lc], gad, ng, help^ );
    u.FreeTagItems (help^);
    IF gad = NIL THEN RETURN 2 END;
    Project0Gadgets[lc] := gad;
    WHILE Project0GTags[tc] # u.done DO INC (tc, 2) END;
    INC (tc);
    INC (lc);
  END; (* WHILE *)
  Project0Wnd := I.OpenWindowTagsA ( NIL,
                    I.waLeft,          Project0Left,
                    I.waTop,           Project0Top,
                    I.waWidth,         Project0Width,
                    I.waHeight,        Project0Height,
                    I.waIDCMP,         gt.cycleIDCMP+gt.checkBoxIDCMP+gt.sliderIDCMP+LONGSET {I.refreshWindow,I.gadgetUp,I.mouseMove,I.menuPick},
                    I.waFlags,         LONGSET {I.activate,I.borderless,I.backDrop},
                    I.waGadgets,       Project0GList,
                    I.waCustomScreen,  Scr,
                    u.done);
  IF Project0Wnd = NIL THEN RETURN 20 END;
  gt.RefreshWindow (Project0Wnd, NIL);

  newMenu := NewMenu;
  menu := gt.CreateMenus(newMenu,u.done);
  IF menu=NIL THEN RETURN 21 END;
  IF ~gt.LayoutMenus(menu,VisualInfo,u.done) THEN RETURN 22 END;
  IF ~I.SetMenuStrip(Project0Wnd,menu^) THEN RETURN 23 END;

  Project0Render;

  RETURN 0;
END OpenProject0Window;

PROCEDURE CloseProject0Window;
BEGIN
  IF menu # NIL THEN gt.FreeMenus(menu) END;

  IF Project0Wnd # NIL THEN
    IF  Project0Wnd.menuStrip#NIL THEN I.ClearMenuStrip(Project0Wnd) END;
    I.CloseWindow (Project0Wnd);
    Project0Wnd := NIL;
  END;
  IF Project0GList # NIL THEN
    gt.FreeGadgets (Project0GList);
    Project0GList := NIL;
  END;
END CloseProject0Window;

(*---------------------------------------------------------------------*)
(*---------------------------------------------------------------------*)

  PROCEDURE Muls (i{0},j{1}: INTEGER): LONGINT; (* $EntryExitCode- *)
  BEGIN
    SYS.INLINE(0C1C1H,04E75H);   (* MULS D1,D0 ; RTS *)
  END Muls;

(*---------------------------------------------------------------------*)

  PROCEDURE InitMeters(nr: INTEGER; byte: SHORTSET);

    VAR i,k: INTEGER;
        p,p0: UNTRACED POINTER TO SHORTSET;
        bpr: INTEGER;
  BEGIN
    p0 := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,helpBM.planes[nr]));
    bpr := helpBM.bytesPerRow;
    FOR i := 1 TO m DO
      p := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p0));
      k := 128;
      REPEAT
        DEC(k);
        p^ := byte;
        p := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p) + bpr);
      UNTIL k=0;
      p0 := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p0) + 1);
    END;
  END InitMeters;


  PROCEDURE DrawMeters;

    VAR i,j,k: INTEGER;
        p,p0: UNTRACED POINTER TO SHORTSET;
        bpr: INTEGER;
  BEGIN
    p0 := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,helpBM.planes[1]));
    bpr := helpBM.bytesPerRow;

    i := 0;
    REPEAT
      INC(i);

      j := SHORT(Muls(aus[i],level) DIV 8192);
      IF j > 128 THEN j := 128 END;
      IF j > hold[i] THEN hold[i] := j ELSE j := hold[i] END;

      IF j > last[i] THEN
        p := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p0) + Muls(bpr,(128-j)));
        k := j-last[i];
        REPEAT
          p^ := SHORTSET{0,1,2,3,4,5,6};
          p := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p) + bpr);
          DEC(k);
        UNTIL k=0;
      ELSIF j < last[i] THEN
        p := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p0) + Muls(bpr,(128-last[i])));
        k := last[i]-j;
        REPEAT
          p^ := SHORTSET{};
          p := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p) + bpr);
          DEC(k);
        UNTIL k=0;
      END;

      last[i] := j;
      DEC(hold[i],d);

      p0 := SYS.VAL(SYS.PTR,SYS.VAL(LONGINT,p0) + 1);
    UNTIL i=m;
    SYS.SETREG(0,g.BltBitMap(helpBM, 0,0, scrBM, 8*8,55, 512,128 , 0C0X, SHORTSET{1}, NIL));

  END DrawMeters;

(*---------------------------------------------------------------------*)

  PROCEDURE Render;

    VAR l   : LONGINT;
        i   : INTEGER;
        num : ARRAY 16 OF CHAR;
  BEGIN
    InitMeters(0,SHORTSET{0,1,2,3,4,5,6,7});
    InitMeters(1,SHORTSET{});

    SYS.SETREG(0,g.BltBitMap(helpBM, 0,0, scrBM, 8*8,55, 512,128 , 0C0X, SHORTSET{0..1}, NIL));
    g.WaitBlit;

    g.SetDrMd(rp,g.jam2);
    g.SetBPen(rp,0);
  
    FOR i := 0 TO 8 DO
      l := -i*5;
      IF (i<4) OR ~ODD(i) THEN
        e.RawDoFmt("%3ld",SYS.ADR(l),RFProc,SYS.ADR(num));
        j := 44 + 128 - SHORT(ENTIER(128*tr.Exp(l * (2.302585/40))+1/2));
        IF i>0 THEN g.SetAPen(rp,0); g.Move(rp,64,j); g.Draw(rp,512+64,j); END;
        g.SetAPen(rp,1); g.Move(rp,34,j+3); g.Text(rp,num,str.Length(num));
      END;
    END;
    g.SetAPen(rp,1);
    g.Move(rp,020,47); g.Text(rp,"dB",2);
    g.Move(rp,600,184); g.Text(rp,"Hz",2);
  END Render;

(*---------------------------------------------------------------------*)

  CONST
    left  = 0;
    right = 1;

  PROCEDURE SetChannel(ch:INTEGER);

  BEGIN
    IF ch=left THEN INCL(hw.ciab.pra,1); EXCL(hw.ciab.pra,2)
               ELSE EXCL(hw.ciab.pra,1); INCL(hw.ciab.pra,2) END;
  END SetChannel;

(*---------------------------------------------------------------------*)

  PROCEDURE SetSampFreq(freq: LONGINT);

    VAR num: ARRAY 16 OF CHAR;
        l: LONGINT;
        i: INTEGER;
        f: REAL;
        len: INTEGER;

  BEGIN
    per := SHORT(ENTIER(3546895 / 2 / freq  + 1/2));

    FOR i := 0 TO 4 DO
      l := ENTIER(freq*(i/8)+1/2);
      e.RawDoFmt(" %ld ",SYS.ADR(l),RFProc,SYS.ADR(num));
      len := SHORT(str.Length(num));
      g.Move(rp,60-len*4+i*(512 DIV 4),184);
      g.Text(rp,num,len);
    END;

  END SetSampFreq;

(*---------------------------------------------------------------------*)

  PROCEDURE Record;

    VAR i,j: INTEGER;
        prb: SHORTINT;
        audci: SHORTINT;
        offs: INTEGER;
  BEGIN
    (* $RangeChk- $OvflChk- $NilChk- *)

    audci := hw.aud0i + chan;

    (* use audio channel for timing: *)

    hw.custom.intena := {audci};    (* disable audio interrupt *)

    hw.custom.aud[chan].ptr := NIL;
    hw.custom.aud[chan].len := 1;   (* one word *)
    hw.custom.aud[chan].per := per;
    hw.custom.aud[chan].vol := 0;

    hw.custom.dmacon := {hw.aud0+chan}; (* DMA off *)

    g.WaitBlit;

    e.Disable;

    hw.custom.intreq := {audci};
    hw.custom.aud[chan].dat := 0;
    prb := SYS.VAL(SHORTINT,hw.ciaa.prb) + SYS.VAL(SHORTINT,-128);

    i := 0;
    REPEAT
      REPEAT UNTIL audci IN hw.custom.intreqr; (* Wait on audio interrupt *)
      hw.custom.intreq := {audci};             (* reset interrupt bit *)
      hw.custom.aud[chan].dat := 0;            (* write to data register *)

      (* read the parallel port: *)

      prb := SYS.VAL(SHORTINT,hw.ciaa.prb) + SYS.VAL(SHORTINT,-128);

      ein[i] := LONG(prb);
      INC(i);
    UNTIL i=n;

    e.Enable;

    (* Get offset: *)

    offs := 0;
    i := n;
    REPEAT
      DEC(i);
      INC(offs,ein[i]);
    UNTIL i=0;

    offs := offs DIV n;

    (* Multiply with hamming function: *)

    i := n;
    REPEAT
      DEC(i);
      j := i * (1024 DIV n) + 256;
      ein[i] := (ein[i]-offs) * (64-sintab[j] DIV 512);
    UNTIL i=0;

    (* $RangeChk= $OvflChk= $NilChk= *)

  END Record;

(*---------------------------------------------------------------------*)

  PROCEDURE Analysis;
  BEGIN
    DHT.DHT(ein,aus,n);      (* Call Hartley Transform *)
    DHT.Abs2(aus,n);         (* Calc Spectrum *)
  END Analysis;

(*---------------------------------------------------------------------*)

  PROCEDURE HandleMessage(msg: I.IntuiMessagePtr);

    CONST
      ES = I.EasyStruct(SIZE(I.EasyStruct),LONGSET{},
                        SYS.ADR("Information"),
                        SYS.ADR("     Spectroscope 2.0\n 1993 by Christian Stiens"),
                        SYS.ADR("Ok"));

    VAR
      gad   : I.GadgetPtr;
      code  : INTEGER;
      class : LONGSET;
      item  : I.MenuItemPtr;

  BEGIN
    class := msg.class;
    code  := msg.code;
    gad   := msg.iAddress;

    gt.ReplyIMsg(msg);

    IF I.refreshWindow IN class THEN
      gt.BeginRefresh(Project0Wnd);
      gt.EndRefresh(Project0Wnd,I.LTRUE);
    END;

    IF I.gadgetUp IN class THEN
      CASE gad.gadgetID OF
      | GDGadget00: SetChannel(code);
      | GDGadget10: IF I.selected IN gad.flags THEN
                      IF e.SetTaskPri(e.FindTask(NIL),oldpri-5)=0 THEN END;;
                    ELSE
                      IF e.SetTaskPri(e.FindTask(NIL),oldpri)=0 THEN END;
                    END;
      | GDGadget20: sync := I.selected IN gad.flags;
      | GDGadget30: SetSampFreq(code*factorSF);
      | GDGadget40: level := code;
      ELSE
      END;
    END;

    IF I.mouseMove IN class THEN
      CASE gad.gadgetID OF
      | GDGadget30: SetSampFreq(code*factorSF);
      | GDGadget40: level := code;
      ELSE
      END;
    END;

    IF I.menuPick IN class THEN
      WHILE code # I.menuNull DO
        item := I.ItemAddress(menu^, code);
        CASE I.ItemNum(code) OF
        | 0: IF I.EasyRequest(Project0Wnd,SYS.ADR(ES),NIL,NIL)=0 THEN END;
        | 1: HALT(0);
        ELSE
        END;
        code := item.nextSelect;
      END;
    END;

  END HandleMessage;

(*---------------------------------------------------------------------*)

  PROCEDURE IoInit (ioa: e.MessagePtr);
    CONST allocMap = "\x01\x02\x04\x08";
  BEGIN
    WITH ioa: au.IOAudioPtr DO
      ioa.data := SYS.ADR(allocMap);
      ioa.length := 4;
      ioa.request.message.node.pri := 127;
    END;
  END IoInit;

(*---------------------------------------------------------------------*)

  PROCEDURE s(x: REAL): REAL;
    VAR y: REAL;
  BEGIN
    WHILE x >= 6 DO x := x-6 END;
    IF x < 3 THEN y := 2-x ELSE y := x-4 END;
    IF y < 0 THEN y := 0 ELSIF y > 1 THEN y := 1 END;
    RETURN y
  END s;

  PROCEDURE LoadCopper(vp: g.ViewPortPtr);
    CONST
      num = 32;
    VAR
      i,j,col: INTEGER;
      cl: g.UCopListPtr;
  BEGIN
    IF vp.uCopIns # NIL THEN RETURN END;
    cl := e.AllocMem(SIZE(cl^),LONGSET{e.public,e.memClear});
    IF cl=NIL THEN HALT(0) END;
    g.CINIT(cl,2*num+4);
    FOR i := 0 TO num-1 DO
      g.CWAIT(cl, i*(128 DIV num)+55,0);
      col := 0;
      FOR j := 2 TO 0 BY -1 DO
        col := col * 16 + SHORT(ENTIER(15.9*s(i*(6/num)+j*2)));
      END;
      g.CMOVE(cl,SYS.ADR(hw.custom.color[3]),col);
    END;
    g.CEND (cl);
    e.Forbid; vp.uCopIns := cl; e.Permit;
    IF g.VideoControlTagList(vp.colorMap,g.vTagUserClipSet,0,g.vTagEndCM,0,u.done) THEN END;
    I.RethinkDisplay;
  END LoadCopper;

(*---------------------------------------------------------------------*)

  PROCEDURE InitHelpBM;
    VAR i: INTEGER;
        p: g.PLANEPTR;
  BEGIN
    NEW(helpBM);
    g.InitBitMap(helpBM^,depth,width,height);
    FOR i := 0 TO depth-1 DO
      p := g.AllocRaster(width,height);
      IF p=NIL THEN HALT(Dos.fail) END;
      helpBM.planes[i] := p;
    END;
  END InitHelpBM;

(*---------------------------------------------------------------------*)

  PROCEDURE FreeHelpBM;
    VAR i: INTEGER;
        p: g.PLANEPTR;
  BEGIN
    IF helpBM # NIL THEN
      g.WaitBlit;
      FOR i := 0 TO depth-1 DO
        p := helpBM.planes[i];
        IF p # NIL THEN
          g.FreeRaster(p,width,height);
        END;
      END;
    END;
  END FreeHelpBM;

(*---------------------------------------------------------------------*)

BEGIN
  ioa := NIL;
  per := 100;
  pp := -1;
  pb := -1;
  sync := TRUE;
  oldpri := e.exec.thisTask.node.pri;

  IF I.int.libNode.version < 37 THEN HALT(Dos.error) END;

  Misc.base := e.OpenResource(Misc.miscName);
  IF Misc.base=NIL THEN HALT(0) END;

  pp := Misc.AllocMiscResource(Misc.parallelPort,"Spectroscope");
  pb := Misc.AllocMiscResource(Misc.parallelBits,"Spectroscope");

  rq.Assert((pp=NIL) & (pb=NIL),"Can't alloc parallel port");

  rq.Assert(SetupScreen()=0,"Can't open screen");
  rq.Assert(OpenProject0Window()=0,"Can't open window");

  scrBM := Scr.rastPort.bitMap;
  InitHelpBM;

  rp := Project0Wnd.rPort;
  vp := I.ViewPortAddress(Project0Wnd);
  LoadCopper(vp);

  Render;


  ioa := ds.OpenDev(au.audioName,0,LONGSET{},SIZE(au.IOAudio),IoInit);
  rq.Assert(ioa # NIL,"Can't open audio.device");

  CASE SYS.VAL(LONGINT,ioa.request.unit) OF
    1: chan := 0 |
    2: chan := 1 |
    4: chan := 2 |
    8: chan := 3     ELSE HALT(0)
  END;

  hw.ciaa.ddrb := SHORTSET{};                   (* All Bits are input *)
  hw.ciab.ddra := hw.ciab.ddra + SHORTSET{1,2}; (* Bit 1 & 2 are output *)


  SetChannel(left);
  SetSampFreq(defaultSampFreq);
  level := defaultLevel;


  LOOP
    (* StopWatch.Start(); *)

    msg := gt.GetIMsg(Project0Wnd.userPort);
    IF msg # NIL THEN HandleMessage(msg) END;

    Record;

    Analysis;

    DrawMeters;

    IF sync THEN g.WaitTOF END;

    (* Dos.PrintF("%ld / 10 fps\n",10000000 DIV StopWatch.Stop()); *)
  END;

CLOSE

  oldpri := e.SetTaskPri(e.FindTask(NIL),oldpri);
  IF (vp#NIL)&(vp.uCopIns#NIL) THEN g.FreeVPortCopLists(vp); I.RemakeDisplay END;
  FreeHelpBM;
  IF (pp=NIL)&(pb=NIL) THEN hw.ciab.ddra := hw.ciab.ddra-SHORTSET{1,2} END;
  ds.CloseDev(ioa);
  CloseProject0Window;
  CloseDownScreen;
  IF pp=NIL THEN Misc.FreeMiscResource(Misc.parallelPort) END;
  IF pb=NIL THEN Misc.FreeMiscResource(Misc.parallelBits) END;

END Spectroscope.

